Package: scRNAseq
Title: Collection of Public Single-Cell RNA-Seq Datasets
Version: 2.24.0
Date: 2025-10-10
Authors@R: c(
    person("Davide", "Risso", email = "risso.davide@gmail.com", role = c("aut", "cph")),
    person("Michael", "Cole", role="aut"),
    person("Aaron", "Lun", role=c("ctb", "cre"), email="infinite.monkeys.with.keyboards@gmail.com"),
    person("Alan", "O'Callaghan", role="ctb"),
    person("Jens", "Preussner", role="ctb"),
    person("Charlotte", "Soneson", role="ctb"),
    person("Stephany", "Orjuela", role="ctb"),
    person("Daniel", "Bunis", role="ctb"),
    person("Milan", "Malfait", role="ctb"))
Description: Gene-level counts for a collection of public scRNA-seq datasets,
    provided as SingleCellExperiment objects with cell- and gene-level metadata.
License: CC0
NeedsCompilation: no
Depends: SingleCellExperiment
Imports: utils, methods, Matrix, BiocGenerics, S4Vectors, SparseArray,
        DelayedArray, GenomicRanges, SummarizedExperiment,
        ExperimentHub (>= 2.3.4), AnnotationHub (>= 3.3.6),
        AnnotationDbi, ensembldb, GenomicFeatures, alabaster.base,
        alabaster.matrix, alabaster.sce, gypsum, jsonlite, DBI, RSQLite
Suggests: BiocStyle, knitr, rmarkdown, testthat, jsonvalidate,
        BiocManager
VignetteBuilder: knitr
Encoding: UTF-8
biocViews: ExperimentHub, ExperimentData, ExpressionData,
        SequencingData, RNASeqData, SingleCellData
BuildResaveData: no
RoxygenNote: 7.3.2
git_url: https://git.bioconductor.org/packages/scRNAseq
git_branch: RELEASE_3_22
git_last_commit: 111920c
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-11-04
Packaged: 2025-11-04 16:43:14 UTC; biocbuild
Author: Davide Risso [aut, cph],
  Michael Cole [aut],
  Aaron Lun [ctb, cre],
  Alan O'Callaghan [ctb],
  Jens Preussner [ctb],
  Charlotte Soneson [ctb],
  Stephany Orjuela [ctb],
  Daniel Bunis [ctb],
  Milan Malfait [ctb]
Maintainer: Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>

# Generated by roxygen2: do not edit by hand

export(AztekinTailData)
export(BachMammaryData)
export(BacherTCellData)
export(BaronPancreasData)
export(BhaduriOrganoidData)
export(BuettnerESCData)
export(BunisHSPCData)
export(CampbellBrainData)
export(ChenBrainData)
export(DarmanisBrainData)
export(ERCCSpikeInConcentrations)
export(ErnstSpermatogenesisData)
export(FletcherOlfactoryData)
export(GiladiHSCData)
export(GrunHSCData)
export(GrunPancreasData)
export(HeOrganAtlasData)
export(HermannSpermatogenesisData)
export(HuCortexData)
export(JessaBrainData)
export(KolodziejczykESCData)
export(KotliarovPBMCData)
export(LaMannoBrainData)
export(LawlorPancreasData)
export(LedergorMyelomaData)
export(LengESCData)
export(LunSpikeInData)
export(MacoskoRetinaData)
export(MairPBMCData)
export(MarquesBrainData)
export(MessmerESCData)
export(MuraroPancreasData)
export(NestorowaHSCData)
export(NowakowskiCortexData)
export(PaulHSCData)
export(PollenGliaData)
export(ReprocessedAllenData)
export(ReprocessedFluidigmData)
export(ReprocessedTh2Data)
export(RichardTCellData)
export(RomanovBrainData)
export(SegerstolpePancreasData)
export(ShekharRetinaData)
export(StoeckiusHashingData)
export(TasicBrainData)
export(UsoskinBrainData)
export(WuKidneyData)
export(XinPancreasData)
export(ZeiselBrainData)
export(ZeiselNervousData)
export(ZhaoImmuneLiverData)
export(ZhongPrefrontalData)
export(ZilionisLungData)
export(countErccMolecules)
export(defineTextQuery)
export(fetchDataset)
export(fetchLatestVersion)
export(fetchMetadata)
export(gsc)
export(listDatasets)
export(listPaths)
export(listVersions)
export(polishDataset)
export(saveDataset)
export(searchDatasets)
export(surveyDatasets)
import(methods)
importClassesFrom(GenomicRanges,GRangesList)
importClassesFrom(Matrix,dgCMatrix)
importClassesFrom(Matrix,lgCMatrix)
importClassesFrom(SparseArray,SVT_SparseMatrix)
importClassesFrom(alabaster.matrix,ReloadedArray)
importClassesFrom(ensembldb,EnsDb)
importFrom(AnnotationDbi,keys)
importFrom(AnnotationDbi,mapIds)
importFrom(AnnotationDbi,select)
importFrom(AnnotationHub,AnnotationHub)
importFrom(BiocGenerics,cbind)
importFrom(BiocGenerics,mean)
importFrom(DBI,dbConnect)
importFrom(DBI,dbDisconnect)
importFrom(DBI,dbGetQuery)
importFrom(DelayedArray,"type<-")
importFrom(DelayedArray,DelayedArray)
importFrom(DelayedArray,is_sparse)
importFrom(DelayedArray,type)
importFrom(ExperimentHub,ExperimentHub)
importFrom(GenomicFeatures,genes)
importFrom(RSQLite,SQLite)
importFrom(S4Vectors,"mcols<-")
importFrom(S4Vectors,DataFrame)
importFrom(SingleCellExperiment,"altExp<-")
importFrom(SingleCellExperiment,"altExps<-")
importFrom(SingleCellExperiment,"colPairs<-")
importFrom(SingleCellExperiment,"reducedDim<-")
importFrom(SingleCellExperiment,"reducedDims<-")
importFrom(SingleCellExperiment,SingleCellExperiment)
importFrom(SingleCellExperiment,altExp)
importFrom(SingleCellExperiment,altExpNames)
importFrom(SingleCellExperiment,altExps)
importFrom(SingleCellExperiment,reducedDim)
importFrom(SingleCellExperiment,reducedDimNames)
importFrom(SingleCellExperiment,splitAltExps)
importFrom(SummarizedExperiment,"assay<-")
importFrom(SummarizedExperiment,"assayNames<-")
importFrom(SummarizedExperiment,"assays<-")
importFrom(SummarizedExperiment,"colData<-")
importFrom(SummarizedExperiment,"rowData<-")
importFrom(SummarizedExperiment,"rowRanges<-")
importFrom(SummarizedExperiment,assay)
importFrom(SummarizedExperiment,assayNames)
importFrom(SummarizedExperiment,assays)
importFrom(SummarizedExperiment,colData)
importFrom(SummarizedExperiment,rowData)
importFrom(SummarizedExperiment,rowRanges)
importFrom(alabaster.base,altReadObject)
importFrom(alabaster.base,altReadObjectFunction)
importFrom(alabaster.base,readObject)
importFrom(alabaster.base,readObjectFile)
importFrom(alabaster.base,saveObject)
importFrom(gypsum,cacheDirectory)
importFrom(gypsum,defineTextQuery)
importFrom(gypsum,fetchLatest)
importFrom(gypsum,fetchMetadataDatabase)
importFrom(gypsum,fetchMetadataSchema)
importFrom(gypsum,formatObjectMetadata)
importFrom(gypsum,gsc)
importFrom(gypsum,listFiles)
importFrom(gypsum,saveFile)
importFrom(gypsum,saveVersion)
importFrom(gypsum,searchMetadataFilter)
importFrom(gypsum,translateTextQuery)
importFrom(gypsum,validateMetadata)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(methods,as)
importFrom(utils,head)
importFrom(utils,read.csv)
importFrom(utils,read.delim)
importMethodsFrom(alabaster.sce,saveObject)

<!-- These are examples of badges you might want to add to your README:
     please update the URLs accordingly

[![Built Status](https://api.cirrus-ci.com/github/<USER>/scrnaseq.svg?branch=main)](https://cirrus-ci.com/github/<USER>/scrnaseq)
[![ReadTheDocs](https://readthedocs.org/projects/scrnaseq/badge/?version=latest)](https://scrnaseq.readthedocs.io/en/stable/)
[![Coveralls](https://img.shields.io/coveralls/github/<USER>/scrnaseq/main.svg)](https://coveralls.io/r/<USER>/scrnaseq)
[![PyPI-Server](https://img.shields.io/pypi/v/scrnaseq.svg)](https://pypi.org/project/scrnaseq/)
[![Conda-Forge](https://img.shields.io/conda/vn/conda-forge/scrnaseq.svg)](https://anaconda.org/conda-forge/scrnaseq)
[![Monthly Downloads](https://pepy.tech/badge/scrnaseq/month)](https://pepy.tech/project/scrnaseq)
[![Twitter](https://img.shields.io/twitter/url/http/shields.io.svg?style=social&label=Twitter)](https://twitter.com/scrnaseq)
-->

[![Project generated with PyScaffold](https://img.shields.io/badge/-PyScaffold-005CA0?logo=pyscaffold)](https://pyscaffold.org/)
[![PyPI-Server](https://img.shields.io/pypi/v/scrnaseq.svg)](https://pypi.org/project/scrnaseq/)

# scrnaseq

The `scrnaseq` package provides convenient access to several publicly available single-cell datasets in the form of [SingleCellExperiment](https://github.com/biocpy/singlecellexperiment) objects. Users can obtain a `SingleCellExperiment` and transform it into analysis-ready representations for immediate use.

To enable discovery, each dataset is decorated with metadata such as the study title/abstract, the species, the number of cells, etc. Users can also contribute their own published datasets to enable re-use by the wider Bioconductor/BiocPy community.

**Also check out the R version of this library [here@scRNAseq](https://bioconductor.org/packages/devel/data/experiment/html/scRNAseq.html) published to Bioconductor.**

## Installation

Package is published to [PyPI](https://pypi.org/project/scrnaseq/),

```shell
pip install scrnaseq
```

## Find Datasets

The `list_datasets()` function will display all available datasets along with their metadata. This can be used to discover interesting datasets for further analysis.

```python
import scrnaseq
datasets = scrnaseq.list_datasets()
print(datasets[["name", "version"]].head(3))

## output
# |    | name                  | version    |
# |---:|:----------------------|:-----------|
# |  0 | romanov-brain-2017    | 2023-12-19 |
# |  1 | campbell-brain-2017   | 2023-12-14 |
# |  2 | zhong-prefrontal-2018 | 2023-12-22 |
```

This returns a pandas `DataFrame` to easily filter and download datasets of interest.

Users can also search on the metadata text using the `search_datasets()` function. This accepts both simple text queries as well as more complicated expressions involving boolean operations.

```python
# Find all datasets involving pancreas.
res = scrnaseq.search_datasets("pancreas")

# Find all mm10 datasets involving pancreas or neurons.
from gypsum_client import define_text_query

res = scrnaseq.search_datasets(
     define_text_query("GRCm38", field="genome")
     & (
          define_text_query("neuro%", partial=True)
          | define_text_query("pancrea%", partial=True)
     )
)
print(res[["name", "version"]].head(3))

## output
# |    | name                    | version    |
# |---:|:------------------------|:-----------|
# |  0 | romanov-brain-2017      | 2023-12-19 |
# |  1 | campbell-brain-2017     | 2023-12-14 |
# |  2 | fletcher-olfactory-2019 | 2023-12-21 |
```

Search results are not guaranteed to be reproducible - more datasets may be added over time, and existing datasets may be updated with new versions. Once a dataset of interest is identified, users should explicitly list the name and version of the dataset in their scripts to ensure reproducibility.

## Fetch Datasets

The `fetch_dataset()` function will download a particular dataset, as `SingleCellExperiment`:

```python
sce = scrnaseq.fetch_dataset("zeisel-brain-2015", "2023-12-14")
print(sce)

## output
# class: SingleCellExperiment
# dimensions: (20006, 3005)
# assays(1): ['counts']
# row_data columns(1): ['featureType']
# row_names(20006): ['Tspan12', 'Tshz1', 'Fnbp1l', ..., 'mt-Rnr2', 'mt-Rnr1', 'mt-Nd4l']
# column_data columns(9): ['tissue', 'group #', 'total mRNA mol', 'well', 'sex', 'age', 'diameter', 'level1class', 'level2class']
# column_names(3005): ['1772071015_C02', '1772071017_G12', '1772071017_A05', ..., '1772063068_D01', '1772066098_A12', '1772058148_F03']
# main_experiment_name: gene
# reduced_dims(0): []
# alternative_experiments(2): ['repeat', 'ERCC']
# row_pairs(0): []
# column_pairs(0): []
# metadata(0):
```

For studies that generate multiple datasets, the dataset of interest must be explicitly requested via the `path` argument:

```python
sce = scrnaseq.fetch_dataset("baron-pancreas-2016", "2023-12-14", path="human")
print(sce)

## output
# class: SingleCellExperiment
# dimensions: (20125, 8569)
# assays(1): ['counts']
# row_data columns(0): []
# row_names(20125): ['A1BG', 'A1CF', 'A2M', ..., 'ZZEF1', 'ZZZ3', 'pk']
# column_data columns(2): ['donor', 'label']
# column_names(8569): ['human1_lib1.final_cell_0001', 'human1_lib1.final_cell_0002', 'human1_lib1.final_cell_0003', ..., 'human4_lib3.final_cell_0699', 'human4_lib3.final_cell_0700', 'human4_lib3.final_cell_0701']
# main_experiment_name:
# reduced_dims(0): []
# alternative_experiments(0): []
# row_pairs(0): []
# column_pairs(0): []
# metadata(0):
```

By default, array data is loaded as a file-backed `DelayedArray` from the [HDF5Array](https://github.com/BiocPy/HDF5Array) package. Setting `realize_assays=True` and/or `realize_reduced_dims=True` will coerce file-backed arrays to numpy or scipy sparse (csr/csc) objects.

```python
sce = scrnaseq.fetch_dataset("baron-pancreas-2016", "2023-12-14", path="human", realize_assays=True)
print(sce)

## output
# class: SingleCellExperiment
# dimensions: (20125, 8569)
# assays(1): ['counts']
# row_data columns(0): []
# row_names(20125): ['A1BG', 'A1CF', 'A2M', ..., 'ZZEF1', 'ZZZ3', 'pk']
# column_data columns(2): ['donor', 'label']
# column_names(8569): ['human1_lib1.final_cell_0001', 'human1_lib1.final_cell_0002', 'human1_lib1.final_cell_0003', ..., 'human4_lib3.final_cell_0699', 'human4_lib3.final_cell_0700', 'human4_lib3.final_cell_0701']
# main_experiment_name:
# reduced_dims(0): []
# alternative_experiments(0): []
# row_pairs(0): []
# column_pairs(0): []
# metadata(0):
```

Users can also fetch the metadata associated with each dataset:

```python
meta = scrnaseq.fetch_metadata("zeisel-brain-2015", "2023-12-14")
print(meta.keys())

## output
# dict_keys(['title', 'description', 'taxonomy_id', 'genome', 'sources', 'maintainer_name', 'maintainer_email', 'bioconductor_version', 'applications'])
```


## Adding New Datasets

Want to contribute your own dataset to this package? It's easy! Just follow these simple steps:

1. Format your dataset as a `SummarizedExperiment` or `SingleCellExperiment`. Let's mock a dataset:

     ```python
     import numpy as np
     from singlecellexperiment import SingleCellExperiment
     from biocframe import BiocFrame

     mat = np.random.poisson(1, (100, 10))
     row_names = [f"GENE_{i}" for i in range(mat.shape[0])]
     col_names = list("ABCDEFGHIJ")
     sce = SingleCellExperiment(
          assays={"counts": mat},
          row_data=BiocFrame(row_names=row_names),
          column_data=BiocFrame(row_names=col_names),
     )
     ```

2. Assemble the metadata for your dataset. This should be a dictionary as specified in the [Bioconductor metadata schema](https://github.com/ArtifactDB/bioconductor-metadata-index). Check out some examples from `fetch_metadata()`  Note that the `application_takane` property will be automatically added later, and so can be omitted from the list that you create.

     ```python
     meta = {
          "title": "My dataset forked from ziesel brain",
          "description": "This is a copy of the ziesel",
          "taxonomy_id": ["10090"],  # NCBI ID
          "genome": ["GRCh38"],  # genome build
          "sources": [{"provider": "GEO", "id": "GSE12345"}],
          "maintainer_name": "Shizuka Mogami",
          "maintainer_email": "mogami.shizuka@765pro.com",
     }
     ```

3. Save your `SummarizedExperiment` or `SingleCellExperiment` object to disk with `save_dataset()`. This saves the dataset into a "staging directory" using language-agnostic file formats - check out the [ArtifactDB](https://github.com/artifactdb) framework for more details. In more complex cases involving multiple datasets, users may save each dataset into a subdirectory of the staging directory.

     ```python
     import tempfile
     from scrnaseq import save_dataset

     # replace tmp with a staging directory
     staging_dir = tempfile.mkdtemp()
     save_dataset(sce, staging_dir, meta)
     ```

     You can check that everything was correctly saved by reloading the on-disk data for inspection:

     ```python
     import dolomite_base as dl

     dl.read_object(staging_dir)
     ```

4. Open a [pull request (PR)](https://github.com/BiocPy/scRNAseq/pulls) for the addition of a new dataset. You will need to provide a few things here:
   - The name of your dataset. This typically follows the format of `{NAME}-{SYSTEM}-{YEAR}`, where `NAME` is the last name of the first author of the study, `SYSTEM` is the biological system (e.g., tissue, cell types) being studied, and `YEAR` is the year of publication for the dataset.
   - The version of your dataset. This is usually just the current date or whenever you started putting together the dataset for upload. The exact date doesn't really matter as long as we can establish a timeline for later versions.
   - An Python file containing the code used to assemble the dataset. This should be added to the [`scripts/`](https://github.com/BiocPy/scRNAseq/tree/master/scripts) directory of this package, in order to provide some record of how the dataset was created.

5. Wait for us to grant temporary upload permissions to your GitHub account.

6. Upload your staging directory to [**gypsum** backend](https://github.com/ArtifactDB/gypsum-worker) with `upload_dataset()`. On the first call to this function, it will automatically prompt you to log into GitHub so that the backend can authenticate you. If you are on a system without browser access (e.g., most computing clusters), a [token](https://github.com/settings/tokens) can be manually supplied via `set_access_token()`.

     ```python
     from scrnaseq import upload_dataset

     upload_dataset(staging_dir, "my_dataset_name", "my_version")
     ```

     You can check that everything was successfully uploaded by calling `fetch_dataset()` with the same name and version:

     ```python
     from scrnaseq import upload_dataset

     fetch_dataset("my_dataset_name", "my_version")
     ```

     If you realized you made a mistake, no worries. Use the following call to clear the erroneous dataset, and try again:

     ```python
     from gypsum_client import reject_probation

     reject_probation("scRNAseq", "my_dataset_name", "my_version")
     ```

7. Comment on the PR to notify us that the dataset has finished uploading and you're happy with it. We'll review it and make sure everything's in order. If some fixes are required, we'll just clear the dataset so that you can upload a new version with the necessary changes. Otherwise, we'll approve the dataset. Note that once a version of a dataset is approved, no further changes can be made to that version; you'll have to upload a new version if you want to modify something.

<!-- pyscaffold-notes -->

## Note

The tests for upload are skipped. To run them, include a GitHub token as an environment variable `gh_token`.

This project has been set up using PyScaffold 4.5. For details and usage
information on PyScaffold see https://pyscaffold.org/.

#' Survey of dataset metadata
#'
#' Metadata survey for all available datasets in the \pkg{scRNAseq} package.
#'
#' @param cache,overwrite Arguments to pass to \code{\link{fetchMetadataDatabase}}.
#' @param latest Whether to only consider the latest version of each dataset.
#'
#' @return 
#' A \linkS4class{DataFrame} where each row corresponds to a dataset, containing various columns of metadata.
#' Some columns may be lists to capture 1:many mappings.
#'
#' @details
#' The returned DataFrame contains the usual suspects like the title and description for each dataset,
#' the number of rows and columns, the organisms and genome builds involved,
#' whether the dataset has any pre-computed reduced dimensions, and so on.
#' More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 
#'
#' @author Aaron Lun
#'
#' @examples
#' surveyDatasets()
#'
#' @seealso
#' \code{\link{searchDatasets}}, to search on the metadata for specific datasets.
#' 
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom gypsum cacheDirectory fetchMetadataDatabase
#' @importFrom DBI dbConnect dbDisconnect dbGetQuery
#' @importFrom RSQLite SQLite
surveyDatasets <- function(cache=cacheDirectory(), overwrite=FALSE, latest=TRUE) {
    bpath <- fetchMetadataDatabase(cache=cache, overwrite=overwrite)
    con <- dbConnect(SQLite(), bpath)
    on.exit(dbDisconnect(con))

    stmt <- "SELECT json_extract(metadata, '$') AS meta, versions.asset AS asset, versions.version AS version, path";
    if (!latest) {
        stmt <- paste0(stmt, ", versions.latest AS latest")
    }
    stmt <- paste0(stmt, " FROM paths LEFT JOIN versions ON paths.vid = versions.vid WHERE versions.project = 'scRNAseq'")
    if (latest) {
        stmt <- paste0(stmt, " AND versions.latest = 1")
    }
    everything <- dbGetQuery(con, stmt)

    sanitize_query_to_output(everything, latest)
}

#' @importFrom S4Vectors DataFrame
#' @importFrom jsonlite fromJSON
sanitize_query_to_output <- function(results, latest, meta.name="meta") {
    path <- results$path
    has.slash <- grepl("/", path)
    path[!has.slash] <- NA_character_
    path[has.slash] <- sub("/[^/]+$", "", path[has.slash])
    df <- DataFrame(name = results$asset, version = results$version, path = path)
    if (!latest) {
        df$latest <- results$latest == 1
    }

    all_meta <- lapply(results[[meta.name]], fromJSON, simplifyVector=FALSE)
    df$object <- extract_atomic_from_json(all_meta, function(x) x$applications$takane$type, "character") 
    df$title <- extract_atomic_from_json(all_meta, function(x) x$title, "character")
    df$description <- extract_atomic_from_json(all_meta, function(x) x$title, "character")
    df$taxonomy_id <- extract_charlist_from_json(all_meta, function(x) x$taxonomy_id)
    df$genome <- extract_charlist_from_json(all_meta, function(x) x$genome)

    df$rows <- extract_atomic_from_json(all_meta, function(x) x$applications$takane$summarized_experiment$rows, "integer")
    df$columns <- extract_atomic_from_json(all_meta, function(x) x$applications$takane$summarized_experiment$columns, "integer")
    df$assays <- extract_charlist_from_json(all_meta, function(x) x$applications$takane$summarized_experiment$assays)
    df$column_annotations <- extract_charlist_from_json(all_meta, function(x) x$applications$takane$summarized_experiment$column_annotations)
    df$reduced_dimensions <- extract_charlist_from_json(all_meta, function(x) x$applications$takane$single_cell_experiment$reduced_dimensions)
    df$alternative_experiments <- extract_charlist_from_json(all_meta, function(x) x$applications$takane$single_cell_experiment$alternative_experiments)

    df$bioconductor_version < extract_atomic_from_json(all_meta, function(x) x$bioconductor_version, "character")
    df$maintainer_name < extract_atomic_from_json(all_meta, function(x) x$maintainer_name, "character")
    df$maintainer_email < extract_atomic_from_json(all_meta, function(x) x$maintainer_email, "character")

    sources <- vector("list", length(all_meta))
    for (i in seq_along(all_meta)) {
        cursources <- all_meta[[i]]$sources
        if (is.null(cursources)) {
            sources[[i]] <- DataFrame(provider=character(0), id=character(0), version=character(0))
        } else {
            sources[[i]] <- DataFrame(
                provider = extract_atomic_from_json(cursources, function(x) x$provider, "character"),
                id = extract_atomic_from_json(cursources, function(x) x$id, "character"),
                version = extract_atomic_from_json(cursources, function(x) x$version, "character")
            )
        }
    }
    df$sources <- as(sources, "CompressedList")

    df
}

extract_atomic_from_json <- function(metadata, extract, type) {
    vapply(metadata, function(y) {
        x <- extract(y)
        if (is.null(x)) {
            as(NA, type)
        } else {
            x
        }
    }, vector(type, 1))
}

extract_charlist_from_json <- function(metadata, extract) {
    output <- lapply(metadata, function(y) {
        x <- extract(y)
        if (is.null(y)) {
            character(0)
        } else {
            x
        }
    })
    as(output, "CompressedList")
}

#' Obtain the Ernst spermatogenesis data
#'
#' Obtain the mouse spermatogenesis single-cell RNA-seq dataset from Ernst et al. (2019).
#'
#' @param method String indicating which cell caller to obtain results for.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' This study contains two analyses done with datasets from different cell calling algorithms.
#' One uses Cellranger version 2 while the other uses \code{emptyDrops} from \pkg{DropletUtils}.
#'
#' Column metadata includes sample information, per-cell QC metrics and cell type labels.
#' In particular, the sample label specifies the developmental stage of the mouse.
#'
#' Note that \code{method="Cellranger"} contains additional data for Tc1 mice.
#' These mice have an additional human chromosome 21 inserted alongside the usual mouse chromosomes.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/ernst-spermatogenesis}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Ernst C et al. (2019). 
#' Staged developmental mapping and X chromosome transcriptional dynamics during mouse spermatogenesis.
#' \emph{Nat Commun} 10, 1251
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- ErnstSpermatogenesisData()
#' }
#' 
#' @export
ErnstSpermatogenesisData <- function(method=c("emptyDrops", "Cellranger"), location=TRUE, legacy=FALSE) {
    method <- match.arg(method)

    if (!legacy) {
        sce <- fetchDataset("ernst-spermatogenesis-2019", "2023-12-21", path=tolower(method), realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("ernst-spermatogenesis", version), suffix=tolower(method))
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

# library(testthat); library(scRNAseq); source('test-he-organ-atlas.R')

test_that("Ernst spermatogenesis getter works as expected", {
    sce <- ErnstSpermatogenesisData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce2 <- ErnstSpermatogenesisData(method="Cellranger")
    expect_true(ncol(sce)!=ncol(sce2))
})

#' Obtain the Aztekin tail data
#'
#' Obtain the Xenopus tail single-cell RNA-seq data from Aztekin et al. (2019).
#'
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in E-MTAB-7761.
#' This contains information such as the treatment condition, batch, putative cell type, putative cell cycle phase.
#'
#' The UMAP results are available as the \code{"UMAP"} entry in the \code{\link[SingleCellExperiment]{reducedDims}}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/aztekin-tail}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Aztekin C et al. (2019).
#' Identification of a regeneration-organizing cell in the Xenopus tail.
#' \emph{Science} 364(6441), 653-658
#'
#' @examples
#' sce <- AztekinTailData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps reducedDim<- 
#' @importFrom SummarizedExperiment colData colData<-
AztekinTailData <- function(legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("aztekin-tail-2019", "2023-12-14", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("aztekin-tail", version), has.rowdata=FALSE)

        # Move UMAP coordinates to the reducedDims.
        reducedDim(sce, "UMAP") <- as.matrix(colData(sce)[,c("X", "Y")])
        colData(sce) <- colData(sce)[,setdiff(colnames(colData(sce)), c("X", "Y"))]
    }

    sce
}

#' Obtain the Fletcher Olfactory data
#'
#' Obtain the mouse olfactory epithelial HBC stem cell differentiation dataset from Fletcher et al. (2017).
#'
#' @param filtered Logical scalar indicating whether to filter out cells that were not used by the authors.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details 
#' Column metadata is scraped from GEO, using both the author-supplied \dQuote{phenoData} per-cell annotations and the author-supplied \dQuote{protocolData} per-cell annotations. 
#' The former includes information about the animals and the instruments, while the latter contains QC statistics.
#' 
#' We also included the clustering results from the authors' analysis.
#'
#' If \code{filtered=TRUE}, only the cells used by the authors in their cluster analysis are returned. 
#' Otherwise, the cells not used by the authors will have \code{NA} in the clustering columns of the \code{\link[SummarizedExperiment]{colData}}.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object. 
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/fletcher-olfactory}.
#'
#' @return 
#' A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Davide Risso
#'
#' @references
#' Fletcher R et al. (2017). 
#' Deconstructing olfactory stem cell trajectories at single-cell resolution. 
#' \emph{Cell Stem Cell} 20, 817-30.
#'
#' @examples
#' sce <- FletcherOlfactoryData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps 
#' @importFrom SummarizedExperiment rowData 
FletcherOlfactoryData <- function(filtered=TRUE, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("fletcher-olfactory-2019", "2023-12-21", realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("fletcher-olfactory", version), has.rowdata=TRUE)
    }

    if (filtered) {
        sce <- sce[,sce$retained]
        sce$retained <- NULL
    }

    type <- ifelse(rowData(sce)$Transcript_Type == "brainProjectControl", "control", "endogenous")
    sce <- splitAltExps(sce, type, ref="endogenous")
    
    .convert_to_ensembl(sce, 
        symbols=rowData(sce)$Gene_Symbol, 
        species="Mm",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source("test-fletcher-olfactory.R")

test_that("FletcherOlfactoryData works as expected", {
    sce <- FletcherOlfactoryData(ensembl=TRUE)
    expect_s4_class(sce, "SingleCellExperiment")

    sce2 <- FletcherOlfactoryData(filtered=FALSE)
    expect_true(ncol(sce2) > ncol(sce))
    expect_true(nrow(sce2) > nrow(sce))
})

#' Obtain the Bach mammary data
#'
#' Obtain the mouse mammary gland single-cell RNA-seq data from Bach et al. (2017).
#'
#' @param samples A character vector with at least one element, specifying which samples(s) to retrieve.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is extracted from the sample annotation in GSE106273,
#' and refers to the developmental stage of the mammary gland.
#'
#' If multiple samples are specified in \code{samples}, the count matrices will be \code{cbind}ed together.
#' Cells originating from different samples are identifiable by the \code{"Sample"} field in the column metadata.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/bach-mammary}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Bach K et al. (2017).
#' Differentiation dynamics of mammary epithelial cells revealed by single-cell RNA sequencing. 
#' \emph{Nat Commun.} 8(1), 2128
#'
#' @examples
#' sce <- BachMammaryData(samples="NP_1")
#' 
#' @export
#' @importFrom SummarizedExperiment rowData<- rowData
#' @importFrom ExperimentHub ExperimentHub
#' @importFrom BiocGenerics cbind
BachMammaryData <- function(samples=c("NP_1", "NP_2", "G_1", "G_2", "L_1", "L_2", "PI_1", "PI_2"), location=TRUE, legacy = FALSE) {
    if (!legacy && length(unique(samples)) == 8) {
        sce <- fetchDataset("bach-mammary-2017", "2023-12-14", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        host <- file.path("bach-mammary", version)
        samples <- match.arg(samples, several.ok=TRUE)

        collected <- vector("list", length(samples))
        for (i in seq_along(samples)) {
           collected[[i]] <- .create_sce(host, has.rowdata=FALSE, suffix=samples[i])
        }
        sce <- do.call(cbind, collected)

        ehub <- ExperimentHub()
        rowData(sce) <- ehub[ehub$rdatapath==file.path("scRNAseq", host, "rowdata.rds")][[1]]
        rownames(sce) <- rowData(sce)$Ensembl
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

#' Obtain the Campbell brain data
#'
#' Obtain the mouse brain single-cell RNA-seq data from Campbell et al. (2017).
#'
#' @param ensembl Logical scalar indicating whether the row names of the returned object should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in GSE93374.
#' This contains information such as the diet of the mice, sex and proposed cell type for each cell.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/campbell-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Campbell R et al. (2017).
#' A molecular census of arcuate hypothalamus and median eminence cell types. 
#' \emph{Nat. Neurosci.} 20, 484-496. 
#'
#' @examples
#' sce <- CampbellBrainData()
#' 
#' @export
CampbellBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("campbell-brain-2017", "2023-12-14", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("campbell-brain", version), has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Paul HSC data
#'
#' Obtain the mouse haematopoietic stem cell single-cell RNA-seq data from Paul et al. (2015).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param discard.multiple Logical scalar indicating whether ambiguous rows should be discarded.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata includes the plate and the mouse of origin, fluoresence intensities from indexed sorting 
#' and the number of cells in each well.
#'
#' Some of the original rownames are concatenated symbols from multiple genes.
#' We consider these rows to represent ambiguously assigned counts and discard them if \code{discard.multiple=TRUE}.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/nestorowa-hsc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Paul F et al. (2015). 
#' Transcriptional heterogeneity and lineage commitment in myeloid progenitors. 
#' \emph{Cell} 163, 1663-77.
#'
#' @examples
#' sce <- PaulHSCData()
#' 
#' @export
PaulHSCData <- function(ensembl=FALSE, discard.multiple=TRUE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("paul-hsc-2015", "2023-12-20", realize.assays=TRUE)
    } else {
        version <- "2.2.0"
        sce <- .create_sce(file.path("paul-hsc", version), has.rowdata=FALSE)
    }

    if (discard.multiple) {
        sce <- sce[!grepl(";", rownames(sce)),]
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Zilionis lung cancer data
#'
#' Obtain the human/mouse lung cancer single-cell RNA-seq data from Zilionis et al. (2019).
#'
#' @param which String specifying the species to get data for.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param filter Logical scalar indicating if the filtered subset should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided and contains information on the library, donor ID/animal ID, replicate and tissue.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#' 
#' If \code{filter=TRUE}, only cells that have been used in the original analysis are returned.
#' The cells used are specified in the \code{Used} column of the \code{\link[SummarizedExperiment]{colData}}.
#' 
#' The \code{\link[SingleCellExperiment]{reducedDim}} contains coordinates of SPRING representations.
#' This may be filled with \code{NA}s for SPRING coordinates computed on a subset of cells (specified in \code{\link[SummarizedExperiment]{colData}}).
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/zilionis-lung}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Jens Preussner
#'
#' @references
#' Zilionis R et al. (2019). 
#' Single-cell transcriptomics of human and mouse lung cancers reveals 
#' conserved myeloid populations across individuals and species. 
#' \emph{Immunity} 50(5), 1317-1334.
#'
#' @examples
#' sce.human <- ZilionisLungData()
#'
#' sce.mouse <- ZilionisLungData("mouse")
#' 
#' @export
#' @importFrom SummarizedExperiment colData colData<-
#' @importFrom SingleCellExperiment reducedDim<- reducedDims<- 
ZilionisLungData <- function(which=c("human", "mouse"), ensembl=FALSE, location=TRUE, filter=FALSE, legacy=FALSE) {
    which <- match.arg(which)

    if (!legacy) {
        sce <- fetchDataset("zilionis-lung-2019", "2023-12-20", path=which, realize.assays=TRUE)
        colnames(sce) <- sce$Barcode

    } else {
        version <- "2.4.0"
        sce <- .create_sce(file.path("zilionis-lung", version), has.rowdata=FALSE, suffix=which)

        if (filter) {
            sce <- sce[,sce$Used]
        }

        cd <- colData(sce)
        cn <- colnames(cd)
        if (which=="human") {
            which.x <- grep("^x_", cn)
            which.y <- grep("^y_", cn)
            grouping.x <- sub("^x_", "", cn[which.x])
            grouping.y <- sub("^y_", "", cn[which.y])
            stopifnot(identical(grouping.x, grouping.y)) # sanity check.

            all.dims <- list()
            for (i in seq_along(grouping.x)) {
                all.dims[[paste0("SPRING_", grouping.x[i])]] <- cbind(x=cd[,which.x[i]], y=cd[,which.y[i]]) 
            }
            reducedDims(sce) <- all.dims
        } else {
            which.x <- which(cn=="x")
            which.y <- which(cn=="y")
            reducedDim(sce, "SPRING") <- cbind(x=cd[,which.x], y=cd[,which.y])
        } 

        colData(sce) <- colData(sce)[,-c(which.x, which.y)]
    }

    .convert_to_ensembl(sce, 
        ensembl=ensembl,
        species=if (which=="human") "Hs" else "Mm",
        symbols=rownames(sce),
        location=location)
}

#' Obtain the Ledergor Myeloma data
#'
#' Obtain the human multiple myeloma single-cell RNA-seq data from Ledergor et al. (2018).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata was created from the sample metadata file in GSE117156. It contains an 'Experiment_ID' column,
#' from which the tissue and subject of origin were extracted, as well as the condition and treatment status of the subject.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/ledergor-myeloma}.
#'
#' @return A \linkS4class{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Milan Malfait
#'
#' @references
#' Ledergor G et al. (2018)
#' Single cell dissection of plasma cell heterogeneity in symptomatic and
#' asymptomatic myeloma. \emph{Nat Med.} 24(12), 1867–1876.
#'
#' @examples
#' sce <- LedergorMyelomaData()
#'
#' @export
#' @importFrom SummarizedExperiment rowData
LedergorMyelomaData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("ledergor-myeloma-2018", "2023-12-20", realize.assays=TRUE)
    } else {
        version <- "2.8.0"
        sce <- .create_sce(file.path("ledergor-myeloma", version), has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce,
        species = "Hs",
        symbols = rownames(sce),
        ensembl = ensembl,
        location = location)
}

# library(testthat); library(scRNAseq); source('test-ledergor-myeloma.R')

test_that("Ledegor myeloma getter works as expected", {
    sce <- LedergorMyelomaData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- LedergorMyelomaData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

#' Obtain the La Manno brain data
#'
#' Obtain the mouse/human brain scRNA-seq data from La Manno et al. (2016).
#'
#' @param which A string specifying which dataset should be obtained.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is provided in the same form as supplied in the supplementary tables in GSE71585.
#' This contains information such as the time point and cell type.
#'
#' The various settings of \code{which} will obtain different data sets.
#' \itemize{
#' \item \code{"human-es"}, human embryonic stem cells.
#' \item \code{"human-embryo"}, human embryo midbrain.
#' \item \code{"human-ips"}, human induced pluripotent stem cells.
#' \item \code{"mouse-adult"}, mouse adult dopaminergic neurons.
#' \item \code{"mouse-embryo"}, mouse embryo midbrain.
#' }
#' Unfortunately, each of these datasets uses a different set of features.
#' If multiple datasets are to be used simultaneously, users will have to decide how to merge them,
#' e.g., by taking the intersection of common features across all datasets.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/lamanno-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' La Manno A et al. (2016).
#' Molecular diversity of midbrain development in mouse, human, and stem cells. 
#' \emph{Cell} 167(2), 566-580.
#'
#' @examples
#' sce.h.es <- LaMannoBrainData()
#'
#' sce.h.em <- LaMannoBrainData("human-embryo")
#'
#' sce.h.ip <- LaMannoBrainData("human-ips")
#'
#' sce.m.ad <- LaMannoBrainData("mouse-adult")
#'
#' sce.m.em <- LaMannoBrainData("mouse-embryo")
#' 
#' @export
LaMannoBrainData <- function(which=c("human-es", "human-embryo", "human-ips", "mouse-adult", "mouse-embryo"), ensembl=FALSE, location=TRUE, legacy=FALSE) {
    which <- match.arg(which)

    if (!legacy) {
        sce <- fetchDataset("lamanno-brain-2016", "2023-12-17", path=which, realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("lamanno-brain", version), has.rowdata=FALSE, suffix=which)
    }

    colnames(sce) <- colData(sce)[[grep("Cell_ID", colnames(colData(sce)), ignore.case=TRUE)]]
    .convert_to_ensembl(sce, 
        species=if (grepl("human", which)) "Hs" else "Mm",
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Nowakowski cortex data
#'
#' Obtain the human cortex single-cell RNA-seq dataset from Nowakowski et al. (2017).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata includes the presumed cell type (\code{WGCNAcluster}), patient and tissue region of origin. 
#' A variety of dimensionality reduction results are also provided.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' This is only performed when \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/nowakowski-cortex}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of TPMs.
#' The \code{\link[SingleCellExperiment]{reducedDims}} contains an assortment of dimensionality reduction results.
#'
#' @author Aaron Lun
#'
#' @references
#' Nowakowski S et al. (2017). 
#' Spatiotemporal gene expression trajectories reveal developmental hierarchies of the human cortex. 
#' \emph{Science} 358, 1318-1323.
#'
#' @examples
#' sce <- NowakowskiCortexData()
#' 
#' @export
#' @importFrom SingleCellExperiment reducedDims<-
NowakowskiCortexData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("nowakowski-cortex-2017", "2023-12-22", realize.assays=TRUE)

    } else {
        version <- "2.6.0"
        hub <- ExperimentHub()
        sce <- .create_sce(file.path("nowakowski-cortex", version), hub=hub, has.rowdata=FALSE, assays="tpm")
        reducedDims(sce) <- hub[[which(hub$rdatapath==file.path("scRNAseq/nowakowski-cortex", version, "reddims.rds"))]]
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-nowakowski-cortex.R')

test_that("Nowakowski cortex getter works as expected", {
    sce <- NowakowskiCortexData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- NowakowskiCortexData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

#' Obtain the Grun HSC data
#'
#' Obtain the mouse haematopoietic stem cell single-cell RNA-seq data from Grun et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Row metadata contains the symbol and chromosomal location for each gene.
#' Column metadata contains the extraction protocol used for each sample, as described in GSE76983.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/grun-hsc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Grun D et al. (2016). 
#' De novo prediction of stem cell identity using single-cell transcriptome data. 
#' \emph{Cell Stem Cell} 19(2), 266-277. 
#'
#' @examples
#' sce <- GrunHSCData()
#' 
#' @export
#' @importFrom SummarizedExperiment rowData rowData<- colData<-
#' @importFrom S4Vectors DataFrame
GrunHSCData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("grun-bone_marrow-2016", "2023-12-14", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("grun-hsc", version), has.rowdata=FALSE, has.coldata=FALSE)

        # Cleaning up the row data.
        symbol <- sub("__.*", "", rownames(sce))
        loc <- sub(".*__", "", rownames(sce))
        rowData(sce) <- DataFrame(symbol=symbol, chr=loc)

        # Cleaning up the col data.
        cn <- colnames(sce)
        sample <- sub("_.*", "", cn)
        protocol <- ifelse(sample %in% c("JC4", "JC48P2", "JC48P4", "JC48P6", "JC48P7"),
            "sorted hematopoietic stem cells", "micro-dissected cells")
        colData(sce) <- DataFrame(sample=sample, protocol=protocol, row.names=cn)
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rowData(sce)$symbol,
        ensembl=ensembl,
        location=location)
}

#' Obtain the Grun pancreas data
#'
#' Obtain the human pancreas single-cell RNA-seq data from Grun et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Row metadata contains fields for the symbol and chromosomal location of each gene,
#' as derived from the row names.
#'
#' Column metadata is derived from the column names of the count matrix with the sample annotations in GSE81076.
#' This includes the donor identity for each cell and the type of sample.
#'
#' The \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}} contains count data for the ERCC spike-in transcripts.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/grun-pancreas}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts. 
#'
#' @author Aaron Lun,
#' using additional metadata obtained by Vladimir Kiselev.
#'
#' @references
#' Grun D et al. (2016). 
#' De novo prediction of stem cell identity using single-cell transcriptome data. 
#' \emph{Cell Stem Cell} 19(2), 266-277. 
#'
#' @examples
#' sce <- GrunPancreasData()
#' 
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom SummarizedExperiment rowData<- colData<-
#' @importFrom SingleCellExperiment splitAltExps
GrunPancreasData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("grun-pancreas-2016", "2023-12-14", realize.assays=TRUE)
        cn <- colnames(colData(sce))
        colnames(colData(sce))[cn == "treatment"] <- "sample" # some protective back-compatibility work.

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("grun-pancreas", version), has.rowdata=FALSE, has.coldata=FALSE)

        # Loading the column metadata
        lib.names <- sub("_.*", "", colnames(sce))
        donor.names <- sub("(D10|D17|D2|D3|D7).*", "\\1", lib.names)
        treatment <- c(
            D10631="CD63+ sorted cells",
            D101="live sorted cells, library 1",
            D102="live sorted cells, library 2",
            D1713="CD13+ sorted cells",
            D172444="CD24+ CD44+ live sorted cells",
            D17All1="live sorted cells, library 1",
            D17All2="live sorted cells, library 2",
            D17TGFB="TGFBR3+ sorted cells",
            D2ex="exocrine fraction, live sorted cells",
            D3en1="live sorted cells, library 1",
            D3en2="live sorted cells, library 2",
            D3en3="live sorted cells, library 3",
            D3en4="live sorted cells, library 4",
            D3ex="exocrine fraction, live sorted cells",
            D71="live sorted cells, library 1",
            D72="live sorted cells, library 2",
            D73="live sorted cells, library 3",
            D74="live sorted cells, library 4"
        )[lib.names]
        colData(sce) <- DataFrame(donor=donor.names, sample=treatment, row.names=colnames(sce))

        # Splitting up gene information.
        symbol <- sub("__.*", "", rownames(sce))
        loc <- sub(".*__", "", rownames(sce))
        rowData(sce) <- DataFrame(symbol=symbol, chr=loc)

        # Splitting spike-ins into an alternative experiment.
        status <- ifelse(grepl("ERCC-[0-9]+", symbol), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")

        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 20, dilution = 50000)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .convert_to_ensembl(sce, 
        symbols=rowData(sce)$symbol, 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

#' Obtain the Richard T cell data
#'
#' Obtain the mouse CD8+ T cell single-cell RNA-seq data from Richard et al. (2018).
#'
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in E-MTAB-6051.
#' This contains information such as the stimulus, time after stimulation, age of the mice and sequencing batch.
#'
#' Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/richard-tcell}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Richard AC et al. (2018). 
#' T cell cytolytic capacity is independent of initial stimulation strength. 
#' \emph{Nat. Immunol.} 19(8), 849-858.
#'
#' @examples
#' sce <- RichardTCellData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps
RichardTCellData <- function(location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("richard-tcell-2018", "2023-12-19", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("richard-tcell", version), has.rowdata=FALSE)

        spike.type <- ifelse(grepl("ERCC", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, spike.type, ref="endogenous")

        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 1000, dilution = 3e07)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

#' Obtain the Lawlor pancreas data
#'
#' Provides the human pancreas single-cell RNA-seq data from Lawlor et al. (2017).
#'
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in GSE86469. 
#' This contains information such as the cell type labels and patient status.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/lawlor-pancreas}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Lawlor N et al. (2017). 
#' Single-cell transcriptomes identify human islet cell signatures and reveal cell-type-specific expression changes in type 2 diabetes.
#' \emph{Genome Res.} 27(2), 208-222.
#'
#' @examples
#' sce <- LawlorPancreasData()
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
LawlorPancreasData <- function(legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("lawlor-pancreas-2017", "2023-12-17", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("lawlor-pancreas", version), has.rowdata=FALSE)
    }
    sce
}

#' Obtain the He organ atlas data
#'
#' Obtain the human cortex single-nuclei RNA-seq data from Hu et al. (2017).
#'
#' @param tissue Character vector specifying the tissues to return.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column data contains the tissue of origin, a variety of per-cell QC metrics well as some cell type annotations.
#' The reclustered annotations required some assembly:
#' \itemize{
#' \item \code{reclustered.broad} was generated based on whether the barcode was present in each \code{*_meta.data.txt} file at \url{https://github.com/bei-lab/scRNA-AHCA}.
#' \item For each barcode that was present in one of those files, \code{reclustered.fine} was generated based on the label in the \code{annotation} field inside that file.
#' }
#'
#' If multiple tissues are requested, counts are only reported for the intersection of genes across all tissues.
#' This is because the gene annotation in the original count matrices differs across tissues.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/he-organ-atlas}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' He S et al. (2020).
#' Single-cell transcriptome profiling of an adult human cell atlas of 15 major organs. 
#' \emph{Genome Biol} 21, 1:294.
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- HeOrganAtlasData()
#' }
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
HeOrganAtlasData <- function(tissue=c("Bladder", "Blood", "Common.bile.duct", "Esophagus", 
    "Heart", "Liver", "Lymph.node", "Marrow", "Muscle", "Rectum", "Skin", "Small.intestine", 
    "Spleen", "Stomach", "Trachea"), ensembl=FALSE, location=TRUE, legacy=FALSE)
{
    if (!legacy) {
        all.sce <- lapply(tissue, function(x) fetchDataset("he-organs-2020", "2023-12-21", path=gsub("\\.", "_", tolower(x)), realize.assays=TRUE))
        if (length(all.sce) > 1) {
            common <- Reduce(intersect, lapply(all.sce, rownames))
            for (i in seq_along(all.sce)) {
                all.sce[[i]] <- all.sce[[i]][common,]
            }
        }
        sce <- do.call(cbind, all.sce)

    } else {
        version <- "2.6.0"
        tissue <- match.arg(tissue, several.ok=TRUE)
        hub <- ExperimentHub()
        host <- file.path("he-organ-atlas", version)

        all.sce <- lapply(tissue, function(x) .create_sce(host, has.rowdata=FALSE, suffix=x, hub=hub))
        if (length(all.sce) > 1) {
            common <- Reduce(intersect, lapply(all.sce, rownames))
            for (i in seq_along(all.sce)) {
                all.sce[[i]] <- all.sce[[i]][common,]
            }
        }
        sce <- do.call(cbind, all.sce)

        # Pulling down reduced dimensions.
        reddim <- hub[hub$rdatapath==file.path("scRNAseq", host, "reddim.rds")][[1]]
        reddim <- lapply(reddim, function(x) x[colnames(sce),,drop=FALSE])
        reducedDims(sce) <- reddim
    }

    .convert_to_ensembl(sce, 
        species="Hs", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-he-organ-atlas.R')

test_that("He organ atlas getter works as expected", {
    sce <- HeOrganAtlasData()
    expect_s4_class(sce, "SingleCellExperiment")

    suppressWarnings(sce2 <- HeOrganAtlasData(ensembl=TRUE))
    expect_true(nrow(sce2) < nrow(sce))

    solo.sce <- HeOrganAtlasData(tissue="Blood")
    expect_identical(reducedDimNames(solo.sce), "tSNE")
})

#' Obtain the Chen brain data
#'
#' Obtain the mouse brain single-cell RNA-seq data from Chen et al. (2017).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in GSE87544.
#' This contains the putative cell type assigned by the original authors.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/chen-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Chen R et al. (2017).
#' Single-Cell RNA-Seq reveals hypothalamic cell diversity. 
#' \emph{Cell Rep.} 18, 3227-3241.
#'
#' @examples
#' sce <- ChenBrainData()
#' 
#' @export
ChenBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("chen-brain-2017", "2023-12-14", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("chen-brain", version), has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Tasic brain data
#'
#' Obtain the mouse brain single-cell RNA-seq data from Tasic et al. (2015).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is provided in the same form as supplied in GSE71585.
#' This contains information such as the reporter gene expressed in each cell, the mouse line, dissection type and so on.
#'
#' Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
#' Note that some of the spike-in rows have \code{NA} observations for some (but not all) cells.
#'
#' The last 9 columns (containing \code{_CTX_} in their names) correspond to no-cell control libraries.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/tasic-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Tasic A et al. (2016). 
#' Adult mouse cortical cell taxonomy revealed by single cell transcriptomics.
#' \emph{Nat. Neurosci.} 19(2), 335-46.
#'
#' @examples
#' sce <- TasicBrainData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps
TasicBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("tasic-brain-2016", "2024-04-18", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("tasic-brain", version), has.rowdata=FALSE)

        status <- ifelse(grepl("^ERCC-[0-9]+$", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 100, dilution = 1000000)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Mm",
        ensembl=ensembl,
        location=location)
}

#' Obtain the Kotliarov CITE-seq data
#'
#' Obtain the Kotliarov PBMC CITE-seq data from Kotliarov et al. (2020).
#'
#' @param mode Character vector specifying whether to return either or both the RNA and ADT counts.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' This dataset contains 20 samples from 2 experimental batches, where each batch contains 5 high and 5 low responders. 
#' The 10 samples per batch were mixed and distributed across the 6 lanes using a cell hashing approach.
#' 
#' The column metadata contains the following fields:
#' \itemize{
#' \item \code{sample*}: identifiers for the sample of origin for each cell.
#' \item \code{adjmfc.time}: type of responder for each sample.
#' \item \code{tenx_lane}: 10X lane from which each cell was collected.
#' \item \code{batch}: the batch of origin.
#' \item \code{barcode_check}: barcode identifier.
#' \item \code{hash_*} and \code{hto_*} columns: \pkg{HTOdemux} outputs.
#' \item \code{DEMUXLET.*} columns: \pkg{demuxlet} outputs.
#' \item \code{joint_classification_global}: \pkg{HTOdemux} and \pkg{demuxlet} joint classification.
#' \item \code{nGene}: number of genes as defined from \pkg{Seurat}'s \code{CreateSeuratObject}.
#' \item \code{nUMI}: number of UMIs as defined from \pkg{Seurat}'s \code{CreateSeuratObject}.
#' \item \code{pctMT}: percent of mitochondrial reads as defined from \pkg{Seurat}'s \code{CreateSeuratObject}.
#' }
#' Note, no filtering has been performed based on the quality control metrics.
#'
#' If \code{ensembl=TRUE}, the gene symbols in the RNA data are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} for the RNA data.
#' Note that this is only performed if \code{ensembl=TRUE}.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/kotliarov-pbmc}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts corresponding to the first \code{mode},
#' with an optional alternative Experiment if there is a second \code{mode}.
#'
#' @author
#' Stephany Orjuela,
#' with modifications from Aaron Lun
#'
#' @references
#' Kotliarov et al. (2020).
#' Broad immune activation underlies shared set point signatures for vaccine responsiveness in healthy individuals and disease activity in patients with lupus.
#' \emph{Nat. Med.} 26, 618–629
#'
#' @examples
#' sce <- KotliarovPBMCData()
#' 
#' @export
#' @importFrom ExperimentHub ExperimentHub
#' @importFrom SummarizedExperiment colData<-
#' @importFrom SingleCellExperiment SingleCellExperiment altExps 
KotliarovPBMCData <- function(mode=c("rna", "adt"), ensembl=FALSE, location=TRUE, legacy=FALSE) {
    mode <- match.arg(mode, c("rna", "adt"), several.ok=TRUE)

    if (!legacy && identical(mode, c("rna", "adt"))) {
        sce <- fetchDataset("kotliarov-pbmc-2020", "2024-04-18", realize.assays=TRUE)

    } else {
        version <- "2.4.0"
        tag <- "kotliarov-pbmc"
        hub <- ExperimentHub()
        
        collated <- list()
        for (x in mode) {
            collated[[x]] <- .create_sce(file.path(tag, version), hub=hub, 
                has.rowdata=FALSE, has.coldata=FALSE, suffix=x)
        }

        if ("rna" %in% names(collated)) {
            collated[["rna"]] <- .convert_to_ensembl(collated[["rna"]],
                symbols=rownames(collated[["rna"]]),
                species="Hs",
                ensembl=ensembl,
                location=location)
        }
        
        sce <- collated[[1]]
        altExps(sce) <- collated[-1]
        colData(sce) <- hub[hub$rdatapath==file.path("scRNAseq", tag, version, "coldata.rds")][[1]] 
    }

    sce
}

#' Fetch a dataset from the gypsum backend
#'
#' Fetch a dataset (or its metadata) from the gypsum backend.
#'
#' @param name String containing the name of the dataset.
#' @param version String containing the version of the dataset.
#' @param path String containing the path to a subdataset, if \code{name} consists of multiple subdatasets.
#' Defaults to \code{NA} if no subdatasets are present.
#' @param package String containing the name of the package.
#' @param cache,overwrite Arguments to pass to \code{\link[gypsum]{saveVersion}} or \code{\link[gypsum]{saveFile}}.
#' @param realize.assays,realize.reduced.dims Logical scalars indicating whether to realize assays and reduced dimensions into memory.
#' Dense and sparse \link[alabaster.matrix]{ReloadedArray} objects are converted into ordinary arrays and dgCMatrix objects, respectively.
#' @param ... Further arguments to pass to \code{\link[alabaster.base]{readObject}}.
#'
#' @return \code{fetchDataset} returns the dataset as a \link[SummarizedExperiment]{SummarizedExperiment} or one of its subclasses.
#'
#' \code{fetchMetadata} returns a named list of metadata for the specified dataset.
#'
#' @seealso
#' \url{https://github.com/ArtifactDB/bioconductor-metadata-index}, on the expected schema for the metadata.
#'
#' \code{\link{saveDataset}} and \code{\link[gypsum]{uploadDirectory}}, to save and upload a dataset.
#'
#' \code{\link{surveyDatasets}} and \code{\link{listVersions}}, to get possible values for \code{name} and \code{version}.
#' 
#' @author Aaron Lun
#' @examples
#' fetchDataset("zeisel-brain-2015", "2023-12-14")
#' fetchMetadata("zeisel-brain-2015", "2023-12-14")
#'
#' @export
#' @importFrom gypsum cacheDirectory saveVersion
#' @importFrom alabaster.base altReadObjectFunction altReadObject
fetchDataset <- function(name, version, path=NA, package="scRNAseq", cache=cacheDirectory(), overwrite=FALSE, realize.assays=FALSE, realize.reduced.dims=TRUE, ...) {
    version_path <- saveVersion(package, name, version, cache=cache, overwrite=overwrite)

    obj_path <- version_path
    if (!is.na(path)) {
        obj_path <- file.path(version_path, gsub("/*$", "", path))
    } else if (!file.exists(file.path(obj_path, "OBJECT"))) {
        stop("need to specify a subdataset via 'path=', see listPaths('", name, "', '", version, "') for options")
    }

    old <- altReadObjectFunction(scLoadObject)
    on.exit(altReadObjectFunction(old))
    altReadObject(obj_path, scRNAseq.realize.assays=realize.assays, scRNAseq.realize.reduced.dims=realize.reduced.dims, ...)
}

#' @export
#' @rdname fetchDataset
#' @importFrom jsonlite fromJSON
#' @importFrom gypsum cacheDirectory saveFile
fetchMetadata <- function(name, version, path=NA, package="scRNAseq", cache=cacheDirectory(), overwrite=FALSE) {
    remote_path <- "_bioconductor.json"
    if (!is.na(path)) {
        remote_path <- paste0(path, "/", remote_path)
    }

    local_path <- saveFile(package, name, version, remote_path, cache=cache, overwrite=overwrite)
    fromJSON(local_path, simplifyVector=FALSE)
}

#' @importFrom alabaster.base readObjectFile readObject
#' @importFrom SummarizedExperiment assay assay<-
#' @importFrom SingleCellExperiment reducedDim reducedDim<-
scLoadObject <- function(path, metadata=NULL, scRNAseq.realize.assays=FALSE, scRNAseq.realize.reduced.dims=TRUE, ...) {
    if (is.null(metadata)) {
        metadata <- readObjectFile(path)
    }
    ans <- readObject(
        path, 
        metadata=metadata, 
        scRNAseq.realize.assays=scRNAseq.realize.assays, 
        scRNAseq.realize.reduced.dims=scRNAseq.realize.reduced.dims, 
        ...
    )

    if (is(ans, "SummarizedExperiment")) {
        if (scRNAseq.realize.assays) {
            for (y in assayNames(ans)) {
                assay(ans, y, withDimnames=FALSE) <- realize_array(assay(ans, y, withDimnames=FALSE))
            }
        }

        if (is(ans, "SingleCellExperiment")) {
            if (scRNAseq.realize.reduced.dims) {
                for (z in reducedDimNames(ans)) {
                    reducedDim(ans, z, withDimnames=FALSE) <- realize_array(reducedDim(ans, z, withDimnames=FALSE))
                }
            }
        }
    }

    ans
}

#' @importClassesFrom alabaster.matrix ReloadedArray
#' @importFrom DelayedArray is_sparse type
#' @importClassesFrom Matrix lgCMatrix dgCMatrix
realize_array <- function(x) {
    if (is(x, "ReloadedArray")) {
        if (is_sparse(x)) {
            if (type(x) == "logical") {
                x <- as(x, "lgCMatrix")
            } else {
                x <- as(x, "dgCMatrix")
            }
        } else {
            x <- as.array(x)
        }
    }
    x
}

# This checks that uploadDirectory works as expected.
# library(testthat); library(scRNAseq); source("test-uploadDirectory.R")

sce <- SingleCellExperiment(list(counts=matrix(rpois(1000, lambda=1), 100, 10)))
sce$foo <- sample(letters, 10)
rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(sce)))
colnames(sce) <- head(LETTERS, 10)

test_that("file listing works as expected without any special elements", {
    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(list(provider="GEO", id="GSE12345")),
        maintainer_name="Shizuka Mogami",
        maintainer_email="mogami.shizuka@765pro.com"
    )

    tmp <- tempfile()
    saveDataset(sce, tmp, meta)

    listing <- gypsum::prepareDirectoryUpload(tmp, links="always")
    expect_identical(nrow(listing$links), 0L)
    expect_identical(sort(listing$files), sort(list.files(tmp, recursive=TRUE)))
})

test_that("file listing works with ReloadedArrays", {
    cache <- tempfile()
    tmp0 <- file.path(cache, gypsum:::BUCKET_CACHE_NAME, "scRNAseq", "test", "foo")
    dir.create(dirname(tmp0), recursive=TRUE)
    alabaster.base::saveObject(sce, tmp0)

    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id=c("10090", "9606"),
        genome=c("GRCh38", "GRCm38"),
        sources=list(list(provider="GEO", id="GSE12345"), list(provider="DOI", id="123asd/231.123")),
        maintainer_name="Kaori Sakuramori",
        maintainer_email="sakuramori.kaori@765pro.com"
    )

    sce2 <- sce
    assay(sce2, "foobar", withDimnames=FALSE) <- alabaster.matrix::ReloadedArray(path=file.path(tmp0, "assays", "0"), seed=matrix(0, 100, 10))

    tmp <- tempfile()
    saveDataset(sce2, tmp, meta)
    expect_error(gypsum::prepareDirectoryUpload(tmp, links="always"), "failed to convert")

    listing <- gypsum::prepareDirectoryUpload(tmp, links="always", cache=cache)
    expect_identical(sort(listing$links$to.path), sort(c("assays/0/array.h5", "assays/0/OBJECT")))
    expect_identical(sort(listing$links$from.path), sort(c("assays/1/array.h5", "assays/1/OBJECT")))
    expect_identical(sort(c(listing$files, listing$links$from.path)), sort(list.files(tmp, recursive=TRUE)))
})

test_that("the actual upload works correctly", {
    gh_token <- gypsum::accessToken(request=FALSE)
    skip_if(is.null(gh_token))

    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(list(provider="GEO", id="GSE12345")),
        maintainer_name="Shizuka Mogami",
        maintainer_email="mogami.shizuka@765pro.com"
    )

    tmp <- tempfile()
    saveDataset(sce, tmp, meta)

    version <- as.character(Sys.Date())
    gypsum::uploadDirectory(tmp, "scRNAseq", "test", version, probation=TRUE)
    on.exit(gypsum::rejectProbation("scRNAseq", "test", version))

    cache <- tempfile() # use a different cache to avoid pulling down the test.
    roundtrip <- fetchDataset("test", version, cache=cache)
    expect_identical(colData(roundtrip), colData(sce))
    expect_identical(as.matrix(counts(roundtrip)), assay(sce))
    expect_s4_class(counts(roundtrip, withDimnames=FALSE), "ReloadedMatrix")
})

# This tests the dataset fetching.
# library(testthat); library(scRNAseq); source("test-fetchDataset.R")

test_that("fetchDataset works as expected", {
    sce <- fetchDataset("zeisel-brain-2015", "2023-12-14")
    expect_s4_class(sce, "SingleCellExperiment")

    # Correctly creates ReloadedMatrix objects.
    ass <- assay(sce, withDimnames=FALSE)
    expect_s4_class(ass, "ReloadedMatrix")
    expect_true(DelayedArray::is_sparse(ass))
    expect_true(grepl("zeisel-brain-2015", ass@seed@path))
    expect_true(grepl("2023-12-14", ass@seed@path))

    # Works with realization options.
    sce <- fetchDataset("zeisel-brain-2015", "2023-12-14", realize.assays=TRUE)
    expect_s4_class(assay(sce, withDimnames=FALSE), "dgCMatrix")
    expect_type(assay(altExp(sce), withDimnames=FALSE), "integer") # also realizes the alternative experiments.
})

test_that("fetchDataset realizes the reduced dimensions", {
    sce <- fetchDataset("aztekin-tail-2019", "2023-12-14", realize.reduced.dims=FALSE)
    expect_s4_class(reducedDim(sce, withDimnames=FALSE), "ReloadedMatrix")

    sce <- fetchDataset("aztekin-tail-2019", "2023-12-14", realize.reduced.dims=TRUE)
    expect_type(reducedDim(sce, withDimnames=FALSE), "double")
})

test_that("fetchDataset fails if it needs a path", {
    expect_error(fetchDataset("lamanno-brain-2016", "2023-12-17"), "listPaths")
    expect_s4_class(fetchDataset("lamanno-brain-2016", "2023-12-17", path="human-es"), "SingleCellExperiment")
})

test_that("fetchMetadata works as expected", {
    meta <- fetchMetadata("zeisel-brain-2015", "2023-12-14")
    expect_match(meta$title, "Brain structure")
    expect_identical(meta$taxonomy_id[[1]], "10090")
})

#' Obtain the Leng ESC data
#'
#' Obtain the human embryonic stem cell single-cell RNA-seq data from Leng et al. (2015).
#'
#' @param ensembl Logical scalar indicating whether gene symbols should be converted to Ensembl annotation.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata contains the cell line, experiment number and experimentally determined cell cycle phase for each cell.
#'
#' If \code{ensembl=TRUE}, the gene symbols in the published annotation are converted to Ensembl.
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/leng-esc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of normalized expected read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Leng F et al. (2015).
#' Oscope identifies oscillatory genes in unsynchronized single-cell RNA-seq experiments. 
#' \emph{Nat. Methods} 12(10), 947-950.
#' 
#' @examples
#' sce <- LengESCData()
#' 
#' @export
LengESCData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("leng-esc-2015", "2024-04-25", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("leng-esc", version), assays="normcounts", has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

#' Obtain the Zhao immune liver data
#'
#' Obtain the human liver immune single-cell RNA-seq data from Zhao et al. (2020).
#'
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param filter Logical scalar indicating if the filtered subset should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata contains various cell labels as provided by the authors.
#' Some of these labels required assembly on our part:
#' \itemize{
#' \item The \code{broad} label was assigned to each barcode based on whether that barcode was present in each \code{*_identities.tsv.gz} in GSE125188's supplementary files.
#' \item For each cell barcode that was present in one of these files, the \code{fine} label was generated from the \code{Group} annotations inside that file.
#' }
#' We guessed the \code{sample} for each cell by assuming that the GEM group numbers match the order of samples in GSE125188.
#' We also assumed that \dQuote{donor 4} is a typo, given that the paper only mentions 3 donors.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#' 
#' If \code{filter=TRUE}, only cells that have been used in the original analysis are returned.
#' Otherwise, the cells used are specified in the \code{retained} column of the \code{\link[SummarizedExperiment]{colData}}.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/zhao-immune-liver}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Zhao J et al. (2020). 
#' Single-cell RNA sequencing reveals the heterogeneity of liver-resident immune cells in human.
#' \emph{Cell Discov} 6, 22.
#'
#' @examples
#' sce.zhao <- ZhaoImmuneLiverData()
#' 
#' @export
ZhaoImmuneLiverData <- function(location=TRUE, filter=FALSE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("zhao-immune-2020", "2023-12-22", realize.assays=TRUE)

    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("zhao-immune-liver", version), has.rowdata=TRUE)

        if (filter) {
            sce <- sce[,sce$retained]
            sce$retained <- NULL
        }

        gem.group <- as.integer(sub(".*-", "", colnames(sce)))
        sce$sample <- c("donor 1 blood", "donor 1 spleen", "donor 1 liver",
            "donor 2 blood", "donor 2 spleen", "donor 2 liver",
            "donor 3 blood", "donor 3 spleen", "donor 3 liver")[gem.group]
    }

    sce
}

# library(testthat); library(scRNAseq); source('test-zhao-immune-liver.R')

test_that("ZhaoImmuneLiverData works as expected", {
    sce <- ZhaoImmuneLiverData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce2 <- ZhaoImmuneLiverData(filter=TRUE)
    expect_s4_class(ncol(sce) > ncol(sce2))
    expect_null(sce2$retained)
})

#' Obtain ERCC molecule counts from concentrations
#'
#' Compute the number of molecules for each transcript in the ERCC spike-in mixture,
#' based on their published concentration as well as the volume of the diluted mixture added to each cell.
#'
#' @param volume Numeric scalar specifying the added volume (in microliters) of ERCC spike-in mixture.
#' @param dilution Numeric scalar specifying the dilution factor used for the added volume of the spike-in mixture.
#' @param mix String specifying whether to compute the number of molecules for mix 1 or 2.
#' @param ... Further arguments to pass to \code{\link{fetchDataset}}.
#'
#' @return A \link[S4Vectors]{DataFrame} object with one row per ERCC spike-in transcript.
#' This contains the estimated concentration and molecule count for each transcript.
#'
#' @author Aaron Lun, based on code from Alan O'Callaghan
#'
#' @examples
#' countErccMolecules(volume = 9, dilution = 300000)
#' 
#' @export
#' @importFrom S4Vectors DataFrame
countErccMolecules <- function(volume, dilution, mix=c("1", "2"), ...) {
    tab <- fetchDataset("ercc", "2023-12-20", ...)

    field <- sprintf("concentration in Mix %s (attomoles/ul)", match.arg(mix))
    molarity <- tab[[field]] * 10^-18 # get concentration in mol/uL.
    molecules_ul <- molarity * (6.02214076 * (10^23))

    # Previously, 'volume' was treated as nanolitres, but I'm pretty sure that
    # was not correct; who pipettes nL anyway? I'm pretty sure uL was intended.
    molecules <- molecules_ul / dilution * volume

    DataFrame(
        row.names=rownames(tab),
        subgroup=tab$subgroup,
        concentration=tab[[field]],
        molecules=molecules
    )
}

#' Obtain the Stoeckius cell hashing data
#'
#' Obtain the (mostly human) cell hashing single-cell RNA-seq data from Stoeckius et al. (2018).
#'
#' @param type String specifying the dataset to obtain.
#' @param mode String specifying the data modalities to obtain, see Details.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param strip.metrics Logical scalar indicating whether quality control metrics should be removed from the HTO/ADT counts.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' When \code{type="pbmc"}, the \code{mode} can be one or more of:
#' \itemize{
#' \item \code{"human"}, the RNA counts for human genes.
#' \item \code{"mouse"}, the RNA counts for mouse genes.
#' Present as the PBMC dataset is actually a mixture of human PBMCs and unlabelled mouse cells.
#' \item \code{"hto"}, the HTO counts.
#' \item \code{"adt1"}, counts for the first set of ADTs (immunoglobulin controls).
#' \item \code{"adt2"}, counts for the second set of ADTs (cell type-specific markers).
#' }
#' If \code{mode=NULL}, the default is to use \code{"human"}, \code{"mouse"} and \code{"hto"}.
#'
#' When \code{type="mixed"}, the \code{mode} can be one or more of:
#' \itemize{
#' \item \code{"rna"}, the RNA counts for the genes;
#' \item \code{"hto"}, the HTO counts.
#' }
#' If \code{mode=NULL}, the default is to use \code{"rna"} and \code{"hto"}.
#'
#' If \code{ensembl=TRUE}, gene symbols for the RNA counts are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE} and only for the RNA counts.
#'
#' For the HTO and ADT matrices, some rows correspond to quality control metrics.
#' If \code{strip.metrics=TRUE}, these rows are removed so that only data for actual HTOs or ADTs are present.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/nestorowa-hsc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a matrix of UMI counts corresponding to the first \code{mode},
#' plus any number of alternative Experiments containing the remaining \code{mode}s.
#' If multiple \code{mode}s are specified, the output object only contains the intersection of their column names.
#'
#' @author Aaron Lun
#'
#' @references
#' Stoeckius et al. (2018). 
#' Cell Hashing with barcoded antibodies enables multiplexing and doublet detection for single cell genomics.
#' \emph{Genome Biol.} 19, 224.
#'
#' @examples
#' sce.pbmc <- StoeckiusHashingData()
#' sce.pbmc
#'
#' sce.mixed <- StoeckiusHashingData(type="mixed")
#' sce.mixed
#' 
#' @export
#' @importFrom utils head
#' @importFrom SingleCellExperiment altExps<- 
StoeckiusHashingData <- function(type=c("pbmc", "mixed"), mode=NULL, ensembl=FALSE, location=TRUE, strip.metrics=TRUE, legacy=FALSE) {
    type <- match.arg(type)
    if (type=="pbmc") {
        acceptable <- c("human", "mouse", "hto", "adt1", "adt2")
    } else {
        acceptable <- c("rna", "hto")
    }

    if (is.null(mode)) {
        mode <- head(acceptable, 3)
    } else {
        mode <- match.arg(mode, acceptable, several.ok=TRUE)
    }

    collated <- list()
    if (!legacy && strip.metrics && type == "mixed") {
        # We just can't do it for PBMC, it's just too much of a mess to
        # preserve back-compatibility, given that we decided to omit the
        # human/mouse splits from the saved rowData. Better to omit it than to
        # give people a misleading impression of a confident split.
        for (m in mode) {
            collated[[m]] <- fetchDataset("stoeckius-hashing-2018", "2024-04-18", path=paste0("mixture/", m), realize.assays=TRUE)
        }

    } else {
        version <- "2.4.0"
        for (m in mode) {
            collated[[m]] <- .create_sce(file.path("stoeckius-hashing", version), 
                suffix=paste0(type, "-", m), has.rowdata=FALSE, has.coldata=FALSE)
        }

        # Stripping metrics from the ADT matrices.
        if (strip.metrics) {
            strippable <- c("hto", "adt1", "adt2")
            metrics <- c("no_match", "ambiguous", "total_reads", "bad_struct")
            for (i in intersect(names(collated), strippable)) {
                collated[[i]] <- collated[[i]][setdiff(rownames(collated[[i]]), metrics),]
            }
        }
    }

    if (length(collated) > 1) {
        common <- Reduce(intersect, lapply(collated, colnames))
        for (m in seq_along(collated)) {
            collated[[m]] <- collated[[m]][,common]
        }
    }

    # Converting everything to Ensembl and adding locations.
    if (type=="pbmc") {
        convertible <- c(human="Hs", mouse="Mm")
    } else {
        convertible <- c(rna="Hs")
    }

    for (i in intersect(names(convertible), names(collated))) {
        collated[[i]] <- .convert_to_ensembl(collated[[i]],
            species=convertible[i],
            symbols=rownames(collated[[i]]),
            ensembl=ensembl,
            location=location)
    }

    primary <- collated[[1]]
    altExps(primary) <- collated[-1]
    primary
}

#' Save a dataset to disk
#'
#' Save a single-cell dataset to disk, usually in preparation for upload.
#'
#' @param x A \link[SummarizedExperiment]{SummarizedExperiment} or one of its subclasses.
#' @param path String containing the path to a new directory in which to save \code{x}.
#' Any existing directory is removed before saving \code{x}.
#' @param metadata Named list containing metadata for this dataset,
#' see the schema returned by \code{\link[gypsum]{fetchMetadataSchema}()}.
#' Note that the \code{applications.takane} property will be automatically added by this function and does not have to be supplied.
#'
#' @return \code{x} and its metadata are saved into \code{path}, and \code{NULL} is invisibly returned.
#'
#' @seealso
#' \url{https://github.com/ArtifactDB/bioconductor-metadata-index}, on the expected schema for the metadata.
#'
#' \code{\link{polishDataset}}, to polish \code{x} before saving it.
#'
#' \code{\link[gypsum]{uploadDirectory}}, to upload the saved contents.
#' 
#' @author Aaron Lun
#' @examples
#' library(SingleCellExperiment)
#' sce <- SingleCellExperiment(list(counts=matrix(rpois(1000, lambda=1), 100, 10)))
#' rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(sce)))
#' colnames(sce) <- head(LETTERS, 10)
#'
#' meta <- list(
#'     title="My dataset",
#'     description="This is my dataset",
#'     taxonomy_id="10090",
#'     genome="GRCh38",
#'     sources=list(list(provider="GEO", id="GSE12345")),
#'     maintainer_name="Shizuka Mogami",
#'     maintainer_email="mogami.shizuka@765pro.com"
#' )
#' 
#' tmp <- tempfile()
#' saveDataset(sce, tmp, meta)
#' list.files(tmp, recursive=TRUE)
#' alabaster.base::readObject(tmp)
#' 
#' @export
#' @importFrom alabaster.base saveObject
#' @importFrom gypsum fetchMetadataSchema validateMetadata formatObjectMetadata
#' @importMethodsFrom alabaster.sce saveObject
#' @importFrom jsonlite toJSON
saveDataset <- function(x, path, metadata) {
    schema <- fetchMetadataSchema()

    if (is.null(metadata$bioconductor_version)) {
        metadata$bioconductor_version <- as.character(BiocManager::version())
    }
    metadata$taxonomy_id <- I(metadata$taxonomy_id)
    metadata$genome <- I(metadata$genome)
    validateMetadata(metadata, schema) # First validation for user-supplied content.

    unlink(path, recursive=TRUE)
    alabaster.base::saveObject(x, path, ReloadedArray.reuse.files="symlink")

    takane <- formatObjectMetadata(x)
    takane$type <- readObjectFile(path)$type

    if (is.null(metadata$applications)) {
        metadata$applications <- list()
    }
    metadata$applications$takane <- takane

    # Second validation with the takane metadata.
    contents <- toJSON(metadata, pretty=4, auto_unbox=TRUE)
    validateMetadata(contents, schema=schema)
    write(contents, file=file.path(path, "_bioconductor.json"))
    invisible(NULL)
}

# This checks that saveDataset works as expected.
# library(testthat); library(scRNAseq); source("test-saveDataset.R")

sce <- SingleCellExperiment(list(counts=matrix(rpois(1000, lambda=1), 100, 10)))
sce$foo <- sample(letters, 10)
rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(sce)))
colnames(sce) <- head(LETTERS, 10)

test_that("saveDataset works as expected", {
    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(list(provider="GEO", id="GSE12345")),
        maintainer_name="Shizuka Mogami",
        maintainer_email="mogami.shizuka@765pro.com"
    )

    tmp <- tempfile()
    saveDataset(sce, tmp, meta)
    roundtrip <- alabaster.base::readObject(tmp)
    expect_identical(colData(roundtrip), colData(sce))
    expect_identical(as.matrix(counts(roundtrip)), assay(sce))

    meta <- jsonlite::fromJSON(file.path(tmp, "_bioconductor.json"), simplifyVector=FALSE)
    expect_identical(meta$bioconductor_version, as.character(BiocManager::version()))

    # Validation fails as expected.
    tmp <- tempfile()
    meta$title <- 1234
    expect_error(saveDataset(sce, tmp, meta), "title")
})

test_that("saveDataset works with ReloadedArray objects", {
    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(list(provider="GEO", id="GSE12345")),
        maintainer_name="Shizuka Mogami",
        maintainer_email="mogami.shizuka@765pro.com"
    )

    # Saving something to link to.
    tmp <- tempfile()
    saveDataset(sce, tmp, meta)

    # Adding some ReloadedArray assays.
    sce2 <- sce
    assay(sce2, "foobar", withDimnames=FALSE) <- alabaster.matrix::ReloadedArray(path=file.path(tmp, "assays", "0"), seed=matrix(0, 100, 10))
    assay(sce2, "blah", withDimnames=FALSE) <- alabaster.matrix::ReloadedArray(path=file.path(tmp, "assays", "0"), seed=matrix(0, 100, 10))

    # Checking that the link targets are created correctly.
    tmp2 <- tempfile()
    saveDataset(sce2, tmp2, meta)
    expect_true(nchar(Sys.readlink(file.path(tmp2, "assays", "1", "array.h5"))) > 0L) # check that these are indeed symlinks.
    expect_true(nchar(Sys.readlink(file.path(tmp2, "assays", "2", "OBJECT"))) > 0L)

    roundtrip <- alabaster.base::readObject(tmp2)
    expect_identical(as.matrix(assay(roundtrip, 2)), assay(sce)) # all assays are just symlinked to the first save!
    expect_identical(as.matrix(assay(roundtrip, 3)), assay(sce))
})

#' Obtain the Macosko retina data
#'
#' Obtain the mouse retina single-cell RNA-seq data from Macosko et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata contains the cluster identity as reported in the paper.
#' Note that some cells will have \code{NA} identities as they are present in the count matrix but not in the metadata file.
#' These are presumably low-quality cells that were discarded prior to clustering.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/macosko-retina}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Macosko E et al. (2016). 
#' Highly parallel genome-wide expression profiling of individual cells using nanoliter droplets. 
#' \emph{Cell} 161(5), 1202-1214.
#'
#' @examples
#' sce <- MacoskoRetinaData()
#' 
#' @export
#' @importFrom AnnotationHub AnnotationHub
#' @importFrom AnnotationDbi select keys
MacoskoRetinaData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("macosko-retina-2015", "2023-12-19", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("macosko-retina", version), has.rowdata=FALSE)
    }

    if (ensembl) {
        # For some bizarre reason, this dataset has all-caps symbols... for mice.
        # So we need to do some custom work to ensure that this converts properly.
        tag <- "AH73905"
        edb <- AnnotationHub()[[tag]]

        anno <- select(edb, keys=keys(edb), keytype="GENEID", columns="SYMBOL")
        all.symbols <- tolower(anno$SYMBOL)
        cur.symbols <- tolower(rownames(sce)) 

        ensid <- anno$GENEID[match(cur.symbols, all.symbols)]
        keep <- !is.na(ensid) & !duplicated(ensid)

        sce <- sce[keep, ]
        rownames(sce) <- ensid[keep]

        sce <- .define_location_from_ensembl(sce, species="Mm", location=location)
    }

    sce
}

#' List all paths for a dataset
#'
#' List the available paths to subdatasets within a version of a given dataset.
#' 
#' @inheritParams fetchDataset
#' @param include.metadata Logical scalar indicating whether to report the metadata for each subdataset.
#'
#' @return If \code{include.metadata=FALSE}, a character vector containing the paths to subdatasets within the specified version of the dataset.
#' If \code{name} does not contain any subdatasets, \code{NA} is returned.
#'
#' Otherwise, a \link[S4Vectors]{DFrame} is returned containing the metadata for each subdataset, e.g., the title and description.
#' More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 
#'
#' @author Aaron Lun
#'
#' @examples
#' listPaths("he-organs-2020", "2023-12-21")
#' listPaths("he-organs-2020", "2023-12-21", include.metadata=TRUE)
#' listPaths("zeisel-brain-2015", "2023-12-14") # no subdatasets
#' listPaths("zeisel-brain-2015", "2023-12-14", include.metadata=TRUE) 
#'
#' @export
#' @importFrom jsonlite fromJSON
#' @importFrom gypsum saveFile listFiles
listPaths <- function(name, version, package="scRNAseq", cache=cacheDirectory(), overwrite=FALSE, include.metadata=FALSE) {
    all.files <- listFiles(package, name, version, include..=FALSE)
    all.files <- all.files[basename(all.files) == "_bioconductor.json"]
    if (!include.metadata) {
        if (identical(all.files, "_bioconductor.json")) {
            return(NA_character_)
        } else {
            return(dirname(all.files))
        }
    }

    metadata <- list()
    for (x in seq_along(all.files)) {
        local.path <- saveFile(package, name, version, path=all.files[x], cache=cache, overwrite=overwrite)
        metadata[[x]] <- paste(readLines(local.path), collapse="\n")
    }

    sanitize_query_to_output(list(
        meta=metadata,
        path=all.files,
        asset=rep(name, length(metadata)),
        version=rep(version, length(metadata))
    ), latest=TRUE)
}

# library(scRNAseq); library(testthat); source("test-listPaths.R")

test_that("listPaths works correctly with names", {
    he.listing <- listPaths("he-organs-2020", "2023-12-21")
    expect_gt(length(he.listing), 2)
    expect_true("blood" %in% he.listing)

    z.listing <- listPaths("zeisel-brain-2015", "2023-12-14") # no subdatasets
    expect_identical(length(z.listing), 1L)
    expect_true(is.na(z.listing))
})

test_that("listPaths works correctly with metadata", {
    he.listing <- listPaths("he-organs-2020", "2023-12-21", include.metadata=TRUE)
    expect_gt(nrow(he.listing), 2)
    expect_true("bladder" %in% he.listing$path)

    z.listing <- listPaths("zeisel-brain-2015", "2023-12-14", include.metadata=TRUE) 
    expect_identical(nrow(z.listing), 1L)
    expect_true(is.na(z.listing$path))
})

#' Reprocessed single-cell data sets
#'
#' Obtain the legacy count matrices for three publicly available single-cell RNA-seq datasets.
#' Raw sequencing data were downloaded from NCBI's SRA or from EBI's ArrayExpress,
#' aligned to the relevant genome build and used to quantify gene expression.
#'
#' @param assays Character vector specifying one or more assays to return.
#' Choices are \code{"tophat_counts"}, \code{"cufflinks_fpkm"}, \code{"rsem_counts"} and \code{"rsem_tpm"}.
#' If \code{NULL}, all assays are returned.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @return
#' A \linkS4class{SingleCellExperiment} object containing one or more expression matrices of counts and/or TPMs,
#' depending on \code{assays}.
#' 
#' @details
#' \code{ReprocessedFluidigmData} returns a dataset of 65 human neural cells from Pollen et al. (2014), 
#' each sequenced at high and low coverage (SRA accession SRP041736).
#' 
#' \code{ReprocessedTh2Data} returns a dataset of 96 mouse T helper cells from Mahata et al. (2014),
#' obtained from ArrayExpress accession E-MTAB-2512.
#' Spike-in counts are stored in the \code{"ERCC"} entry of the \code{\link{altExps}}.
#'
#' \code{ReprocessedAllenData} return a dataset of 379 mouse brain cells from Tasic et al. (2016).
#' This is a re-processed subset of the data from \code{\link{TasicBrainData}},
#' and contains spike-in information stored as in the \code{\link{altExps}}.
#'
#' In each dataset, the first columns of the \code{colData} are sample quality metrics from FastQC and Picard.
#' The remaining fields were obtained from the original study in their GEO/SRA submission
#' and/or as Supplementary files in the associated publication.
#' These two categories of \code{colData} are distinguished by a \code{which_qc} element in the \code{\link{metadata}},
#' which contains the names of the quality-related columns in each object.
#' 
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/legacy-allen},
#' \code{scRNAseq/legacy-fluidigm} or \code{scRNAseq/legacy-th2}.
#'
#' @section Pre-processing details:
#' FASTQ files were either obtained directly from ArrayExpress, 
#' or converted from SRA files (downloaded from the Sequence Read Archive) using the SRA Toolkit.
#'
#' Reads were aligned with TopHat (v. 2.0.11) to the appropriate reference genome (GRCh38 for human samples, GRCm38 for mouse). 
#' RefSeq mouse gene annotation (GCF_000001635.23_GRCm38.p3) was downloaded from NCBI on Dec. 28, 2014. 
#' RefSeq human gene annotation (GCF_000001405.28) was downloaded from NCBI on Jun. 22, 2015.
#'
#' featureCounts (v. 1.4.6-p3) was used to compute gene-level read counts.
#' Cufflinks (v. 2.2.0) was used to compute gene-leve FPKMs.
#' Reads were also mapped to the transcriptome using RSEM (v. 1.2.19) to compute read counts and TPM's.
#' 
#' FastQC (v. 0.10.1) and Picard (v. 1.128) were used to compute sample quality control (QC) metrics. 
#' However, no filtering on the QC metrics has been performed for any dataset.
#'
#' @references
#' Pollen AA et al. (2014). 
#' Low-coverage single-cell mRNA sequencing reveals cellular heterogeneity and activated signaling pathways in developing cerebral cortex. 
#' \emph{Nat. Biotechnol.} 32(10), 1053-8.
#'
#' Mahata B et al. (2014).
#' Single-cell RNA sequencing reveals T helper cells synthesizing steroids de novo to contribute to immune homeostasis. 
#' \emph{Cell Rep}, 7(4), 1130-42.
#'
#' Tasic A et al. (2016). 
#' Adult mouse cortical cell taxonomy revealed by single cell transcriptomics.
#' \emph{Nat. Neurosci.} 19(2), 335-46.
#'
#' @examples
#' sce <- ReprocessedAllenData()
#'
#' @export
#' @rdname ReprocessedData
#' @importFrom SingleCellExperiment splitAltExps
#' @importFrom SummarizedExperiment assays assays<-
ReprocessedAllenData <- function(assays=NULL, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("legacy", "2023-12-18", "allen", realize.assays=TRUE)
        if (!is.null(assays)) {
            assays(sce, withDimnames=FALSE) <- assays(sce, withDimnames=FALSE)[assays]
        }
    } else {
        version <- "1.10.0"
        sce <- .create_sce_legacy(file.path("legacy-allen", version), assays)
        status <- ifelse(grepl("^ERCC-[0-9]+$", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' @export
#' @rdname ReprocessedData
#' @importFrom SingleCellExperiment splitAltExps
ReprocessedTh2Data <- function(assays=NULL, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("legacy", "2023-12-18", "th2", realize.assays=TRUE)
        if (!is.null(assays)) {
            assays(sce, withDimnames=FALSE) <- assays(sce, withDimnames=FALSE)[assays]
        }
    } else {
        version <- "1.10.0"
        sce <- .create_sce_legacy(file.path("legacy-th2", version), assays)
        status <- ifelse(grepl("^ERCC-[0-9]+$", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' @export
#' @rdname ReprocessedData
ReprocessedFluidigmData <- function(assays=NULL, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("legacy", "2023-12-18", "fluidigm", realize.assays=TRUE)
        if (!is.null(assays)) {
            assays(sce, withDimnames=FALSE) <- assays(sce, withDimnames=FALSE)[assays]
        }
    } else {
        version <- "1.10.0"
        sce <- .create_sce_legacy(file.path("legacy-fluidigm", version), assays)
    }

    .convert_to_ensembl(sce, 
        species="Hs", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

# This checks through all of the getter functions.
# Note that we don't re-run any of the code in the Examples.
# library(testthat); library(scRNAseq); source("test-loading.R")

CHECK <- function(sce) {
    expect_true(all(dim(sce) > 0))
    expect_true(length(assayNames(sce)) > 0)
}

test_that("BaronPancreasData works", {
    CHECK(BaronPancreasData(ensembl=TRUE))
    CHECK(BaronPancreasData("mouse", ensembl=TRUE))
})

test_that("BuettnerESCData works", {
    out <- BuettnerESCData()
    CHECK(out)

    out <- BuettnerESCData(remove.htseq=FALSE)
    CHECK(out)
    expect_s4_class(rowRanges(out), "GRangesList")
})

test_that("BunisHSPCData works", {
    CHECK(BunisHSPCData())
})

test_that("CampbellBrainData works", {
    CHECK(CampbellBrainData(ensembl=TRUE))
})

test_that("ChenBrainData works", {
    CHECK(ChenBrainData(ensembl=TRUE))
})

test_that("GrunHSCData works", {
    CHECK(GrunHSCData(ensembl=TRUE))
})

test_that("GrunPancreasData works", {
    CHECK(GrunPancreasData(ensembl=TRUE))
})

test_that("HermannSpermatogenesisData works", {
    CHECK(HermannSpermatogenesisData())
})

test_that("KolodziejczykESCData works", {
    out <- KolodziejczykESCData()
    CHECK(out)

    out <- KolodziejczykESCData(remove.htseq=FALSE)
    CHECK(out)
    expect_s4_class(rowRanges(out), "GRangesList")
})

test_that("LaMannoBrainData works", {
    CHECK(LaMannoBrainData(ensembl=TRUE, "human-es"))
    CHECK(LaMannoBrainData(ensembl=TRUE, "human-embryo"))
    CHECK(LaMannoBrainData(ensembl=TRUE, "human-ips"))
    CHECK(LaMannoBrainData(ensembl=TRUE, "mouse-adult"))
    CHECK(LaMannoBrainData(ensembl=TRUE, "mouse-embryo"))
})

test_that("LunSpikeInData works", {
    out <- LunSpikeInData()
    CHECK(out)
    expect_s4_class(rowRanges(out), "GRangesList")

    out <- LunSpikeInData(split.oncogene=TRUE)
    expect_true("oncogene" %in% altExpNames(out))
    CHECK(out)

    CHECK(LunSpikeInData("tropho"))
})

test_that("MacoskoRetinaData works", {
    CHECK(MacoskoRetinaData(ensembl=TRUE))
})

test_that("MarquesBrainData works", {
    CHECK(MarquesBrainData(ensembl=TRUE))
})

test_that("MuraroPancreasData works", {
    CHECK(MuraroPancreasData(ensembl=TRUE))
})

test_that("NestorowaHSCData works", {
    CHECK(NestorowaHSCData(remove.htseq=FALSE))
})

test_that("PaulHSCData works", {
    CHECK(PaulHSCData(ensembl=TRUE))
    CHECK(PaulHSCData(discard.multiple=FALSE))
})

test_that("Reprocessed*Data works", {
    CHECK(ReprocessedAllenData(assays="tophat_counts", ensembl=TRUE))
    CHECK(ReprocessedTh2Data(assays="tophat_counts", ensembl=TRUE))
    CHECK(ReprocessedFluidigmData(assays="tophat_counts", ensembl=TRUE))
})

test_that("RomanovBrainData works", {
    CHECK(RomanovBrainData(ensembl=TRUE))
})

test_that("SegerstolpePancreasData works", {
    CHECK(SegerstolpePancreasData(ensembl=TRUE))
})

test_that("ShekharRetinaData works", {
    CHECK(ShekharRetinaData(ensembl=TRUE))
})

test_that("TasicBrainData works", {
    CHECK(TasicBrainData(ensembl=TRUE))
})

test_that("UsoskinBrainData works", {
    CHECK(UsoskinBrainData(ensembl=TRUE))
})

test_that("XinPancreasData works", {
    CHECK(XinPancreasData(ensembl=TRUE))
})

test_that("ZeiselBrainData works", {
    CHECK(ZeiselBrainData(ensembl=TRUE))
})

test_that("ZilionisBrianData works", {
    CHECK(ZilionisLungData(ensembl=TRUE))

    CHECK(ZilionisLungData("mouse", ensembl=TRUE))
})

test_that("HermannSpermatogenesisData works", {
    CHECK(HermannSpermatogenesisData(strip=TRUE))
})

test_that("StoeckiusHashingData works", {
    CHECK(StoeckiusHashingData(ensembl=TRUE))

    CHECK(StoeckiusHashingData(mode=c("adt1", "hto"), ensembl=TRUE))

    CHECK(out <- StoeckiusHashingData(mode=c("adt1", "adt2"), strip.metrics=FALSE))
    expect_true("no_match" %in% rownames(out))

    # Handles the human or mouse as a non-primary experiment.
    CHECK(out <- StoeckiusHashingData(mode=c("hto", "human"), ensembl=TRUE))
    expect_true(all(grepl("ENSG", rownames(altExp(out, "human")))))

    CHECK(out <- StoeckiusHashingData(mode=c("adt1", "mouse"), ensembl=TRUE))
    expect_true(all(grepl("ENSMUSG", rownames(altExp(out, "mouse")))))

    # Repeating this for the cell line mixtures.
    CHECK(StoeckiusHashingData(type="mixed", strip.metrics=FALSE))

    CHECK(StoeckiusHashingData(type="mixed", ensembl=TRUE))
})

test_that("MairPBMCData works", {
    CHECK(MairPBMCData(ensembl=TRUE))

    CHECK(MairPBMCData(mode=c("adt", "rna"), ensembl=TRUE))

    CHECK(MairPBMCData(mode="adt"))
})

test_that("KotliarovPBMCData works", {
    CHECK(KotliarovPBMCData(ensembl=TRUE))

    CHECK(KotliarovPBMCData(mode=c("adt", "rna"), ensembl=TRUE))

    CHECK(KotliarovPBMCData(mode="adt"))
})

test_that("HuCortexData works", {
    skip_on_os("windows") # because 32-bit Windows CAN'T HANDLE THE TRUTH!

    CHECK(HuCortexData(ensembl=TRUE))

    CHECK(HuCortexData(samples=c("cell-3T3")))

    CHECK(HuCortexData(samples=c("cell-3T3", "nuclei-ctx-1")))
})

test_that("WuKidneyData works", {
    CHECK(WuKidneyData(ensembl=TRUE))
})

test_that("ERCCSpikeInConcentrations works", {
    table <- ERCCSpikeInConcentrations()
    expect_s4_class(table, "DFrame")
})

#' Obtain the Mair CITE-seq data
#'
#' Obtain the Mair PBMC targeted CITE-seq data from Mair et al. (2020).
#'
#' @param mode Character vector specifying whether to return either or both the RNA and ADT counts.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata contains the donor identity and cartridge of origin.
#' Some libraries may also be classified as multiplets or have undeterminate origins after hash tag debarcoding.
#'
#' If \code{ensembl=TRUE}, the gene symbols in the RNA data are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} for the RNA data.
#' Note that this is only performed if \code{ensembl=TRUE}.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/mair-pbmc}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts corresponding to the first \code{mode},
#' with an optional alternative Experiment if there is a second \code{mode}.
#'
#' @author
#' Stephany Orjuela,
#' with modifications from Aaron Lun
#'
#' @references
#' Mair C et al. (2020).
#' A targeted multi-omic analysis approach measures protein expression and low-abundance transcripts on the single-cell level.
#' \emph{Cell Rep.} 31, 107499
#'
#' @examples
#' sce <- MairPBMCData()
#' 
#' @export
#' @importFrom ExperimentHub ExperimentHub
#' @importFrom SummarizedExperiment colData<- rowData
#' @importFrom SingleCellExperiment SingleCellExperiment altExps 
MairPBMCData <- function(mode=c("rna", "adt"), ensembl=FALSE, location=TRUE, legacy=FALSE) {
    mode <- match.arg(mode, c("rna", "adt"), several.ok=TRUE)

    if (!legacy && identical(mode, c("rna", "adt"))) {
        sce <- fetchDataset("mair-pbmc-2020", "2024-04-18", realize.assays=TRUE)

    } else {
        version <- "2.4.0"
        tag <- "mair-pbmc"
        hub <- ExperimentHub()

        collated <- list()
        for (x in mode) {
            collated[[x]] <- .create_sce(file.path(tag, version), hub=hub, 
                has.rowdata=TRUE, has.coldata=FALSE, suffix=x)
        }

        if ("rna" %in% names(collated)) {
            collated[["rna"]] <- .convert_to_ensembl(collated[["rna"]],
                symbols=rowData(collated[["rna"]])$Symbol,
                species="Hs",
                ensembl=ensembl,
                location=location)
        }

        sce <- collated[[1]]
        altExps(sce) <- collated[-1]
        colData(sce) <- hub[hub$rdatapath==file.path("scRNAseq", tag, version, "coldata.rds")][[1]] 
    }

    sce
}

#' @importFrom AnnotationDbi mapIds
#' @importFrom SummarizedExperiment rowData rowData<-
.convert_to_ensembl <- function(sce, symbols, species, ahub.id=NULL, keytype="SYMBOL", ensembl=FALSE, location=TRUE) {
    if (!ensembl) {
        return(sce)
    }

    edb <- .pull_down_ensdb(species, ahub.id=ahub.id)
    ensid <- mapIds(edb, keys=symbols, keytype=keytype, column="GENEID")
    keep <- !is.na(ensid) & !duplicated(ensid)
    if (!all(keep)) {
        sce <- sce[keep,]
    }

    old <- rownames(sce)
    rownames(sce) <- ensid[keep]
    rowData(sce)$originalName <- old

    .define_location_from_ensembl(sce, edb=edb, location=location)
}

#' @importFrom SummarizedExperiment rowRanges<- rowRanges rowData
#' @importFrom S4Vectors mcols<-
#' @importFrom BiocGenerics cbind
#' @importFrom methods as
#' @importClassesFrom GenomicRanges GRangesList
#' @importFrom GenomicFeatures genes
.define_location_from_ensembl <- function(sce, species, ahub.id=NULL, edb=NULL, location=TRUE) {
    if (!location) {
        return(sce)
    }

    if (is.null(edb)) {
        edb <- .pull_down_ensdb(species, ahub.id=ahub.id)
    }

    # Need to switch between a GRL and a GR, depending on whether
    # there are non-valid keys in 'sce'.
    ginfo <- genes(edb, columns=character(0))
    mcols(ginfo) <- NULL
    m <- match(rownames(sce), names(ginfo))
    present <- !is.na(m)

    if (!all(present)) {
        replacement <- rowRanges(sce)
        mcols(replacement) <- NULL
        ginfo <- as(ginfo, "GRangesList")
        replacement[present] <- ginfo[m[present]]
    } else {
        replacement <- ginfo[m]        
    }

    mcols(replacement) <- rowData(sce)
    rowRanges(sce) <- replacement
    sce
}

#' @importClassesFrom ensembldb EnsDb
#' @importFrom AnnotationHub AnnotationHub
.pull_down_ensdb <- function(species, ahub.id) {
    if (is.null(ahub.id)) {
        if (species=="Mm") {
            ahub.id <- "AH73905"
        } else if (species=="Hs") {
            ahub.id <- "AH73881"
        }
    }
    AnnotationHub()[[ahub.id]]
}

#' Obtain the Buettner ESC data
#'
#' Obtain the mouse embryonic stem cell single-cell RNA-seq data from Buettner et al. (2015).
#'
#' @param remove.htseq Logical scalar indicating whether HT-seq alignment statistics should be removed.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Rows corresponding to HT-seq's alignment statistics are removed by default.
#' These can be retained by setting \code{remove.htseq=FALSE}.
#'
#' Column metadata contains the experimentally determined cell cycle phase for each cell.
#'
#' Counts for ERCC spike-ins are stored in the \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/buettner-esc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Buettner F et al. (2015).
#' Computational analysis of cell-to-cell heterogeneity in single-cell RNA-sequencing data reveals hidden subpopulations of cells.
#' \emph{Nat. Biotechnol.} 33(2), 155-160.
#' 
#' @examples
#' sce <- BuettnerESCData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps 
BuettnerESCData <- function(remove.htseq=TRUE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("buettner-esc-2015", "2024-04-18", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("buettner-esc", version), has.rowdata=TRUE)

        if (remove.htseq) {
            sce <- sce[grepl("^ENSMUS", rownames(sce)) | grepl("^ERCC", rownames(sce)),]
        }

        status <- ifelse(grepl("^ERCC-[0-9]+", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 1, dilution = 1000)
        spikedata <- spikedata[rownames(spike.exp),]

        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        rowData(spike.exp)$featureType <- NULL # redundant field; what else would it be!?
        altExp(sce, "ERCC") <- spike.exp
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

#' Obtain the Segerstolpe pancreas data
#'
#' Download the human pancreas single-cell RNA-seq (scRNA-seq) dataset from Segerstolpe et al. (2016)
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Row data contains fields for the gene symbol and RefSeq transcript IDs corresponding to each gene.
#' The rows of the output object are named with the symbol, but note that these are not unique.
#'
#' Column metadata were extracted from the \code{Characteristics} fields of the SDRF file for ArrayExpress E-MTAB-5061.
#' This contains information such as the cell type labels and patient status.
#'
#' Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
#' Estimated numbers of spike-in molecules are provided in the \code{\link[SummarizedExperiment]{rowData}} of this entry. 
#' Note that these concentrations are incorrect for donor H1, as 100 uL of spike-in mixture were added for this donor, rather than 25 uL for all others.
#' 
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/segerstolpe-pancreas}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Segerstolpe A et al. (2016). 
#' Single-cell transcriptome profiling of human pancreatic islets in health and type 2 diabetes. 
#' \emph{Cell Metab.} 24(4), 593-607.
#'
#' @examples
#' sce <- SegerstolpePancreasData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps
#' @importFrom SummarizedExperiment rowData
SegerstolpePancreasData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("segerstolpe-pancreas-2016", "2023-12-19", realize.assays=TRUE)
        rowData(sce)$symbol <- rownames(sce)
        cn <- colnames(colData(sce))
        cn[cn == "inferred cell type"] <- "cell type"
        cn[cn == "single cell quality"] <- "single cell well quality"
        colnames(colData(sce)) <- cn

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("segerstolpe-pancreas", version))
        rownames(sce) <- rowData(sce)$symbol

        status <- ifelse(grepl("^ERCC-[0-9]+", rowData(sce)$refseq), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
        
        ## This is wrong for one donor - donor "H1" has 100ul rather than 25ul
        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 25, dilution = 40000)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

#' Obtain the Shekhar retina data
#'
#' Obtain the mouse retina single-cell RNA-seq dataset from Shekhar et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata contains the cluster identities as reported in the paper.
#' Note that some cells will have \code{NA} identities as they are present in the count matrix but not in the metadata file.
#' These are presumably low-quality cells that were discarded prior to clustering.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/shekhar-retina}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Shekhar K et al. (2016). 
#' Comprehensive classification of retinal bipolar neurons by single-cell transcriptomics.
#' \emph{Cell} 166(5), 1308-1323.
#'
#' @examples
#' sce <- ShekharRetinaData()
#' 
#' @export
ShekharRetinaData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("shekhar-retina-2016", "2023-12-19", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("shekhar-retina", version), has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Romanov brain data
#'
#' Obtain the mouse brain single-cell RNA-seq dataset from Romanov et al. (2017).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in GSE74672.
#' This contains information such as the reporter gene expressed in each cell, the mouse line, dissection type and so on.
#'
#' Counts for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
#' Note that some of the spike-in rows have \code{NA} observations for some (but not all) cells.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/romanov-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun,
#' based on code by Vladimir Kiselev and Tallulah Andrews.
#'
#' @references
#' Romanov RA et al. (2017).
#' Molecular interrogation of hypothalamic organization reveals distinct dopamine neuronal subtypes. 
#' \emph{Nat. Neurosci.} 20, 176-188.
#'
#' @examples
#' sce <- RomanovBrainData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps
RomanovBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("romanov-brain-2017", "2023-12-19", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("romanov-brain", version), has.rowdata=FALSE)
        status <- ifelse(grepl("^ERCC-[0-9]+", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

samples <- c(
    "cell-3T3",
    "nuclei-3T3",
    "nuclei-ctx-1",
    "nuclei-ctx-2",
    "nuclei-ctx-3", 
    "nuclei-ctx-4",
    "nuclei-ctx-5",
    "nuclei-ctx-6",
    "nuclei-ctx-7",
    "nuclei-ctx-8",
    "nuclei-ctx-9",
    "nuclei-ctx-10",
    "nuclei-ctx-11",
    "nuclei-ctx-12",
    "nuclei-ctx-13",
    "nuclei-ctx-saline1", 
    "nuclei-ctx-PTZ1", 
    "nuclei-ctx-saline2",
    "nuclei-ctx-PTZ2" 
)

details <- c(
    "3T3 cells",
    "3T3 nuclei",
    sprintf("mouse cortex cells (animal %i)", seq_len(13)),
    sprintf("mouse cortex cells (%s-treated, animal %i)",
        rep(c("saline", "PTZ"), rep=2),
        rep(1:2, each=2)
    )
)

files <- paste0("GSM28453", 58 + seq_along(samples))

write.csv(file="../../extdata/2.4.0/metadata-hu-cortex.csv", 
    data.frame(
        Title = sprintf("Hu %s counts", sub("cortex cells", "cortex", details)),
        Description = sprintf("Count matrix for %s in the Hu single-nucleus RNA-seq dataset", details),
        RDataPath = file.path("scRNAseq", "hu-cortex", "2.4.0", sprintf("counts-%s.rds", samples)),
        BiocVersion="3.12",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE106678",
        SourceVersion=sprintf("%s_%s_gene_exonic.intronic_tagged.dge.txt.gz", files, samples),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass="dgCMatrix",
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Obtain the Bunis haematopoietic stem and progenitor cell data
#'
#' Obtain the human fetal, newborn, and adult haematopoietic stem and progenitor cell single-cell RNA-seq dataset from Bunis et al. (2021).
#'
#' @param filtered Logical scalar or "cells" indicating whether to:
#' \itemize{
#' \item \code{TRUE}: filter out cells that were not used by the authors.
#' \item \code{"cells"}: filter out empty droplets as filtered out by cell ranger.
#' \item \code{FALSE}: no filtering
#' }
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is recreated from GEO using the author-supplied TSV of per-cell annotations, or retrieved from a processed version of the data shared by authors via figshare.
#' This contains information such as the tissue & sample of origin, age group, likely cell type, and Developmental Stage Scoring.
#' Within DevStageScoring element of the column metadata are the applied results ('<cell_type>_scores') of random forest regression trained on the fetal (score = 0) and adult (score = 1) cells of individual cell types indicated by ('<cell_type>_inTraining').
#'
#' If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
#' Otherwise, an additional \code{retained} field will be present in the \code{\link[SummarizedExperiment]{colData}}, indicating whether the cell was retained by the authors.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/bunis-hspc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Daniel Bunis
#'
#' @references
#' Bunis DG et al. (2021). 
#' Single-Cell Mapping of Progressive Fetal-to-Adult Transition in Human Naive T Cells
#' \emph{Cell Rep.} 34(1): 108573
#' 
#' @examples
#' sce <- BunisHSPCData()
#' 
#' @export
BunisHSPCData <- function(filtered=TRUE, legacy=FALSE) {
    if (!legacy && !isFALSE(filtered)) {
        sce <- fetchDataset("bunis-hspc-2021", "2023-12-21", realize.assays=TRUE)
        if (isTRUE(filtered)) {
            sce <- sce[,sce$retained]
            sce$retained <- NULL
        }

    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("bunis-hspc", version), has.rowdata = TRUE, has.coldata = FALSE)

        # Manually grabbing the colData as it doesn't have the right shape.
        hub <- ExperimentHub()
        colData.path <- file.path("scRNAseq", "bunis-hspc", version, "coldata.rds")
        colData <- hub[hub$rdatapath==colData.path][[1]]

        if (isTRUE(filtered)) {
            keep <- colnames(sce) %in% rownames(colData)[colData$retained]
            sce <- sce[,keep]
            colData$retained <- NULL
        } else if (identical(filtered, "cells")) {
            keep <- colnames(sce) %in% rownames(colData)
            sce <- sce[,keep]
        }

        # Weird performance issue when directly subsetting with rownames.
        # Also, preserve names when filtered=FALSE, though this takes some time.
        m <- match(colnames(sce), rownames(colData))
        colData <- colData[m,, drop = FALSE]
        rownames(colData) <- colnames(sce)
        colData(sce) <- colData
    }

    sce
}

# library(testthat); library(scRNAseq); source('test-bunis-hspc.R')

test_that("Bunis HSPC getter works as expected", {
  sce <- BunisHSPCData()
  expect_s4_class(sce, "SingleCellExperiment")
  
  sce2 <- BunisHSPCData(filtered = "cells")
  expect_s4_class(sce2, "SingleCellExperiment")
  
  sce3 <- BunisHSPCData(filtered=FALSE)
  
  # Checks cell filtering and that colData is added all at once
  expect_true( nrow(colData(sce)) < nrow(colData(sce2)) )
  expect_true( nrow(colData(sce2)) < nrow(colData(sce3)) )
  
  expect_true(all(grepl("^ENSG", rownames(sce))))
})

write.csv(file="../extdata/2.0.0/metadata-grun-pancreas.csv",
    data.frame(
        Title = "Grun pancreas counts",
        Description = "Count matrix for the Grun pancreas single-cell RNA-seq dataset", 
        RDataPath = file.path("scRNAseq", "grun-pancreas", "2.0.0", "counts.rds"),
        BiocVersion="3.10",
        Genome="hg19",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81076",
        SourceVersion="GSE81076_D2_3_7_10_17.txt.gz",
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass="dgCMatrix",
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Obtain the Kolodziejcyzk ESC data
#'
#' Obtain the mouse embryonic stem cell single-cell RNA-seq data from Kolodziejczyk et al. (2015).
#'
#' @param remove.htseq Logical scalar indicating whether HT-seq alignment statistics should be removed.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is generated from the column names,
#' and contains the culture conditions and the plate of origin for each cell.
#'
#' Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/kolodziejczyk-esc}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun 
#'
#' @references
#' Messmer T et al. (2019). 
#' Transcriptional heterogeneity in naive and primed human pluripotent stem cells at single-cell resolution. 
#' \emph{Cell Rep} 26(4), 815-824.e4
#'
#' @examples
#' sce <- KolodziejczykESCData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps
KolodziejczykESCData <- function(remove.htseq=TRUE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("kolodziejczyk-esc-2015", "2023-12-17", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("kolodziejczyk-esc", version), has.rowdata=FALSE, has.coldata=FALSE)

        sce$culture <- sub(".*mES_([^_]+)_.*", "\\1", colnames(sce))
        sce$plate <- sub(".*mES_[^_]+_([^_]+)_.*", "\\1", colnames(sce))

        if (remove.htseq) {
            sce <- sce[grep("^__", rownames(sce), invert=TRUE),]
        }

        spike.type <- ifelse(grepl("ERCC", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, spike.type, ref="endogenous")

        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 1, dilution = 25e6)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

#' Obtain the Hu cortex data
#'
#' Obtain the mouse cortex single-nuclei RNA-seq data from Hu et al. (2017).
#'
#' @param mode Character vector indicating whether to return data for the 3T3 cells or the mouse cortex.
#' @param samples Character vector indicating whether to return data for specific samples, see Details.
#' If specified, this overrides \code{mode}.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata includes the mode and sample corresponding to each cell/nuclei.
#' Available samples are:
#' \itemize{
#' \item \code{"cell-3T3"} and \code{"nuclei-3T3"}, generated from the 3T3 cell line.
#' \item \code{"nuclei-ctx-X"}, nuclei generated from the cortex of animal number X (from 1 to 13).
#' \item \code{"nuclei-ctx-salineX"} or \code{"nuclei-ctx-PTZX"}, nuclei generated from the cortex of saline- or PTZ-treated mice.
#' X represents the replicate number and can be 1 or 2.
#' }
#'
#' If multiple modes are requested, counts are only reported for the intersection of genes across all modes.
#' This is because the gene annotation in the original count matrices differs across modes.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/wu-kidney}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Hu P et al. (2017). 
#' Dissecting cell-type composition and activity-dependent transcriptional state in mammalian brains by massively parallel single-nucleus RNA-seq.
#' \emph{Mol. cell} 68, 1006-1015.
#'
#' @examples
#' sce <- HuCortexData("3T3")
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
HuCortexData <- function(mode=c("ctx", "3T3"), samples=NULL, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    mode <- match.arg(mode)

    if (!legacy) {
        if (mode == "ctx") {
            mode <- "cortex"
        }
        sce <- fetchDataset("hu-cortex-2017", "2023-12-20", path=mode, realize.assays=TRUE)

    } else {
        version <- "2.4.0"

        avail.samples <- c(
            "cell-3T3", "nuclei-3T3",
            sprintf("nuclei-ctx-%i", seq_len(13)),
            "nuclei-ctx-saline1", "nuclei-ctx-PTZ1", "nuclei-ctx-saline2", "nuclei-ctx-PTZ2"
        )
        avail.modes <- sub("^[^-]+-", "", avail.samples)
        avail.modes <- sub("-.*", "", avail.modes)
        
        if (!is.null(samples)) {
            samples <- match.arg(samples, avail.samples, several.ok=TRUE)
        } else {
            mode <- match.arg(mode, several.ok=TRUE)
            samples <- character(0)
            for (m in mode) { # convoluted to respect ordering of 'mode'.
                samples <- c(samples, avail.samples[avail.modes==m])
            }
        }
        sample.idx <- match(samples, avail.samples)

        all.sce <- list()
        for (s in sample.idx) {
            current <- .create_sce(file.path("hu-cortex", version), has.rowdata=FALSE, has.coldata=FALSE, suffix=avail.samples[s])
            current$Mode <- avail.modes[s]
            current$Sample <- avail.samples[s]
            all.sce <- c(all.sce, list(current))
        }

        if (length(all.sce) > 1) {
            common <- Reduce(intersect, lapply(all.sce, rownames))
            for (i in seq_along(all.sce)) {
                all.sce[[i]] <- all.sce[[i]][common,]
            }
        }

        sce <- do.call(cbind, all.sce)
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Hermann spermatogenesis data
#'
#' Obtain the mouse spermatogenesis single-cell RNA-seq data from Hermann et al. (2018).
#'
#' @param strip Logical scalar indicating whether to strip the \code{.X} notation from the row names.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' Only used if \code{strip=TRUE}.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata contains cell types provided by the data generators
#' at \url{https://data.mendeley.com/datasets/kxd5f8vpt4/1#file-fe79c10b-c42e-472e-9c7e-9a9873d9b3d8}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/hermann-spermatogenesis}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with two matrices, containing spliced and unspliced counts, respectively.
#'
#' @author Charlotte Soneson
#'
#' @references
#' Hermann B.P. et al. (2018).
#' The Mammalian Spermatogenesis Single-Cell Transcriptome, from Spermatogonial Stem Cells to Spermatids. 
#' \emph{Cell Rep.} 25(6), 1650-1667.e8. 
#'
#' @examples
#' sce <- HermannSpermatogenesisData()
#' 
#' @export
HermannSpermatogenesisData <- function(strip=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("splicing-demonstration-2020", "2023-12-20", realize.assays=TRUE)
    } else {
        version <- "2.4.0"
        sce <- .create_sce(file.path("hermann-spermatogenesis", version), 
            assays = c("spliced", "unspliced"), has.rowdata=FALSE)
    }

    if (strip) {
        rownames(sce) <- sub("\\..*", "", rownames(sce))

        # We know we mapped it to Gencode vM24 --> Ensembl 99, 
        # hence the hard-coded AnnotationHub ID.
        sce <- .define_location_from_ensembl(sce, ahub.id="AH78811", location=location)
    }

    sce 
}

write.csv(file="../../extdata/2.6.0/metadata-zhao-immune-liver.csv",
    data.frame(
        Title = sprintf("Zhao immune liver %s", c("counts", "colData", "rowData")),
        Description = sprintf("%s for the Zhao immune liver single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata", "Per-gene metadata")),
        RDataPath = file.path("scRNAseq", "zhao-immune-liver", "2.6.0", 
            c("counts.rds", "coldata.rds", "rowdata.rds")),
        BiocVersion="3.13",
        Genome="hg38",
        SourceType=c("MTX", "TSV", "TSV"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE125188",
        SourceVersion=c(
            "GSE125188_CD45+_matrix.mtx.gz",
            "GSE125188_CD45+_barcodes.tsv.gz;GSE125188_CD45+_cell_identities.tsv.gz;GSE125188_ASC_cell_identities.tsv.gz;GSE125188_B_cell_identities.tsv.gz;GSE125188_Mo_cell_identities.tsv.gz;GSE125188_NKT_cell_identities.tsv.gz",
            "GSE125188_CD45+_genes.tsv.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.6.0/metadata-darmanis-brain.csv",
    data.frame(
        Title = sprintf("Darmanis brain %s", c("counts", "colData")),
        Description = sprintf("%s for the Darmanis human brain single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "darmanis-brain", "2.6.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.13",
        Genome="hg38",
        SourceType=c("TSV", "TXT"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE162086",
        SourceVersion=c(
            "GSE67835_RAW.tar",
            "GSE67835-GPL15520_series_matrix.txt.gz;GSE67835-GPL18573_series_matrix.txt.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Polish dataset for saving
#'
#' Prepare a \link[SummarizedExperiment]{SummarizedExperiment} or \link[SingleCellExperiment]{SingleCellExperiment} to be saved with \code{\link{saveDataset}}.
#' This performs some minor changes to improve storage efficiency.
#'
#' @param x A \link[SummarizedExperiment]{SummarizedExperiment} or one of its subclasses.
#' @param strip.inner.names Logical scalar indicating whether to strip redundant names from internal objects, 
#' e.g., dimnames of the assays, row names of reduced dimensions, column names of alternative experiments.
#' This saves some space in the on-disk representation.
#' @param reformat.assay.by.density Numeric scalar indicating whether to optimize assay formats based on the density of non-zero values.
#' Assays with densities above this number are converted to ordinary dense arrays (if they are not already), while those with lower densities are converted to sparse matrices.
#' This can be disabled by setting it to \code{NULL}.
#' @param attempt.integer.conversion Logical scalar indicating whether to convert double-precision assays containing integer values to actually have the integer type.
#' This can improve efficiency of downstream applications by avoiding the need to operate in double precision.
#' @param forbid.nested.altexp Logical scalar indicating whether nested alternative experiments (i.e., alternative experiments of alternative experiments) should be forbidden.
#' This defaults to \code{TRUE} as nested alternative experiments are usually the result of some mistake in \code{\link[SingleCellExperiment]{altExp}} preparation.
#' @param remove.altexp.coldata Logical scalar indicating whether column data for alternative experiments should be removed.
#' This defaults to \code{TRUE} as the alternative experiment column data is usually redundant with that of the main experiment.
#'
#' @return A modified copy of \code{x}.
#'
#' @author Aaron Lun
#'
#' @examples
#' mat <- matrix(rpois(1000, lambda=0.2), 100, 10) * 1.0
#' rownames(mat) <- sprintf("GENE_%i", seq_len(nrow(mat)))
#' colnames(mat) <- head(LETTERS, 10)
#'
#' library(SingleCellExperiment)
#' sce <- SingleCellExperiment(list(counts=mat))
#' str(assay(sce, withDimnames=FALSE))
#'
#' polished <- polishDataset(sce)
#' str(assay(polished, withDimnames=FALSE))
#'
#' @export
polishDataset <- function(x, strip.inner.names=TRUE, reformat.assay.by.density=0.3, attempt.integer.conversion=TRUE, remove.altexp.coldata=TRUE, forbid.nested.altexp=TRUE) {
    .polish_dataset(x, 
        strip.inner.names=strip.inner.names, 
        reformat.assay.by.density=reformat.assay.by.density, 
        attempt.integer.conversion=attempt.integer.conversion, 
        remove.altexp.coldata=remove.altexp.coldata, 
        forbid.nested.altexp=forbid.nested.altexp
    )
}

#' @import methods
#' @importFrom BiocGenerics mean
#' @importFrom DelayedArray is_sparse type<- DelayedArray
#' @importClassesFrom SparseArray SVT_SparseMatrix
#' @importFrom SummarizedExperiment assay assayNames
#' @importFrom SingleCellExperiment reducedDim reducedDimNames reducedDim<- altExp altExpNames altExp<-
.polish_dataset <- function(x, strip.inner.names, reformat.assay.by.density, attempt.integer.conversion, remove.altexp.coldata, forbid.nested.altexp, level=0) {
    for (i in assayNames(x)) {
        ass <- assay(x, i, withDimnames=FALSE)

        if (!is.null(reformat.assay.by.density)) {
            density <- mean(ass != 0 | is.na(ass)) # NAs are included as non-zero.
            if (density < reformat.assay.by.density) {
                if (!is_sparse(ass)) {
                    ass <- as(ass, "SVT_SparseMatrix")
                }
            } else {
                if (is_sparse(ass)) {
                    ass <- as.array(ass)
                }
            }
        }

        if (attempt.integer.conversion) {
            if (type(ass) == "double" && !any(ass %% 1 != 0, na.rm=TRUE)) {
                converter <- selectMethod(`type<-`, class(ass), optional=TRUE)
                if (is.null(converter)) {
                    ass <- DelayedArray(ass)
                    type(ass) <- "integer"
                } else {
                    ass <- converter(ass, "integer")
                }
            }
        }

        if (strip.inner.names) {
            dimnames(ass) <- NULL
        }

        assay(x, i, withDimnames=FALSE) <- ass
    }

    if (is(x, "SingleCellExperiment")) {
        if (strip.inner.names) {
            for (i in reducedDimNames(x)) {
                red <- reducedDim(x, i, withDimnames=FALSE)
                rownames(red) <- NULL
                reducedDim(x, i, withDimnames=FALSE) <- red
            }
        }

        for (i in altExpNames(x)) {
            if (forbid.nested.altexp && level > 0L) {
                stop("nested alternative experiments are forbidden")
            }

            alt <- altExp(x, i, withDimnames=FALSE)
            if (remove.altexp.coldata) {
                colData(alt) <- colData(alt)[,0]
            }
            if (strip.inner.names) {
                colnames(alt) <- NULL
            }

            alt <- .polish_dataset(alt, 
                strip.inner.names=strip.inner.names,
                reformat.assay.by.density=reformat.assay.by.density, 
                attempt.integer.conversion=attempt.integer.conversion,
                remove.altexp.coldata=remove.altexp.coldata,
                forbid.nested.altexp=forbid.nested.altexp,
                level=level + 1L
            )

            altExp(x, i, withDimnames=FALSE) <- alt
        }
    }

    x
}

# This checks that saveDataset works as expected.
# library(testthat); library(scRNAseq); source("test-polishDataset.R")

mat <- matrix(rpois(1000, lambda=1), ncol=10)

test_that("polishDataset strips assay dimnames", {
    rownames(mat) <- sprintf("GENE_%i", seq_len(nrow(mat)))
    colnames(mat) <- head(LETTERS, 10)
    sce <- SingleCellExperiment(list(counts=mat))

    y <- polishDataset(sce)
    expect_identical(dimnames(y), dimnames(mat))
    expect_null(dimnames(assay(y, withDimnames=FALSE)))

    # Unless we disable it.
    rownames(mat) <- tolower(rownames(mat))
    colnames(mat) <- tolower(colnames(mat))
    assay(sce, withDimnames=FALSE) <- mat
    y <- polishDataset(sce, strip.inner.names=FALSE)
    expect_identical(dimnames(y), dimnames(sce))
    expect_identical(dimnames(assay(y, withDimnames=FALSE)), dimnames(mat))
})

test_that("polishDataset strips reduced dimension names", {
    sce <- SingleCellExperiment(list(counts=mat))
    rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(mat)))
    colnames(sce) <- head(LETTERS, 10)

    pca <- matrix(rnorm(50), ncol=5)
    colnames(pca) <- sprintf("PC%i", seq_len(5))
    rownames(pca) <- seq_len(nrow(pca))
    reducedDim(sce, "PCA", withDimnames=FALSE) <- pca

    y <- polishDataset(sce)
    rdim <- reducedDim(y, "PCA", withDimnames=FALSE)
    expect_null(rownames(rdim))
    expect_identical(colnames(rdim), colnames(pca))

    # Unless we disable it.
    y <- polishDataset(sce, strip.inner.names=FALSE)
    rdim <- reducedDim(y, "PCA", withDimnames=FALSE)
    expect_identical(dimnames(rdim), dimnames(pca))
})

test_that("polishDataset strips alternative experiment names", {
    sce <- SingleCellExperiment(list(counts=mat))
    rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(mat)))
    colnames(sce) <- head(LETTERS, 10)

    sub <- sce[1:10,]
    rownames(sub) <- paste0("X-", rownames(sub))
    altExp(sce, "ERCC", withDimnames=FALSE) <- sub

    y <- polishDataset(sce)
    ae <- altExp(y, "ERCC", withDimnames=FALSE)
    expect_identical(rownames(ae), rownames(sub))
    expect_null(colnames(ae))

    # Unless we disable it.
    y <- polishDataset(sce, strip.inner.names=FALSE)
    ae <- altExp(y, "ERCC", withDimnames=FALSE)
    expect_identical(rownames(ae), rownames(sub))
    expect_identical(colnames(ae), colnames(sub))
})

library(DelayedArray)
test_that("polishDataset converts dense to sparse", {
    mat <- matrix(rpois(1000, lambda=0.2), ncol=10)
    sce <- SingleCellExperiment(list(counts=mat))

    y <- polishDataset(sce)
    expect_true(is_sparse(assay(y)))

    # No conversion.
    y <- polishDataset(sce, reformat.assay.by.density=0)
    expect_false(is_sparse(assay(y)))
    y <- polishDataset(sce, reformat.assay.by.density=NULL)
    expect_false(is_sparse(assay(y)))
})

library(Matrix)
test_that("polishDataset converts sparse to dense", {
    mat <- matrix(rpois(1000, lambda=3), ncol=10)
    sce <- SingleCellExperiment(list(counts=as(mat, "dgCMatrix")))

    y <- polishDataset(sce)
    expect_false(is_sparse(assay(y)))

    # No conversion.
    y <- polishDataset(sce, reformat.assay.by.density=1)
    expect_true(is_sparse(assay(y)))
    y <- polishDataset(sce, reformat.assay.by.density=NULL)
    expect_true(is_sparse(assay(y)))
})

test_that("polishDataset attempts integer conversions", {
    mat <- matrix(rpois(1000, lambda=3), ncol=10) * 1.0
    sce <- SingleCellExperiment(list(counts=mat))

    y <- polishDataset(sce)
    a <- assay(y)
    expect_true(is.matrix(a))
    expect_type(a, "integer")

    # Handles the conversion safely.
    mat <- matrix(rpois(1000, lambda=0.1), ncol=10) * 1.0
    sce <- SingleCellExperiment(list(counts=mat))

    y <- polishDataset(sce)
    a <- assay(y)
    expect_s4_class(a, "SVT_SparseMatrix")
    expect_identical(type(a), "integer")

    # Handles the conversion when there's no type<- method:
    mat <- round(as(mat, "dgCMatrix") * 100)
    sce <- SingleCellExperiment(list(counts=mat))

    y <- polishDataset(sce)
    a <- assay(y)
    expect_s4_class(a, "DelayedMatrix")
    expect_identical(type(a), "integer")

    # Doesn't attempt the conversion.
    mat <- matrix(rpois(1000, lambda=3), ncol=10) * 1.5
    sce <- SingleCellExperiment(list(counts=mat))

    y <- polishDataset(sce)
    a <- assay(y)
    expect_true(is.matrix(a))
    expect_type(a, "double")
})

test_that("polishDataset works with NA values in the matrices", {
    mat <- matrix(rpois(1000, lambda=0.1), ncol=10)
    mat[sample(length(mat), 10)] <- NA
    sce <- SingleCellExperiment(list(counts=as(mat, "dgCMatrix")))

    y <- polishDataset(sce)
    expect_true(is_sparse(assay(y)))
    expect_identical(type(assay(y)), "integer")
})

test_that("polishDataset strips out alternative experiment's colData", {
    sce <- SingleCellExperiment(list(counts=mat))
    rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(mat)))
    colnames(sce) <- head(LETTERS, 10)

    sub <- sce[1:10,]
    rownames(sub) <- paste0("X-", rownames(sub))
    sub$foo <- runif(10)
    altExp(sce, "ERCC", withDimnames=FALSE) <- sub

    y <- polishDataset(sce)
    expect_identical(ncol(colData(altExp(y))), 0L)

    # Unless we disable it.
    y <- polishDataset(sce, remove.altexp.coldata=FALSE)
    expect_identical(ncol(colData(altExp(y))), 1L)
})

test_that("polishDataset forbids highly nested altexps", {
    sce0 <- SingleCellExperiment(list(counts=mat))
    sce1 <- SingleCellExperiment(list(counts=mat[1:10,]+1))
    sce2 <- SingleCellExperiment(list(counts=mat[1:5,]+2))
    altExp(sce1, "FOO") <- sce2
    altExp(sce0, "BAR") <- sce1

    expect_error(polishDataset(sce0), "nested")
    y <- polishDataset(sce0, forbid.nested.altexp=FALSE)
    expect_equal(y, sce0)
})

#' Obtain the Zhong prefrontal cortex data
#'
#' Obtain the human prefrontal cortex single-cell RNA-seq dataset from Zhong et al. (2018).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is scraped from GEO and includes week of gestation, gender and likely cell type.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' This is only performed when \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/zhong-prefrontal}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Zhong S et al. (2018). 
#' A single-cell RNA-seq survey of the developmental landscape of the human prefrontal cortex. 
#' \emph{Nature} 555, 524-528.
#'
#' @examples
#' sce <- ZhongPrefrontalData()
#' 
#' @export
ZhongPrefrontalData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("zhong-prefrontal-2018", "2023-12-22", realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("zhong-prefrontal", version), has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-zhong-prefrontal.R')

test_that("Zhong prefrontal getter works as expected", {
    sce <- ZhongPrefrontalData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- ZhongPrefrontalData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

write.csv(file="../extdata/2.0.0/metadata-leng-esc.csv", 
    data.frame(
        Title = sprintf("Leng ESC %s", c("normcounts", "colData")),
        Description = sprintf("%s for the Leng embryonic stem cell single-cell RNA-seq dataset", 
            c("Normalized count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "leng-esc", "2.0.0", c("normcounts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="hg19",
        SourceType="CSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE64016",
        SourceVersion="GSE64016_H1andFUCCI_normalized_EC.csv.gz",
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

#' Obtain the Marques brain data
#'
#' Obtain the mouse brain single-cell RNA-seq data from Marques et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in GSE75330. 
#' This contains information such as the cell type and age/sex of the mouse of origin for each cell.
#' 
#' Note that some genes may be present in multiple rows corresponding to different genomic locations.
#' These additional rows are identified by a \code{_loc[2-9]} suffix in their row names.
#' Users may wish to consider either removing them or merging them, e.g., with \code{scater::sumCountsAcrossFeatures}.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#' All searching is performed after removing the \code{_loc[2-9]} suffix. 
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/marques-brain}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Marques A et al. (2016).
#' Oligodendrocyte heterogeneity in the mouse juvenile and adult central nervous system. 
#' \emph{Science} 352(6291), 1326-1329. 
#'
#' @examples
#' sce <- MarquesBrainData()
#' 
#' @export
MarquesBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("marques-brain-2016", "2023-12-19", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("marques-brain", version), assays="counts", has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=sub("_loc[0-9]+$", "", rownames(sce)),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Xin pancreas data
#'
#' Obtain the human pancreas single-cell RNA-seq dataset from Xin et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Row data contains fields for the Entrez ID and symbol for each gene.
#' Column metadata was obtained from the authors (indirectly, via the Hemberg group) 
#' and contains information such as the cell type labels and donor status.
#'
#' If \code{ensembl=TRUE}, the Entrez IDs are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/xin-pancreas}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of RPKMs.
#'
#' @author Aaron Lun,
#' using additional metadata obtained by Vladimir Kiselev.
#'
#' @references
#' Xin A et al. (2016).
#' RNA sequencing of single human islet cells reveals type 2 diabetes genes.
#' \emph{Cell Metab.} 24(4), 608-615.
#'
#' @examples
#' sce <- XinPancreasData()
#' 
#' @export
XinPancreasData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("xin-pancreas-2016", "2023-12-19", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("xin-pancreas", version), assays="rpkm")
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs", 
        keytype="ENTREZID",
        ensembl=ensembl,
        location=location)
}

#' Obtain the Jessa brain data
#'
#' Obtain the mouse brain single-cell RNA-seq dataset from Jessa et al. (2019).
#'
#' @param filtered Logical scalar indicating whether to filter out cells that were not used by the authors.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
#' Otherwise, an additional \code{filtered} field will be present in the \code{\link[SummarizedExperiment]{colData}}, indicating whether the cell was retained by the authors. 
#'
#' The column data contains sample of origin, some QC metrics and various cluster assignments for each cell.
#' Cluster assignments starting with \code{Sample_*} are derived from per-sample analyses and cannot be compared sensibly across samples.
#' Other clusterings (\code{Forebrain_*} and \code{Pons_*}) are derived from joint analyses across all samples involving the named tissue.
#'
#' The \code{reducedDims} of the output contains various dimensionality reduction results.
#' Coordinates for entries prefixed with \code{Sample_*} were generated from per-sample analyses and cannot be compared across samples.
#' Coordinates for entries prefixed with \code{Forebrain_*} and \code{Pons_*} were generated from joint analyses from the corresponding tissue. 
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/jessa-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Jessa S et al. (2019).
#' Stalled developmental programs at the root of pediatric brain tumors
#' \emph{Nat Genet} 51, 1702-1713
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- JessaBrainData()
#' } 
#' @export
#' @importFrom SingleCellExperiment reducedDims<-
JessaBrainData <- function(filtered=TRUE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("jessa-brain-2019", "2023-12-22", realize.assays=TRUE)

    } else {
        version <- "2.6.0"
        hub <- ExperimentHub()
        sce <- .create_sce(file.path("jessa-brain", version), hub=hub)

        reddims <- hub[hub$rdatapath==file.path("scRNAseq", "jessa-brain", version, "reddims.rds")][[1]]
        reducedDims(sce) <- reddims
    }

    if (filtered) {
        sce <- sce[,sce$retained]
        sce$filtered <- NULL
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

# library(testthat); library(scRNAseq); source("test-jessa-brain.R")

test_that("JessaBrainData works as expected", {
    sce <- JessaBrainData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce2 <- JessaBrainData(filtered=FALSE)
    expect_true(ncol(sce2) > ncol(sce))
})

FUN <- function(type, short) {
    data.frame(
        Title = sprintf("Lun %s plus spike-in %s", type, c("counts", "rowData", "colData")),
        Description = sprintf("%s for the Lun %s (plus spike-ins) single-cell RNA-seq dataset", 
            c("Count matrix", "Per-gene metadata", "Per-cell metadata"), type),
        RDataPath = file.path("scRNAseq", "lun-spikein", "2.0.0", 
            sprintf(c("counts-%s.rds", "rowdata-%s.rds", "coldata-%s.rds"), short)),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-5522/",
        SourceVersion=c(
            "E-MTAB-5522.processed.1.zip",
            "E-MTAB-5522.processed.1.zip",
            "E-MTAB-5522.sdrf.txt"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="ArrayExpress",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DataFrame", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    )
}

write.csv(file="../extdata/2.0.0/metadata-lun-spikein.csv", 
    rbind(FUN("416B", "416b"), FUN("trophoblast", "tropho")),
    row.names=FALSE)

#' Obtain the Pollen radial glia data
#'
#' Obtain the human radial glia single-cell RNA-seq dataset from Pollen et al. (2017).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata includes the anatomical source, sample of origin, presumed cell type and assorted alignment statistics.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' This is only performed when \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/pollen-glia}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Pollen A et al. (2017). 
#' Molecular identity of human outer radial glia during cortical development.
#' \emph{Cell} 163, 55-67.
#'
#' @examples
#' sce <- PollenGliaData()
#' 
#' @export
PollenGliaData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("pollen-glia-2015", "2023-12-22", realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("pollen-glia", version), has.rowdata=FALSE)
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-pollen-glia.R')

test_that("Pollen glia getter works as expected", {
    sce <- PollenGliaData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- PollenGliaData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

write.csv(file="../extdata/2.0.0/metadata-zeisel-brain.csv", 
    data.frame(
        Title = sprintf("Zeisel brain %s", c("counts", "rowData", "colData")),
        Description = sprintf("%s for the Zeisel brain single-cell RNA-seq dataset", 
            c("Count matrix", "Per-gene metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "zeisel-brain", "2.0.0", 
            c("counts.rds", "rowdata.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://storage.googleapis.com/linnarsson-lab-www-blobs/blobs/cortex",
        SourceVersion=paste(
            "expression_mRNA_17-Aug-2014.txt",
            "expression_mito_17-Aug-2014.txt",
            "expression_rep_17-Aug-2014.txt",
            "expression_spikes_17-Aug-2014.txt", sep=";"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="Sten Linnarsson",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DataFrame", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Obtain the Zeisel nervous system data
#'
#' Obtain the mouse nervous system single-cell RNA-seq dataset from Zeisel et al. (2018).
#'
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Row data contains the gene symbol as well as some relevant per-gene statistics,
#' e.g., the squared coefficient of variance, mean, and whether it was selected for downstream analyses.
#'
#' Column data contains a wide variety of fields including patient-level information, sample-level sequencing statistics and many flavors of cell type classification.
#' Note that many numeric columns may have \code{NA} values if they could not be successfully parsed form the source file.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/zeisel-nervous}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Zeisel A et al. (2018). 
#' Molecular architecture of the mouse nervous system.
#' \emph{Cell} 174(4), 999-1014.
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- ZeiselNervousData()
#' }
#' @export
#' @importFrom SingleCellExperiment splitAltExps altExp altExp<- reducedDims<- colPairs<-
#' @importFrom SummarizedExperiment rowData rowData<-
#' @importFrom BiocGenerics cbind
ZeiselNervousData <- function(location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("zeisel-nervous-2018", "2023-12-22", realize.assays=TRUE)

    } else {
        version <- "2.6.0"

        hub <- ExperimentHub()
        sce <- .create_sce(file.path("zeisel-nervous", version), hub=hub)

        reducedDims(sce) <- hub[[which(hub$rdatapath==file.path("scRNAseq/zeisel-nervous", version, "reddims.rds"))]]
        colPairs(sce) <- hub[[which(hub$rdatapath==file.path("scRNAseq/zeisel-nervous", version, "colpairs.rds"))]]
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

# library(testthat); library(scRNAseq); source('test-zeisel-nervous.R')

test_that("Zeisel nervous getter works as expected", {
    sce <- ZeiselNervousData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- ZeiselNervousData(location=FALSE)
    expect_true(all(lengths(sce)==0))
})

#' List available versions
#'
#' List the available and latest versions for a dataset.
#'
#' @param name String containing the name of the dataset.
#'
#' @return 
#' For \code{listVersions}, a character vector containing the names of the available versions of the dataset.
#'
#' For \code{fetchLatestVersion}, a string containing the name of the latest version.
#'
#' @author Aaron Lun
#'
#' @examples
#' listVersions("zeisel-brain-2015")
#' fetchLatestVersion("zeisel-brain-2015")
#'
#' @export
listVersions <- function(name) {
    gypsum::listVersions("scRNAseq", name)
}

#' @export
#' @rdname listVersions
#' @importFrom gypsum fetchLatest
fetchLatestVersion <- function(name) {
    fetchLatest("scRNAseq", name)
}

write.csv(file="../../extdata/2.6.0/metadata-bacher-tcell.csv",
    data.frame(
        Title = sprintf("Bacher COVID T cell %s", c("counts", "colData")),
        Description = sprintf("%s for the Bacher COVID T cell single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "bacher-tcell", "2.6.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.13",
        Genome="hg38",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE162086",
        SourceVersion=c(
            "GSE162086_RAW.tar",
            "GSE162086_seurat_metadata.tsv.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Search dataset metadata
#'
#' Search for datasets of interest based on matching text in the associated metadata.
#'
#' @param query String containing a query in a human-readable syntax or a \link[gypsum]{gypsum.search.clause}, see Examples.
#' @inheritParams surveyDatasets
#'
#' @return 
#' A \link[S4Vectors]{DataFrame} where each row corresponds to a (sub)dataset, containing various columns of metadata.
#' Some columns may be lists to capture 1:many mappings.
#'
#' @details
#' The returned DataFrame contains the usual suspects like the title and description for each dataset,
#' the number of rows and columns, the organisms and genome builds involved,
#' whether the dataset has any pre-computed reduced dimensions, and so on.
#' More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 
#'
#' If a dataset contains multiple subdatasets, each subdataset is reported as a separate row in the DataFrame.
#' The location of subdataset is provided in the \code{path} column.
#' If a dataset does not contain any subdatasets, the \code{path} entry will be set to \code{NA}.
#'
#' @author Aaron Lun
#'
#' @examples
#' searchDatasets("brain")[,c("name", "title")]
#' searchDatasets("Neuro%")[,c("name", "title")]
#' searchDatasets("taxonomy_id:10090")[,c("name", "title")]
#' searchDatasets("(genome: GRCm38 AND neuro%) OR pancrea%")[,c("name", "title")]
#'
#' # We can also use gypsum search clauses via the gsc() function.
#' # Here, 'asset' is analogous to the 'name' of the dataset.
#' searchDatasets(gsc(asset="he-organs-2020"))[,c("path")]
#' searchDatasets(
#'     (gsc(asset="%brain%", partial=TRUE) |
#'     gsc(asset="%neur%", partial=TRUE) |
#'     gsc(text="%neur%", partial=TRUE) |
#'     gsc(text="%brain%", partial=TRUE)) &
#'     gsc(field="taxonomy_id", text="10090")
#' )[,c("name", "title")]
#' 
#' @seealso
#' \code{\link{surveyDatasets}}, to easily obtain a listing of all available datasets.
#'
#' \code{\link[gypsum]{translateTextQuery}}, for details on the human-readable query syntax.
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom gypsum cacheDirectory fetchMetadataDatabase searchMetadataFilter translateTextQuery
#' @importFrom DBI dbConnect dbDisconnect dbGetQuery
#' @importFrom RSQLite SQLite
searchDatasets <- function(query, cache=cacheDirectory(), overwrite=FALSE, latest=TRUE) {
    if (is.character(query)) {
        query <- translateTextQuery(query)
    }
    filter <- searchMetadataFilter(query)

    bpath <- fetchMetadataDatabase(cache=cache, overwrite=overwrite)
    con <- dbConnect(SQLite(), bpath)
    on.exit(dbDisconnect(con))

    stmt <- "SELECT json_extract(metadata, '$') AS meta, versions.asset AS asset, versions.version AS version, path";
    if (!latest) {
        stmt <- paste0(stmt, ", versions.latest AS latest")
    }
    stmt <- paste0(stmt, " FROM paths LEFT JOIN versions ON paths.vid = versions.vid WHERE versions.project = 'scRNAseq'")
    if (latest) {
        stmt <- paste0(stmt, " AND versions.latest = 1")
    }
    if (!is.null(filter)) {
        stmt <- paste0(stmt, " AND ", filter$where)
        everything <- dbGetQuery(con, stmt, params=filter$parameters)
    } else {
        everything <- dbGetQuery(con, stmt)
    }

    sanitize_query_to_output(everything, latest)
}

#' @export
#' @importFrom gypsum defineTextQuery
gypsum::defineTextQuery

#' @export
#' @importFrom gypsum gsc
gypsum::gsc

#' Obtain the Muraro pancreas data
#'
#' Obtain the human pancreas single-cell RNA-seq data from Muraro et al. (2016).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Row data contains fields for the symbol and chromosomal location of each gene.
#'
#' Column metadata is derived from the columns of the count matrix provided in GSE85241,
#' with additional cell type labels obtained from the authors (indirectly, via the Hemberg group).
#' Some cells have \code{NA} labels and were presumably removed prior to downstream analyses.
#'
#' Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/muraro-pancreas}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun,
#' using additional metadata obtained by Vladimir Kiselev.
#'
#' @references
#' Muraro MJ et al. (2016). 
#' A single-cell transcriptome atlas of the human pancreas.
#' \emph{Cell Syst.} 3(4), 385-394.
#'
#' @examples
#' sce <- MuraroPancreasData()
#' 
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom SummarizedExperiment rowData<-
#' @importFrom SingleCellExperiment splitAltExps
MuraroPancreasData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("muraro-pancreas-2016", "2023-12-19", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("muraro-pancreas", version), has.rowdata=FALSE) 

        symbol <- sub("__.*", "", rownames(sce))
        loc <- sub(".*__", "", rownames(sce))
        rowData(sce) <- DataFrame(symbol=symbol, chr=loc)

        status <- ifelse(grepl("^ERCC-[0-9]+", symbol), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
    }

    .convert_to_ensembl(sce,
        symbols=rowData(sce)$symbol,
        species="Hs",
        ensembl=ensembl,
        location=location)
}

write.csv(file="../../extdata/2.6.0/metadata-pollen-glia.csv",
    data.frame(
        Title = sprintf("Pollen radial glia %s", c("counts", "colData")),
        Description = sprintf("%s for the Pollen radial glia single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "pollen-glia", "2.6.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.13",
        Genome="hg19",
        SourceType=c("TSV", "XLS/XLSX"),
        SourceUrl="https://www.pollenlab.org/datasets",
        SourceVersion=c(
            "oRG paper - counts.txt",
            "Pollen et al 2015 updated metadata.xlsx"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' @importFrom ExperimentHub ExperimentHub
#' @importFrom SingleCellExperiment SingleCellExperiment
.create_sce <- function(dataset, hub=ExperimentHub(), assays="counts", has.rowdata=TRUE, has.coldata=TRUE, suffix=NULL) {
    host <- file.path("scRNAseq", dataset)
    if (is.null(suffix)) {
        suffix <- ""
    } else {
        suffix <- paste0("-", suffix)
    }

    all.assays <- list()
    for (a in assays) {
        all.assays[[a]] <- hub[hub$rdatapath==file.path(host, sprintf("%s%s.rds", a, suffix))][[1]]
    }

    args <- list()
    if (has.coldata) {
        args$colData <- hub[hub$rdatapath==file.path(host, sprintf("coldata%s.rds", suffix))][[1]]
    }
    if (has.rowdata) {
        args$rowData <- hub[hub$rdatapath==file.path(host, sprintf("rowdata%s.rds", suffix))][[1]]
    }

    do.call(SingleCellExperiment, c(list(assays=all.assays), args))
}

#' @importFrom ExperimentHub ExperimentHub
#' @importFrom SingleCellExperiment SingleCellExperiment
.create_sce_legacy <- function(dataset, assays, hub=ExperimentHub()) {
    choices <- c("tophat_counts", "cufflinks_fpkm", "rsem_counts", "rsem_tpm")
    if (is.null(assays)) {
        assays <- choices
    } else {
        assays <- match.arg(assays, choices, several.ok=TRUE)
    }

    host <- file.path("scRNAseq", dataset)
    all.assays <- list()
    for (i in assays) {
        path <- file.path(host, sprintf("%s.rds", i))
        all.assays[[i]] <- hub[hub$rdatapath==path][[1]]
    }

    coldata <- hub[hub$rdatapath==file.path(host, "coldata.rds")][[1]]
    metadata <- hub[hub$rdatapath==file.path(host, "metadata.rds")][[1]]
    SingleCellExperiment(all.assays, colData=coldata, metadata=metadata)
}

#' Obtain the Lun spike-in data
#'
#' Obtain the spike-in single-cell RNA-seq data from Lun et al. (2017).
#'
#' @param which String specifying whether the 416B or trophoblast data should be obtained.
#' @param split.oncogene Logical scalar indicating whether the oncogene should be split to a separate \code{\link[SingleCellExperiment]{altExp}}.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Row data contains a single \code{"Length"} field describing the total exonic length of each feature.
#'
#' Column metadata is provided in the same form as supplied in E-MTAB-5522.
#' This contains information such as the cell type, plate of origin, spike-in addition order and oncogene induction. 
#'
#' Two sets of spike-ins were added to each cell in each dataset.
#' These are available as the \code{"SIRV"} and \code{"ERCC"} entries in the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{split.oncogene=TRUE} and \code{which="416b"},
#' the CBFB-MYH11-mcherry oncogene is moved to extra \code{"oncogene"} entry in the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/lun-spikein}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Lun ATL et al. (2017). 
#' Assessing the reliability of spike-in normalization for analyses of single-cell RNA sequencing data. 
#' \emph{Genome Res.} 27(11), 1795-1806.
#'
#' @examples
#' sce <- LunSpikeInData()
#'
#' sce <- LunSpikeInData("tropho")
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
#' @importFrom SingleCellExperiment splitAltExps
LunSpikeInData <- function(which=c("416b", "tropho"), split.oncogene=FALSE, location=TRUE, legacy=FALSE) {
    which <- match.arg(which)

    if (!legacy && !split.oncogene) {
        sce <- fetchDataset("lun-spikein-2017", "2023-12-18", path=which, realize.assays=TRUE) 

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("lun-spikein", version), suffix=which)

        spike.type <- rep("endogenous", nrow(sce))
        spike.type[grep("ERCC", rownames(sce))] <- "ERCC"
        spike.type[grep("SIRV", rownames(sce))] <- "SIRV"

        if (split.oncogene) {
            spike.type[rownames(sce)=="CBFB-MYH11-mcherry"] <- "oncogene"
        }

        sce <- splitAltExps(sce, spike.type, ref="endogenous")

        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 100, dilution = 3e6)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

write.csv(file="../../extdata/2.8.0/metadata-ledergor-myeloma.csv",
    data.frame(
        Title = sprintf("Ledergor multiple myeloma %s", c("counts", "colData")),
        Description = sprintf("%s for the Ledergor multiple myeloma single-cell RNA-seq dataset",
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "ledergor-myeloma", "2.8.0",
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.14",
        Genome="hg38",
        SourceType=c("MTX", "TSV"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE117156",
        SourceVersion=c(
            "GSE117156_RAW.tar",
            "GSE117156_metadata.txt.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Obtain the Messmer ESC data
#'
#' Obtain the human embryonic stem cell single-cell RNA-seq data from Messmer et al. (2019).
#'
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Row data contains a single \code{"Length"} field describing the total exonic length of each feature.
#'
#' Column metadata is provided in the same form as supplied in E-MTAB-6819.
#' This contains information such as the cell phenotype (naive or primed) and the batch of origin.
#' Note that counts for technical replicates have already been summed together.
#'
#' Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/messmer-esc}.
#' 
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun 
#'
#' @references
#' Messmer T et al. (2019). 
#' Transcriptional heterogeneity in naive and primed human pluripotent stem cells at single-cell resolution. 
#' \emph{Cell Rep} 26(4), 815-824.e4
#'
#' @examples
#' sce <- MessmerESCData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps
MessmerESCData <- function(location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("messmer-esc-2019", "2023-12-19", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("messmer-esc", version))

        spike.type <- ifelse(grepl("ERCC", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, spike.type, ref="endogenous")
        
        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 1, dilution = 25000000)
        spikedata <- spikedata[rownames(spike.exp), ]
        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        altExp(sce, "ERCC") <- spike.exp
    }

    .define_location_from_ensembl(sce, species="Hs", location=location)
}

#' Obtain the Bacher T cell data
#'
#' Obtain the human COVID T cell single-cell RNA-seq dataset from Bacher et al. (2020).
#'
#' @param filtered Logical scalar indicating whether to filter out cells that were not used by the authors.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is scraped from GEO, using both the author-supplied TSV of per-cell annotations and the sample-level metadata.
#' This contains information such as the diagnosis, severity, WHO class, clustering and clonotype.
#'
#' If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
#' Otherwise, an additional \code{filtered} field will be present in the \code{\link[SummarizedExperiment]{colData}}, indicating whether the cell was retained by the authors. 
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/bacher-tcell}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Bacher P et al. (2020). 
#' Low avidity T cell responses to SARS-CoV-2 in unexposed individuals and severe COVID-19
#' \emph{Immunity} 53, 1258-1271
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- BacherTCellData()
#' }
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps altExp altExp<-
#' @importFrom SummarizedExperiment rowData rowData<-
#' @importFrom BiocGenerics cbind
BacherTCellData <- function(filtered=TRUE, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("bacher-tcell-2020", "2023-12-21", realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("bacher-tcell", version), has.rowdata=FALSE)
    }

    if (filtered) {
        sce <- sce[,sce$retained]
        sce$retained <- NULL
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-bacher-tcell.R')

test_that("Bacher T cell getter works as expected", {
    sce <- BacherTCellData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce2 <- BacherTCellData(filtered=FALSE)
    expect_true(ncol(sce) < ncol(sce2))

    sce <- BacherTCellData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

#' Obtain the Giladi HSC data
#'
#' Obtain the mouse haematopoietic stem cell single-cell RNA-seq and CRISPR-seq dataset from Giladi et al. (2018).
#'
#' @param filtered Logical scalar indicating whether to filter out cells that were not used by the authors.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers, when \code{mode} contains \code{"rna"}.
#' @param location Logical scalar indicating whether genomic coordinates should be returned, when \code{mode} contains \code{"rna"}.
#' @param mode Character vector specifying which modalities should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is scraped from GEO using the author-supplied TSV of per-cell annotations. 
#' This contains information such as the batch of origin for each cell plus an array of FACS measurements per cell.
#'
#' If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
#' Otherwise, an additional \code{filtered} field will be present in the \code{\link[SummarizedExperiment]{colData}}, indicating whether the cell was retained by the authors. 
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#' For row names with multiple semi-colon-delimited symbols, the last symbol is used for matching against the Ensembl annotation.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' This is only relevant when \code{ensembl=TRUE}.
#'
#' If \code{mode} contains multiple modalities, the intersection of cells that are present in both modalities is returned.
#' This is because not all cells have data across both modalities.
#' If \code{mode} contains only one modality, all cells for that modality are returned.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/giladi-hsc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a matrix of UMI counts for the scRNA-seq or CRISPR-seq data.
#' Alternatively, an object with both count matrices, where the second modality is stored as an alternative Experiment.
#'
#' @author Aaron Lun
#'
#' @references
#' Giladi A et al. (2018).
#' Single-cell characterization of haematopoietic progenitors and their trajectories in homeostasis and perturbed haematopoiesis. 
#' \emph{Nat Cell Biol} 20, 836-846
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- GiladiHSCData()
#' }
#' @export
#' @importFrom SingleCellExperiment splitAltExps altExp altExp<-
#' @importFrom SummarizedExperiment rowData rowData<-
#' @importFrom BiocGenerics cbind
GiladiHSCData <- function(mode=c("rna", "crispr"), filtered=TRUE, ensembl=FALSE, location=TRUE, legacy=FALSE) {
    mode <- match.arg(mode, several.ok=TRUE)
    version <- "2.6.0"

    collated <- list()
    if ("rna" %in% mode) {
        if (!legacy) {
            sce <- fetchDataset("giladi-hsc-2018", "2023-12-21", path="rna", realize.assays=TRUE)
        } else {
            sce <- .create_sce(file.path("giladi-hsc", version), has.rowdata=FALSE, suffix="rna")
        }

        if (filtered) {
            sce <- sce[,sce$retained]
            sce$filtered <- NULL
        }

        sce <- .convert_to_ensembl(sce, 
            symbols=sub(".*;", "", rownames(sce)),
            species="Mm",
            ensembl=ensembl,
            location=location)

        collated$rna <- sce
    } 

    if ("crispr" %in% mode) {
        if (!legacy) {
            sce <- fetchDataset("giladi-hsc-2018", "2023-12-21", path="crispr", realize.assays=TRUE)
        } else {
            sce <- .create_sce(file.path("giladi-hsc", version), suffix="crispr")
        }
        collated$crispr <- sce
    }

    if (length(collated) > 1) {
        keep <- Reduce(intersect, lapply(collated, colnames))
        for (i in seq_along(collated)) {
            collated[[i]] <- collated[[i]][,keep]
        }
    }

    primary <- collated[[1]]
    altExps(primary) <- collated[-1]
    primary
}

# library(testthat); library(scRNAseq); source('test-giladi-hsc.R')

test_that("Giladi HSC getter works as expected", {
    sce <- GiladiHSCData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce2 <- GiladiHSCData(mode="rna")
    expect_true(ncol(sce2) > ncol(sce))

    sce.filt <- GiladiHSCData(mode='rna', filtered=TRUE)
    expect_identical(colnames(sce.filt), colnames(sce2)[sce2$retained])

    sce3 <- GiladiHSCData(mode="crispr")
    expect_true(ncol(sce3) > ncol(sce))

    sceE <- GiladiHSCData(mode="rna", ensembl=TRUE)
    expect_true(all(grepl("^ENSMUSG", rownames(sceE))))
})

write.csv(file="../extdata/2.0.0/metadata-kolodziejczyk-esc.csv", 
    data.frame(
        Title = "Kolodziejczyk ESC counts",
        Description = "Count matrix for the Kolodziejczyk embryonic stem cell single-cell RNA-seq dataset", 
        RDataPath = file.path("scRNAseq", "kolodziejczyk-esc", "2.0.0", "counts.rds"),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="TXT",
        SourceUrl="https://espresso.teichlab.sanger.ac.uk/static",
        SourceVersion="counttable_es.csv",
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="Sarah Teichmann",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass="matrix", 
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

#' Obtain the Baron pancreas data
#'
#' Obtain the human/mouse pancreas single-cell RNA-seq data from Baron et al. (2017).
#'
#' @param which String specifying the species to get data for.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in GSE84133.
#' This contains information such as the cell type labels and donor ID (for humans) or strain (for mouse).
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/baron-pancreas}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Baron M et al. (2017). 
#' Single-cell transcriptomic map of the human and mouse pancreas reveals inter- and intra-cell population structure. 
#' \emph{Cell Syst.} 3(4), 346-360.
#'
#' @examples
#' sce.human <- BaronPancreasData()
#'
#' sce.mouse <- BaronPancreasData("mouse")
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
BaronPancreasData <- function(which=c("human", "mouse"), ensembl=FALSE, location=TRUE, legacy=FALSE) {
    which <- match.arg(which)

    if (!legacy) {
        sce <- fetchDataset("baron-pancreas-2016", "2023-12-14", path=which, realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("baron-pancreas", version), has.rowdata=FALSE, suffix=which)
    }

    .convert_to_ensembl(sce, 
        ensembl=ensembl,
        species=if (which=="human") "Hs" else "Mm",
        symbols=rownames(sce),
        location=location)
}

#' Obtain the Darmanis brain data
#'
#' Obtain the human brain single-cell RNA-seq dataset from Darmanis et al. (2015).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param remove.htseq Logical scalar indicating whether HT-seq alignment statistics should be removed.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column metadata is scraped from GEO and includes patient information, tissue of origin and likely cell type. 
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' This is only performed when \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/darmanis-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Darmanis S et al. (2015). 
#' A survey of human brain transcriptome diversity at the single cell level. 
#' \emph{Proc Natl Acad Sci USA} 112, 7285-90.
#'
#' @examples
#' sce <- DarmanisBrainData()
#' 
#' @export
DarmanisBrainData <- function(ensembl=FALSE, location=TRUE, remove.htseq=TRUE, legacy=FALSE) {
    if (!legacy && remove.htseq) {
        sce <- fetchDataset("darmanis-brain-2015", "2023-12-21", realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("darmanis-brain", version), has.rowdata=FALSE)

        if (remove.htseq) {
            to.drop <- c("no_feature", "ambiguous", "alignment_not_unique")
            sce <- sce[!rownames(sce) %in% to.drop,]
        }
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-darmanis-brain.R')

test_that("Darmanis brain getter works as expected", {
    sce <- DarmanisBrainData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- DarmanisBrainData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

write.csv(file="../../extdata/2.4.0/metadata-kotliarov-pbmc.csv", 
    data.frame(
        Title = sprintf("Kotliarov PBMCs %s", c("RNA counts", "ADT counts", "colData")),
        Description = sprintf("%s for the Kotliarov influenza-vaccinated healthy PBMCs CITE-seq dataset", 
            c("RNA count matrix", "ADT count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "kotliarov-pbmc", "2.4.0",
            c("counts-rna.rds", "counts-adt.rds", "coldata.rds")),
        BiocVersion="3.12",
        Genome="hg19",
        SourceType="RDS",
        SourceUrl="https://nih.figshare.com/ndownloader/files/20706642",
        SourceVersion="H1_day0_demultilexed_singlets.RDS",
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="NIH",
        Maintainer="Stephany Orjuela <sorjuelal@gmail.com>",
        RDataClass=c("dgCMatrix", "dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

#' Obtain the Usoskin brain data
#'
#' Obtain the mouse brain single-cell RNA-seq dataset from Usoskin et al. (2015).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata is provided in the same form as supplied in External Table 2 of \url{http://linnarssonlab.org/drg/}.
#' This contains information such as the library of origin and the cell type.
#'
#' The count matrix contains information for repeats, marked with \code{r_} prefixes in the row names;
#' as well as mitochondrial transcripts, marked with \code{mt-} prefixes.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/usoskin-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of RPMs.
#'
#' @author Aaron Lun
#'
#' @references
#' Usoskin A et al. (2015).
#' Unbiased classification of sensory neuron types by large-scale single-cell RNA sequencing. 
#' \emph{Nat. Neurosci.} 18(1), 145-53.
#'
#' @examples
#' sce <- UsoskinBrainData()
#' 
#' @export
#' @importFrom SummarizedExperiment rowData colData
UsoskinBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("usoskin-brain-2015", "2023-12-19", realize.assays=TRUE)
    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("usoskin-brain", version), assays="rpm")
        rownames(sce) <- rowData(sce)[,1]
    }

    colnames(sce) <- colData(sce)[["Sample ID"]]
    
    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Mm",
        ensembl=ensembl,
        location=location)
}

#' Obtain the Bhaduri cortical organoid data
#'
#' Obtain the human cortical organoid single-cell RNA-seq dataset from Bhaduri et al. (2020).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Column data contains sample-level information.
#' In theory, there is also cell-level metadata for this dataset but it could not be unambiguously mapped to the column names.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/bhaduri-organoid}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of normalized expression values.
#'
#' @author Aaron Lun
#'
#' @references
#' Bhaduri A et al. (2020). 
#' Cell stress in cortical organoids impairs molecular subtype specification.
#' \emph{Nature} 578(7793), 142-148.
#'
#' @examples
#' if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
#'     sce <- BhaduriOrganoidData()
#' }
#' @export
#' @importFrom SingleCellExperiment splitAltExps altExp altExp<- reducedDims<- colPairs<-
#' @importFrom SummarizedExperiment rowData rowData<- assayNames<-
#' @importFrom BiocGenerics cbind
BhaduriOrganoidData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("bhaduri-organoid-2020", "2023-12-21", realize.assays=TRUE)
    } else {
        version <- "2.6.0"
        sce <- .create_sce(file.path("bhaduri-organoid", version), has.rowdata=FALSE)
        assayNames(sce) <- "normalized"
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Hs",
        ensembl=ensembl,
        location=location)
}

# library(testthat); library(scRNAseq); source('test-bhaduri-organoid.R')

test_that("Bhaduri organoid getter works as expected", {
    sce <- BhaduriOrganoidData()
    expect_s4_class(sce, "SingleCellExperiment")

    sce <- BhaduriOrganoidData(ensembl=TRUE)
    expect_true(all(grepl("^ENSG", rownames(sce))))
})

write.csv(file="../extdata/2.0.0/metadata-lawlor-pancreas.csv",
    data.frame(
        Title = sprintf("Lawlor pancreas %s", c("counts", "colData")),
        Description = sprintf("%s for the Lawlor pancreas single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "lawlor-pancreas", "2.0.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="hg19",
        SourceType="TXT",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE86469",
        SourceVersion=c(
            "GSE86469_GEO.islet.single.cell.processed.data.RSEM.raw.expected.counts.csv.gz",
            "GSE86469_series_matrix.txt.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Obtain the Nestorowa HSC data
#'
#' Obtain the mouse haematopoietic stem cell single-cell RNA-seq data from Nestorowa et al. (2015).
#'
#' @param remove.htseq Logical scalar indicating whether HT-seq alignment statistics should be removed.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Rows corresponding to HT-seq's alignment statistics are removed by default.
#' These can be retained by setting \code{remove.htseq=FALSE}.
#'
#' Column metadata includes the cell type mapping, as described on the website (see References),
#' and the FACS expression levels of selected markers.
#' Note that these are stored as nested matrices within the \code{\link[SummarizedExperiment]{colData}}.
#'
#' Diffusion map components are provided as the \code{"diffusion"} entry in the \code{\link[SingleCellExperiment]{reducedDims}}.
#'
#' Counts for ERCC spike-ins are stored in the \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/nestorowa-hsc}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Nestorowa S et al. (2016). 
#' A single-cell resolution map of mouse hematopoietic stem and progenitor cell differentiation
#' \emph{Blood} 128, e20-e31.
#'
#' Gene and protein expression in adult haematopoiesis: Data.
#' \url{http://blood.stemcells.cam.ac.uk/single_cell_atlas.html#data}.
#' 
#' @examples
#' sce <- NestorowaHSCData()
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
#' @importFrom SingleCellExperiment splitAltExps reducedDim<-
NestorowaHSCData <- function(remove.htseq=TRUE, location=TRUE, legacy=FALSE) {
    if (!legacy && remove.htseq) {
        sce <- fetchDataset("nestorowa-hsc-2016", "2024-04-18", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("nestorowa-hsc", version), has.rowdata=FALSE)

        if (remove.htseq) {
            sce <- sce[grep("^__", rownames(sce), invert=TRUE),]
        }

        reducedDim(sce, "diffusion") <- sce$diffusion
        sce$diffusion <- NULL

        status <- ifelse(grepl("^ERCC-[0-9]+", rownames(sce)), "ERCC", "endogenous")
        sce <- splitAltExps(sce, status, ref="endogenous")
    }

    .define_location_from_ensembl(sce, species="Mm", location=location)
}

write.csv(file="../extdata/2.0.0/metadata-messmer-esc.csv", row.names=FALSE,
    data.frame(
        Title = sprintf("Messmer ESC %s", c("counts", "rowData", "colData")),
        Description = sprintf("%s for the Messmer embryonic stem cell single-cell RNA-seq dataset", 
            c("Count matrix", "Per-gene metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "messmer-esc", "2.0.0", 
            c("counts.rds", "rowdata.rds", "coldata.rds")), 
        BiocVersion="3.10",
        Genome="hg38",
        SourceType="TSV",
        SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6819/",
        SourceVersion=c(
            "E-MTAB-6819.processed.1.zip",
            "E-MTAB-6819.processed.1.zip",
            "E-MTAB-6819.sdrf.txt"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="ArrayExpress",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    )
)

write.csv(file="../extdata/2.0.0/metadata-romanov-brain.csv", 
    data.frame(
        Title = sprintf("Romanov brain %s", c("counts", "colData")),
        Description = sprintf("%s for the Romanov brain single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "romanov-brain", "2.0.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="XLS/XLSX",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE74672",
        SourceVersion="GSE74672_expressed_mols_with_classes.xlsx.gz",
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

#' Obtain ERCC concentrations
#'
#' Obtain ERCC spike-in concentrations from the Thermo Fisher Scientific website.
#'
#' @param volume Numeric scalar specifying the added volume (in nanoliters) of ERCC spike-in mixture.
#' Only used if \code{dilution} is specified.
#' @param dilution Numeric scalar specifying the dilution factor used for the added volume of the spike-in mixture.
#' Only used if \code{volume} is specified.
#' @param mix String specifying whether to compute the number of molecules for mix 1 or 2.
#' Only used if both \code{dilution} and \code{volume} are specified.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' If \code{volume} and \code{dilution} are specified,
#' an additional column is added to the output specifying the number of molecules of spike-in transcipt for the specified mix.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/ercc-concentrations}.
#'
#' @return A \link[S4Vectors]{DataFrame} object with one row per ERCC spike-in transcript.
#' This contains information such as the spike-in concentration in each mix.
#'
#' @author Alan O'Callaghan
#'
#' @examples
#' df <- ERCCSpikeInConcentrations()
#' 
#' @export
#' @importFrom utils read.delim
#' @importFrom SingleCellExperiment splitAltExps
ERCCSpikeInConcentrations <- function(volume = NULL, dilution = NULL, mix=c("1", "2"), legacy=FALSE) {
    if (!legacy) {
        table <- fetchDataset("ercc", "2023-12-20")
    } else {
        version <- "2.0.0"
        hub <- ExperimentHub()
        host <- "scRNAseq/ercc-concentrations"
        file <- hub[hub$rdatapath==file.path(host, "2.2.0", "cms_095046.txt")][[1]]

        table <- read.delim(file, check.names = FALSE)
        rownames(table) <- table[,"ERCC ID"]
        table <- table[,-(1:2)]
    }

    if (!is.null(volume) && !is.null(dilution)) {
        field <- sprintf("concentration in Mix %s (attomoles/ul)", match.arg(mix))
        molarity <- table[[field]] * 10^-18 # get concentration in mol/uL.
        molecules_ul <- molarity * (6.02214076 * (10^23))
        table$molecules <- molecules_ul / dilution * (volume / 1000) # get volume in uL.
    }

    DataFrame(table, check.names=FALSE)
}

FUN <- function(species, short, long, id) {
  data.frame(
    Title = sprintf("Zilionis %s lung %s", species, c("counts", "colData")),
    Description = sprintf("%s for the Zilionis %s lung single-cell RNA-seq dataset", 
                          c("Count matrix", "Per-cell metadata"), species),
    RDataPath = file.path("scRNAseq", "zilionis-lung", "2.4.0", 
                          sprintf(c("counts-%s.rds", "coldata-%s.rds"), species)),
    BiocVersion="3.12",
    Genome=short,
    SourceType="CSV",
    SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE127465",
    SourceVersion="GSE127465_RAW.tar", 
    Species=long,
    TaxonomyId=id,
    Coordinate_1_based=NA,
    DataProvider="GEO",
    Maintainer="Jens Preussner <jens.preussner@mpi-bn.mpg.de>",
    RDataClass=c("dgCMatrix", "DFrame"),
    DispatchClass="Rds",
    stringsAsFactors = FALSE
  )
}

write.csv(file="../../extdata/2.4.0/metadata-zilionis-lung.csv", 
    rbind(FUN("human", "hg38", "Homo sapiens", "9606"), FUN("mouse", "mm10", "Mus musculus", "10090")),
    row.names=FALSE)

#' Obtain the Wu kidney data
#'
#' Obtain the mouse kidney single-nuclei RNA-seq data from Wu et al. (2019).
#'
#' @param mode String indicating whether to return data for healthy and/or diseased donors.
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#'
#' @details
#' Column metadata includes the single-cell technology and whether they came from a diseased or healthy individual.
#' 
#' If \code{mode} specifies both healthy and disease donors,
#' counts are only reported for the intersection of genes that are present for both donors.
#' This is because the original count matrices had differences in their annotation.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#' Note that this is only performed if \code{ensembl=TRUE}.
#'
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/wu-kidney}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Wu H et al. (2019). 
#' Advantages of single-nucleus over single-cell RNA sequencing of adult kidney: rare cell types and novel cell states revealed in fibrosis.
#' \emph{J. Am. Soc. Nephrol.} 30, 23-32.
#'
#' @examples
#' sce <- WuKidneyData("disease")
#' 
#' @export
#' @importFrom SummarizedExperiment rowData
WuKidneyData <- function(mode=c("healthy", "disease"), ensembl=FALSE, location=TRUE, legacy=FALSE) {
    mode <- match.arg(mode, several.ok=TRUE)
    all.sce <- list()

    if (!legacy) {
        for (m in mode) {
            current <- fetchDataset("wu-kidney-2019", "2023-12-20", m, realize.assays=TRUE)
            current$Status <- m
            if (m != "healthy") {
                current$Technology <- "sNuc-10x"
            }
            all.sce[[m]] <- current
        }

    } else {
        version <- "2.4.0"

        for (m in mode) {
            current <- .create_sce(file.path("wu-kidney", version), has.rowdata=FALSE, has.coldata=FALSE, suffix=m)

            if (m=="healthy") {
                current$Technology <- sub("_.*", "", colnames(current))
            } else {
                current$Technology <- "sNuc-10x"
            }
            current$Status <- m

            all.sce[[m]] <- current
        }
    }

    if (length(all.sce) > 1) {
        common <- Reduce(intersect, lapply(all.sce, rownames))
        for (i in seq_along(all.sce)) {
            all.sce[[i]] <- all.sce[[i]][common,]
        }
    }

    sce <- do.call(cbind, all.sce)

    .convert_to_ensembl(sce, 
        species="Mm", 
        symbols=rownames(sce),
        ensembl=ensembl,
        location=location)
}

#' Obtain the Zeisel brain data
#'
#' Obtain the mouse brain single-cell RNA-seq dataset from Zeisel et al. (2015).
#'
#' @param ensembl Logical scalar indicating whether the output row names should contain Ensembl identifiers.
#' @param location Logical scalar indicating whether genomic coordinates should be returned.
#' @param legacy Logical scalar indicating whether to pull data from ExperimentHub.
#' By default, we use data from the gypsum backend.
#' 
#' @details
#' Row data contains a single \code{"featureType"} field describing the type of each feature
#' (endogenous genes, mitochondrial genes, spike-in transcripts and repeats).
#' Spike-ins and repeats are stored as separate entries in the \code{\link[SingleCellExperiment]{altExps}}.
#'
#' Column metadata is provided in the same form as supplied in \url{http://linnarssonlab.org/cortex/}.
#' This contains information such as the cell diameter and the published cell type annotations.
#'
#' If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
#' Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
#'
#' If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
#'
#' Spike-in metadata is added using \code{\link{ERCCSpikeInConcentrations}},
#' with molecule counts computed using a volume of 9 nL per cell at a dilution of 1:20000.
#' 
#' All data are downloaded from ExperimentHub and cached for local re-use.
#' Specific resources can be retrieved by searching for \code{scRNAseq/zeisel-brain}.
#'
#' @return A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
#'
#' @author Aaron Lun
#'
#' @references
#' Zeisel A et al. (2015). 
#' Brain structure. Cell types in the mouse cortex and hippocampus revealed by single-cell RNA-seq. 
#' \emph{Science} 347(6226), 1138-42.
#'
#' @examples
#' sce <- ZeiselBrainData()
#' 
#' @export
#' @importFrom SingleCellExperiment splitAltExps altExp altExp<-
#' @importFrom SummarizedExperiment rowData rowData<-
#' @importFrom BiocGenerics cbind
ZeiselBrainData <- function(ensembl=FALSE, location=TRUE, legacy=FALSE) {
    if (!legacy) {
        sce <- fetchDataset("zeisel-brain-2015", "2023-12-14", realize.assays=TRUE)

    } else {
        version <- "2.0.0"
        sce <- .create_sce(file.path("zeisel-brain", version))

        status <- rowData(sce)$featureType
        status[status=="mito"] <- "endogenous"
        sce <- splitAltExps(sce, status, "endogenous")

        # Decorating spike-ins with row metadata. Probably could be less
        # verbose but this is pretty clear.
        spike.exp <- altExp(sce, "ERCC")
        spikedata <- ERCCSpikeInConcentrations(volume = 9, dilution = 20000)
        spikedata <- spikedata[rownames(spike.exp),]

        rowData(spike.exp) <- cbind(rowData(spike.exp), spikedata)
        rowData(spike.exp)$featureType <- NULL # redundant field; what else would it be!?
        altExp(sce, "ERCC") <- spike.exp
    }

    .convert_to_ensembl(sce, 
        symbols=rownames(sce), 
        species="Mm",
        ensembl=ensembl,
        location=location)
}

write.csv(file="../../extdata/2.6.0/metadata-bunis-hspc.csv",
    data.frame(
        Title = sprintf("Bunis human HSPC %s", c("counts", "colData", "rowData")),
        Description = sprintf("%s for the Bunis human haematopoietic stem-progenitor single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata", "Per-gene metadata")),
        RDataPath = file.path("scRNAseq", "bunis-hspc", "2.6.0", 
            c("counts.rds", "coldata.rds", "rowdata.rds")),
        BiocVersion="3.13",
        Genome="hg38",
        SourceType=c("MTX", "TSV", "TSV"),
        SourceUrl=c(
            "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE162086",
            "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE162086;https://ndownloader.figshare.com/files/25953740",
            "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE162086"),
        SourceVersion=c(
            "GSE158490_matrix.mtx.gz",
            "GSE158490_HSPC.best.txt.gz;GSE158490_barcodes.tsv.gz",	
            "GSE158490_genes.tsv.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.4.0/metadata-stoeckius-hashing.csv", 
    data.frame(
        Title = sprintf("Stoeckius %s counts", 
            c("PBMC human RNA", "PBMC mouse RNA", "PBMC HTO", "PBMC ADT-1", "PBMC ADT-2", 
                "mixed cell line RNA", "mixed cell line HTO")),
        Description = sprintf("%s count matrix for %s in the Stoeckius single-cell RNA-seq dataset", 
            c("RNA", "RNA", "HTO", "First ADT", "Second ADT", "RNA", "HTO"),
            c("hashed PBMCs", "unlabelled mouse controls", "hashed PBMCs", "hashed PBMCs", "hashed PBMCs", 
                "a hashed cell line mixture", "a hashed cell line mixture")),
        RDataPath = file.path("scRNAseq", "stoeckius-hashing", "2.4.0", 
            c("counts-pbmc-human.rds", "counts-pbmc-mouse.rds",
                "counts-pbmc-hto.rds", "counts-pbmc-adt1.rds", "counts-pbmc-adt2.rds",
                "counts-mixed-rna.rds", "counts-mixed-hto.rds")),
        BiocVersion="3.12",
        Genome=c("hg19", "mm10", "hg19", "hg19", "hg19", "hg19", "hg19"),
        SourceType=c("TSV", "TSV", "CSV", "CSV", "CSV", "TSV", "CSV"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE108313",
        SourceVersion=c(
            "GSM2895282_Hashtag-RNA.umi.txt.gz",
            "GSM2895282_Hashtag-RNA.umi.txt.gz",
            "GSM2895283_Hashtag-HTO-count.csv.gz",
            "GSM2895284_Hashtag-ADT1-count.csv.gz",
            "GSM2895284_Hashtag-ADT2-count.csv.gz",
            "GSM3501446_MixCellLines-RNA.umi.txt.gz",
            "GSM3501447_MixCellLines-HTO-counts.csv.gz"
        ),
        Species=c("Homo sapiens", "Mus musculus", "Homo sapiens", "Homo sapiens", "Homo sapiens", "Homo sapiens", "Homo sapiens"),
        TaxonomyId=c("9606", "10090", "9606", "9606", "9606", "9606", "9606"),
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "dgCMatrix", "matrix", "matrix", "matrix", "dgCMatrix", "matrix"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.6.0/metadata-fletcher-olfactory.csv",
    data.frame(
        Title = sprintf("Fletcher olfactory epithelial %s", c("counts", "colData", "rowData")),
        Description = sprintf("%s for the Fletcher olfactory epithelial HBC stem cell differentiation dataset", 
            c("Count matrix", "Per-cell metadata", "Per-gene metadata")),
        RDataPath = file.path("scRNAseq", "fletcher-olfactory", "2.6.0", 
            c("counts.rds", "coldata.rds", "rowdata.rds")),
        BiocVersion="3.13",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE95601",
        SourceVersion=c(
            "GSE95601_oeHBCdiff_Cufflinks_eSet_counts_table.txt.gz",
            "GSE95601_oeHBCdiff_phenoData.txt.gz",
            "GSE95601_oeHBCdiff_Cufflinks_eSet_featureData.txt.gz"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Davide Risso <risso.davide@gmail.com>",
        RDataClass=c("matrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.6.0/metadata-zeisel-nervous.csv",
    data.frame(
        Title = sprintf("Zeisel nervous system %s", c("counts", "rowData", "colData", "colPairs", "reducedDims")),
        Description = sprintf("%s for the Zeisel mouse nervous system single-cell RNA-seq dataset", 
            c("Count matrix", "Per-gene metadata", "Per-cell metadata", "Cell-cell pairings", "Reduced dimensions")),
        RDataPath = file.path("scRNAseq", "zeisel-nervous", "2.6.0", 
            c("counts.rds", "rowdata.rds", "coldata.rds", "colpairs.rds", 'reddims.rds')),
        BiocVersion="3.13",
        Genome="mm10",
        SourceType="HDF5",
        SourceUrl="http://mousebrain.org/downloads.html",
        SourceVersion="L5_All.loom",
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame", "DFrame", "list", "list"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

# mkdir -p scRNAseq/ercc-concentrations/2.2.0/
# cd scRNAseq/ercc-concentrations/2.2.0/
# wget https://assets.thermofisher.com/TFS-Assets/LSG/manuals/cms_095046.txt

write.csv(file = "../extdata/2.2.0/metadata-ercc-concentrations.csv", 
    data.frame(
        Title = "ERCC spike-in concentrations",
        Description = "Concentrations for ERCC spike-in RNA molecules in 2 mixes",
        RDataPath = file.path("scRNAseq", "ercc-concentrations", "2.2.0", "cms_095046.txt"),
        BiocVersion = "3.11",
        Genome = NA,
        SourceType = "TSV",
        SourceUrl = "https://assets.thermofisher.com/TFS-Assets/LSG/manuals/",
        SourceVersion = "cms_095046.txt",
        Species = NA,
        TaxonomyId = NA,
        Coordinate_1_based = NA,
        DataProvider = "Thermo Fisher Scientific",
        Maintainer = "Alan O'Callaghan <alan.ocallaghan@outlook.com>",
        RDataClass = "DFrame",
        DispatchClass = "FilePath",
        stringsAsFactors = FALSE
    ),
    row.names = FALSE)

FUN <- function(species, short, long, id) {
    data.frame(
        Title = sprintf("Baron %s pancreas %s", species, c("counts", "colData")),
        Description = sprintf("%s for the Baron %s pancreas single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata"), species),
        RDataPath = file.path("scRNAseq", "baron-pancreas", "2.0.0", 
            sprintf(c("counts-%s.rds", "coldata-%s.rds"), species)),
        BiocVersion="3.10",
        Genome=short,
        SourceType="CSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE84133",
        SourceVersion="GSE84133_RAW.tar", 
        Species=long,
        TaxonomyId=id,
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    )
}

write.csv(file="../extdata/2.0.0/metadata-baron-pancreas.csv", 
    rbind(FUN("human", "hg38", "Homo sapiens", "9606"), FUN("mouse", "mm10", "Mus musculus", "10090")),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-richard-tcell.csv", 
    data.frame(
        Title = sprintf("Richard CD8+ T cell %s", c("counts", "colData")),
        Description = sprintf("%s for the Richard CD8+ T cell single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "richard-tcell", "2.0.0", c("counts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6051/",
        SourceVersion=c("E-MTAB-6051.processed.1.zip", "E-MTAB-6051.sdrf.txt"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="ArrayExpress",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

write.csv(file="../extdata/2.0.0/metadata-aztekin-tail.csv", 
    data.frame(
        Title = sprintf("Aztekin Xenopus tail %s", c("counts", "colData")),
        Description = sprintf("%s for the Aztekin Xenopus tail single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "aztekin-tail", "2.0.0", c("counts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="v9.1",
        SourceType="ZIP",
        SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-7716",
        SourceVersion="E-MTAB-7716.processed.1.zip",
        Species="Xenopus laevis",
        TaxonomyId="8355",
        Coordinate_1_based=NA,
        DataProvider="ArrayExpress",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

write.csv(file="../../extdata/2.6.0/metadata-jessa-brain.csv",
    data.frame(
        Title = sprintf("Jessa mouse brain %s", c("counts", "colData", "rowData", "reducedDims")),
        Description = sprintf("%s for the Jessa mouse brain single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata", "Per-gene metadata", "Reduced dimensions")),
        RDataPath = file.path("scRNAseq", "jessa-brain", "2.6.0", 
            c("counts.rds", "coldata.rds", "rowdata.rds", "reddims.rds")),
        BiocVersion="3.13",
        Genome="mm10",
        SourceType=c("MTX", "TSV", "TSV", "TSV"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE133531",
        SourceVersion=c(
            "GSE133531_RAW.tar",
            "GSE133531_RAW.tar;GSE133531_Forebrain_join.2D.tsv.gz;GSE133531_Pons_join.2D.tsv.gz"), # recycle
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame", "DFrame", "list"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-nestorowa-hsc.csv", 
    data.frame(
        Title = sprintf("Nestorowa HSC %s", c("counts", "colData")),
        Description = sprintf("%s for the Nestorowa haematopoietic stem cell single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "nestorowa-hsc", "2.0.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl=c(
            "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81682",
            "http://blood.stemcells.cam.ac.uk/data"
        ),
        SourceVersion=c(
            "GSE81682_HTSeq_counts.txt.gz",
            "all_cell_types.txt;coordinates_gene_counts_flow_cytometry.txt.gz"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider=c("GEO", "Bertie Göttgens"),
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

collected <- list()
for (tissue in c("Bladder", "Blood", "Common.bile.duct", "Esophagus", "Heart", 
        "Liver", "Lymph.node", "Marrow", "Muscle", "Rectum", "Skin",
        "Small.intestine", "Spleen", "Stomach", "Trachea")) {    
    nice.tissue <- tolower(sub("\\.", " ", tissue))
    collected[[tissue]] <- data.frame(
            Title = sprintf("He human organ atlas %s %s", nice.tissue, c("counts", "colData")),
            Description = sprintf("%s for the %s subset of the He human organ atlas single-cell RNA-seq dataset", 
                c("Count matrix", "Per-cell metadata"), nice.tissue),
            RDataPath = file.path("scRNAseq", "he-organ-atlas", "2.6.0", 
                paste0(c("counts", "coldata"), "-", tissue, ".rds")),
            BiocVersion="3.13",
            Genome="hg38",
            SourceType=c("CSV", "TSV"),
            SourceUrl=c(
                "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE159929",
                "https://github.com/bei-lab/scRNA-AHCA/tree/master/Cell_barcode_and_corresponding_cell_types_of_AHCA"),
            SourceVersion=c(
                "GSE159929_RAW.tar",
                "Annotation_AHCA_alltissues_meta.data_84363_cell.txt;B_and_plasma.meta.data.txt;CD4_meta.data.txt;CD8_meta.data.txt;Endothelial_cell.meta.data.txt;Epithelial_cells.meta.data.txt;FibSmo.meta.data.txt;Myeloid.meta.data.txt"),
            Species="Homo sapiens",
            TaxonomyId="9606",
            Coordinate_1_based=NA,
            DataProvider="GEO",
            Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
            RDataClass=c("dgCMatrix", "DFrame"),
            DispatchClass="Rds",
            stringsAsFactors = FALSE
        )
}

collected <- do.call(rbind, collected)

extra <- data.frame(
    Title = "He human organ atlas reducedDims",
    Description = "Reduced dimensions for the He human organ atlas single-cell RNA-seq dataset",
    RDataPath = file.path("scRNAseq", "he-organ-atlas", "2.6.0", "reddim.rds"),
    BiocVersion="3.13",
    Genome="hg38",
    SourceType="TSV",
    SourceUrl="https://github.com/bei-lab/scRNA-AHCA/tree/master/Cell_barcode_and_corresponding_cell_types_of_AHCA",
    SourceVersion="Annotation_AHCA_alltissues_meta.data_84363_cell.txt",
    Species="Homo sapiens",
    TaxonomyId="9606",
    Coordinate_1_based=NA,
    DataProvider="GEO",
    Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
    RDataClass="list",
    DispatchClass="Rds",
    stringsAsFactors = FALSE
)

write.csv(file="../../extdata/2.6.0/metadata-he-organ-atlas.csv", rbind(collected, extra), row.names=FALSE)

write.csv(file="../../extdata/2.4.0/metadata-wu-kidney.csv", 
    data.frame(
        Title = sprintf("Wu %s mouse kidney %s", 
            rep(c("healthy", "diseased"), each=2),
            rep(c("counts", "colData"), 2)),
        Description = sprintf("%s for %s in the Wu mouse kidney single-nucleus RNA-seq dataset", 
            rep(c("Count matrix", "Per-cell metadata"), 2),
            rep(c("healthy tissue", "fibrotic tissue"), each=2)),
        RDataPath = file.path("scRNAseq", "wu-kidney", "2.4.0", 
            paste0(c("counts-healthy", "coldata-healthy", "counts-disease", "coldata-disease"), ".rds")),
        BiocVersion="3.12",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE119531",
        SourceVersion=c(
            "GSE119531_Healthy.combined.dge.txt.gz",
            "GSE119531_Healthy.combined.cell.annotation.txt.gz",
            "GSE119531_UUO.dge.txt.gz",
            "GSE119531_UUO.cell.annotation.txt.gz"
        ),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "data.frame", "dgCMatrix", "data.frame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

GENERATOR <- function(short, long, target, src) {
    data.frame(
        Title = sprintf("Bach mammary %s", short),
        Description = sprintf("%s for the Bach mammary gland single-cell RNA-seq dataset", long), 
        RDataPath = file.path("scRNAseq", "bach-mammary", "2.0.0", target), 
        BiocVersion="3.10",
        Genome="mm10",
        SourceType=if (grepl("counts", target)) "MTX" else "TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE106273",
        SourceVersion=src,
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=if (grepl("counts", target)) "dgCMatrix" else "DFrame",
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    )
}

accessions <- c(
    "GSM2834498", "GSM2834499",
    "GSM2834500", "GSM2834501",
    "GSM2834502", "GSM2834503",
    "GSM2834504", "GSM2834505")

samples <- c(
    "NP_1", "NP_2",
    "G_1", "G_2",
    "L_1", "L_2",
    "PI_1", "PI_2")

conditions <- c(NP="nulliparous",
    G="gestation",
    L="lactation",
    PI="post-involution")[sub("_.*", "", samples)]

collected <- list()
for (i in seq_along(accessions)) {
    n <- sub(".*_", "", samples[i])
    collected[[i]] <- rbind(
        GENERATOR(paste(samples[i], "counts"), paste0("Count matrix (", conditions[i], " replicate ", n, ")"), 
            paste0("counts-", samples[i], ".rds"), paste0(accessions[i], "_", samples[i], "_counts.mtx.gz")),
        GENERATOR(paste(samples[i], "colData"), paste0("Per-cell metadata (", conditions[i], " replicate ", n, ")"), 
            paste0("coldata-", samples[i], ".rds"), paste0(accessions[i], "_", samples[i], "_barcodes.tsv.gz"))
    )
} 
collected[[i+1]] <- GENERATOR("rowData", "Per-gene metadata", "rowdata.rds", 
    paste0(accessions[1], "_", samples[1], "_genes.tsv.gz"))

write.csv(file="../extdata/2.0.0/metadata-bach-mammary.csv", row.names=FALSE, do.call(rbind, collected))

write.csv(file="../extdata/2.0.0/metadata-chen-brain.csv", 
    data.frame(
        Title = sprintf("Chen brain %s", c("counts", "colData")),
        Description = sprintf("%s for the Chen brain single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "chen-brain", "2.0.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType=c("TSV", "CSV"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE87544",
        SourceVersion=c(
            "GSE87544_Merged_17samples_14437cells_count.txt.gz",
            "GSE87544_1443737Cells.SVM.cluster.identity.renamed.csv.gz"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.6.0/metadata-giladi-hsc.csv",
    data.frame(
        Title = sprintf("Giladi HSC %s", c("RNA counts", "RNA colData", "CRISPR counts", "CRISPR colData", "CRISPR rowData")),
        Description = sprintf("%s for the Giladi haematopoietic stem cell single-cell RNA-seq and CRISPR-seq dataset", 
            c("RNA count matrix", "Per-cell metadata (RNA)", "CRISPR count matrix", "Per-cell metadata (CRISPR)", "Per-guide metadata")),
        RDataPath = file.path("scRNAseq", "giladi-hsc", "2.6.0", 
            c("counts-rna.rds", "coldata-rna.rds", "counts-crispr.rds", "coldata-crispr.rds", "rowdata-crispr.rds")),
        BiocVersion="3.13",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl=rep(c(
            "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE113494",
            "https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE113494"),
            c(2,3)),
        SourceVersion=c(
            "GSE92575_RAW.tar",
            "GSE92575_metadata.txt.gz",
            rep("GSE113494_crispseq_count.txt.gz", 3)),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame", "dgCMatrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.6.0/metadata-bhaduri-organoid.csv",
    data.frame(
        Title = sprintf("Bhaduri organoid %s", c("counts", "colData")),
        Description = sprintf("%s for the Bhaduri cortical organoid single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "bhaduri-organoid", "2.6.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.13",
        Genome="hg38",
        SourceType=c("TSV", "TXT"),
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE132672",
        SourceVersion=c(
            "GSE132672_allorganoids_withnew_matrix_txt.gz",
            "GSE132672_series_matrix.txt.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-xin-pancreas.csv",
    data.frame(
        Title = sprintf("Xin pancreas %s", c("RPKMs", "rowData", "colData")),
        Description = sprintf("%s for the Xin pancreas single-cell RNA-seq dataset", 
            c("RPKM matrix", "Per-gene metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "xin-pancreas", "2.0.0", 
            c("rpkm.rds", "rowdata.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="hg19",
        SourceType=c("TSV", "CSV", "TSV"),
        SourceUrl=c("https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81608",
            rep("https://s3.amazonaws.com/scrnaseq-public-datasets/manual-data/xin", 2)), 
        SourceVersion=c(
            "GSE81608_human_islets_rpkm.txt.gz",
            "human_gene_annotation.csv",
            "human_islet_cell_identity.txt"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider=c("GEO", rep("Martin Hemberg", 2)),
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", rep("DataFrame", 2)),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-usoskin-brain.csv", 
    data.frame(
        Title = sprintf("Usoskin brain %s", c("RPMs", "rowData", "colData")),
        Description = sprintf("%s for the Usoskin brain single-cell RNA-seq dataset", 
            c("RPM matrix", "Per-gene metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "usoskin-brain", "2.0.0", 
            c("rpm.rds", "rowdata.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="XLS/XLSX",
        SourceUrl="http://linnarssonlab.org/drg/",
        SourceVersion="External resource Table 1",
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="Sten Linnarsson",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DataFrame", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-buettner-esc.csv", 
    data.frame(
        Title = sprintf("Buettner ESC %s", c("counts", "rowData", "colData")),
        Description = sprintf("%s for the Buettner embryonic stem cell single-cell RNA-seq dataset", 
            c("Count matrix", "Per-gene metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "buettner-esc", "2.0.0", c("counts.rds", "rowdata.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="ZIP",
        SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-2805",
        SourceVersion="E-MTAB-2805.processed.1.zip",
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="ArrayExpress",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

write.csv(file="../../extdata/2.6.0/metadata-nowakowski-cortex.csv",
    data.frame(
        Title = sprintf("Nowakowski cortex %s", c("TPMs", "colData", "reducedDims")),
        Description = sprintf("%s for the Nowakowski cortex single-cell RNA-seq dataset", 
            c("TPM matrix", "Per-cell metadata", "Reduced dimensions")),
        RDataPath = file.path("scRNAseq", "nowakowski-cortex", "2.6.0", 
            c("tpm.rds", "coldata.rds", "reddims.rds")),
        BiocVersion="3.13",
        Genome="hg38",
        SourceType="TSV",
        SourceUrl="http://cells.ucsc.edu/?ds=cortex-dev",
        SourceVersion=c(
            "exprMatrix.tsv.gz",
            "meta.tsv",
            "tMinusSNE_on_WGCNA.coords.tsv.gz;TriMap.coords.tsv.gz;TMinusSNE_scanpy.coords.tsv.gz;UMAP_scanpy.coords.tsv.gz;PAGAPlusForceAtlas2_scanpy.coords.tsv.gz"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame", "list"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

base <- c("Tophat count", "Cufflinks FPKM", "RSEM count", "RSEM TPM")
file.titles <- paste0(base, "s")
file.descs <- paste(base, "matrix")
file.paths <- paste0(c("tophat_counts", "cufflinks_fpkm", "rsem_counts", "rsem_tpm"), ".rds")

CREATOR <- function(desc, path, genome, src, species, taxid, append=TRUE) {
    write.table(file="../extdata/2.0.0/metadata-legacy.csv",
        data.frame(
            Title = sprintf("%s %s", desc, c(file.titles, "colData", "metadata")),
            Description = sprintf("%s for the %s single-cell RNA-seq dataset", 
                c(file.descs, "Per-cell metadata", "Experiment metadata"), desc),
            RDataPath = file.path("scRNAseq", path, c(file.paths, "coldata.rds", "metadata.rds")),
            BiocVersion="3.10",
            Genome=genome,
            SourceType="RData",
            SourceUrl="http://bioconductor.org/packages/3.9/data/experiment/src/contrib/scRNAseq_1.10.0.tar.gz",
            SourceVersion=src,
            Species=species,
            TaxonomyId=taxid,
            Coordinate_1_based=NA,
            DataProvider="Michael Cole",
            Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
            RDataClass=c(rep("matrix", length(file.titles)), "DataFrame", "list"),
            DispatchClass="Rds",
            stringsAsFactors = FALSE
        ),
        row.names=FALSE, sep=",", append=append, col.names=!append)
}

CREATOR("Allen brain", "legacy-allen/1.10.0", "mm10", "data/allen.rda", "Mus musculus", "10090", append=FALSE)
CREATOR("T helper 2", "legacy-th2/1.10.0", "mm10", "data/th2.rda", "Mus musculus", "10090")
CREATOR("Fluidigm", "legacy-fluidigm/1.10.0", "hg38", "data/pollen.rda", "Homo sapiens", "9606")

#' List all available datasets
#'
#' Summary information for all available datasets in the \pkg{scRNAseq} package.
#'
#' @details
#' A study may contribute multiple datasets if they cannot be reasonably combined (e.g., different species).
#' The reported number of cells refers only to the dataset as it is stored in \pkg{scRNAseq};
#' this may be different to the number of cells used by the authors in their analysis, e.g., due to filtering.
#'
#' @return 
#' A \linkS4class{DataFrame} where each row corresponds to a dataset, containing the fields:
#' \itemize{
#' \item \code{Reference}, a Markdown-formatted citation to \code{scripts/ref.bib} in the \pkg{scRNAseq} installation directory.
#' \item \code{Taxonomy}, an identifier for the organism.
#' \item \code{Part}, the part of the organism being studied.
#' \item \code{Number}, the total number of cells in the dataset.
#' \item \code{Call}, the relevant R call required to construct the dataset.
#' }
#'
#' @author Aaron Lun
#'
#' @examples
#' listDatasets()
#' 
#' @export
#' @importFrom S4Vectors DataFrame
#' @importFrom utils read.csv
listDatasets <- function() {
    path <- system.file("extdata", "manifest.csv", package="scRNAseq")
    DataFrame(read.csv(path, stringsAsFactors=FALSE))
}

.move_cache <- function(pkg, envvar) { 
    if (!is.na(Sys.getenv(envvar, NA))) {
        return(NULL)
    }

    olddir <- path.expand(rappdirs::user_cache_dir(appname=pkg))
    if (!file.exists(olddir)) {
        return(NULL)
    }

    newdir <- tools::R_user_dir(pkg, which="cache")
    if (file.exists(newdir)) {
        # Shouldn't happen that we have both new and old caches, 
        # but whatever, you can deal with it.
        return(NULL)
    }

    dir.create(path=newdir, recursive=TRUE)
    destroyme <- newdir
    on.exit(unlink(destroyme, recursive=TRUE, force=TRUE))

    files <- list.files(olddir, full.names =TRUE)
    for (f in files) {
        newname <- file.path(newdir, basename(f))
        if (!file.exists(newname)) { # weak protection against concurrent evaluation.
            if (!file.link(f, newname) && !file.copy(f, newname)) {
                return(NULL)
            }
        }
    }

    destroyme <- olddir
    NULL
}

collected <- list()
for (analysis in c("Cellranger", "emptyDrops")) {
    collected[[analysis]] <- data.frame(
            Title = sprintf("Ernst spermatogenesis %s (%s cell calls)", c("counts", "colData", "rowData"), analysis),
            Description = sprintf("%s for Ernst mouse spermatogenesis single-cell RNA-seq dataset with the %s-derived cell calls", 
                c("Count matrix", "Per-cell metadata", "Per-gene metadata"), analysis),
            RDataPath = file.path("scRNAseq", "ernst-spermatogenesis", "2.6.0", 
                paste0(c("counts", "coldata", "rowdata"), "-", tolower(analysis), ".rds")),
            BiocVersion="3.13",
            Genome="mm10",
            SourceType=c("MTX", "TXT", "TSV"),
            SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6946",
            SourceVersion=if (analysis=="Cellranger") {
                    c("E-MTAB-6946.processed.3.zip", 
                    "E-MTAB-6946.processed.1.zip", 
                    "E-MTAB-6946.processed.2.zip")
                } else {
                    c("E-MTAB-6946.processed.4.zip",
                    "E-MTAB-6946.processed.5.zip",
                    "E-MTAB-6946.processed.6.zip")
                },
            Species="Mus musculus",
            TaxonomyId="10090",
            Coordinate_1_based=NA,
            DataProvider="ArrayExpress",
            Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
            RDataClass=c("dgCMatrix", "DFrame", "DFrame"),
            DispatchClass="Rds",
            stringsAsFactors = FALSE
        )
}

write.csv(file="../../extdata/2.6.0/metadata-ernst-spermatogenesis.csv", rbind(collected[[1]], collected[[2]]), row.names=FALSE)

spawn_df <- function(type, suffix, src, file="TSV") {
    data.frame(
        Title = sprintf("La Manno %s %s", type, c("counts", "colData")),
        Description = sprintf("%s for the La Manno %s single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata"), type),
        RDataPath = file.path("scRNAseq", "lamanno-brain", "2.0.0", 
            sprintf(c("counts-%s.rds", "coldata-%s.rds"), suffix)),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType=file,
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE76381",
        SourceVersion=src,
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    )
}

write.csv(file="../extdata/2.0.0/metadata-lamanno-brain.csv", 
    rbind(
        spawn_df("human ES", "human-es", "GSE76381_ESMoleculeCounts.cef.txt.gz"),
        spawn_df("human embryo midbrain", "human-embryo", "GSE76381_EmbryoMoleculeCounts.cef.txt.gz"),
        spawn_df("mouse adult dopaminergic neuron", "mouse-adult", "GSE76381_MouseAdultDAMoleculeCounts.cef.txt.gz", file="CSV"),
        spawn_df("mouse embryo midbrain", "mouse-embryo", "GSE76381_MouseEmbryoMoleculeCounts.cef.txt.gz"),
        spawn_df("human iPS", "human-ips", "GSE76381_iPSMoleculeCounts.cef.txt.gz")
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-segerstolpe-pancreas.csv",
    data.frame(
        Title = sprintf("Segerstolpe pancreas %s", c("counts", "rowData", "colData")),
        Description = sprintf("%s for the Segerstolpe pancreas single-cell RNA-seq dataset", 
            c("Count matrix", "Per-gene metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "segerstolpe-pancreas", "2.0.0", 
            c("counts.rds", "rowdata.rds", "coldata.rds")),
        BiocVersion="3.10",
        Genome="hg19",
        SourceType="TSV",
        SourceUrl="https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-5061/",
        SourceVersion=c(
            "E-MTAB-5061.processed.1.zip",
            "E-MTAB-5061.processed.1.zip",
            "E-MTAB-5061.sdrf.txt"),
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="ArrayExpress",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("matrix", "DataFrame", "DataFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.0.0/metadata-grun-hsc.csv", 
    data.frame(
        Title = "Grun HSC counts",
        Description = "Count matrix for the Grun haematopoietic stem cell single-cell RNA-seq dataset", 
        RDataPath = file.path("scRNAseq", "grun-hsc", "2.0.0", "counts.rds"),
        BiocVersion="3.10",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE76983",
        SourceVersion="GSE76983_expdata_BMJhscC.csv.gz",
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass="dgCMatrix",
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../extdata/2.2.0/metadata-paul-hsc.csv", 
    data.frame(
        Title = sprintf("Paul HSC %s", c("counts", "colData")),
        Description = sprintf("%s for the Paul haematopoietic stem cell single-cell RNA-seq dataset", 
            c("Count matrix", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "paul-hsc", "2.2.0", 
            c("counts.rds", "coldata.rds")),
        BiocVersion="3.11",
        Genome="mm10",
        SourceType="TSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE72857",
        SourceVersion=c(
            "GSE72857_umitab.txt.gz",
            "GSE72857_experimental_design.txt.gz"),
        Species="Mus musculus",
        TaxonomyId="10090",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Aaron Lun <infinite.monkeys.with.keyboards@gmail.com>",
        RDataClass=c("dgCMatrix", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE)

write.csv(file="../../extdata/2.4.0/metadata-hermann-spermatogenesis.csv", 
          data.frame(
              Title = sprintf("Hermann Spermatogenesis %s", c("spliced counts", "unspliced counts", "colData")),
              Description = sprintf("%s for the Hermann spermatogenesis single-cell RNA-seq dataset", 
                                    c("Spliced count matrix", "Unspliced count matrix", "Per-cell metadata")),
              RDataPath = file.path("scRNAseq", "hermann-spermatogenesis", "2.4.0", 
                                    c("spliced.rds", "unspliced.rds", "coldata.rds")),
              BiocVersion="3.12",
              Genome="GRCm38",
              SourceType=c("BAM","BAM","TXT"),
              SourceUrl=c(rep("https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSM2928341,https://sra-pub-src-1.s3.amazonaws.com/SRR6459157/AdultMouse_Rep3_possorted_genome_bam.bam.1",2),
                          "https://data.mendeley.com/datasets/kxd5f8vpt4/1#file-fe79c10b-c42e-472e-9c7e-9a9873d9b3d8"),
              SourceVersion="",
              Species="Mus musculus",
              TaxonomyId="10090",
              Coordinate_1_based=NA,
              DataProvider="GEO",
              Maintainer="Charlotte Soneson <charlottesoneson@gmail.com>",
              RDataClass=c("dgCMatrix", "dgCMatrix", "DFrame"),
              DispatchClass="Rds",
              stringsAsFactors = FALSE
          ),
          row.names=FALSE)

write.csv(file="../../extdata/2.4.0/metadata-mair-pbmc.csv", 
    data.frame(
        Title = sprintf("Mair PBMC %s", c("RNA counts", "RNA rowData", "ADT counts", "ADT rowData", "colData")),
        Description = sprintf("%s for the Mair healthy PBMC targeted CITE-seq dataset", 
            c("RNA count matrix", "RNA gene metadata","ADT count matrix","ADT feature metadata", "Per-cell metadata")),
        RDataPath = file.path("scRNAseq", "mair-pbmc", "2.4.0",
            c("counts-rna.rds", "rowdata-rna.rds", "counts-adt.rds", "rowdata-adt.rds","coldata.rds")),
        BiocVersion="3.12",
        Genome="hg38",
        SourceType="CSV",
        SourceUrl="https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE135325",
        SourceVersion="GSM4005486_Combined_PBMC_AbSeq_1_DBEC_MolsPerCell_with_SampleTag.csv.gz;GSM4005487_Combined_PBMC_AbSeq_2_DBEC_MolsPerCell_with_SampleTag.csv.gz",
        Species="Homo sapiens",
        TaxonomyId="9606",
        Coordinate_1_based=NA,
        DataProvider="GEO",
        Maintainer="Stephany Orjuela <sorjuelal@gmail.com>",
        RDataClass=c("matrix", "DFrame", "matrix", "DFrame", "DFrame"),
        DispatchClass="Rds",
        stringsAsFactors = FALSE
    ),
    row.names=FALSE
)

---
title: Processing the Ernst spermatogenesis dataset
author: Aaron Lun
date: 22 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of mouse spermatogenesis from @ernst2019staged.
Counts for endogenous genes are available from ArrayExpress using the accession number [E-MTAB-6946](https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6946/).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
cr.zpath <- bfcrpath(bfc, "https://www.ebi.ac.uk/arrayexpress/files/E-MTAB-6946/E-MTAB-6946.processed.3.zip")
ed.zpath <- bfcrpath(bfc, "https://www.ebi.ac.uk/arrayexpress/files/E-MTAB-6946/E-MTAB-6946.processed.4.zip")
```

We read each of the count matrices into memory.

```{r}
library(Matrix)
unzip(cr.zpath, exdir=tempdir())
cr.mat <- as(readMM(file.path(tempdir(), "raw_counts.mtx")), "dgCMatrix")
dim(cr.mat)
unzip(ed.zpath, exdir=tempdir())
ed.mat <- as(readMM(file.path(tempdir(), "raw_counts_emptyDrops.mtx")), "dgCMatrix")
dim(ed.mat)
```

We then slap all the objects together in a `SingleCellExperiment` object.

```{r}
library(SingleCellExperiment)
cellranger <- SingleCellExperiment(list(counts=cr.mat))
cellranger
emptydrops <- SingleCellExperiment(list(counts=ed.mat))
emptydrops
```

# Filling in the metadata

Adding gene-level metadata.

```{r}
gene.path <- bfcrpath(bfc, "https://www.ebi.ac.uk/arrayexpress/files/E-MTAB-6946/E-MTAB-6946.processed.2.zip")
unzip(gene.path, exdir=tempdir())
gene.info <- read.delim(file.path(tempdir(), "genes.tsv"))
rowData(cellranger) <- gene.info
rownames(cellranger) <- gene.info$ID
rowData(cellranger)

gene.path <- bfcrpath(bfc, "https://www.ebi.ac.uk/arrayexpress/files/E-MTAB-6946/E-MTAB-6946.processed.6.zip")
unzip(gene.path, exdir=tempdir())
gene.info <- read.delim(file.path(tempdir(), "genes_emptyDrops.tsv"))
rowData(emptydrops) <- gene.info
rownames(emptydrops) <- gene.info$ID
rowData(emptydrops)
```

Adding cell-level metadata.

```{r}
cell.path <- bfcrpath(bfc, "https://www.ebi.ac.uk/arrayexpress/files/E-MTAB-6946/E-MTAB-6946.processed.1.zip")
unzip(cell.path, exdir=tempdir())
cell.info <- read.table(file.path(tempdir(), "cell_metadata.txt"))
colData(cellranger) <- DataFrame(cell.info, check.names=FALSE)
colData(cellranger)

cell.path <- bfcrpath(bfc, "https://www.ebi.ac.uk/arrayexpress/files/E-MTAB-6946/E-MTAB-6946.processed.5.zip")
unzip(cell.path, exdir=tempdir())
cell.info <- read.table(file.path(tempdir(), "cell_metadata_emptyDrops.txt"))
colData(emptydrops) <- DataFrame(cell.info)
colData(emptydrops)
```

# Saving to file

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "ernst-spermatogenesis", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(cellranger), file=file.path(path, "counts-cellranger.rds"))
saveRDS(colData(cellranger), file=file.path(path, "coldata-cellranger.rds"))
saveRDS(rowData(cellranger), file=file.path(path, "rowdata-cellranger.rds"))

saveRDS(assay(emptydrops), file=file.path(path, "counts-emptydrops.rds"))
saveRDS(colData(emptydrops), file=file.path(path, "coldata-emptydrops.rds"))
saveRDS(rowData(emptydrops), file=file.path(path, "rowdata-emptydrops.rds"))
```

# Session information {-}

```{r}
sessionInfo()
```

# References

# scrnaseq

The `scRNAseq` package provides convenient access to several publicly available single-cell datasets in the form of [SingleCellExperiment](https://github.com/biocpy/singlecellexperiment) objects. Users can obtain a `SingleCellExperiment` and transform it into analysis-ready representations for immediate use.

To enable discovery, each dataset is decorated with metadata such as the study title/abstract, the species, the number of cells, etc. Users can also contribute their own published datasets to enable re-use by the wider Bioconductor/BiocPy community.

**Also check out the R version of this library [here@scRNAseq](https://bioconductor.org/packages/devel/data/experiment/html/scRNAseq.html) published to Bioconductor.**


## Contents

```{toctree}
:maxdepth: 2

Overview <readme>
Module Reference <api/modules>
Contributions & Help <contributing>
License <license>
Authors <authors>
Changelog <changelog>
```

## Indices and tables

* {ref}`genindex`
* {ref}`modindex`
* {ref}`search`

[Sphinx]: http://www.sphinx-doc.org/
[Markdown]: https://daringfireball.net/projects/markdown/
[reStructuredText]: http://www.sphinx-doc.org/en/master/usage/restructuredtext/basics.html
[MyST]: https://myst-parser.readthedocs.io/en/latest/

---
title: Processing the Grun HSC dataset
author: Aaron Lun
date: 18 May 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the mouse haematopoietic stem cells from @grun2016denovo.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE76983](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE76983).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
count.file <- bfcrpath(bfc, file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE76nnn/GSE76983/suppl/GSE76983_expdata_BMJhscC.csv.gz"))
```

We read this into memory as a sparse matrix.

```{r}
library(scater)
counts <- readSparseCounts(count.file)
dim(counts)
```

# Saving to file 

We save the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "grun-hsc", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Jessa brain dataset
author: Aaron Lun
date: 22 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the mouse brain from @jessa2019stalled.
Counts for endogenous genes are available from ArrayExpress using the accession number [E-MTAB-6946](https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6946/).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
raw.path <- bfcrpath(bfc, "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE133531&format=file")
tmp <- tempfile()
untar(raw.path, exdir=tmp)
list.files(tmp)
```

Creating a `SingleCellExperiment` from each entry.

```{r}
library(DropletUtils)
nms <- c("GSM3934450_ET_CT_12", "GSM3934451_ET_CT_15",
    "GSM3934452_PT_CT_0", "GSM3934453_C57BL6-P3-cortex",
    "GSM3934454_C57BL6-P6-cortex", "GSM3934455_ET_PO_12",
    "GSM3934456_ET_PO_15", "GSM3934457_PT_PO_0",
    "GSM3934458_C57BL6-P3-pons", "GSM3934459_C57BL6-P6-pons")

library(BiocParallel)
all.prefixes <- file.path(tmp, paste0(nms, "_"))
names(all.prefixes) <- nms
sce <- read10xCounts(all.prefixes, type="prefix", BPPARAM=MulticoreParam())
sce
```

# Filling in the metadata

Filling in the per-sample metadata.

```{r}
all.meta <- list()
for (i in seq_along(nms)) {
    all.meta[[i]] <- read.delim(file.path(tmp, paste0(nms[i], ".metadata.tsv.gz")))
    all.meta[[i]]$.Sample <- nms[i]
}
all.meta <- do.call(rbind, all.meta)
head(all.meta)
```

Matching to the columns of `sce`.

```{r}
obs <- DataFrame(Sample=all.meta$.Sample, Barcode=paste0(all.meta$Cell, "-1"))
stopifnot(all(obs %in% colData(sce)))
m <- match(colData(sce), obs)
all.meta <- DataFrame(all.meta[m,], row.names=NULL)

# Auto-filling cell and sample information, even for the cells that weren't used.
all.meta$Cell <- sub("-1", "", sce$Barcode)
stopifnot(all(!grepl("-", all.meta$Cell)))

rename <- split(all.meta$Sample, all.meta$.Sample)
rename <- lapply(rename, unique)
stopifnot(all(lengths(rename)==1L))
all.meta$Sample <- unlist(rename)[sce$Sample]
table(all.meta$Sample, useNA="always")

colData(sce) <- all.meta
colnames(sce) <- NULL
sce$retained <- !is.na(m)
sce$.Sample <- NULL
summary(sce$retained)
```

Moving the dimensionality reduction results to a better home.

```{r}
cd <- colData(sce)
pc.names <- c("PC1", "PC2")
t.names <- c("tSNE_1", "tSNE_2")
reducedDims(sce) <- list(Sample_PC=as.matrix(cd[,pc.names]), Sample_tSNE=as.matrix(cd[,t.names]))

keep <- colnames(cd) %in% c(pc.names, t.names)
cd <- cd[,!keep,drop=FALSE]
colnames(cd)[colnames(cd)=="Cluster"] <- "Sample_Cluster"
colnames(cd)[colnames(cd)=="Cluster_number"] <- "Sample_Cluster_number"
colData(sce) <- cd
sce
```

# Adding joint metadata

Adding the joint forebrain data.

```{r}
fb.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE133531&format=file&file=GSE133531%5FForebrain%5Fjoin%2E2D%2Etsv%2Egz")
fb.meta <- DataFrame(read.delim(fb.path))
fb.meta

obs <- fb.meta[,c("Cell", "Sample")]
obs$Cell <- sub(".*_", "", obs$Cell)
ref <- colData(sce)[,c("Cell", "Sample")]
stopifnot(all(obs %in% ref))
m <- match(ref, obs)
summary(is.na(m))
fb.meta <- fb.meta[m,]

u.names <- c("UMAP1", "UMAP2")
reducedDim(sce, "Forebrain_PC") <- as.matrix(fb.meta[,pc.names])
reducedDim(sce, "Forebrain_tSNE") <- as.matrix(fb.meta[,t.names])
reducedDim(sce, "Forebrain_UMAP") <- as.matrix(fb.meta[,u.names])

sce$Forebrain_Joint_cluster_number <- fb.meta$Joint_cluster_number
table(sce$Forebrain_Joint_cluster_number)
sce$Forebrain_Joint_cluster <- fb.meta$Joint_cluster
table(sce$Forebrain_Joint_cluster)
```

Repeating the dose with the Pons data.

```{r}
pons.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE133531&format=file&file=GSE133531%5FPons%5Fjoin%2E2D%2Etsv%2Egz")
pons.meta <- DataFrame(read.delim(pons.path))
pons.meta

obs <- pons.meta[,c("Cell", "Sample")]
obs$Cell <- sub(".*_", "", obs$Cell)
ref <- colData(sce)[,c("Cell", "Sample")]
stopifnot(all(obs %in% ref))
m <- match(ref, obs)
summary(is.na(m))
pons.meta <- pons.meta[m,]

u.names <- c("UMAP1", "UMAP2")
reducedDim(sce, "Pons_PC") <- as.matrix(pons.meta[,pc.names])
reducedDim(sce, "Pons_tSNE") <- as.matrix(pons.meta[,t.names])
reducedDim(sce, "Pons_UMAP") <- as.matrix(pons.meta[,u.names])

sce$Pons_Joint_cluster_number <- pons.meta$Joint_cluster_number
table(sce$Pons_Joint_cluster_number)
sce$Pons_Joint_cluster <- pons.meta$Joint_cluster
table(sce$Pons_Joint_cluster)
```

# Saving to file

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "jessa-brain", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
saveRDS(rowData(sce), file=file.path(path, "rowdata.rds"))
saveRDS(as.list(reducedDims(sce)), file=file.path(path, "reddims.rds"))
```

# Session information {-}

```{r}
sessionInfo()
```

---
title: Processing the Muraro pancreas dataset
author: Aaron Lun
date: 28 June 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human pancreas from @muraro2016singlecell.
A count matrix is provided from the Gene Expression Omnibus under the accession code GSE85241.
We download it using `r Biocpkg("BiocFileCache")` to cache the results:

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask=FALSE)
muraro.fname <- bfcrpath(bfc, file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE85nnn/GSE85241/suppl",
    "GSE85241%5Fcellsystems%5Fdataset%5F4donors%5Fupdated%2Ecsv%2Egz"))
```

We first read the table into memory as a sparse matrix.

```{r}
library(scater)
counts <- readSparseCounts(muraro.fname, quote="\"")
dim(counts)
```

# Loading the column metadata

We extract the metadata from the column names.

```{r}
donor.names <- sub("^(D[0-9]+).*", "\\1", colnames(counts))
plate.id <- sub("^D[0-9]+-([0-9]+)_.*", "\\1", colnames(counts))
table(donor.names, plate.id)
```

We also load in some cell annotations provided by the authors (via Vladimir Wikislev and Martin Hemberg).

```{r}
muraro.cell <- bfcrpath(bfc, file.path("https://s3.amazonaws.com",
    "scrnaseq-public-datasets/manual-data/muraro",
    "cell_type_annotation_Cels2016.csv"))
coldata <- read.delim(muraro.cell, row.names=1, stringsAsFactors=FALSE)

library(S4Vectors)
coldata <- as(coldata, "DataFrame")
coldata
```

We clean up the names of `coldata` and make sure to synchronize the rows with `counts`.

```{r}
colnames(coldata) <- "label"
rownames(coldata) <- sub("\\.", "-", rownames(coldata))
m <- match(colnames(counts), rownames(coldata))
coldata <- coldata[m,,drop=FALSE]
rownames(coldata) <- colnames(counts)
summary(is.na(m))
```

Finally, we add the extra annotations.

```{r}
coldata$donor <- donor.names
coldata$plate <- factor(plate.id)
coldata
```

# Saving to file

We save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "muraro-pancreas", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Grun pancreas dataset
author: Aaron Lun
date: 28 June 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human pancreas from @grun2016denovo.
A count matrix for endogenous genes and spike-ins is provided from the Gene Expression Omnibus 
using accession code [GSE81076](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81076).
We download it using `r Biocpkg("BiocFileCache")` to cache the results:

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask=FALSE)
grun.fname <- bfcrpath(bfc, file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE81nnn/GSE81076/suppl/GSE81076%5FD2%5F3%5F7%5F10%5F17%2Etxt%2Egz"))
```

We read the table into memory as a sparse matrix.

```{r}
library(scater)
counts <- readSparseCounts(grun.fname, quote="\"")
dim(counts)
```

# Saving to file

We save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "grun-pancreas", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Usoskin brain dataset
author: Aaron Lun
date: "June 8, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of the mouse brain from @usoskin2015unbiased.
RPMs for endogenous genes and repeat regions are available as External Resource Table 1 at http://linnarssonlab.org/drg/.
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("https://storage.googleapis.com",
    "linnarsson-lab-www-blobs/blobs/drg")
count.file <- bfcrpath(bfc, file.path(base.url,
    "Usoskin%20et%20al.%20External%20resources%20Table%201.xlsx"))
```

We load them into memory.

```{r}
library(readxl)
all.raw <- read_xlsx(count.file, sheet=2, col_names=FALSE)
```

# Unpacking the format

We need to unpack the column data:

```{r}
meta.raw <- head(all.raw, 10)
coldata <- data.frame(t(meta.raw[,-(1:9)]), stringsAsFactors=FALSE)

library(S4Vectors)
coldata <- DataFrame(coldata)
colnames(coldata) <- meta.raw[,9,drop=TRUE]
rownames(coldata) <- NULL

coldata[,5] <- as.numeric(coldata[,5])
coldata
```

And then the rowdata:

```{r}
gene.info <- tail(all.raw, -11)
rowdata <- as.data.frame(gene.info[,1:8])
rowdata.names <- all.raw[11,1:8,drop=TRUE]
rowdata.names <- sub("^[0-9]\\. ", "", rowdata.names)
rowdata <- DataFrame(rowdata)
colnames(rowdata) <- rowdata.names
rowdata
```

And finally, the RPMs (since no raw counts have been provided):

```{r}
rpms <- as.matrix(gene.info[,-(1:9)])
storage.mode(rpms) <- "numeric"
colnames(rpms) <- NULL
dim(rpms)
```

# Saving to file 

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "usoskin-brain", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(rpms, file=file.path(path, "rpms.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
saveRDS(rowdata, file=file.path(path, "rowdata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Stoeckius hashing dataset
author: Aaron Lun
date: 23 July 2020 
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the hashed PBMCs and cell lines from @stoeckius2018hashing.
Counts for endogenous genes, ADTs and HTOs are available from the Gene Expression Omnibus
using the accession number [GGSE108313](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE108313).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)

pbmc.rna <- bfcrpath(bfc, file.path("https://ftp.ncbi.nlm.nih.gov/geo/samples",
    "GSM2895nnn/GSM2895282/suppl/GSM2895282_Hashtag-RNA.umi.txt.gz"))
pbmc.hto <- bfcrpath(bfc, file.path("https://ftp.ncbi.nlm.nih.gov/geo/samples",
    "GSM2895nnn/GSM2895283/suppl/GSM2895283_Hashtag-HTO-count.csv.gz"))
pbmc.adt1 <- bfcrpath(bfc, file.path("https://ftp.ncbi.nlm.nih.gov/geo/samples",
    "GSM2895nnn/GSM2895284/suppl/GSM2895284_Hashtag-ADT1-count.csv.gz"))
pbmc.adt2 <- bfcrpath(bfc, file.path("https://ftp.ncbi.nlm.nih.gov/geo/samples",
    "GSM2895nnn/GSM2895284/suppl/GSM2895284_Hashtag-ADT2-count.csv.gz"))

cell.rna <- bfcrpath(bfc, file.path("https://ftp.ncbi.nlm.nih.gov/geo/samples",
    "GSM3501nnn/GSM3501446/suppl/GSM3501446_MixCellLines-RNA.umi.txt.gz"))
cell.hto <- bfcrpath(bfc, file.path("https://ftp.ncbi.nlm.nih.gov/geo/samples",
    "GSM3501nnn/GSM3501447/suppl/GSM3501447_MixCellLines-HTO-counts.csv.gz"))
```

We read the RNA counts into memory as sparse matrices,
while the HTO and ADT counts are read in as dense matrices owing to, well, their density.

```{r}
library(scuttle)
pbmc.rna.mat <- readSparseCounts(pbmc.rna, row.names=1)
dim(pbmc.rna.mat)
pbmc.hto.mat <- as.matrix(read.csv(pbmc.hto, row.names=1))
dim(pbmc.hto.mat)
pbmc.adt1.mat <- as.matrix(read.csv(pbmc.adt1, row.names=1))
dim(pbmc.adt1.mat)
pbmc.adt2.mat <- as.matrix(read.csv(pbmc.adt2, row.names=1))
dim(pbmc.adt2.mat)

cell.rna.mat <- readSparseCounts(cell.rna, row.names=1)
dim(cell.rna.mat)
cell.hto.mat <- t(as.matrix(read.csv(cell.hto, row.names=1)))
dim(cell.hto.mat)
```

# Sanity checks

Somehow, none of the matrices have the same set of columns!
Incredible.
This will need to be filtered out before use.

```{r}
length(intersect(colnames(pbmc.hto.mat), colnames(pbmc.adt1.mat)))
length(intersect(colnames(pbmc.hto.mat), colnames(pbmc.adt2.mat)))
length(intersect(colnames(pbmc.hto.mat), colnames(pbmc.rna.mat)))
length(intersect(colnames(cell.hto.mat), colnames(cell.rna.mat)))
```

Even more fun is the fact that the PBMC dataset contains both human and mouse transcripts.
Let's try to figure out which is which based on capitalization and the fact that the cell line data is human-only.
Even this is not straightforward because the cell line data does not have the full set of human features!

```{r}
probably.human <- !grepl("[a-z]+", rownames(pbmc.rna.mat)) |
    grepl("^hsa-", rownames(pbmc.rna.mat)) |
    rownames(pbmc.rna.mat) %in% rownames(cell.rna.mat)
summary(probably.human)
head(rownames(pbmc.rna.mat)[probably.human])
tail(rownames(pbmc.rna.mat)[probably.human])

head(rownames(pbmc.rna.mat)[!probably.human])
tail(rownames(pbmc.rna.mat)[!probably.human])
```

Well, whatever.

```{r}
pbmc.human.mat <- pbmc.rna.mat[probably.human,]
pbmc.mouse.mat <- pbmc.rna.mat[!probably.human,]
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "stoeckius-hashing", "2.4.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)

saveRDS(pbmc.human.mat, file=file.path(path, "counts-pbmc-human.rds"))
saveRDS(pbmc.mouse.mat, file=file.path(path, "counts-pbmc-mouse.rds"))
saveRDS(pbmc.adt1.mat, file=file.path(path, "counts-pbmc-adt1.rds"))
saveRDS(pbmc.adt2.mat, file=file.path(path, "counts-pbmc-adt2.rds"))
saveRDS(pbmc.hto.mat, file=file.path(path, "counts-pbmc-hto.rds"))

saveRDS(cell.rna.mat, file=file.path(path, "counts-mixed-rna.rds"))
saveRDS(cell.hto.mat, file=file.path(path, "counts-mixed-hto.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Segerstolpe pancreas dataset
author: Aaron Lun
date: 18 May 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human pancreas from @segerstolpe2016single. 
A count matrix is provided in the ArrayExpress entry for this project.
We download it using `r Biocpkg("BiocFileCache")` to cache the results:

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask=FALSE)    
emat <- bfcrpath(bfc, file.path("https://www.ebi.ac.uk/arrayexpress",
    "experiments/E-MTAB-5061/files/E-MTAB-5061.processed.1.zip"))
unzip(emat, list=TRUE)
```

The file itself is quite complex:

- The first 2 columns contain the gene symbol and NCBI GenBank transcript identifiers for each row.
- The next `X` columns are the RPKMs, for `X` cells.
- The remaining `X` columns are the counts.

This requires some additional work to extract the useful data.
The first line contains the names of the cells, 
so we can use this to determine the number and indices of the columns with per-cell counts.

```{r}
count.file <- "pancreas_refseq_rpkms_counts_3514sc.txt"
col.names <- read.table(unz(emat, count.file), header=FALSE, sep="\t", 
    stringsAsFactors=FALSE, comment.char="", nrows = 1)[,-1]
ncells <- length(col.names)

what <- vector("list", ncells*2 + 2)
what[[1]] <- "character"
what[[2]] <- "character"
what[seq_len(ncells) + ncells + 2] <- "integer"
```

We then read in the gene symbols and the counts.

```{r}
emtab.df <- read.table(unz(emat, count.file), header=FALSE, sep="\t", 
    stringsAsFactors=FALSE, colClasses=what, skip=1)
gene.info <- emtab.df[,1:2]
emtab.df <- emtab.df[,-(1:2)]
colnames(emtab.df) <- col.names
dim(emtab.df)
```

Some coercion is performed to yield a count matrix and a row-level `DataFrame`. 

```{r}
library(S4Vectors)
counts <- as.matrix(emtab.df)
rowdata <- DataFrame(symbol=gene.info[,1], refseq=gene.info[,2])
```

# Preparing the column metadata

We retrieve the column metadata fields from ArrayExpress using the same accession number.

```{r}
meta.fname <- bfcrpath(bfc, file.path("https://www.ebi.ac.uk/arrayexpress",
    "files/E-MTAB-5061/E-MTAB-5061.sdrf.txt"))
emtab.sdrf <- read.delim(meta.fname, stringsAsFactors=FALSE, check.names=FALSE)
colnames(emtab.sdrf)
```

We make sure that the sample IDs match with the column names of the count matrix.

```{r}
stopifnot(identical(sort(emtab.sdrf[["Source Name"]]), sort(colnames(emtab.df))))
emtab.sdrf <- emtab.sdrf[match(colnames(counts), emtab.sdrf[["Source Name"]]),]
```

We only keep the `Source Name` and `Characteristics` fields.
The other fields describe relationships to other files/identifiers within ArrayExpress and are not of (primary) interest.

```{r}
keep <- grep("Characteristics", colnames(emtab.sdrf))
emtab.sdrf <- emtab.sdrf[,c(1, keep)]
colnames(emtab.sdrf) <- sub(".*\\[(.*)\\]", "\\1", colnames(emtab.sdrf))
```

We also remove fields that only have one level and thus are not really useful.

```{r}
has.multi.levels <- vapply(emtab.sdrf, function(x) length(unique(x))>1L, TRUE)
emtab.sdrf <- emtab.sdrf[,has.multi.levels]
```

The `cell type` field has some `"not applicable"` entries that should be more formally represented as `NA`s.

```{r}
lost <- emtab.sdrf$`cell type`=="not applicable"
emtab.sdrf$`cell type`[lost] <- NA_character_
```

We coerce this into a column-level `DataFrame`.

```{r}
coldata <- as(emtab.sdrf, "DataFrame")
coldata
```

# Saving to file

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "segerstolpe-pancreas", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(rowdata, file=file.path(path, "rowdata.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Nestorowa HSC dataset
author: Aaron Lun
date: 18 May 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the mouse haematopoietic stem cells from @nestorowa2016singlecell.
Counts for endogenous genes and spike-in transcripts are available from the Gene Expression Omnibus
using the accession number [GSE81682](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81682).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
count.file <- bfcrpath(bfc, file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE81nnn/GSE81682/suppl/GSE81682_HTSeq_counts.txt.gz"))
```

We read this into memory as a sparse matrix.

```{r}
library(scater)
counts <- readSparseCounts(count.file)
dim(counts)
```

# Downloading the per-cell metadata

We download the per-cell metadata from the website.

```{r}
celltype.file <- bfcrpath(bfc, file.path(
    "http://blood.stemcells.cam.ac.uk/data/all_cell_types.txt"))
celltype <- read.delim(celltype.file)
celltype <- celltype[match(colnames(counts), rownames(celltype)),]
stopifnot(identical(colnames(counts), rownames(celltype)))
head(celltype)
```

We convert this into a sparse logical matrix of identities.

```{r}
library(Matrix)
celltype <- as(as.matrix(celltype), "lgCMatrix")
head(celltype)
```

We also obtain a matrix of flow cytometry intensities:

```{r}
flowcyt.file <- bfcrpath(bfc, file.path("http://blood.stemcells.cam.ac.uk",
    "data/coordinates_gene_counts_flow_cytometry.txt.gz"))
flowcyt <- read.delim(flowcyt.file, row.names=1)
flowcyt <- flowcyt[match(colnames(counts), rownames(flowcyt)),]
rownames(flowcyt) <- colnames(counts)

is.diffusion <- grep("^DC[0-9]+$", colnames(flowcyt))
colnames(flowcyt)[is.diffusion]
is.facs <- setdiff(grep("^ENSMUSG[0-9]+$", colnames(flowcyt), invert=TRUE), is.diffusion)
colnames(flowcyt)[is.facs]
```

We pack this into a single `DataFrame`:

```{r}
coldata <- DataFrame(cell.type=I(celltype),
    diffusion=I(as.matrix(flowcyt[,is.diffusion])),
    FACS=I(as.matrix(flowcyt[,is.facs])))
coldata
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "nestorowa-hsc", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Marques brain dataset
author: Aaron Lun
date: "June 8, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of mouse oligodendrocytes from @marques2016oligodendrocyte.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE75330](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE75330).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
count.file <- bfcrpath(bfc, file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE75nnn/GSE75330/suppl/GSE75330_Marques_et_al_mol_counts2.tab.gz"))
```

We load them into memory.

```{r}
library(scater)
counts <- readSparseCounts(count.file)
dim(counts)
```

# Extracting the metadata

We extract the metadata for this study using the `r Biocpkg("GEOquery")` package.

```{r}
library(GEOquery)
coldata <- pData(getGEO("GSE75330")[[1]])

library(S4Vectors)
coldata <- as(coldata, "DataFrame")
rownames(coldata) <- NULL
colnames(coldata)
```

Unfortunately, some of the cells are named differently in the `coldata` compared to the `counts`, which needs correction.

```{r}
coldata[,1] <- sub("T[0-9]-", "-", coldata[,1])
coldata <- coldata[match(colnames(counts), coldata[,1]),]
stopifnot(identical(colnames(counts), coldata[,1]))
```

We remove the constant columns, as these are unlikely to be interesting.

```{r}
nlevels <- vapply(coldata, FUN=function(x) length(unique(x[!is.na(x)])), 1L)
coldata <- coldata[,nlevels > 1L]
```

We also remove the columns related to the GEO accession itself, as well as some redundant fields.

```{r}
coldata <- coldata[,! colnames(coldata) %in%
    c("geo_accession", "relation", "relation.1")]
coldata <- coldata[,!grepl("^characteristics_ch", colnames(coldata))]
```

We convert all factors into character vectors, and we clean up the column names.

```{r}
for (i in colnames(coldata)) {
    if (is.factor(coldata[[i]])) {
        coldata[[i]] <- as.character(coldata[[i]])
    }
}
colnames(coldata) <- sub("[:_]ch1$", "", colnames(coldata))
coldata
```

# Saving to file 

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "marques-brain", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Zhao immune liver dataset
author: Aaron Lun
date: 19 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of liver-resident immune cells from multiple donors from @zhao2020singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE125188](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE125188).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)

mat.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE125188&format=file&file=GSE125188%5FCD45%2B%5Fmatrix%2Emtx%2Egz")
mat <- Matrix::readMM(mat.path)
mat <- as(mat, "dgCMatrix")
```

Slapping together a `SingleCellExperiment` object.

```{r}
feat.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE125188&format=file&file=GSE125188%5FCD45%2B%5Fgenes%2Etsv%2Egz")
feat.data <- read.table(feat.path, header=FALSE)
colnames(feat.data) <- c("ID", "Symbol")
rownames(feat.data) <- feat.data$ID

barcode.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE125188&format=file&file=GSE125188%5FCD45%2B%5Fbarcodes%2Etsv%2Egz")
colnames(mat) <- readLines(barcode.path)

library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=mat), rowData=feat.data)
sce
```

# Downloading the metadata

Pulling down the metadata. 
Note that not all cells are present in the metadata so some expansion is required.

```{r}
meta.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE125188&format=file&file=GSE125188%5FCD45%2B%5Fcell%5Fidentities%2Etsv%2Egz")
meta <- read.table(meta.path, header=TRUE)

m <- match(colnames(sce), meta$Barcode)
colData(sce) <- cbind(colData(sce), meta[m,])
sce$Barcode <- colnames(sce)
sce$retained <- !is.na(m)

colData(sce)
```

We also need to collate the fine metadata for the other cell types.

```{r}
pattern1 <- "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE125188&format=file&file=GSE125188%5F" 
pattern2 <- "%5Fcell%5Fidentities%2Etsv%2Egz"

broad <- fine <- rep(NA_character_, ncol(sce))
for (type in c("ASC", "B", "Mo", "NKT")) {
    sub.path <- bfcrpath(bfc, paste0(pattern1, type, pattern2))
    out <- read.delim(sub.path)
    m <- match(out$Barcode, colnames(sce))
    stopifnot(all(!is.na(m)))
    broad[m] <- type
    fine[m] <- out$Group
}

table(broad, useNA="always")
table(fine, useNA="always")

sce$broad <- broad
sce$fine <- fine
```

# Saving to file

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "zhao-immune-liver", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
saveRDS(rowData(sce), file=file.path(path, "rowdata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Zeisel nervous system dataset
author: Aaron Lun
date: 5 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the mouse nervous system from @zeisel2018molecular.
Counts for endogenous genes are available from http://mousebrain.org/downloads.html as a `loom` file.
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
loom.path <- bfcrpath(bfc, "https://storage.googleapis.com/linnarsson-lab-loom/l5_all.loom")
```

We load this into our R session using the `r Biocpkg("LoomExperiment")` package.

```{r}
library(LoomExperiment)
scle <- import(loom.path, type="SingleCellLoomExperiment")
scle 
```

Then it's just a matter of peeling apart the bits that we need.
We do need to clean up the `rowData`:

```{r}
rd <- rowData(scle)
colnames(rd) <- sub("^X_", "", colnames(rd))
rownames(rd) <- rd$Accession
rd
```

We also need to clean up the `colData`.
This is, sadly, a gargantuan effort - so much for `loom` being a ready-to-use file format!

```{r}
cd <- colData(scle)

# Useless pieces of information, or pieces that are obvious from the experimental design.
cd$Bucket <- NULL
cd$Species <- NULL
cd$AnalysisProject <- NULL
cd$Transcriptome <- NULL
cd$TimepointPool <- NULL
cd$NGI_PlateWell <- NULL
cd$PlugDate <- NULL
cd$Plug_Date <- NULL
cd$Project <- NULL

# Redundant fields.
cd$Cell_Conc <- NULL
cd$ngperul_cDNA <- NULL
cd$cDNA_Lib_Ok <- NULL
cd$Target_Num_Cells <- NULL
cd$Date_Captured <- NULL
cd$Num_Pooled_Animals <- NULL
cd$PCR_Cycles <- NULL
cd$Sample_Index <- NULL
cd$Seq_Comment <- NULL
cd$Seq_Lib_Date <- NULL
cd$Seq_Lib_Ok <- NULL

# Converting various character fields into numbers, where applicable.
# On occassion, we have to get rid of some nonsense fields with quotation marks;
# something probably got corrupted when the file was saved.
options(warn=2)
for (i in colnames(cd)) {
    current <- cd[[i]]

    if (is.character(current)) {
        converted <- current
        is.bad <- converted=="" | grepl('"', converted)
        converted[is.bad] <- NA
        changed <- TRUE

        if (any(has.pct <- grepl("%$", current))) {
            converted[!has.pct] <- NA
            converted <- sub("%$", "", converted)
            converted <- as.numeric(converted)/100
            cd[[i]] <- converted
        } else if (any(has.comma <- grepl(",[0-9]{3}", current))) {
            converted[!has.comma] <- NA # these are probably corruptions of some sort.
            converted <- gsub(",([0-9]{3})", "\\1", converted)
            converted <- as.numeric(converted)
            cd[[i]] <- converted
        } else if (any(grepl("^[0-9]+,[0-9]+$", current))){
            converted <- sub(",", ".", converted)
            converted <- as.numeric(converted)
            cd[[i]] <- converted
        } else if (any(grepl("^[0-9]+(\\.[0-9]+)?$", current))) {
            cd[[i]] <- as.numeric(converted)
        } else {
            changed <- FALSE
        }

#        # For debugging purposes, to check that the corruptions are purged correctly.
#        if (changed) {
#            print(i)
#            print(names(table(current)))
#            print(names(table(converted)))
#        }
    }
}
options(warn=1)

# Replacing the 'nan's with NA's.
for (i in colnames(cd)) {
    current <- cd[[i]]
    if (is.character(current) && any(lost <- current=="nan")) {
        current[lost] <- NA_character_
        cd[[i]] <- current
    }    
}

# Stripping out the class probabilities into a nested matrix.
clcols <- grep("ClassProbability", colnames(cd))
clprobs <- cd[,clcols]
clprobs <- as.matrix(clprobs)
colnames(clprobs) <- sub("ClassProbability_", "", colnames(clprobs))
cd <- cd[,-clcols]
cd$ClassProbability <- clprobs

# Hacking out the reduced dimensions.
pca.cols <- c("X_PC1", "X_PC2")
tsne.cols <- c("X_tSNE1", "X_tSNE2")
other.cols <- c("X_X", "X_Y")
reddim <- list(
    PCA=as.matrix(cd[,pca.cols]),
    tSNE=as.matrix(cd[,tsne.cols]),
    unnamed=as.matrix(cd[,other.cols])
)
cd <- cd[,!colnames(cd) %in% c(pca.cols, tsne.cols, other.cols)]

# Why the prefixing with X_? Well, whatever.
colnames(cd) <- sub("X_", "", colnames(cd))

cd
```

# Saving to file

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "zeisel-nervous", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(rd, file=file.path(path, "rowdata.rds"))
saveRDS(cd, file=file.path(path, "coldata.rds"))
saveRDS(reddim, file=file.path(path, "reddims.rds"))
```

The column pairs are a bit tricky, but we just coerce them into `SelfHits` objects and save that.

```{r}
colpairs <- lapply(colGraphs(scle), function(x) as(x, "SelfHits"))
saveRDS(colpairs, file=file.path(path, "colpairs.rds"))
```

The trickiest part is resaving the HDF5 file as a sparse matrix.

```{r}
saveRDS(as(assay(scle), 'dgCMatrix'), file=file.path(path, "counts.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Leng ESC dataset
author: Aaron Lun
date: "October 13, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of mouse ESCs from @leng2015oscope.

```{r}
library(GEOquery)
out <- getGEOSuppFiles("GSE64016", baseDir=tempdir())
rownames(out)
```

# Processing the read counts

We read in the count matrix for cells in each cell cycle phase.

```{r}
count.file <- rownames(out)[1]
hs.counts <- read.csv(count.file, header=TRUE, row.names=1)
hs.counts <- as.matrix(hs.counts)
dim(hs.counts)
```

# Creating column metadata

Pulling information out of GSE64016's description.

```{r}
cellline <- rep(c("H1", "H1-Fucci"), c(213, 247))
experiment <- sub(".*_Exp([0-9]+)\\..*", "\\1", colnames(hs.counts))
phase <- sub("_Exp.*", "", colnames(hs.counts))
phase[!phase %in% c("G1", "S", "G2M")] <- NA

library(S4Vectors)
coldata <- DataFrame(CellLine=cellline, Experiment=experiment, Phase=phase)
coldata
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
repath <- file.path("scRNAseq", "leng-esc", "2.0.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)
saveRDS(hs.counts, file=file.path(repath, "normcounts.rds"))
saveRDS(coldata, file=file.path(repath, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Bach mammary gland dataset
author: Aaron Lun
date: "September 6, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Specifying the samples

We obtain a single-cell RNA sequencing dataset of the mouse mammary gland from @bach2017differentiation.
we define all of the samples to be pulled down.

```{r}
accessions <- c(
    "GSM2834498", "GSM2834499", 
    "GSM2834500", "GSM2834501", 
    "GSM2834502", "GSM2834503", 
    "GSM2834504", "GSM2834505")

samples <- c(
    "NP_1", "NP_2",
    "G_1", "G_2",
    "L_1", "L_2",
    "PI_1", "PI_2")

conditions <- c(NP="Nulliparous",
    G="Gestation",
    L="Lactation",
    PI="Post-involution")[sub("_.*", "", samples)]

data.frame(accessions, samples, conditions)
```

# Downloading the count data

We then download and cache the assorted count matrices using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)

library(Matrix)
library(S4Vectors)
template.path <- "ftp://ftp.ncbi.nlm.nih.gov/geo/samples/%snnn/%s/suppl"
collected.counts <- collected.rowdata <- collected.coldata <- list()
for (i in seq_along(accessions)) {
    curacc <- accessions[i]
    subacc <- substr(curacc, 1, 7)
    base.path <- sprintf(template.path, subacc, curacc)
    header <- paste0(curacc, "%5F", sub("_", "%5F", samples[i]), "%5F")

    barcode.fname <- bfcrpath(bfc, file.path(base.path,
        paste0(header, "barcodes%2Etsv%2Egz")))
    gene.fname <- bfcrpath(bfc, file.path(base.path,
        paste0(header, "genes%2Etsv%2Egz")))
    counts.fname <- bfcrpath(bfc, file.path(base.path,
        paste0(header, "matrix%2Emtx%2Egz")))

    collected.counts[[i]] <- as(readMM(counts.fname), "dgCMatrix")
    gene.info <- read.table(gene.fname, stringsAsFactors=FALSE)
    colnames(gene.info) <- c("Ensembl", "Symbol")
    collected.rowdata[[i]] <- DataFrame(gene.info)
    collected.coldata[[i]] <- DataFrame(
        Barcode=readLines(barcode.fname),
        Sample=samples[i],
        Condition=conditions[i])
}
```

We verify that all of the row data matches up, and that all the dimensions are consistent.

```{r}
stopifnot(length(unique(collected.rowdata))==1L)

X <- vapply(collected.coldata, nrow, 0L)
Y <- vapply(collected.counts, ncol, 0L)
stopifnot(identical(X, Y))
X
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "bach-mammary", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(collected.rowdata[[1]], file=file.path(path, "rowdata.rds"))
for (i in seq_along(samples)) {
    saveRDS(collected.counts[[i]], file=file.path(path, sprintf("counts-%s.rds", samples[i])))
    saveRDS(collected.coldata[[i]], file=file.path(path, sprintf("coldata-%s.rds", samples[i])))
}
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the La Manno brain dataset
author: Aaron Lun
date: "June 8, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of the mouse and human midbrains from @lamanno2016molecular.
Counts for cells from various developmental stages in both species are available from the Gene Expression Omnibus 
using the accession number [GSE76381](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE76381).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE76nnn/GSE76381/suppl")

es.count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE76381_ESMoleculeCounts.cef.txt.gz"))
embryo.count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE76381_EmbryoMoleculeCounts.cef.txt.gz"))
ips.count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE76381_iPSMoleculeCounts.cef.txt.gz"))

madult.count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE76381_MouseAdultDAMoleculeCounts.cef.txt.gz"))
membryo.count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE76381_MouseEmbryoMoleculeCounts.cef.txt.gz"))
```

# Reading the data in

We create a function to extract data from each file.

```{r}
library(S4Vectors)
FUN <- function(path, as.csv=FALSE, skip=0) {
    if (as.csv) {
        FUN <- read.csv
    } else {
        FUN <- read.delim
    }

    x <- FUN(path, header=FALSE, stringsAsFactors=FALSE, skip=skip)
    is.gene <- which(x[,1]=="Gene")
    
    metadata <- t(x[2:(is.gene-1L),-(1:2)])
    df <- data.frame(metadata, stringsAsFactors=FALSE)
    df <- DataFrame(df)
    colnames(df) <- x[2:(is.gene-1L),2]
    rownames(df) <- NULL

    counts <- as.matrix(x[-(1:(is.gene+1L)),])
    rownames(counts) <- counts[,1]
    colnames(counts) <- NULL
    
    # checking that second column has nothing interesting.
    stopifnot(length(unique(counts[,2]))==1L) 
    counts <- counts[,-(1:2)]
    storage.mode(counts) <- "integer"

    list(counts=counts, coldata=df)
}    
```

We run this on all the human datasets:

```{r}
es.data <- FUN(es.count.file)
dim(es.data$counts)
es.data$coldata
embryo.data <- FUN(embryo.count.file)
dim(embryo.data$counts)
embryo.data$coldata
ips.data <- FUN(ips.count.file)
dim(ips.data$counts)
ips.data$coldata
```

We repeat the process for the mouse data.

```{r}
madult.data <- FUN(madult.count.file, as.csv=TRUE, skip=1)
dim(madult.data$counts)
madult.data$coldata
membryo.data <- FUN(membryo.count.file, skip=1)
dim(membryo.data$counts)
membryo.data$coldata
```

# Saving objects

Rather frustratingly, each of the stages has a different set of genes, so we need to save them separately.
We set up a simple function do to so:

```{r}
path <- file.path("scRNAseq", "lamanno-brain", "2.0.0")
SAVEFUN <- function(input, suffix) {
    dir.create(path, showWarnings=FALSE, recursive=TRUE)
    saveRDS(input$counts, file=file.path(path, 
        sprintf("counts-%s.rds", suffix)))
    saveRDS(input$coldata, file=file.path(path, 
        sprintf("coldata-%s.rds", suffix)))
}
```

We save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
SAVEFUN(es.data, "human-es")
SAVEFUN(embryo.data, "human-embryo")
SAVEFUN(ips.data, "human-ips")
SAVEFUN(madult.data, "mouse-adult")
SAVEFUN(membryo.data, "mouse-embryo")
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Bunis HSPC dataset
author: Aaron Lun and Daniel Bunis
date: 21 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of haematopoietic stem-progenitor cells from @bunis2021singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE158490](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE158490).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
mat.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE158490&format=file&file=GSE158490%5Fmatrix%2Emtx%2Egz")
mat <- Matrix::readMM(mat.path)
mat <- as(mat, "dgCMatrix")
dim(mat)
```

Creating a `SingleCellExperiment` object:

```{r}
library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=mat))

cd.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE158490&format=file&file=GSE158490%5Fbarcodes%2Etsv%2Egz")
colnames(sce) <- readLines(cd.path)

rd.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE158490&format=file&file=GSE158490%5Fgenes%2Etsv%2Egz")
rd <- read.table(rd.path)
colnames(rd) <- c("ID", "Symbol")
rowData(sce) <- rd
rownames(sce) <- rowData(sce)$ID

sce
```

# Pulling down the metadata

Attaching some metadata.
The `SNG.1ST` column specifies the sample for each cell, ranging from adult bone marrow (`APB`), fetal bone marrow (`FS`) and newborn umbilical cord (`UCB`).
Various other _demuxlet_ statistics are also reported here.
We won't expand this out to all 720k columns of `mat` to save some space, given that most downstream applications will only care about the observed cells.

We can import this data into our `sce`, using the `importDemux()` function of `r Biocpkg("dittoSeq")` to retain selected metadata.
(Indeed, yes it is an odd placement for this function which should be split off into some other demultiplexing-focused package.)

We can then convert the ABM/UCB/FBM of Samples into age groups of samples.

```{r}
demux.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE158490&format=file&file=GSE158490%5FHSPC%2Ebest%2Etxt%2Egz")
demux <- read.delim(demux.path, check.names=FALSE)
stopifnot(all(demux$BARCODE %in% colnames(sce)))
dim(demux)

sce <- dittoSeq::importDemux(sce, demuxlet.best = demux, verbose = FALSE)

# Remove the unnecessary (single-value here) Lane metadata
sce$Lane <- NULL

# Remove the "CD4_" from start of sample names, and replace "APB" and "FS"
# with "ABM" and "FBM" which reflect the cells put through bulk RNAseq to
# generate genotyping for Demuxlet, rather than these cells.
sce$Sample <- gsub("^CD4_", "", sce$Sample)
sce$Sample <- gsub("^FS", "FBM", sce$Sample)
sce$Sample <- gsub("^APB", "ABM", sce$Sample)

# Add age metadata
sce$age <- NA
sce$age[grep("^F", sce$Sample)] <- "fetal"
sce$age[grep("^U", sce$Sample)] <- "newborn"
sce$age[grep("^A", sce$Sample)] <- "adult"

colData(sce)
```

# Adding additional metadata

From the fully processed objects shared on figshare, we will obtain cell type annotations and developmental stage scores.

```{r}
full.path <- bfcrpath(bfc,
    "https://ndownloader.figshare.com/files/25953740")
raw_meta <- readRDS(full.path)@meta.data
stopifnot(all(rownames(raw_meta) %in% colnames(sce)))
dim(raw_meta)

# Add 'retained' metadata representing barcodes retained by authors.
sce$retained <- colnames(sce) %in% rownames(raw_meta)
summary(sce$retained)

# Add celltypes
authors_meta <- raw_meta[colnames(sce),]
rownames(authors_meta) <- colnames(sce)
sce$labels <- authors_meta$trajectory_calls
table(sce$labels)

# Add Developmental Stage Scoring as a DataFrame
# Retrieve and rename a bit better
DevStageScores <- DataFrame(
  HSCMPP_scores = authors_meta$MPP.RFScore,
  HSCMPP_inTraining = authors_meta$MPP.inTraining,
  GMP_scores = authors_meta$GMP.RFScore,
  GMP_inTraining = authors_meta$GMP.inTraining,
  MEP_scores = authors_meta$MEP.RFScore,
  MEP_inTraining = authors_meta$MEP.inTraining,
  row.names = rownames(authors_meta)
)
sce$DevStageScoring <- DevStageScores
sce$DevStageScoring[sce$retained,]
```

To save space, we only save the `colData` for non-`NA` rows.
We will fill these in later.

```{r}
keep <- colnames(sce) %in% c(demux$BARCODE, rownames(raw_meta))
summary(keep)
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "bunis-hspc", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce)[keep,], file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Nestorowa HSC dataset
author: Aaron Lun
date: 18 May 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the mouse haematopoietic stem cells from @paul2015transcriptional.
Counts for endogenous genes and spike-in transcripts are available from the Gene Expression Omnibus
using the accession number [GSE72857](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE72857).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo",
    "series/GSE72nnn/GSE72857/suppl")
fname.A <- bfcrpath(bfc, file.path(base.url, "GSE72857_umitab.txt.gz"))
```

We read this into memory as a sparse matrix.

```{r}
library(scater)
counts <- readSparseCounts(fname.A, quote='"')
dim(counts)
```

# Downloading the metadata

We pull down the metadata from GEO as well. 

```{r}
meta.A <- bfcrpath(bfc, file.path(base.url, 
    "GSE72857_experimental_design.txt.gz"))
meta <- read.delim(meta.A, skip=19, header=TRUE, stringsAsFactors=FALSE)
meta <- DataFrame(meta)
meta
```

We check that the cell names match up with the matrix.

```{r}
m <- match(colnames(counts), meta$Well_ID)
stopifnot(all(!is.na(m)))
meta <- meta[m,]
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "paul-hsc", "2.2.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(meta, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Aztekin Xenopus tail dataset
author: Aaron Lun
date: "September 6, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of Xenopus tail cells from @aztekin2019identification.
We download and cache the count matrix and metadata using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
contents <- bfcrpath(bfc,
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-7716/E-MTAB-7716.processed.1.zip"))

unzip(contents, exdir=tempdir())
unzip(file.path(tempdir(), "arrayExpressUpload.zip"), exdir=tempdir())
unzip(file.path(tempdir(), "ArrayExpressV2.zip"), exdir=tempdir())
```

# Processing the read counts

We load the counts into memory.
Despite the name of the file, it is not actually comma-separated!

```{r}
library(Matrix)
path <- file.path(tempdir(), "ArrayExpress")
counts <- readMM(file.path(path, "countsMatrix.mtx"))
counts <- as(counts, "dgCMatrix")
rownames(counts) <- read.csv(file.path(path, "genes.csv"), 
    stringsAsFactors=FALSE, header=FALSE)[,1]
colnames(counts) <- read.csv(file.path(path, "cells.csv"), 
    stringsAsFactors=FALSE, header=FALSE)[,1]
dim(counts)
```

# Processing the metadata

Processing the per-cell metadata, and merging in the per-sample metadata along with it.

```{r}
library(S4Vectors)
meta <- read.csv(file.path(path, "meta.csv"), stringsAsFactors=FALSE)
labels <- read.csv(file.path(path, "labels.csv"), stringsAsFactors=FALSE)
meta <- cbind(meta, labels[match(meta$sample, labels$Sample),-1])
meta <- DataFrame(meta)
rownames(meta) <- NULL
meta
```

Now enforcing consistency checks with the column names of `counts`.

```{r}
stopifnot(identical(colnames(counts), meta$cell))
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
repath <- file.path("scRNAseq", "aztekin-tail", "2.0.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(repath, "counts.rds"))
saveRDS(meta, file=file.path(repath, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Ledergor Myeloma data
author: Milan Malfait
date: 6 July 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a MARS-seq single-cell RNA sequencing dataset of human bone marrow
plasma cells and circulating plasma cells from @ledergor2018single.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE117156](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE117156).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
mat.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE117156&format=file"
)
```

We read this into memory as a sparse matrix.

Each matrix corresponds to a 384-well plate from the MARS-seq sequencing.
Rows correspond to the features (identical across all matrices), columns
correspond to the wells. To obtain the full count matrix, we `cbind` the
matrices together.

```{r}
tmp <- tempfile()
untar(mat.path, exdir=tmp)
all.files <- list.files(tmp)

library(BiocParallel)
library(scuttle)
all.counts <- bplapply(file.path(tmp, all.files), readSparseCounts)
names(all.counts) <- sub(".txt.gz$", "", all.files)
do.call(rbind, lapply(all.counts, dim))

counts <- do.call(cbind, all.counts)
dim(counts)
```

Creating a `SingleCellExperiment` object:

```{r}
library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=counts))
```

# Adding the metadata

```{r}
meta.path <- bfcrpath(bfc,
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE117156&format=file&file=GSE117156%5Fmetadata%2Etxt%2Egz"
)
meta <- read.delim(meta.path)
meta <- DataFrame(meta, check.names=FALSE)
meta

meta$X <- NULL
rownames(meta) <- meta$well
```

We check that the cell names match up with the matrix.

```{r}
m <- match(colnames(counts), rownames(meta))
stopifnot(all(!is.na(m)))
meta <- meta[m,]
```

The `Experiment_ID` column encodes information on the subject, tissue and
treatment status. We'll extract this into separate columns for easier access.

```{r}
splt <- strsplit(meta$Experiment_ID, split = "_")

meta$Subject_ID <- vapply(splt, `[[`, character(1), 2)
meta$Condition <- sub("\\d+$", "", meta$Subject_ID)
meta$Condition[which(meta$Condition == "hip")] <- "Control"

## Treated IDs have 4 components, with `postRx` between subject ID and tissue
meta$Tissue <- vapply(splt, function(x) x[[length(x)]], character(1))
meta$Tissue <- sub("#\\d$", "", meta$Tissue)

meta$Treated <- grepl("postRx", meta$Experiment_ID)

meta
```

```{r}
colData(sce) <- meta
sce
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "ledergor-myeloma", "2.8.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Pollen radial glia dataset
author: Aaron Lun
date: 11 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of human radial glial cells from @pollen2015molecular.
Counts for endogenous genes are available from Dropbox (why would you do that!?) via https://www.pollenlab.org/datasets.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
exprs.path <- bfcrpath(bfc, "https://www.dropbox.com/s/rjrkq96li4j4rvn/oRG%20paper%20-%20counts.txt?dl=1")

library(scuttle)
mat <- readSparseCounts(exprs.path, row.names=1)
dim(mat)
str(dimnames(mat))
```

We also read in the per-cell metadata from an Excel file (why!?).

```{r}
meta.path <- bfcrpath(bfc, "https://www.dropbox.com/s/rb9tl4gjswrxfy9/Pollen%20et%20al%202015%20updated%20metadata.xlsx?dl=1")

library(readxl)
meta <- DataFrame(read_excel(meta.path), check.names=FALSE)
meta$`AlignmentRate, Pairs` <- as.numeric(sub("%$", "", meta$`AlignmentRate, Pairs`))/100
rownames(meta) <- meta$Cell
stopifnot(all(colnames(mat) %in% sort(meta$Cell)))

meta <- meta[colnames(mat),]
meta
```

Making sure we can assemble the final SCE:

```{r}
sce <- SingleCellExperiment(list(counts=mat), colData=meta)
sce 
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "pollen-glia", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(mat, file=file.path(path, "counts.rds"))
saveRDS(meta, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Campbell brain dataset
author: Aaron Lun
date: "June 24, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of the mouse brain from @campbell2017molecular.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE93374](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE93374).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE93nnn/GSE93374/suppl")
count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE93374_Merged_all_020816_DGE.txt.gz"))
```

Reading them in as a sparse matrix.

```{r}
library(scater)
counts <- readSparseCounts(count.file)
dim(counts)
```

# Downloading the metadata

We also download the cluster identities.

```{r}
cluster.file <- bfcrpath(bfc, file.path(base.url,
    "GSE93374_cell_metadata.txt.gz"))
colClasses <- vector("list", 15)
colClasses[1:11] <- "character"
coldata <- read.delim(cluster.file, stringsAsFactors=FALSE, 
    colClasses=colClasses)
coldata <- as(coldata, "DataFrame")
colnames(coldata) <- sub("X[0-9]+\\.", "", colnames(coldata))
coldata
```

We check that the columns are in the same order.

```{r}
m <- match(colnames(counts), coldata$ID)
coldata <- coldata[m,]
stopifnot(identical(colnames(counts), coldata$ID))
```

# Saving to file 

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "campbell-brain", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: "Overview of the scRNAseq dataset collection"
author: 
- name: Davide Risso
  affiliation: Division of Biostatistics and Epidemiology, Weill Cornell Medicine
- name: Aaron Lun
  email: infinite.monkeys.with.keyboards@gmail.com
date: "Created: May 25, 2016; Compiled: `r Sys.Date()`"
output:
  BiocStyle::html_document:
    toc: true
package: scRNAseq
vignette: >
  %\VignetteIndexEntry{User's Guide}
  %\VignetteEngine{knitr::rmarkdown}
---

```{r style, echo=FALSE}
knitr::opts_chunk$set(error=FALSE, warning=FALSE, message=FALSE)
```

# Introduction

The `r Biocpkg("scRNAseq")` package provides convenient access to several publicly available single-cell datasets in the form of `SingleCellExperiment` objects.
We do all of the necessary data munging for each dataset beforehand, so that users can obtain a `SingleCellExperiment` for immediate use in further analyses.
To enable discovery, each dataset is decorated with metadata such as the study title/abstract, the species involved, the number of cells, etc.
Users can also contribute their own published datasets to enable re-use by the wider Bioconductor community. 

# Finding datasets 

The `surveyDatasets()` function will show all available datasets along with their metadata.
This can be used to discover interesting datasets for further analysis.

```{r}
library(scRNAseq)
all.ds <- surveyDatasets()
all.ds
```

Users can also search on the metadata text using the `searchDatasets()` function.
This accepts both simple text queries as well as more complicated expressions involving boolean operations.

```{r}
# Find all datasets involving pancreas.
searchDatasets("pancreas")[,c("name", "title")]

# Find all mm10 datasets involving pancreas or neurons.
searchDatasets(
   defineTextQuery("GRCm38", field="genome") &
   (defineTextQuery("neuro%", partial=TRUE) | 
    defineTextQuery("pancrea%", partial=TRUE))
)[,c("name", "title")]
```

Keep in mind that the search results are not guaranteed to be reproducible -
more datasets may be added over time, and existing datasets may be updated with new versions.
Once a dataset of interest is identified, users should explicitly list the name and version of the dataset in their scripts to ensure reproducibility.

# Loading a dataset

The `fetchDataset()` function will download a particular dataset, returning it as a `SingleCellExperiment`:

```{r}
sce <- fetchDataset("zeisel-brain-2015", "2023-12-14")
sce
```

For studies that generate multiple datasets, the dataset of interest must be explicitly requested via the `path=` argument:

```{r}
sce <- fetchDataset("baron-pancreas-2016", "2023-12-14", path="human")
sce
```

By default, array data is loaded as a file-backed `DelayedArray` from the `r Biocpkg("HDF5Array")` package.
Setting `realize.assays=TRUE` and/or `realize.reduced.dims=TRUE` will coerce these to more conventional in-memory representations like ordinary arrays or `dgCMatrix` objects.

```{r}
assay(sce)
sce <- fetchDataset("baron-pancreas-2016", "2023-12-14", path="human", realize.assays=TRUE)
class(assay(sce))
```

Users can also fetch the metadata associated with each dataset:

```{r}
str(fetchMetadata("zeisel-brain-2015", "2023-12-14"))
```

# Adding new datasets

Want to contribute your own dataset to this package?
It's easy!
Just follow these simple steps for instant fame and prestige. 

1. Format your dataset as a `SummarizedExperiment` or `SingleCellExperiment`.
   Let's just make up something here.

    ```{r}
    library(SingleCellExperiment)
    sce <- SingleCellExperiment(list(counts=matrix(rpois(1000, lambda=1), 100, 10)))
    rownames(sce) <- sprintf("GENE_%i", seq_len(nrow(sce)))
    colnames(sce) <- head(LETTERS, 10)
    ```

2. Assemble the metadata for your dataset.
   This should be a list structured as specified in the [Bioconductor metadata schema](https://artifactdb.github.io/bioconductor-metadata-index/bioconductor/v1.json)
   Check out some examples from `fetchMetadata()` - note that the `application.takane` property will be automatically added later, and so can be omitted from the list that you create.

    ```{r}
    meta <- list(
        title="My dataset",
        description="This is my dataset",
        taxonomy_id="10090",
        genome="GRCh38",
        sources=list(
            list(provider="GEO", id="GSE12345"),
            list(provider="PubMed", id="1234567")
        ),
        maintainer_name="Chihaya Kisaragi",
        maintainer_email="kisaragi.chihaya@765pro.com"
    )
    ```

3. Save your `SummarizedExperiment` (or whatever object contains your dataset) to disk with `saveDataset()`.
   This saves the dataset into a "staging directory" using language-agnostic file formats - check out the [**alabaster**](https://github.com/ArtifactDB/alabaster.base) framework for more details.
   In more complex cases involving multiple datasets, users may save each dataset into a subdirectory of the staging directory.

    ```{r}
    # Simple case: you only have one dataset to upload.
    staging <- tempfile()
    saveDataset(sce, staging, meta)
    list.files(staging, recursive=TRUE)
    
    # Complex case: you have multiple datasets to upload.
    staging <- tempfile()
    dir.create(staging)
    saveDataset(sce, file.path(staging, "foo"), meta)
    saveDataset(sce, file.path(staging, "bar"), meta) # etc.
    ```

   You can check that everything was correctly saved by reloading the on-disk data into the R session for inspection:

    ```{r}
    alabaster.base::readObject(file.path(staging, "foo"))
    ```

4. Open a [pull request (PR)](https://github.com/LTLA/scRNAseq/pulls) for the addition of a new dataset.
   You will need to provide a few things here:
   - The name of your dataset.
     This typically follows the format of `{NAME}-{SYSTEM}-{YEAR}`, where `NAME` is the last name of the first author of the study,
     `SYSTEM` is the biological system (e.g., tissue, cell types) being studied,
     and `YEAR` is the year of publication for the dataset.
   - The version of your dataset.
     This is usually just the current date... or whenever you started putting together the dataset for upload.
     The exact date doesn't really matter as long as we can establish a timeline for later versions.
   - An Rmarkdown file containing the code used to assemble the dataset.
     This should be added to the [`scripts/`](https://github.com/LTLA/scRNAseq/tree/master/scripts) directory of this package,
     in order to provide some record of how the dataset was created.

5. Wait for us to grant temporary upload permissions to your GitHub account.
6. Upload your staging directory to [**gypsum** backend](https://github.com/ArtifactDB/gypsum-worker) with `gypsum::uploadDirectory()`.
   On the first call to this function, it will automatically prompt you to log into GitHub so that the backend can authenticate you.
   If you are on a system without browser access (e.g., most computing clusters), a [token](https://github.com/settings/tokens) can be manually supplied via `gypsum::setAccessToken()`.

    ```{r, eval=FALSE}
    gypsum::uploadDirectory(staging, "scRNAseq", "my_dataset_name", "my_version")
    ```

   You can check that everything was successfully uploaded by calling `fetchDataset()` with the same name and version:

    ```{r, eval=FALSE}
    fetchDataset("my_dataset_name", "my_version")
    ```

   If you realized you made a mistake, no worries.
   Use the following call to clear the erroneous dataset, and try again:

    ```{r, eval=FALSE}
    gypsum::rejectProbation("scRNAseq", "my_dataset_name", "my_version")
    ```

7. Comment on the PR to notify us that the dataset has finished uploading and you're happy with it.
   We'll review it and make sure everything's in order.
   If some fixes are required, we'll just clear the dataset so that you can upload a new version with the necessary changes.
   Otherwise, we'll approve the dataset.
   Note that once a version of a dataset is approved, no further changes can be made to that version;
   you'll have to upload a new version if you want to modify something.

# Session information {-}

```{r}
sessionInfo()
```

---
title: Processing the Richard CD8^+^ T cell dataset
author: Aaron Lun
date: "September 6, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of CD8^+^ T cells from @richard2018tcell.
Counts for endogenous genes and spike-in transcripts are available from ArrayExpress
using the accession number [E-MTAB-6051](https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6051/).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
richard.zip <- bfcrpath(bfc, 
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-6051/E-MTAB-6051.processed.1.zip"))
unzip(richard.zip, exdir=tempdir())
```

# Reading in the metadata

We read in the metadata from the SDRF file:

```{r}
richard.sdrf <- bfcrpath(bfc, 
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-6051/E-MTAB-6051.sdrf.txt"))
coldata <- read.delim(richard.sdrf, check.names=FALSE, stringsAsFactors=FALSE)
libnames <- coldata[["Assay Name"]]

library(S4Vectors)
coldata <- as(coldata, "DataFrame")
colnames(coldata)
```

We keep only the experimentally interesting metadata, discarding columns that are duplicated or only have one level.
We also discard some ArrayExpress-specific columns.

```{r}
keep <- grep("(Characteristics|Factor|Parameter Value|Comment)", colnames(coldata))
coldata <- coldata[,keep] 

new.colnames <- sub(".*\\[(.*)\\]", "\\1", colnames(coldata))
u <- !duplicated(new.colnames)
coldata <- coldata[,u]
colnames(coldata) <- new.colnames[u]

has.multi.levels <- vapply(coldata, FUN=function(x) length(unique(x))>1L, TRUE)
coldata <- coldata[,has.multi.levels]
coldata <- coldata[,setdiff(colnames(coldata), c("ENA_SAMPLE", 
    "BioSD_SAMPLE", "technical replicate group", "ENA_EXPERIMENT", 
    "SUBMITTED_FILE_NAME", "ENA_RUN", "FASTQ_URI",
    "single cell identifier"))]
coldata
```

We convert all of the FACS intensity measurements to numeric values.

```{r}
for (i in grep("^CD[0-9]+", colnames(coldata))) {
    suppressWarnings(coldata[[i]] <- as.numeric(coldata[[i]]))
}
```

Many of these are actually technical replicates, so we only need to keep the first one of each pair.

```{r}
libnames <- sub("(_[iS][0-9]{3}\\.).*", "\\1", libnames)
first <- !duplicated(libnames)
last <- !duplicated(libnames, fromLast=TRUE)
stopifnot(identical(coldata[first,], coldata[last,]))

coldata <- coldata[first,]
libnames <- libnames[first]
dim(coldata)
```

# Processing the read counts

We load the counts into memory.

```{r}
batch1 <- read.delim(file.path(tempdir(), 
    "counts_table_sortdate20160727.txt"),
    header=TRUE, row.names=1, check.names=FALSE)
batch2 <- read.delim(file.path(tempdir(),
    "counts_table_sortdate20161026.txt"),
    header=TRUE, row.names=1, check.names=FALSE)
stopifnot(identical(rownames(batch1), rownames(batch2)))
```

We combine the matrices together, and make sure they match up with the `coldata` order.

```{r}
combined <- cbind(batch1, batch2)
combined <- as.matrix(combined)
stopifnot(identical(sort(colnames(combined)), sort(libnames)))
m <- match(libnames, colnames(combined))
combined <- combined[,m]
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "richard-tcell", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(combined, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the He organ atlas dataset
author: Aaron Lun
date: 22 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of human organs from @he2020singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE159929](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE159929).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
mat.path <- bfcrpath(bfc, "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE159929&format=file")
```

We load this into memory.
Sadly, each matrix does not contain the same set of features.
This is because they filtered out low-expressing genes - it would not be correct to zero-fill.

```{r}
tmp <- tempfile()
untar(mat.path, exdir=tmp)
all.files <- list.files(tmp)

library(BiocParallel)
library(scuttle)
all.counts <- bplapply(file.path(tmp, all.files), readSparseCounts, sep=",", row.names=1, quote='"')
names(all.counts) <- sub(".*_([^_]+)_Counts.csv.gz", "\\1", all.files)
do.call(rbind, lapply(all.counts, dim))
```

# Downloading the metadata

We obtain the metadata from the GitHub repository accompanying the study 
(see [here](https://github.com/bei-lab/scRNA-AHCA/tree/master/Cell_barcode_and_corresponding_cell_types_of_AHCA)).

```{r}
meta.path <- bfcrpath(bfc,
    "https://github.com/bei-lab/scRNA-AHCA/raw/master/Cell_barcode_and_corresponding_cell_types_of_AHCA/Annotation_AHCA_alltissues_meta.data_84363_cell.txt")
meta <- read.delim(meta.path)
meta <- DataFrame(meta, check.names=FALSE)
meta

rownames(meta) <- meta$X
meta <- meta[,-1]
colnames(meta)[1] <- "Tissue"
meta$Tissue <- sub("_cDNA", "", meta$Tissue)

ref.names <- lapply(all.counts, colnames)
ref.names <- paste0(
    rep(names(all.counts), vapply(all.counts, ncol, 0L)),
    "_cDNA_", 
    unlist(ref.names)
)
stopifnot(identical(sort(ref.names), sort(rownames(meta))))

# Don't need this crap.
meta$Color_of_tissues <- NULL

reddims <- list(tSNE=cbind(meta$tSNE_1, meta$tSNE_2))
rownames(reddims[[1]]) <- rownames(meta)
meta <- meta[,setdiff(colnames(meta), c("tSNE_1", "tSNE_2"))]
```

We also obtain finer cell types from reclustering.

```{r}
fine.anno <- broad.anno <- rep(NA_character_, nrow(meta))

for (sub in c("B_and_plasma.meta.data.txt", "CD4_meta.data.txt", "CD8_meta.data.txt",
    "Endothelial_cell.meta.data.txt", "Epithelial_cells.meta.data.txt", 
    "FibSmo.meta.data.txt", "Myeloid.meta.data.txt")) 
{
    sub.path <- bfcrpath(bfc,
        file.path("https://github.com/bei-lab/scRNA-AHCA/raw/master/Cell_barcode_and_corresponding_cell_types_of_AHCA", sub))
    sub.df <- read.delim(sub.path)

    m <- match(sub.df$X, rownames(meta))
    stopifnot(all(is.na(fine.anno[m])))
    stopifnot(all(is.na(broad.anno[m])))
    broad.anno[m] <- sub(".meta\\.data.txt", "", sub)
    fine.anno[m] <- sub.df$annotation
}

meta$reclustered.broad <- broad.anno
meta$reclustered.fine <- fine.anno
```

# Saving to file

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "he-organ-atlas", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
for (tissue in names(all.counts)) {
    current <- all.counts[[tissue]]
    colnames(current) <- paste0(tissue, "_cDNA_", colnames(current))
    saveRDS(current, file=file.path(path, paste0("counts-", tissue, ".rds")))

    m <- match(colnames(current), rownames(meta))
    stopifnot(all(!is.na(m)))
    stopifnot(all(meta$Tissue[m]==tissue))
    saveRDS(meta[m,], file=file.path(path, paste0("coldata-", tissue, ".rds")))
}
saveRDS(reddims, file=file.path(path, "reddim.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Tasic brain dataset
author: Aaron Lun
date: "June 8, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of the mouse brain from @tasic2016adult. 
Counts for endogenous genes and spike-in transcripts are available from the Gene Expression Omnibus
using the accession number [GSE71585](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE71585).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE71nnn/GSE71585/suppl")
count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE71585_RefSeq_counts.csv.gz"))
spike.file <- bfcrpath(bfc, file.path(base.url,
    "GSE71585_ERCC_and_tdTomato_counts.csv.gz"))
```

We load them into memory.

```{r}
count.mat <- read.csv(count.file, row.names=1, header=TRUE, check.names=FALSE)
count.mat <- as.matrix(count.mat)
dim(count.mat)
spike.mat <- read.csv(spike.file, row.names=1, header=TRUE, check.names=FALSE)
spike.mat <- as.matrix(spike.mat)
dim(spike.mat)
```

We check that all objects are in the same order and combine them.

```{r}
stopifnot(identical(colnames(count.mat), colnames(spike.mat)))
counts <- rbind(count.mat, spike.mat)
```

# Downloading the per-cell metadata

We also download a file containing the metadata for each cell.

```{r}
meta.file <- bfcrpath(bfc, file.path(base.url,
    "GSE71585_Clustering_Results.csv.gz"))
metadata <- read.csv(meta.file, stringsAsFactors=FALSE)
nrow(metadata)
head(metadata)
```

Some clean-up is necessary to replace `"N/A"` with actual `NA_character_` entries, 
which are more appropriate for conveying missingness.

```{r}
for (i in colnames(metadata)) {
    current <- metadata[,i]
    to.rename <- current %in% c("N/A")
    current[to.rename] <- NA
    metadata[,i] <- current
}
```

We check that all objects are in the same order, and use this to create a column-level `DataFrame`.

```{r}
m <- match(colnames(counts), metadata$sample_title)
stopifnot(all(!is.na(m)))
metadata <- metadata[m,]
library(S4Vectors)
coldata <- as(metadata, "DataFrame")
```

# Saving to file 

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "tasic-brain", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Darmanis brain dataset
author: Aaron Lun
date: 7 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of human cerebral cortex cells from @darmanis2015survey.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE67835](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE67835).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
tarball <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE67835&format=file")

fname <- tempfile()
untar(tarball, exdir=fname)
all.files <- list.files(fname, full=TRUE)
length(all.files)
```

Reading all the counts in as sparse matrices:

```{r}
library(BiocParallel)
counts <- bplapply(all.files, read.table, sep="\t", header=FALSE, row.names=1,
    BPPARAM=MulticoreParam())

# Sanity check:
gene.ids <- lapply(counts, rownames)
stopifnot(length(unique(gene.ids))==1L)

combined <- as.matrix(do.call(cbind, counts))
rownames(combined) <- sub(" +$", "", rownames(combined))
colnames(combined) <- sub("_.*", "\\1", basename(all.files))
str(combined)
```

We pull down some metadata in SOFT format.
This requires some cleaning to get rid of the useless bits of information.

```{r}
library(GEOquery)
out <- GEOquery::getGEO("GSE67835")

df <- rbind(
    as(phenoData(out[[1]]), "data.frame"),
    as(phenoData(out[[2]]), "data.frame")
) 
stopifnot(anyDuplicated(rownames(df))==0)
stopifnot(identical(sort(rownames(df)), sort(colnames(combined))))

# Getting rid of fields that are either too specific or too general.
keep <- vapply(df, function(x) {
    n <- length(unique(x))
    n > 1 & n < length(x)
}, TRUE)
df <- df[,keep]

keep <- grep(":ch1$", colnames(df))
df <- df[,keep]

library(S4Vectors)
colnames(df) <- sub(":ch1$", "", colnames(df))
df <- DataFrame(df)

df <- df[colnames(combined),] # already know that rownames are valid.
df 
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "darmanis-brain", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(combined, file=file.path(path, "counts.rds"))
saveRDS(df, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Giladi HSC dataset
author: Aaron Lun
date: 6 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data (RNA-seq)

We obtain a single-cell RNA sequencing dataset of haematopoietic stem cells from @giladi2018singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE92575](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE92575).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
collected <- bfcrpath(bfc, "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE92575&format=file")

dir <- tempfile()
untar(collected, exdir=dir)
all.files <- list.files(dir, full=TRUE)
length(all.files)
```

Loading them all into memory.

```{r}
library(scuttle)
library(BiocParallel)
out <- bplapply(all.files, readSparseCounts)

# Sanity check:
stopifnot(length(unique(lapply(out, rownames)))==1)

# Checking that the names are unique after combining.
all.cn <- unlist(lapply(out, colnames))
stopifnot(anyDuplicated(all.cn)==0)

combined <- do.call(cbind, out)
```

Constructing a `SingleCellExperiment` object.

```{r}
sce <- SingleCellExperiment(list(counts=combined))
sce
```

Filling in the metadata as well.

```{r}
meta.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE92575&format=file&file=GSE92575%5Fmetadata%2Etxt%2Egz")
meta <- read.delim(meta.path, skip=14)

# Expanding to cover all cells in the SCE.
true.samples <- sub(".*_", "", sub(".txt.gz", "", basename(all.files)))
true.samples <- rep(true.samples, vapply(out, ncol, 0L))
ref <- DataFrame(well=colnames(combined), sample=true.samples)
obs <- DataFrame(well=meta$well, sample=meta$Amp_batch_ID)
stopifnot(all(obs %in% ref))

m <- match(ref, obs)
meta <- meta[m,]
meta$retained <- !is.na(m)
meta$sample <- true.samples

stopifnot(all(meta$well == colnames(combined) | !meta$retained))
meta$well <- colnames(combined)
rownames(meta) <- meta$well

colData(sce) <- DataFrame(meta)
colData(sce)
```

# Downloading the data (CRISPR-seq)

Counts for CRISPR barcodes are also available from the Gene Expression Omnibus
using the accession number [GSE113494](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE113494).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
crisp.path <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE113494&format=file&file=GSE113494%5Fcrispseq%5Fcount%2Etxt%2Egz")
crisp.df <- read.delim(crisp.path, skip=9)
head(crisp.df)
```

For some bizarre reason, this is not actually a count matrix, but rather, a molecule information file.
So we convert this into a count matrix in the simplest way - namely, counting the number of unique UMIs for each cell in each barcode.

```{r}
library(DropletUtils)
crisp.mat <- makeCountMatrix(crisp.df$grna, crisp.df$well)
dim(crisp.mat)

crisp.cd <- DataFrame(crisp.df[,c("well", "amplification.batch")])
crisp.cd <- crisp.cd[match(colnames(crisp.mat), crisp.cd$well),]
rownames(crisp.cd) <- crisp.cd$well
crisp.cd <- crisp.cd[,2,drop=FALSE]
head(crisp.cd)

crisp.rd <- DataFrame(crisp.df[,c("grna", "ugi")])
crisp.rd <- crisp.rd[match(rownames(crisp.mat), crisp.rd$grna),]
rownames(crisp.rd) <- crisp.rd$grna
crisp.rd <- crisp.rd[,2,drop=FALSE]
head(crisp.rd)
```

# Saving to file

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "giladi-hsc", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts-rna.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata-rna.rds"))

saveRDS(crisp.mat, file=file.path(path, "counts-crispr.rds"))
saveRDS(crisp.cd, file=file.path(path, "coldata-crispr.rds"))
saveRDS(crisp.rd, file=file.path(path, "rowdata-crispr.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: "Processing Mair-2020 CITEseq data (One-SENSE paper)"
author: "Stephany Orjuela"
date: "June 24, 2020"
output: 
 BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r setup, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Download the data

We obtain a single-cell RNA sequencing dataset of the human PBMCs from @mair2020targeted.
Counts for endogenous genes and antibody-derived tags are available from the Gene Expression Omnibus
using the accession number [GSE13525](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE135325).
Of particular interest are the `*Combined_PBMC_AbSeq*.csv.gz` files that contain the data for Figure 3A.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/samples/",
    "GSM4005nnn/GSM4005486/suppl/")
Abseq_1 <- count.file <- bfcrpath(bfc, file.path(base.url,
    "GSM4005486_Combined_PBMC_AbSeq_1_DBEC_MolsPerCell_with_SampleTag.csv.gz"))

base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/samples/",
    "GSM4005nnn/GSM4005487/suppl/")
Abseq_2 <- count.file <- bfcrpath(bfc, file.path(base.url,
    "GSM4005487_Combined_PBMC_AbSeq_2_DBEC_MolsPerCell_with_SampleTag.csv.gz"))
```

# Processing the data

We set up a function to process each of these files.
This breaks up the data frame into the two matrices (RNA and ADT) as well as the sample-level metadata;
it also harvests some row-level metadata from the feature names.

```{r}
library(S4Vectors)
ingester <- function(fname) {
    df <- read.csv(fname, row.names = 1, check.names = FALSE)
    meta <- df[,tail(colnames(df), 2)]
    mat <- t(as.matrix(df[,head(colnames(df), -2)]))

    is.protein <- grepl("pAbO", rownames(mat))
    P <- mat[is.protein,]
    RNA <- mat[!is.protein,]

    rna.rowdata <- strsplit(rownames(RNA), "\\|")
    rna.rowdata <- DataFrame(do.call(rbind, rna.rowdata))
    colnames(rna.rowdata) <- c("Symbol", "RefSeq", "Type")
    rownames(RNA) <- with(rna.rowdata, paste0(Symbol, "_", ifelse(Type=="PolyA_1", Type, "RNA")))

    p.rowdata <- strsplit(rownames(P), "\\|")
    p.rowdata <- DataFrame(do.call(rbind, p.rowdata))[,1:3]
    colnames(p.rowdata) <- c("Symbol", "Alternative", "ID")
    rownames(P) <- with(p.rowdata, paste0(Symbol, "_", Alternative))

    list(rna.mat=RNA, rna.rowdata=rna.rowdata, 
        p.mat=P, p.rowdata=p.rowdata, 
        coldata=DataFrame(meta))
}
```

We run the ingester on each of the two files.

```{r}
ingested1 <- ingester(Abseq_1)
dim(ingested1$rna.mat)
dim(ingested1$p.mat)
ingested1$coldata

ingested2 <- ingester(Abseq_2)
dim(ingested2$rna.mat)
dim(ingested2$p.mat)
ingested2$coldata
```

We check that the row metadata is consistent across both files.

```{r}
stopifnot(identical(ingested1$rna.rowdata, ingested2$rna.rowdata))
stopifnot(identical(ingested1$p.rowdata, ingested2$p.rowdata))
stopifnot(identical(colnames(ingested1$coldata), colnames(ingested2$coldata)))
ingested1$p.rowdata
ingested1$rna.rowdata
```

We then combine all of the pieces of information into one matrix per set of features and a common sample metadata dataframe.

```{r}
final.p.mat <- cbind(ingested1$p.mat, ingested2$p.mat)
dim(final.p.mat)

final.rna.mat <- cbind(ingested1$rna.mat, ingested2$rna.mat)
dim(final.rna.mat)

final.coldata <- rbind(ingested1$coldata, ingested2$coldata)
final.coldata$Cartridge <- rep(as.character(1:2), 
    c(nrow(ingested1$coldata), nrow(ingested2$coldata)))
final.coldata

stopifnot(identical(nrow(final.coldata), ncol(final.p.mat)))
stopifnot(identical(nrow(final.coldata), ncol(final.rna.mat)))
```

We also coerce all sample names to the same set of values to avoid later confusion.

```{r}
final.names <- paste0(colnames(final.p.mat), "-", final.coldata$Cartridge)
rownames(final.coldata) <- colnames(final.p.mat) <- colnames(final.rna.mat) <- final.names
head(final.names)
```

# Save for upload
 
We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.
Note that it doesn't matter which row data we use, as they should be the same.

```{r}
repath <- file.path("scRNAseq", "mair-pbmc", "2.4.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)
saveRDS(final.coldata, file=file.path(repath, "coldata.rds"))

saveRDS(ingested1$rna.rowdata, file=file.path(repath, "rowdata-rna.rds"))
saveRDS(final.rna.mat, file=file.path(repath, "counts-rna.rds"))

saveRDS(ingested1$p.rowdata, file=file.path(repath, "rowdata-adt.rds"))
saveRDS(final.p.mat, file=file.path(repath, "counts-adt.rds"))
```

# Session info

```{r}
sessionInfo()
```

# References

---
title: "Processing Kotliarov-2020 CITEseq data (used in dsb paper)"
author: "Stephany Orjuela"
date: "August 27, 2020"
output: 
 BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r setup, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Download the data

We obtain a single-cell RNA sequencing dataset of the human PBMCs from @kotliarov2020citeseq.
Counts for endogenous genes and antibody-derived tags (ADTs) are available from [figshare](https://nih.figshare.com/collections/Data_and_software_code_repository_for_Broad_immune_activation_underlies_shared_set_point_signatures_for_vaccine_responsiveness_in_healthy_individuals_and_disease_activity_in_patients_with_lupus_Kotliarov_Y_Sparks_R_et_al_Nat_Med_DOI_https_d/4753772).

Code used to analyze the data is available in the same link.

Of particular interest is the `H1_day0_demultilexed_singlets.RDS` file, 
which is a Seurat object that contains demultiplexed counts (filtered for singlets). 
Demultiplexing was performed with HTO counts and verified with demuxlet. 
Counts available have not been filtered by QC or normalized.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
contents <- bfcrpath(bfc,
    "https://nih.figshare.com/ndownloader/files/20706642")

seuratObj <- readRDS(contents)
```

# Extract elements from Seurat object

First we extract the RNA counts:

```{r}
library(S4Vectors)
rna.mat <- seuratObj@data
dim(rna.mat)

coldata <- DataFrame(seuratObj@meta.data)
nrow(coldata)
colnames(coldata)
```

Then the ADT counts:

```{r}
adt.mat <- seuratObj@assay$CITE@raw.data
dim(adt.mat)
```

# Save for upload
 
We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
repath <- file.path("scRNAseq", "kotliarov-pbmc", "2.4.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)
saveRDS(coldata, file=file.path(repath, "coldata.rds"))
saveRDS(rna.mat, file=file.path(repath, "counts-rna.rds"))
saveRDS(adt.mat, file=file.path(repath, "counts-adt.rds"))
```

# Session info

```{r}
sessionInfo()
```

# References

---
title: Processing the Zhong prefrontal cortex dataset
author: Aaron Lun
date: 7 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of human prefrontal cortex cells from @zhong2018singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE104276](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE104276).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
fname <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE104276&format=file&file=GSE104276%5Fall%5Fpfc%5F2394%5FUMI%5Fcount%5FNOERCC%2Exls%2Egz")
tmp <- tempfile(fileext=".xls")
R.utils::gunzip(fname, destname=tmp, remove=FALSE)
```

Reading all the counts in as sparse matrices.
Despite its name, this is not actually an XLS file.

```{r}
library(scuttle)
counts <- readSparseCounts(tmp, skip.row=1, col.names=FALSE)
colnames(counts) <- strsplit(readLines(tmp, n=1), "\t")[[1]]
dim(counts)
```

We pull down some sample-level metadata in SOFT format.

```{r}
library(GEOquery)
out <- GEOquery::getGEO("GSE104276")

df <- as(phenoData(out[[1]]), "data.frame")
sampdata <- DataFrame(
    developmental_stage=df[["developmental stage:ch1"]],
    gender=df[["gender:ch1"]],
    sample=df$title
)

# Fixing an error in their annotation.
sampdata$developmental_stage[sampdata$developmental_stage == "week gestation"] <- "23 weeks after gestation"

sampdata
```

Unfortunately, it's not enough to link the exact samples to their cells.
We take the nuclear option of rolling through the per-sample TPM files to figure out who lives where.

```{r}
fname <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE104276&format=file")

tmp <- tempfile()
untar(fname, exdir=tmp)

all.files <- list.files(tmp, full=TRUE)
out <- lapply(all.files, function(x) {
    strsplit(readLines(x, n=1), "\t")[[1]]
})

# So many errors in their sample names...
samples <- sub("\\..*", "", basename(all.files))
samples <- sub("GSM[0-9]+_", "", samples)
samples <- sub("GW8", "GW08", samples)
samples <- sub("GW9", "GW09", samples)
samples <- sub("GW19_PFC", "GW19_PFC1_", samples)

mapping <- rep(basename(samples), lengths(out))
names(mapping) <- unlist(out)

converted <- mapping[colnames(counts)]
stopifnot(all(!is.na(converted)))
m <- match(converted, sampdata$sample)
stopifnot(all(!is.na(m)))

coldata <- sampdata[m,]
rownames(coldata) <- colnames(counts)
coldata
```

But that's not all, because we can add even more annotation about the cell type.

```{r}
fname <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE104276&format=file&file=GSE104276%5Freadme%5Fsample%5Fbarcode%2Exlsx")

library(readxl)
more.data <- read_xlsx(fname, sheet="SampleInfo")
more.data <- as.data.frame(more.data)
head(more.data)

m <- match(rownames(coldata), more.data[,1])
coldata <- cbind(coldata, more.data[m,c(-1, -ncol(more.data))])
colnames(coldata)
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "zhong-prefrontal", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Kolodziejczyk ESC dataset
author: Aaron Lun
date: "September 6, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of mouse embryonic stem cells from @kolodziejczyk2015singlecell.
We download and cache the count matrix using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
kolod.counts <- bfcrpath(bfc,
    file.path("https://espresso.teichlab.sanger.ac.uk",
        "static/counttable_es.csv"))
```

# Processing the read counts

We load the counts into memory.
Despite the name of the file, it is not actually comma-separated!

```{r}
counts <- read.table(kolod.counts, row.names=1, check.names=FALSE)
counts <- as.matrix(counts)
dim(counts)
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "kolodziejczyk-esc", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing legacy datasets
author: Aaron Lun
date: June 9, 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Overview

Here, we re-process datasets that were previously stored inside `r Biocpkg("scRNAseq")` itself,
in preparation for relocating them into `r Biocpkg("ExperimentHub")`.
This aims to reduce the size of the package and improve consistency with the other ExperimentHub-hosted datasets.

# Downloading the data

We download and cache an old version of the `r Biocpkg("scRNAseq")` package using the `r Biocpkg("BiocFileCache")` package.
We choose the latest version that still contains the serialized `SingleCellExperiment` objects.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
tarpath <- bfcrpath(bfc, file.path("http://bioconductor.org/packages",
    "3.9/data/experiment/src/contrib/scRNAseq_1.10.0.tar.gz"))
```

We unpack this in preparation for extraction of the serialized objects.

```{r}
tmploc <- tempfile()
dir.create(tmploc, showWarnings=FALSE)
untar(tarpath, exdir=tmploc)
path <- file.path(tmploc, "scRNAseq", "data")
list.files(path)
```

# Extracting objects

We set up a function to extract the `SingleCellExperiment` object from each Rdata file.

```{r}
library(SingleCellExperiment)
extractor <- function(fname) {
    env <- new.env()
    load(fname, envir=env)
    env[[ls(env)[1]]]
}
```

Using this function, we read in the counts for the endogenous genes, ERCC spike-in transcripts and mitochondrial genes.

```{r}
allen <- extractor(file.path(path, "allen.rda"))
allen
th2 <- extractor(file.path(path, "th2.rda"))
th2
fluidigm <- extractor(file.path(path, "fluidigm.rda"))
fluidigm
```

# Saving objects

We set up another function that peels apart the `SingleCellExperiment` object and saves its components.
Components are used to reconstruct a `SingleCellExperiment` object on the client side,
which provides some robustness against changes to the class structure.

```{r}
# No rowData for any of these objects.
saver <- function(sce, target) {
    dir.create(target, recursive=TRUE, showWarnings=FALSE)
    saveRDS(colData(sce), file=file.path(target, "coldata.rds"))
    saveRDS(metadata(sce), file=file.path(target, "metadata.rds"))
    for (x in assayNames(sce)) {
        saveRDS(assay(sce, x), file=file.path(target, paste0(x, ".rds")))
    }
}
```

We then apply this function to all of the objects.

```{r}
saver(allen, file.path("scRNAseq", "legacy-allen", "1.10.0"))
saver(th2, file.path("scRNAseq", "legacy-th2", "1.10.0"))
saver(fluidigm, file.path("scRNAseq", "legacy-fluidigm", "1.10.0"))
```

# Session information

```{r}
sessionInfo()
```

---
title: Processing the Bhaduri cortical organoid dataset
author: Aaron Lun
date: 14 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of human cortical organoids from @bhaduri2020cell.
Counts for endogenous genes are available from GEO using the accession [GSE132672](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE132672). 
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
fname <- bfcrpath(bfc, "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE132672&format=file&file=GSE132672%5Fallorganoids%5Fwithnew%5Fmatrix%2Etxt%2Egz")
```

Reading the counts in as a sparse matrix - this may take some time.

```{r}
library(scuttle)
counts <- readSparseCounts(fname)
sce <- SingleCellExperiment(list(counts=counts))
sce
```

# Grabbing the metadata

We use `r Biocpkg("GEOquery")` to scrape the relevant sample-level metadata.

```{r}
library(GEOquery)
out <- getGEO("GSE132672")
meta <- as(phenoData(out[[1]]), "data.frame")
meta <- meta[,c(1L, grep(":ch1$", colnames(meta)))]
colnames(meta) <- sub(":ch1$", "", colnames(meta))
meta <- DataFrame(meta, check.names=FALSE)
meta
```

Then it's just a simple matter to expand it.

```{r}
observed.donor <- sub("_.*", "", colnames(sce))

# Well, not so simple, because these authors couldn't 
# keep their names straight if their lives depended on it.
mappings <- c(
    H1SWeek24="wk24H1S_S2",
    L13234PWeek24="wk2413234P",
    L13234SWeek15="L13234controlweek15",
    L13234SWeek24="wk2413234S",
    Week10P="Week10P_1323_4",
    Week10S="Week10S_1323_4",
    Week5P="Week5P_1323_4",
    Week5S="Week5S_1323_4",
    Week8P="Week8P_1323_4",
    Week8S="Week8S_1323_4"
)

failed <- observed.donor %in% names(mappings)
observed.donor[failed] <- unname(mappings[observed.donor[failed]])
table(observed.donor)

m <- match(observed.donor, meta$title)
stopifnot(all(!is.na(m)))
coldata <- meta[m,,drop=FALSE]
rownames(coldata) <- colnames(sce)
colData(sce) <- coldata
```

# Further notes

There is, in fact, another version of this dataset, available at https://cells.ucsc.edu/organoidreportcard.
Briefly put: this was hell.
The column names of the count matrix simply do not match with the cell names in the metadata table.
Attempts to reconstruct the cell names failed to yield an unambiguous mapping, and so this was abandoned.
The code used is reported below as a warning to future generations.

```{r, eval=FALSE}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
fname <- bfcrpath(bfc, "https://cells.ucsc.edu/organoidreportcard/primary10X/primarymatrix_nonorm.txt.gz")

library(scuttle)
counts <- readSparseCounts(fname)
sce <- SingleCellExperiment(list(counts=counts))

meta.path <- bfcrpath(bfc, "https://cells.ucsc.edu/organoidreportcard/primary10X/meta.tsv")
meta <- read.delim(meta.path)
meta <- DataFrame(meta)
table(meta$Area)

ids <- colnames(sce)
ids <- sub("Central", "Central cortex", ids)
ids <- sub("Frontal", "Frontal cortex", ids)
ids <- sub("Occipital", "Occipital cortex", ids)
ids <- sub("Hippocampus", "hippocampus", ids)
ids <- sub("Motor", "motor", ids)
ids <- sub("motordeeper", "motor", ids)
ids <- sub("PFCdeeper", "PFC", ids)
ids <- sub("LPFC", "PFC", ids)
ids <- sub("parietaldeeper", "parietal", ids)
ids <- sub("Parietal", "parietal", ids)
ids <- sub("(S|s)omato", "somatosensory", ids)
ids <- sub("Temporal", "temporal", ids)

converted <- sub(".* ([A-Z]{2}[0-9]{2})","", ids)
table(converted)

idx <- sub(".*_", "", meta$Cell)

cell <- sub("([ACGT]+)_[^_]+$", "\\1", meta$Cell)
cell <- sub("^[^_]+_([ACTG]+)", "\\1", cell)
ids2 <- paste0(cell, " ", meta$Individual, meta$Area)

# Really poor matches here, but whatever. 
m <- match(ids, ids2)
table(converted, is.na(m))
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "bhaduri-organoid", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Buettner ESC dataset
author: Aaron Lun
date: "October 13, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of mouse ESCs from @buettner2015computational,
by downloading the files from ArrayExpress and caching the results with `r Biocpkg("BiocFileCache")`.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask=FALSE)
zip.path <- bfcrpath(bfc, file.path("https://www.ebi.ac.uk/arrayexpress",
    "files/E-MTAB-2805/E-MTAB-2805.processed.1.zip"))
exdir <- tempdir()
unzip(zip.path, exdir=exdir)
```

# Processing the read counts

We read in the count matrix for cells in each cell cycle phase.

```{r}
g1 <- read.delim(file.path(exdir, "G1_singlecells_counts.txt"), row.names=1)
gene.g1 <- g1[,1:3]
g1 <- as.matrix(g1[,-(1:3)])
dim(g1)

s <- read.delim(file.path(exdir, "S_singlecells_counts.txt"), row.names=1)
gene.s <- s[,1:3]
s <- as.matrix(s[,-(1:3)])
dim(s)

g2m <- read.delim(file.path(exdir, "G2M_singlecells_counts.txt"), row.names=1)
gene.g2m <- g2m[,1:3]
g2m <- as.matrix(g2m[,-(1:3)])
dim(g2m)
```

Applying some sanity checks to ensure that the rows are the same across matrices.

```{r}
stopifnot(identical(gene.g1, gene.s))
stopifnot(identical(gene.g1, gene.g2m))
```

# Combining and storing metadata

```{r}
combined <- cbind(g1, s, g2m)
library(S4Vectors)
coldata <- DataFrame(phase=rep(c("G1", "S", "G2M"), c(ncol(g1), ncol(s), ncol(g2m))))
coldata
rowdata <- DataFrame(gene.g1)
rowdata
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
repath <- file.path("scRNAseq", "buettner-esc", "2.0.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)
saveRDS(combined, file=file.path(repath, "counts.rds"))
saveRDS(coldata, file=file.path(repath, "coldata.rds"))
saveRDS(rowdata, file=file.path(repath, "rowdata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Hu cortex snRNA-seq data
author: "Aaron Lun"
date: "August 1, 2020"
output: 
 BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r setup, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Download the data

We obtain a single-nucleus RNA sequencing dataset of mouse brains from @hu2017dissecting.
Counts for endogenous genes and antibody-derived tags are available from the Gene Expression Omnibus
using the accession number [GSE106678](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE106678).

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
tarred <- bfcrpath(bfc, "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE106678&format=file")

temp <- tempfile()
dir.create(temp)
untar(tarred, exdir=temp)
list.files(temp)
```

# Processing the data

We load in each of the files.

```{r}
library(scuttle)
counts <- list()
for (x in list.files(temp, full.names=TRUE)) {
    prefix <- sub("^[^_]*_", "", x)
    prefix <- sub("_.*", "", prefix)
    counts[[prefix]] <- readSparseCounts(x)
}
do.call(rbind, lapply(counts, dim))
```

For some unknown reason, each matrix has its own set of features!
Crazy.
At least the intersection is of a reasonable size.

```{r}
length(Reduce(intersect, lapply(counts, rownames)))
```

# Save for upload
 
We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
repath <- file.path("scRNAseq", "hu-cortex", "2.4.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)
for (x in names(counts)) {
    saveRDS(counts[[x]], file=file.path(repath, sprintf("counts-%s.rds", x)))
}
```

# Session info

```{r}
sessionInfo()
```

# References

---
title: Processing the Nowakowski cortex dataset
author: Aaron Lun
date: 11 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the human cotex from @nowakowski2017spatiotemporal.
TPMs for endogenous genes are available from the the UCSC cell browser at http://cells.ucsc.edu/?ds=cortex-dev.
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
exprs.path <- bfcrpath(bfc, "http://cells.ucsc.edu/cortex-dev/exprMatrix.tsv.gz")

library(scuttle)
mat <- readSparseCounts(exprs.path, row.names=1)
dim(mat)
```

We also read in various pieces of metadata; firstly, the per-cell metadata.

```{r}
meta.path <- bfcrpath(bfc, "http://cells.ucsc.edu/cortex-dev/meta.tsv")
meta <- read.delim(meta.path, row.names=1)
stopifnot(identical(colnames(mat), rownames(meta))) 
meta <- DataFrame(meta)
meta
```

And then all of the various reduced dimensions:

```{r}
red.dim.list <- c(
    "tMinusSNE_on_WGCNA",
    "TriMap",
    "TMinusSNE_scanpy",
    "UMAP_scanpy",
    "PAGAPlusForceAtlas2_scanpy"
)

reddims <- list()
for (i in red.dim.list) { 
    X <- bfcrpath(bfc, sprintf("http://cells.ucsc.edu/cortex-dev/%s.coords.tsv.gz", i))
    Y <- as.matrix(read.table(X, row.names=1, sep="\t"))
    stopifnot(all(rownames(Y) %in% colnames(mat)))

    expanded <- Y[match(colnames(mat), rownames(Y)),,drop=FALSE]
    rownames(expanded) <- colnames(mat)

    stopifnot(identical(colnames(expanded), c("V2", "V3")))
    colnames(expanded) <- NULL
    reddims[[i]] <- expanded
}
```

Making sure we can assemble the final SCE:

```{r}
sce <- SingleCellExperiment(list(tpm=mat), colData=meta, reducedDims=reddims)
sce 
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "nowakowski-cortex", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(mat, file=file.path(path, "tpm.rds"))
saveRDS(meta, file=file.path(path, "coldata.rds"))
saveRDS(reddims, file=file.path(path, "reddims.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Zilionis lung cancer dataset
author: Jens Preussner
date: "May 4, 2020"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human and mouse lung cancer from @zilionis2019singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE127465](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE127465).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache(ask=FALSE)    
tarball <- bfcrpath(bfc, 
    file.path("https://www.ncbi.nlm.nih.gov/geo",
        "download/?acc=GSE127465&format=file"))
```

We unpack it to a temporary directory.

```{r}
temp <- tempfile()
untar(tarball, exdir=temp)
```

# Reading in human data

We read in all the human datasets as sparse matrices.

```{r}
hs.files <- c(
    "GSM3635278_human_p1t1_raw_counts.tsv.gz",
    "GSM3635279_human_p1t2_raw_counts.tsv.gz",
    "GSM3635280_human_p1t3_raw_counts.tsv.gz",
    "GSM3635281_human_p1t4_raw_counts.tsv.gz",
    "GSM3635282_human_p1b1_raw_counts.tsv.gz",
    "GSM3635283_human_p1b2_raw_counts.tsv.gz",
    "GSM3635284_human_p1b3_raw_counts.tsv.gz",
    "GSM3635285_human_p2t1_raw_counts.tsv.gz",
    "GSM3635286_human_p2t2_raw_counts.tsv.gz",
    "GSM3635287_human_p2b1_raw_counts.tsv.gz",
    "GSM3635288_human_p3t1_raw_counts.tsv.gz",
    "GSM3635289_human_p3t2_raw_counts.tsv.gz",
    "GSM3635290_human_p3t3_raw_counts.tsv.gz",
    "GSM3635291_human_p3b1_raw_counts.tsv.gz",
    "GSM3635292_human_p4t1_raw_counts.tsv.gz",
    "GSM3635293_human_p4t2_raw_counts.tsv.gz",
    "GSM3635294_human_p4t3_raw_counts.tsv.gz",
    "GSM3635295_human_p4b1_raw_counts.tsv.gz",
    "GSM3635296_human_p5t1_raw_counts.tsv.gz",
    "GSM3635297_human_p5t2_raw_counts.tsv.gz",
    "GSM3635298_human_p6t1_raw_counts.tsv.gz",
    "GSM3635299_human_p6t2_raw_counts.tsv.gz",
    "GSM3635300_human_p6b1_raw_counts.tsv.gz",
    "GSM3635301_human_p7t1_raw_counts.tsv.gz",
    "GSM3635302_human_p7t2_raw_counts.tsv.gz",
    "GSM3635303_human_p7b1_raw_counts.tsv.gz"
)

library(scater)
all.human <- lapply(file.path(temp, hs.files), readSparseCounts)

library(Matrix) # Because the values are transposed.
all.human <- lapply(all.human, t)

t(sapply(all.human, dim))
```

We verify that the gene order is the same, and combine the counts.

```{r}
stopifnot(length(unique(lapply(all.human, rownames)))==1L)
counts <- do.call(cbind, all.human)
dim(counts)
```

We derive some metadata from each file name and apply them to all of the constituent barcodes.

```{r}
samples <- vapply(strsplit(hs.files, "_"), "[", i=3, "")
barcode <- lapply(all.human, colnames)
origin <- rep(samples, times = lengths(barcode))

donor <- sub("[bt].*", "", origin)
tissue <- ifelse(grepl("t", origin), "tumor", "blood")
barcode <- unlist(barcode)

library(S4Vectors)
coldata <- DataFrame(Library=origin, Barcode=barcode,
    Patient=donor, Tissue=tissue)
coldata
```

We then add additional metadata for a subset of cells that were used in the original paper.
We convert some of the fields to logical values.

```{r}
bfc <- BiocFileCache(ask=FALSE)
tarball <- bfcrpath(bfc, 
    file.path("https://ftp.ncbi.nlm.nih.gov/geo/series",
        "GSE127nnn/GSE127465/suppl",
        "GSE127465_human_cell_metadata_54773x25.tsv.gz"))

metadata <- read.delim(tarball, stringsAsFactors=FALSE, check.names = FALSE)
for (u in grep("^used", colnames(metadata))) {
    metadata[[u]] <- metadata[[u]]=="True"
}

metadata <- DataFrame(metadata, check.names=FALSE)
metadata
```

We merge this with our file name-derived metadata:

```{r}
keys <- c("Library", "Barcode")
m <- match(coldata[,keys], metadata[,keys])
coldata$Used <- !is.na(m)

discard <- c(keys, "Patient", "Tissue")
colData <- cbind(coldata, metadata[m,setdiff(colnames(metadata), discard)])
colData
```

We save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "zilionis-lung", "2.4.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts-human.rds"))
saveRDS(colData, file=file.path(path, "coldata-human.rds"))
```

```{r, echo=FALSE, results="hide"}
rm(counts, all.human)
gc()
```

# Reading in mouse data

We read in all the mouse datasets.

```{r}
mm.files <- c(
    "GSM3635304_mouse_h_1_1_raw_counts.tsv.gz",
    "GSM3635305_mouse_h_1_2_raw_counts.tsv.gz",
    "GSM3635306_mouse_h_2_1_raw_counts.tsv.gz",
    "GSM3635307_mouse_h_2_2_raw_counts.tsv.gz",
    "GSM3635308_mouse_h_2_3_raw_counts.tsv.gz",
    "GSM3635309_mouse_t_1_1_raw_counts.tsv.gz",
    "GSM3635310_mouse_t_1_2_raw_counts.tsv.gz",
    "GSM3635311_mouse_t_1_3_raw_counts.tsv.gz",
    "GSM3635312_mouse_t_1_4_raw_counts.tsv.gz",
    "GSM3635313_mouse_t_1_5_raw_counts.tsv.gz",
    "GSM3635314_mouse_t_2_1_raw_counts.tsv.gz",
    "GSM3635315_mouse_t_2_2_raw_counts.tsv.gz",
    "GSM3635316_mouse_t_2_3_raw_counts.tsv.gz",
    "GSM3635317_mouse_t_2_4_raw_counts.tsv.gz"
)

all.mouse <- lapply(file.path(temp, mm.files), readSparseCounts)
all.mouse <- lapply(all.mouse, t)
t(sapply(all.mouse, dim))
```

We verify that the gene order is the same, and combine the counts.

```{r}
stopifnot(length(unique(lapply(all.mouse, rownames)))==1L)
counts <- do.call(cbind, all.mouse)
dim(counts)
```

We derive some metadata from each file name and apply them to all of the constituent barcodes.

```{r}
separated <- strsplit(mm.files, "_")

tissue <- vapply(separated, "[", i=3, "")
animal <- vapply(separated, "[", i=4, "")
replicate <- vapply(separated, "[", i=5, "")
barcode <- lapply(all.mouse, colnames)

animal <- rep(sprintf("%s_%s", tissue, animal), lengths(barcode))
replicate <- rep(replicate, lengths(barcode))
library <- sprintf("%s_%s", animal, replicate)
tissue <- rep(ifelse(tissue == "t", "tumor", "healthy"), times = lengths(barcode))
barcode <- unlist(barcode)

coldata <- DataFrame(Library=library, Barcode=barcode, Animal = animal, 
    Run = replicate, Tissue=tissue)
coldata
```

We next add additional metadata for a subset of cells that were used in the original paper.
We keep only the experimentally interesting metadata, discarding columns that are duplicated or only have one level.

```{r}
bfc <- BiocFileCache(ask=FALSE)
tarball <- bfcrpath(bfc, 
    file.path("https://ftp.ncbi.nlm.nih.gov/geo/series",
        "GSE127nnn/GSE127465/suppl",
        "GSE127465_mouse_cell_metadata_15939x12.tsv.gz"))

metadata <- read.delim(tarball, stringsAsFactors=FALSE, check.names = FALSE)
metadata <- DataFrame(metadata, check.names=FALSE)
metadata
```

We merge this with our file name-derived metadata:

```{r}
keys <- c("Library", "Barcode")
m <- match(coldata[,keys], metadata[,keys])
coldata$Used <- !is.na(m)

discard <- c(keys, "Tumor or healthy", "Biological replicate")
colData <- cbind(coldata, metadata[m,setdiff(colnames(metadata), discard)])
colData
```

We save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "zilionis-lung", "2.4.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts-mouse.rds"))
saveRDS(colData, file=file.path(path, "coldata-mouse.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Macosko retina dataset
author: Aaron Lun
date: "June 8, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of the mouse retina from @shekhar2016comprehensive. 
Counts for endogenous genes and spike-in transcripts are available from the Gene Expression Omnibus
using the accession number [GSE81904](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81904).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
count.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE81nnn/GSE81904/suppl/GSE81904_BipolarUMICounts_Cell2016.txt.gz")
count.file <- bfcrpath(bfc, count.url)
```

We load them into memory.

```{r}
library(scater)
counts <- readSparseCounts(count.file)
dim(counts)
```

# Downloading the per-cell metadata

We also download a file containing the metadata for each cell.
(Courtesy of Vladimir Kiselev and Martin Hemberg, as the original annotation has disappeared.)

```{r}
meta.file <- bfcrpath(bfc, file.path("https://s3.amazonaws.com",
    "scrnaseq-public-datasets/manual-data/shekhar/clust_retinal_bipolar.txt"))
coldata <- read.delim(meta.file, stringsAsFactors=FALSE, check.names=FALSE)

library(S4Vectors)
coldata <- as(coldata, "DataFrame")
coldata
```

We match the metadata to the columns.

```{r}
m <- match(colnames(counts), coldata$NAME)
coldata <- coldata[m,]
coldata$NAME <- colnames(counts)
summary(is.na(m))
```

# Saving to file 

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "shekhar-retina", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Baron pancreas dataset
author: Aaron Lun
date: "June 27, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human pancreas from @baron2016singlecell.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE84133](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE84133).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache(ask=FALSE)    
tarball <- bfcrpath(bfc, 
    "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE84133&format=file")
```

We unpack it to a temporary directory.

```{r}
temp <- tempfile()
untar(tarball, exdir=temp)
```

# Reading in human data

We set up a function to load in each set of counts as a sparse matrix.

```{r}
library(Matrix)
FUN <- function(X) {
    input <- read.csv(X, stringsAsFactors=FALSE)
    rownames(input) <- input[,1]
    labels <- as.character(input$assigned_cluster)
    input <- input[,4:ncol(input)]
    input <- t(input)
    list(mat=as(input, "dgCMatrix"), label=labels)
}
```

We read in all the human datasets.

```{r}
hs.files <- c(
    "GSM2230757_human1_umifm_counts.csv.gz",
    "GSM2230759_human3_umifm_counts.csv.gz",
    "GSM2230758_human2_umifm_counts.csv.gz",  
    "GSM2230760_human4_umifm_counts.csv.gz"
)
all.human <- lapply(file.path(temp, hs.files), FUN)
sapply(all.human, function(x) dim(x$mat))
```

We verify that the gene order is the same, and combine the counts.

```{r}
stopifnot(length(unique(lapply(all.human, function(x) rownames(x$mat))))==1L)
counts <- do.call(cbind, lapply(all.human, function(x) x$mat))
dim(counts)
```

We do the same thing with the column metadata.

```{r}
labels <- lapply(all.human, function(x) x$label)
donor <- rep(sub("_.*", "", hs.files), lengths(labels))
labels <- unlist(labels)

library(S4Vectors)
coldata <- DataFrame(donor=donor, label=labels)
coldata
```

We save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "baron-pancreas", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts-human.rds"))
saveRDS(coldata, file=file.path(path, "coldata-human.rds"))
```

# Reading in mouse data

We read in all the mouse datasets.

```{r}
mm.files <- c(
    "GSM2230761_mouse1_umifm_counts.csv.gz",
    "GSM2230762_mouse2_umifm_counts.csv.gz"
)
all.mouse <- lapply(file.path(temp, mm.files), FUN)
sapply(all.mouse, function(x) dim(x$mat))
```

We verify that the gene order is the same, and combine the counts.

```{r}
stopifnot(length(unique(lapply(all.mouse, function(x) rownames(x$mat))))==1L)
counts <- do.call(cbind, lapply(all.mouse, function(x) x$mat))
dim(counts)
```

We do the same thing with the column metadata.

```{r}
labels <- lapply(all.mouse, function(x) x$label)
strain <- rep(c("ICR", "C57BL/6"), lengths(labels))
labels <- unlist(labels)
coldata <- DataFrame(strain=strain, label=labels)
coldata
```

We save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "baron-pancreas", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts-mouse.rds"))
saveRDS(coldata, file=file.path(path, "coldata-mouse.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Wu kidney snRNA-seq data
author: "Aaron Lun"
date: "August 1, 2020"
output: 
 BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r setup, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Download the data

We obtain a single-nucleus RNA sequencing dataset of mouse kidneys from @wu2019advantages.
Counts for endogenous genes and antibody-derived tags are available from the Gene Expression Omnibus
using the accession number [GSE119531](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE119531).

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- "https://ftp.ncbi.nlm.nih.gov/geo/series/GSE119nnn/GSE119531/suppl"

healthy.raw <- bfcrpath(bfc, file.path(base.url, "GSE119531_Healthy.combined.dge.txt.gz"))
healthy.ann <- bfcrpath(bfc, file.path(base.url, "GSE119531_Healthy.combined.cell.annotation.txt.gz"))

disease.raw <- bfcrpath(bfc, file.path(base.url, "GSE119531_UUO.dge.txt.gz"))
disease.ann <- bfcrpath(bfc, file.path(base.url, "GSE119531_UUO.cell.annotation.txt.gz"))
```

# Processing the data

We load in each of the two sets of files.
There are some mild discrepancies with the cell barcodes that require some adjustment.

```{r}
library(scater)
disease.mat <- readSparseCounts(disease.raw)
dim(disease.mat)

disease.tab <- read.table(disease.ann, header=TRUE, sep="\t", stringsAsFactors=FALSE)
head(disease.tab)

colnames(disease.mat) <- sub("1_", "_", colnames(disease.mat))
stopifnot(identical(colnames(disease.mat), disease.tab$CellBarcode))
```

And again, for the healthy samples.

```{r}
healthy.mat <- readSparseCounts(healthy.raw)
dim(healthy.mat)

healthy.tab <- read.table(healthy.ann, header=TRUE, sep="\t", stringsAsFactors=FALSE)
head(healthy.tab)

colnames(healthy.mat) <- sub("sNuc.10x", "sNuc-10x", colnames(healthy.mat))
stopifnot(identical(colnames(healthy.mat), healthy.tab$CellBarcode))
```

# Save for upload
 
We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
repath <- file.path("scRNAseq", "wu-kidney", "2.4.0")
dir.create(repath, showWarnings=FALSE, recursive=TRUE)

saveRDS(healthy.tab, file=file.path(repath, "coldata-healthy.rds"))
saveRDS(healthy.mat, file=file.path(repath, "counts-healthy.rds"))

saveRDS(disease.tab, file=file.path(repath, "coldata-disease.rds"))
saveRDS(disease.mat, file=file.path(repath, "counts-disease.rds"))
```

# Session info

```{r}
sessionInfo()
```

# References

---
title: Processing the Bacher T cell dataset
author: Aaron Lun
date: 5 January 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of T cells from multiple patients from @bacher2020low.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE162086](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE162086).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)

# Manually transcribed from GEO:
samples <- matrix(ncol=4, byrow=TRUE, 
    c(
        "GSM4932900","J09835","WHO 1","non-hospitalized",
        "GSM4932901","J09836","WHO 1","non-hospitalized",
        "GSM4932902","J10535","WHO 4","mild-moderate",
        "GSM4932903","J10624","WHO 2","non-hospitalized",
        "GSM4932904","J10625","WHO 2","non-hospitalized",
        "GSM4932905","J10886","WHO 5","mild-moderate",
        "GSM4932906","J10887","WHO 6","severe",
        "GSM4932907","J10888","WHO 4","mild-moderate",
        "GSM4932908","J11689","WHO 2","non-hospitalized",
        "GSM4932909","J14204","WHO 7","severe",
        "GSM4932910","J14205","WHO 5","mild-moderate",
        "GSM4932911","J15890","WHO 2","non-hospitalized",
        "GSM4932912","J15891","healthy","healthy",
        "GSM4932913","J15892","healthy","healthy",
        "GSM4932914","J15899","healthy","healthy",
        "GSM4932915","J15900","healthy","healthy",
        "GSM4932916","J15893","WHO 5","mild-moderate",
        "GSM4932917","J21854","WHO 7","severe",
        "GSM4932918","J21855","healthy","healthy",
        "GSM4932919","J21856","healthy","healthy"
    )
)

counts <- vector("list", nrow(samples))
for (x in seq_along(counts)) {
    url <- sprintf("https://www.ncbi.nlm.nih.gov/geo/download/?acc=%s&format=file&file=%s%%5F%s%%5Fcounts%%2Etsv%%2Egz", 
        samples[x,1], samples[x,1], samples[x,2])
    counts[[x]] <- bfcrpath(bfc, url) 
}
```

Reading all the counts in as sparse matrices:

```{r}
library(scuttle)
library(BiocParallel)
counts <- bplapply(counts, readSparseCounts, quote="\"", BPPARAM=MulticoreParam())

# Sanity check:
gene.ids <- lapply(counts, rownames)
stopifnot(length(unique(gene.ids))==1L)

# Checking that the names are unique after combining.
all.cn <- unlist(lapply(counts, colnames))
stopifnot(anyDuplicated(all.cn)==0)

combined <- do.call(cbind, counts)
```

Creating a `SingleCellExperiment` object:

```{r}
library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=combined))
true.patient <- rep(samples[,2], vapply(counts, ncol, 0L))
```

Attaching some metadata:

```{r}
meta.path <- bfcrpath(bfc, "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE162086&format=file&file=GSE162086%5Fseurat%5Fmetadata%2Etsv%2Egz")
meta <- read.delim(meta.path, check.names=FALSE)

ID <- paste0(sub("-1", "", meta$barcode), "-", meta$sample)
ref <- gsub("\"", "", colnames(combined))
stopifnot(all(ID %in% ref))

m <- match(ref, ID)
meta <- meta[m,]
rownames(meta) <- ref
meta$retained <- !is.na(m)

# Filling in some of the patient-level attributes for missing cells.
# This is done by extrapolating from cells in the same patient. 
meta$sample <- sub(".*-", "", ref)
meta$barcode <- sub("-.*", "", ref)

for (field in c("batch", "seq_run", "diagnosis")) {
    tab <- table(meta$sample, meta[[field]])
    best <- colnames(tab)[max.col(tab)]
    best <- as(best, typeof(meta[[field]]))
    names(best) <- rownames(tab)
    meta[[field]] <- best[meta$sample]
}

m <- match(meta$sample,samples[,2])
stopifnot(all(!is.na(m)))
meta$who_class <- samples[m,3]
meta$severity <- samples[m,4]

colData(sce) <- DataFrame(meta)
sce
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "bacher-tcell", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Hermann spermatogenesis dataset for RNA velocity analysis
author: Charlotte Soneson
date: "June 5, 2020"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Loading required R packages

```{r}
suppressPackageStartupMessages({
    library(Biostrings)
    library(BSgenome)
    library(eisaR)
    library(GenomicFeatures)
    library(SummarizedExperiment)
    library(tximeta)
    library(rjson)
    library(SingleCellExperiment)
})
```

# Setting the working directory

```{r}
workdir <- tempdir()
Sys.setenv(workdir = workdir)
```

# Downloading the reference sequences

In order to perform the quantification of spliced and unspliced abundances with alevin [@srivastava2019alevin], we need to first prepare a suitable set of reference features (transcript isoform and intron sequences) to generate an index from. 
We start by downloading the mouse reference genome and annotation gtf file from GENCODE.

```{bash}
wget ftp://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_mouse/release_M24/gencode.vM24.annotation.gtf.gz -P $workdir
wget ftp://ftp.ebi.ac.uk/pub/databases/gencode/Gencode_mouse/release_M24/GRCm38.primary_assembly.genome.fa.gz -P $workdir
gunzip $workdir/GRCm38.primary_assembly.genome.fa.gz
```

# Extracting the transcript and intron sequences

Next, we extract a GRanges object containing the genomic coordinates of each annotated transcript and intron. 

```{r}
gtf <- file.path(workdir, "gencode.vM24.annotation.gtf.gz")
grl <- eisaR::getFeatureRanges(
    gtf = gtf,
    featureType = c("spliced", "intron"), 
    intronType = "separate", 
    flankLength = 90L, 
    joinOverlappingIntrons = FALSE, 
    verbose = TRUE
)
```

After defining the genomic positions of all features of interest, we can extract the sequences of these, and write to a fasta file for later indexing with _Salmon_ [@patro2017salmon].
In addition, we generate reference files that will be helpful for reading the data with _tximeta_ [@love2020tximeta] and splitting the joint count matrix into a spliced and an unspliced one.

```{r}
genome <- Biostrings::readDNAStringSet(
    file.path(workdir, "GRCm38.primary_assembly.genome.fa")
)
names(genome) <- sapply(strsplit(names(genome), " "), .subset, 1)
seqs <- GenomicFeatures::extractTranscriptSeqs(
    x = genome, 
    transcripts = grl
)
Biostrings::writeXStringSet(
    seqs, filepath = file.path(workdir, "gencode.vM24.annotation.expanded.fa")
)
eisaR::exportToGtf(
    grl, 
    filepath = file.path(workdir, "gencode.vM24.annotation.expanded.gtf")
)
write.table(
    metadata(grl)$corrgene, 
    file = file.path(workdir, "gencode.vM24.annotation.expanded.features.tsv"),
    row.names = FALSE, col.names = TRUE, quote = FALSE, sep = "\t"
)
df <- eisaR::getTx2Gene(
    grl, filepath = file.path(workdir, "gencode.vM24.annotation.expanded.tx2gene.tsv")
)
```

# Creating the quantification index

We use _Salmon_ (v1.2.0, https://github.com/COMBINE-lab/salmon/releases/tag/v1.2.0) to generate a quantification index based on the expanded fasta file assembled above.
To improve the quantification, the full genome sequence is provided as a decoy sequence [@srivastava2019mm]. 
This is recommended in order to avoid reads truly originating from intergenic regions being assigned to a suboptimal transcriptome location.

```{bash}
grep ">" $workdir/GRCm38.primary_assembly.genome.fa | cut -d ">" -f 2 | cut -d " " -f 1 > $workdir/GRCm38.primary_assembly.genome.chrnames.txt

salmon index \
-t <(cat $workdir/gencode.vM24.annotation.expanded.fa $workdir/GRCm38.primary_assembly.genome.fa) \
-i $workdir/gencode.vM24.annotation.expanded.sidx --gencode -p 32 \
-d $workdir/GRCm38.primary_assembly.genome.chrnames.txt
```

We also create a linked transcriptome for _tximeta_, to be able to add annotations when loading the quantified data into R.

```{r}
tximeta::makeLinkedTxome(
    indexDir = file.path(workdir, "gencode.vM24.annotation.expanded.sidx"), 
    source = "GENCODEtxintron", genome = "GRCm38", 
    organism = "Mus musculus", release = "M24", 
    fasta = file.path(workdir, "gencode.vM24.annotation.expanded.fa"), 
    gtf = file.path(workdir, "gencode.vM24.annotation.expanded.gtf"), 
    write = TRUE, jsonFile = file.path(workdir, "gencode.vM24.annotation.expanded.json")
)
```

# Downloading the raw data

We download the BAM file for the "AdultMouse3" sample (dataset GSE109033, sample ID GSM2928341) from @hermann2018spermatogenesis from SRA, and convert the downloaded BAM file into FASTQ files using the [bamtofastq](https://github.com/10XGenomics/bamtofastq) utility.

```{bash}
wget https://sra-pub-src-1.s3.amazonaws.com/SRR6459157/AdultMouse_Rep3_possorted_genome_bam.bam.1 -P $workdir
mv $workdir/AdultMouse_Rep3_possorted_genome_bam.bam.1 $workdir/AdultMouse_Rep3_possorted_genome_bam.bam
bamtofastq --reads-per-fastq=500000000 $workdir/AdultMouse_Rep3_possorted_genome_bam.bam $workdir/FASTQtmp
mv $workdir/FASTQtmp/Ad-Ms-Total-Sorted_20k_count_MissingLibrary_1_HK2GNBBXX/bamtofastq_S1_L006_I1_001.fastq.gz $workdir/AdultMouseRep3_S1_L001_I1_001.fastq.gz
mv $workdir/FASTQtmp/Ad-Ms-Total-Sorted_20k_count_MissingLibrary_1_HK2GNBBXX/bamtofastq_S1_L006_R1_001.fastq.gz $workdir/AdultMouseRep3_S1_L001_R1_001.fastq.gz
mv $workdir/FASTQtmp/Ad-Ms-Total-Sorted_20k_count_MissingLibrary_1_HK2GNBBXX/bamtofastq_S1_L006_R2_001.fastq.gz $workdir/AdultMouseRep3_S1_L001_R2_001.fastq.gz
```

# Quantifying spliced and unspliced abundances

Next, we run _alevin_ to obtain spliced and unspliced gene-level counts for each gene and cell.

```{bash}
salmon alevin -l ISR -i $workdir/gencode.vM24.annotation.expanded.sidx \
-1 $workdir/AdultMouseRep3_S1_L001_R1_001.fastq.gz \
-2 $workdir/AdultMouseRep3_S1_L001_R2_001.fastq.gz \
-o $workdir/alevin_out -p 36 \
--tgMap $workdir/gencode.vM24.annotation.expanded.tx2gene.tsv \
--chromium --dumpFeatures --expectCells 1850
```

# Importing quantifications into R

We use _tximeta_ to import the quantification files into R, after loading the linked transcriptome created previously.

```{r}
tximeta::loadLinkedTxome(file.path(workdir, "gencode.vM24.annotation.expanded.json"))
txi <- tximeta::tximeta(coldata = data.frame(
    names = "AdultMouseRep3",
    files = file.path(workdir, "alevin_out/alevin/quants_mat.gz"), 
    stringsAsFactors = FALSE
), type = "alevin")
```

The txi object contains a single assay ('counts') containing both spliced and unspliced abundances. 
We need to split this into two matrices, one with spliced and one with unspliced abundances, with corresponding rows. 
This can be done using the `splitSE()` function from _tximeta_, providing the data frame linking spliced and unspliced gene identifiers that we created above. 

```{r}
cg <- read.delim(file.path(workdir, "gencode.vM24.annotation.expanded.features.tsv"),
                 header = TRUE, as.is = TRUE)
## Rename the 'intron' column 'unspliced' to make assay names compatible with scVelo
colnames(cg)[colnames(cg) == "intron"] <- "unspliced"
txis <- tximeta::splitSE(txi, cg, assayName = "counts")
```

Next, we download cell type labels provided by the data generators and add to the colData of the created object.
The cloupe file generated by the authors can be obtained from https://data.mendeley.com/datasets/kxd5f8vpt4/1#file-fe79c10b-c42e-472e-9c7e-9a9873d9b3d8 and loaded into the loupe cell browser, from which cell type labels can be exported into a file, here named "Spermatogenesis_Loupe_CellTypes.csv". 

```{r}
ctp <- read.csv("Spermatogenesis_Loupe_CellTypes.csv", header = TRUE, as.is = TRUE)
## Keep only Replicate 3
ctp <- ctp[grep("-3", ctp$Barcode), , drop = FALSE]
ctp$index <- gsub("-3", "", ctp$Barcode)
ctp$celltype <- ctp$Cell.Types
ctp <- ctp[, c("index", "celltype")]
txis$celltype <- ctp$celltype[match(colnames(txis), ctp$index)]
```

Finally, we save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "hermann-spermatogenesis", "2.4.0")
dir.create(path, showWarnings = FALSE, recursive = TRUE)
saveRDS(assay(txis, "spliced"), file = file.path(path, "spliced.rds"))
saveRDS(assay(txis, "unspliced"), file = file.path(path, "unspliced.rds"))
saveRDS(colData(txis), file = file.path(path, "coldata.rds"))
```


# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Xin pancreas dataset
author: Aaron Lun
date: 28 June 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human pancreas from @xin2016rna. 
A matrix of RPKMs is provided in the Gene Expression Omnibus
under the accession [GSE81608](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE81608).
We download it using `r Biocpkg("BiocFileCache")` to cache the results:

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask=FALSE)    
rpkm.txt <- bfcrpath(bfc, file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE81nnn/GSE81608/suppl/GSE81608_human_islets_rpkm.txt.gz"))
```

We read the RPKMs into memory as a sparse matrix.

```{r}
library(scater)
mat <- readSparseCounts(rpkm.txt)
dim(mat)
```

# Preparing the column metadata

We download the metadata, which was supplied by the authors to Vladimir Kiselev, Tallulah Andrews and Martin Hemberg.

```{r}
col.path <- bfcrpath(bfc, file.path("https://s3.amazonaws.com/",
    "scrnaseq-public-datasets/manual-data/xin",
    "human_islet_cell_identity.txt"))
coldata <- read.delim(col.path, stringsAsFactors=FALSE, check.names=FALSE)
colnames(coldata)
```

We check that this is consistent:

```{r}
refnames <- sub(" ", "_", coldata[,1])
stopifnot(identical(colnames(mat), refnames))
```

We clean out uninteresting columns that have either all unique values or only one value.
We hold onto the sample ID for verification purposes later.

```{r}
keep <- vapply(coldata, function(x) {
    !length(unique(x)) %in% c(1L, length(x))
}, TRUE)
coldata <- coldata[,c(1, which(keep))]
```

Finally, we coerce it into a `DataFrame` for storage.

```{r}
library(S4Vectors)
coldata <- as(coldata, "DataFrame")
coldata
```

# Preparing the row metadata

We do the same for the row metadata, which is pretty straightforward.

```{r}
row.path <- bfcrpath(bfc, file.path("https://s3.amazonaws.com/",
    "scrnaseq-public-datasets/manual-data/xin",
    "human_gene_annotation.csv"))
rowdata <- read.csv(row.path, stringsAsFactors=FALSE, check.names=FALSE)
rowdata <- as(rowdata, "DataFrame")
stopifnot(identical(rownames(mat), as.character(rowdata[,1])))
rowdata
```

# Saving to file

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "xin-pancreas", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(mat, file=file.path(path, "rpkm.rds"))
saveRDS(rowdata, file=file.path(path, "rowdata.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Zeisel brain dataset
author: Aaron Lun
date: 18 May 2019
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of the mouse brain from @zeisel2015brain. 
Counts for endogenous genes, mitochondrial genes, repeats and spike-in transcripts are available from http://linnarssonlab.org/cortex.
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("https://storage.googleapis.com",
    "linnarsson-lab-www-blobs/blobs/cortex")
mRNA.path <- bfcrpath(bfc, file.path(base.url, 
    "expression_mRNA_17-Aug-2014.txt"))
rep.path <- bfcrpath(bfc, file.path(base.url, 
    "expression_rep_17-Aug-2014.txt"))
mito.path <- bfcrpath(bfc, file.path(base.url, 
    "expression_mito_17-Aug-2014.txt"))
spike.path <- bfcrpath(bfc, file.path(base.url, 
    "expression_spikes_17-Aug-2014.txt"))
```

We define a simple utility function for loading data in from each file. 
Each file contains some metadata, so we create a `SingleCellExperiment` object to accommodate both the counts and the metadata.

```{r}
library(SingleCellExperiment)
readFormat <- function(infile) { 
    # First column is empty.
    metadata <- read.delim(infile, stringsAsFactors=FALSE, header=FALSE, nrow=10)[,-1]
    rownames(metadata) <- metadata[,1]
    metadata <- metadata[,-1]
    metadata <- DataFrame(t(metadata), check.names=FALSE)

    # Cleaning up aspects of coercion to a DataFrame.
    to.coerce <- c("group #", "total mRNA mol", "well", "sex", "age", "diameter")
    for (x in colnames(metadata)) {
        if (x %in% to.coerce) {
            FUN <- as.numeric
        } else {
            FUN <- as.character
        }
        metadata[[x]] <- FUN(metadata[[x]])
    }
    rownames(metadata) <- NULL

    # First column after row names is some useless filler.
    counts <- read.delim(infile, stringsAsFactors=FALSE, 
        header=FALSE, row.names=1, skip=11)[,-1] 
    counts <- as.matrix(counts)
    colnames(counts) <- metadata$cell_id

    SingleCellExperiment(list(counts=counts), colData=metadata)
}
```

Using this function, we read in the counts for the endogenous genes, ERCC spike-in transcripts and mitochondrial genes.

```{r}
endo.data <- readFormat(mRNA.path)
endo.data
rep.data <- readFormat(rep.path)
rep.data
spike.data <- readFormat(spike.path)
spike.data
mito.data <- readFormat(mito.path)
mito.data
```

# Saving objects

We need to rearrange the columns for the mitochondrial data, as the order is not consistent with the other files.

```{r}
m <- match(endo.data$cell_id, mito.data$cell_id)
mito.data <- mito.data[,m]
```

```{r, echo=FALSE}
# Should all be the same.
stopifnot(identical(colData(endo.data), colData(spike.data))) 
stopifnot(identical(colData(endo.data), colData(rep.data)))
stopifnot(identical(colData(endo.data), colData(mito.data)))
```

We combine all components into a single `SingleCellExperiment` object.

```{r}
sce <- rbind(endo.data, mito.data, spike.data, rep.data)
rowData(sce)$featureType <- rep(c("endogenous", "mito", "ERCC", "repeat"),
    c(nrow(endo.data), nrow(mito.data), nrow(spike.data), nrow(rep.data)))
sce
```

We now save all of the relevant components of `sce` to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "zeisel-brain", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts(sce), file=file.path(path, "counts.rds"))
saveRDS(rowData(sce), file=file.path(path, "rowdata.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Fletcher Olfactory Epithelium (OE) dataset
author: Davide Risso
date: 9 February 2021
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the data

We obtain a single-cell RNA sequencing dataset of Olfactory Epithelium cells from multiple mice from @fletcher2017deconstructing.
Counts for the endogenous genes, spike-ins, and gene constructs are available from the Gene Expression Omnibus using the accession number [GSE95601](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE95601).
We download and cache them using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)

base <- "https://www.ncbi.nlm.nih.gov/geo/download/?acc=GSE95601&format=file&file="
fname <- bfcrpath(bfc, 
    paste0(base, "GSE95601%5FoeHBCdiff%5FCufflinks%5FeSet%5Fcounts%5Ftable%2Etxt%2Egz"))

protocol_data_file <- bfcrpath(bfc, 
    paste0(base, "GSE95601%5FoeHBCdiff%5FCufflinks%5FeSet%5FprotocolData%2Etxt%2Egz"))

pheno_data_file <- bfcrpath(bfc, 
    paste0(base, "GSE95601%5FoeHBCdiff%5FphenoData%2Etxt%2Egz"))

rowdata_file <- bfcrpath(bfc, 
    paste0(base, "GSE95601%5FoeHBCdiff%5FCufflinks%5FeSet%5FfeatureData%2Etxt%2Egz"))
```

Reading the counts in a matrix:

```{r}
counts <- read.table(fname)
counts <- as.matrix(counts)
dim(counts)
```

For some reason there are many `NA` values. 
Author response here:

> NA is equivalent to 0 in these matrices. I always produced matrices that had all the annotated genes in the reference transcriptome, and if no reads were detected than it was set to NA. Originally, I thought that distinguishing NAs and 0s for two slightly different cases (no reads in the current sample / no reads in any of the samples) could be useful, but this turned out not to be the case. So in the downstream analyses, I just overwrote the NA's with 0's right after reading the matrix.

So we'll just replace them as well.

```{r}
lost <- is.na(counts)
mean(lost)
counts[lost] <- 0L
```

# Adding metadata

Reading in the column data in dataframes:

```{r}
protocol_data <- read.table(protocol_data_file, header = TRUE)
pheno_data <- read.table(pheno_data_file, header = TRUE)

m1 <- match(colnames(counts), pheno_data$sample_sequencing_id)
m2 <- match(colnames(counts), protocol_data$sample_sequencing_id)
stopifnot(all(!is.na(m1)))
stopifnot(all(!is.na(m2)))

library(S4Vectors)
coldata <- cbind(pheno_data[m1,], protocol_data[m2,])
rownames(coldata) <- colnames(counts)
coldata <- DataFrame(coldata, check.names=FALSE)
colnames(coldata)
```

Repeating the dose for the row data:

```{r}
rowdata <- read.table(rowdata_file, header = TRUE)
rowdata <- DataFrame(rowdata, check.names=FALSE)
stopifnot(
    all(
        rownames(counts) == rowdata$Gene_Symbol |
        sub("\\.[0-9]$", "", rownames(counts)) == rowdata$Gene_Symbol
    )
)
rowdata
```

Attaching the cluster results from the original analysis (cluster labels available [here](https://raw.githubusercontent.com/rufletch/p63-HBC-diff/master/ref/clusterKey.md)):

```{r}
cluster_file <- bfcrpath(bfc, 
    "https://raw.githubusercontent.com/rufletch/p63-HBC-diff/master/ref/oeHBCdiff_clusterLabels.txt")
cluster_id <- read.table(cluster_file, col.names = c("sample_id", "cluster_id"))

cluster_labels <- data.frame(cluster_id = c(1:5, 7:12, 14, 15),
    cluster_label = c("HBC", 
        "INP1", 
        "GBC", 
        "mSUS", 
        "HBC2", 
        "iSUS", 
        "HBC1", 
        "iOSN", 
        "INP3", 
        "MVC1", 
        "mOSN", 
        "INP2", 
        "MVC2"),
    cluster_description = c("Resting Horizontal Basal Cells", 
        "Immediate Neuronal Precursor 1", 
        "Globose Basal Cells", 
        "Mature Sustentacular Cells", 
        "Transitional HBC 2", 
        "Immature Sustentacular Cells", 
        "Transitional HBC 1", 
        "Immature Olfactory Sensory Neurons", 
        "Immediate Neuronal Precursor 3", 
        "Microvillous Cells, type 1", 
        "Mature Olfactory Sensory Neurons", 
        "Immediate Neuronal Precursor 2", 
        "Microvillous Cells, type 2"))

m <- match(colnames(counts), cluster_id$sample_id)
coldata$retained <- !is.na(m)
coldata$cluster_id <- cluster_id$cluster_id[m]

mc <- match(coldata$cluster_id, cluster_labels$cluster_id)
stopifnot(all(!is.na(mc) | !coldata$retained))
coldata <- cbind(coldata, cluster_labels[mc,-1])
coldata
```

Creating a `SingleCellExperiment` object:

```{r}
library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=counts),
    colData = coldata, rowData = rowdata)
```

# Saving to file 

We now save all of the relevant components to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "fletcher-olfactory", "2.6.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(assay(sce), file=file.path(path, "counts.rds"))
saveRDS(colData(sce), file=file.path(path, "coldata.rds"))
saveRDS(rowData(sce), file=file.path(path, "rowdata.rds"))
```

# Session information {-}

```{r}
sessionInfo()
```

# References

---
title: Processing the Romanov brain dataset
author: Aaron Lun
date: "June 8, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---

```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of the mouse hypothalamus from @romanov2017molecular.
Counts for endogenous genes are available from the Gene Expression Omnibus
using the accession number [GSE74672](https://www.ncbi.nlm.nih.gov/geo/query/acc.cgi?acc=GSE74672).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
base.url <- file.path("ftp://ftp.ncbi.nlm.nih.gov/geo/series",
    "GSE74nnn/GSE74672/suppl")
count.file <- bfcrpath(bfc, file.path(base.url,
    "GSE74672_expressed_mols_with_classes.xlsx.gz"))
```

We load them into memory.

```{r}
library(R.utils)
tmp.file <- tempfile(fileext=".xlsx")
gunzip(count.file, destname=tmp.file, remove=FALSE)

# WARNING: this requires >20 GB of memory!
library(readxl)
all.data <- read_xlsx(tmp.file, col_names=FALSE)
dim(all.data)
```

# Extracting the metadata

The first 9 rows correspond to metadata and need to be extracted.

```{r}
coldata <- all.data[1:9,]
colfields <- coldata[,1,drop=TRUE]
coldata <- coldata[,-1]

coldata <- t(coldata)
coldata <- data.frame(coldata, stringsAsFactors=FALSE)
rownames(coldata) <- NULL

library(S4Vectors)
coldata <- DataFrame(coldata)
colnames(coldata) <- colfields

for (i in 4:9) {
    coldata[,i] <- as.numeric(coldata[,i])
}
coldata
```

The next three rows are empty, so we just skip them.
We convert the remaining counts into a sparse matrix.

```{r}
raw.counts <- all.data[-(1:12),]
counts <- as.matrix(raw.counts[,-1])

library(Matrix)
counts <- as(counts, "dgCMatrix")
rownames(counts) <- raw.counts[,1,drop=TRUE]
colnames(counts) <- NULL
dim(counts)
```

# Saving to file 

We now save all of the components to file for upload to `r Biocpkg("ExperimentHub")`.
These will be used to construct a `SingleCellExperiment` on the client side when the dataset is requested.

```{r}
path <- file.path("scRNAseq", "romanov-brain", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Messmer human ESC dataset
author: Aaron Lun
date: "September 6, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of human embryonic stem cells from @messmer2019transcriptional.
Counts for endogenous genes and spike-in transcripts are available from ArrayExpress
using the accession number [E-MTAB-6819](https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-6819/).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
messmer.zip <- bfcrpath(bfc, 
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-6819/E-MTAB-6819.processed.1.zip"))
unzip(messmer.zip, exdir=tempdir())
```

# Reading in the metadata

We read in the metadata from the SDRF file:

```{r}
messmer.sdrf <- bfcrpath(bfc, 
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-6819/E-MTAB-6819.sdrf.txt"))
coldata <- read.delim(messmer.sdrf, check.names=FALSE, stringsAsFactors=FALSE)

library(S4Vectors)
coldata <- as(coldata, "DataFrame")
colnames(coldata)
```

We sort by the batch number.
This is important for making sure that the `libnames` match up with the column names of the count matrix later.

```{r}
libnames <- coldata[["Assay Name"]]
o <- order(coldata[["Comment[sequencing run]"]], libnames)
coldata <- coldata[o,]
libnames <- libnames[o]
```

We keep only the experimentally interesting metadata, discarding columns that are duplicated or only have one level.
We also discard some ArrayExpress-specific columns.

```{r}
keep <- grep("(Characteristics|Factor|Parameter Value|Comment)", colnames(coldata))
coldata <- coldata[,c(1, keep)]  # keeping the cell ID.

new.colnames <- sub(".*\\[(.*)\\]", "\\1", colnames(coldata))
u <- !duplicated(new.colnames)
coldata <- coldata[,u]
colnames(coldata) <- new.colnames[u]

has.multi.levels <- vapply(coldata, FUN=function(x) length(unique(x))>1L, TRUE)
coldata <- coldata[,has.multi.levels]
coldata <- coldata[,setdiff(colnames(coldata), c("ENA_SAMPLE", 
    "BioSD_SAMPLE", "technical replicate group", "ENA_EXPERIMENT", 
    "SUBMITTED_FILE_NAME", "ENA_RUN", "FASTQ_URI",
    "single cell identifier", "RUN"))]
coldata
```

Many of these are actually technical replicates or multiple entries for paired data, 
so we only need to keep the first one of each set.

```{r}
first <- !duplicated(coldata[,1])
coldata <- coldata[first,]
libnames <- libnames[first]
dim(coldata)
```

# Processing the read counts

```{r, datain}
library(edgeR)
all.counts <- list()
cell.names <- list()
gene.names <- NULL
gene.length <- NULL

for (sample in c("2383", "2384", "2677", "2678", "2739", "2740", "2780", "2781")) {
    cur.file <- file.path(tempdir(), paste0("genic_counts_", sample, ".tsv"))
    current_counts <- read.table(cur.file, sep="\t", header=TRUE, row.names=1)

    # Checking gene names and length are the same as those in other files.
    if (is.null(gene.names)){
        gene.names <- rownames(current_counts)
        gene.length <- current_counts$Length
    } else {
        stopifnot(identical(gene.names, rownames(current_counts)))
        stopifnot(identical(gene.length, current_counts$Length))
    }
    current_counts$Length <- NULL

    # Take the technical replicates and merge them, if they exist.
    cellname <- colnames(current_counts)
    cellname <- sub("^lane[0-9]_", "", cellname)
    cellname <- sub("_L00[0-9]_", "_", cellname)
    cellname <- sub("_[12]$", "", cellname)

    if (any(duplicated(cellname))) {
        oldnames <- colnames(current_counts)
        current_counts <- sumTechReps(current_counts, ID=cellname)

        m <- match(colnames(current_counts), cellname)
        cellname <- colnames(current_counts)
        colnames(current_counts) <- oldnames[m]
        gc()
    }

    # Adding to the list.
    all.counts[[sample]] <- as.matrix(current_counts)
    cell.names[[sample]] <- cellname
}
sapply(all.counts, ncol)
```

We then merge technical replicates across batches (2677 + 2678, 2739 + 2740, 2780 + 2781).

```{r mergetech}
stopifnot(identical(cell.names[["2677"]], cell.names[["2678"]]))
all.counts[["2677"]] <- all.counts[["2677"]] + all.counts[["2678"]]
all.counts[["2678"]] <- NULL

stopifnot(identical(cell.names[["2739"]], cell.names[["2740"]]))
all.counts[["2739"]] <- all.counts[["2739"]] + all.counts[["2740"]]
all.counts[["2740"]] <- NULL

stopifnot(identical(cell.names[["2780"]], cell.names[["2781"]]))
all.counts[["2780"]] <- all.counts[["2780"]] + all.counts[["2781"]]
all.counts[["2781"]] <- NULL

sapply(all.counts, ncol)
```

Finally, we `cbind`  everything together into one large matrix.

```{r cbindmat}
combined.counts <- do.call(cbind, all.counts)
stopifnot(identical(colnames(combined.counts), libnames))
```

# Saving for upload 

We save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "messmer-esc", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(combined.counts, file=file.path(path, "counts.rds"))
saveRDS(coldata, file=file.path(path, "coldata.rds"))
saveRDS(DataFrame(Length=gene.length), file=file.path(path, "rowdata.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

---
title: Processing the Lun spike-in dataset
author: Aaron Lun
date: "June 24, 2019"
output:
  BiocStyle::html_document:
    titlecaps: false
    toc_float: true
bibliography: ../ref.bib
---


```{r style, echo=FALSE, results='hide', message=FALSE}
library(BiocStyle)
knitr::opts_chunk$set(error=FALSE, message=FALSE, warning=FALSE)
```

# Downloading the count data

We obtain a single-cell RNA sequencing dataset of 416B cells and trophoblasts from @lun2017assessing.
Counts for endogenous genes and spike-in transcripts are available from ArrayExpress
using the accession number [E-MTAB-5522](https://www.ebi.ac.uk/arrayexpress/experiments/E-MTAB-5522/).
We download and cache it using the `r Biocpkg("BiocFileCache")` package.

```{r}
library(BiocFileCache)
bfc <- BiocFileCache("raw_data", ask = FALSE)
lun.zip <- bfcrpath(bfc, 
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-5522/E-MTAB-5522.processed.1.zip"))
unzip(lun.zip, exdir=tempdir())
```

# Reading in the metadata

We read in the metadata from the SDRF file:

```{r}
lun.sdrf <- bfcrpath(bfc, 
    file.path("https://www.ebi.ac.uk/arrayexpress/files",
        "E-MTAB-5522/E-MTAB-5522.sdrf.txt"))
coldata <- read.delim(lun.sdrf, check.names=FALSE, stringsAsFactors=FALSE)

library(S4Vectors)
coldata <- as(coldata, "DataFrame")
colnames(coldata)
```

We keep only the experimentally interesting metadata, discarding columns that are duplicated or only have one level.

```{r}
keep <- grep("Characteristics|Factor", colnames(coldata))
coldata <- coldata[,c(1, keep)] # keeping the cell IDs.

new.colnames <- sub(".*\\[(.*)\\]", "\\1", colnames(coldata))
u <- !duplicated(new.colnames)
coldata <- coldata[,u]
colnames(coldata) <- new.colnames[u]

has.multi.levels <- vapply(coldata, FUN=function(x) length(unique(x))>1L, TRUE)
coldata <- coldata[,has.multi.levels]
head(coldata)
```

# Processing the 416B data

We load the counts into memory for the 416B cells.

```{r}
plate1.416b <- read.delim(file.path(tempdir(), "counts_Calero_20160113.tsv"),
    header=TRUE, row.names=1, check.names=FALSE)
plate2.416b <- read.delim(file.path(tempdir(), "counts_Calero_20160325.tsv"),
    header=TRUE, row.names=1, check.names=FALSE)
stopifnot(identical(rownames(plate1.416b), rownames(plate2.416b)))
```

We extract the gene lengths and combine the matrices together:

```{r}
gene.lengths <- plate1.416b$Length 
rowdata.416b <- DataFrame(Length=gene.lengths)
stopifnot(identical(gene.lengths, plate2.416b$Length))

plate1.416b <- as.matrix(plate1.416b[,-1])
plate2.416b <- as.matrix(plate2.416b[,-1])
counts.416b <- cbind(plate1.416b, plate2.416b)
dim(counts.416b)
```

We extract the relevant column data.

```{r}
m <- match(colnames(counts.416b), coldata[,1])
coldata.416b <- coldata[m,]
stopifnot(identical(colnames(counts.416b), coldata.416b[,1]))
```

And we save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "lun-spikein", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts.416b, file=file.path(path, "counts-416b.rds"))
saveRDS(rowdata.416b, file=file.path(path, "rowdata-416b.rds"))
saveRDS(coldata.416b, file=file.path(path, "coldata-416b.rds"))
```

# Processing the trophoblast data

We repeat this process for the trophoblasts.

```{r}
plate1.tropho <- read.delim(file.path(tempdir(), "counts_Liora_20160906.tsv"),
    header=TRUE, row.names=1, check.names=FALSE)
plate2.tropho <- read.delim(file.path(tempdir(), "counts_Liora_20170201.tsv"),
    header=TRUE, row.names=1, check.names=FALSE)
stopifnot(identical(rownames(plate1.tropho), rownames(plate2.tropho)))
```

We extract the gene lengths and combine the matrices together:

```{r}
gene.lengths <- plate1.tropho$Length 
rowdata.tropho <- DataFrame(Length=gene.lengths)
stopifnot(identical(gene.lengths, plate2.tropho$Length))

plate1.tropho <- as.matrix(plate1.tropho[,-1])
plate2.tropho <- as.matrix(plate2.tropho[,-1])
counts.tropho <- cbind(plate1.tropho, plate2.tropho)
dim(counts.tropho)
```

We extract the relevant column data.

```{r}
m <- match(colnames(counts.tropho), coldata[,1])
coldata.tropho <- coldata[m,]
stopifnot(identical(colnames(counts.tropho), coldata.tropho[,1]))
```

And we save these to file for upload to `r Biocpkg("ExperimentHub")`.

```{r}
path <- file.path("scRNAseq", "lun-spikein", "2.0.0")
dir.create(path, showWarnings=FALSE, recursive=TRUE)
saveRDS(counts.tropho, file=file.path(path, "counts-tropho.rds"))
saveRDS(rowdata.tropho, file=file.path(path, "rowdata-tropho.rds"))
saveRDS(coldata.tropho, file=file.path(path, "coldata-tropho.rds"))
```

# Session information

```{r}
sessionInfo()
```

# References

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LengESCData.R
\name{LengESCData}
\alias{LengESCData}
\title{Obtain the Leng ESC data}
\usage{
LengESCData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether gene symbols should be converted to Ensembl annotation.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of normalized expected read counts.
}
\description{
Obtain the human embryonic stem cell single-cell RNA-seq data from Leng et al. (2015).
}
\details{
Column metadata contains the cell line, experiment number and experimentally determined cell cycle phase for each cell.

If \code{ensembl=TRUE}, the gene symbols in the published annotation are converted to Ensembl.
If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/leng-esc}.
}
\examples{
sce <- LengESCData()

}
\references{
Leng F et al. (2015).
Oscope identifies oscillatory genes in unsynchronized single-cell RNA-seq experiments. 
\emph{Nat. Methods} 12(10), 947-950.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MairPBMCData.R
\name{MairPBMCData}
\alias{MairPBMCData}
\title{Obtain the Mair CITE-seq data}
\usage{
MairPBMCData(
  mode = c("rna", "adt"),
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{mode}{Character vector specifying whether to return either or both the RNA and ADT counts.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts corresponding to the first \code{mode},
with an optional alternative Experiment if there is a second \code{mode}.
}
\description{
Obtain the Mair PBMC targeted CITE-seq data from Mair et al. (2020).
}
\details{
Column metadata contains the donor identity and cartridge of origin.
Some libraries may also be classified as multiplets or have undeterminate origins after hash tag debarcoding.

If \code{ensembl=TRUE}, the gene symbols in the RNA data are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} for the RNA data.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/mair-pbmc}.
}
\examples{
sce <- MairPBMCData()

}
\references{
Mair C et al. (2020).
A targeted multi-omic analysis approach measures protein expression and low-abundance transcripts on the single-cell level.
\emph{Cell Rep.} 31, 107499
}
\author{
Stephany Orjuela,
with modifications from Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/DarmanisBrainData.R
\name{DarmanisBrainData}
\alias{DarmanisBrainData}
\title{Obtain the Darmanis brain data}
\usage{
DarmanisBrainData(
  ensembl = FALSE,
  location = TRUE,
  remove.htseq = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{remove.htseq}{Logical scalar indicating whether HT-seq alignment statistics should be removed.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human brain single-cell RNA-seq dataset from Darmanis et al. (2015).
}
\details{
Column metadata is scraped from GEO and includes patient information, tissue of origin and likely cell type. 

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
This is only performed when \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/darmanis-brain}.
}
\examples{
sce <- DarmanisBrainData()

}
\references{
Darmanis S et al. (2015). 
A survey of human brain transcriptome diversity at the single cell level. 
\emph{Proc Natl Acad Sci USA} 112, 7285-90.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PaulHSCData.R
\name{PaulHSCData}
\alias{PaulHSCData}
\title{Obtain the Paul HSC data}
\usage{
PaulHSCData(
  ensembl = FALSE,
  discard.multiple = TRUE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{discard.multiple}{Logical scalar indicating whether ambiguous rows should be discarded.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse haematopoietic stem cell single-cell RNA-seq data from Paul et al. (2015).
}
\details{
Column metadata includes the plate and the mouse of origin, fluoresence intensities from indexed sorting 
and the number of cells in each well.

Some of the original rownames are concatenated symbols from multiple genes.
We consider these rows to represent ambiguously assigned counts and discard them if \code{discard.multiple=TRUE}.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/nestorowa-hsc}.
}
\examples{
sce <- PaulHSCData()

}
\references{
Paul F et al. (2015). 
Transcriptional heterogeneity and lineage commitment in myeloid progenitors. 
\emph{Cell} 163, 1663-77.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KotliarovPBMCData.R
\name{KotliarovPBMCData}
\alias{KotliarovPBMCData}
\title{Obtain the Kotliarov CITE-seq data}
\usage{
KotliarovPBMCData(
  mode = c("rna", "adt"),
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{mode}{Character vector specifying whether to return either or both the RNA and ADT counts.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts corresponding to the first \code{mode},
with an optional alternative Experiment if there is a second \code{mode}.
}
\description{
Obtain the Kotliarov PBMC CITE-seq data from Kotliarov et al. (2020).
}
\details{
This dataset contains 20 samples from 2 experimental batches, where each batch contains 5 high and 5 low responders. 
The 10 samples per batch were mixed and distributed across the 6 lanes using a cell hashing approach.

The column metadata contains the following fields:
\itemize{
\item \code{sample*}: identifiers for the sample of origin for each cell.
\item \code{adjmfc.time}: type of responder for each sample.
\item \code{tenx_lane}: 10X lane from which each cell was collected.
\item \code{batch}: the batch of origin.
\item \code{barcode_check}: barcode identifier.
\item \code{hash_*} and \code{hto_*} columns: \pkg{HTOdemux} outputs.
\item \code{DEMUXLET.*} columns: \pkg{demuxlet} outputs.
\item \code{joint_classification_global}: \pkg{HTOdemux} and \pkg{demuxlet} joint classification.
\item \code{nGene}: number of genes as defined from \pkg{Seurat}'s \code{CreateSeuratObject}.
\item \code{nUMI}: number of UMIs as defined from \pkg{Seurat}'s \code{CreateSeuratObject}.
\item \code{pctMT}: percent of mitochondrial reads as defined from \pkg{Seurat}'s \code{CreateSeuratObject}.
}
Note, no filtering has been performed based on the quality control metrics.

If \code{ensembl=TRUE}, the gene symbols in the RNA data are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} for the RNA data.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/kotliarov-pbmc}.
}
\examples{
sce <- KotliarovPBMCData()

}
\references{
Kotliarov et al. (2020).
Broad immune activation underlies shared set point signatures for vaccine responsiveness in healthy individuals and disease activity in patients with lupus.
\emph{Nat. Med.} 26, 618–629
}
\author{
Stephany Orjuela,
with modifications from Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NestorowaHSCData.R
\name{NestorowaHSCData}
\alias{NestorowaHSCData}
\title{Obtain the Nestorowa HSC data}
\usage{
NestorowaHSCData(remove.htseq = TRUE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{remove.htseq}{Logical scalar indicating whether HT-seq alignment statistics should be removed.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse haematopoietic stem cell single-cell RNA-seq data from Nestorowa et al. (2015).
}
\details{
Rows corresponding to HT-seq's alignment statistics are removed by default.
These can be retained by setting \code{remove.htseq=FALSE}.

Column metadata includes the cell type mapping, as described on the website (see References),
and the FACS expression levels of selected markers.
Note that these are stored as nested matrices within the \code{\link[SummarizedExperiment]{colData}}.

Diffusion map components are provided as the \code{"diffusion"} entry in the \code{\link[SingleCellExperiment]{reducedDims}}.

Counts for ERCC spike-ins are stored in the \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}}.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/nestorowa-hsc}.
}
\examples{
sce <- NestorowaHSCData()

}
\references{
Nestorowa S et al. (2016). 
A single-cell resolution map of mouse hematopoietic stem and progenitor cell differentiation
\emph{Blood} 128, e20-e31.

Gene and protein expression in adult haematopoiesis: Data.
\url{http://blood.stemcells.cam.ac.uk/single_cell_atlas.html#data}.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZeiselBrainData.R
\name{ZeiselBrainData}
\alias{ZeiselBrainData}
\title{Obtain the Zeisel brain data}
\usage{
ZeiselBrainData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq dataset from Zeisel et al. (2015).
}
\details{
Row data contains a single \code{"featureType"} field describing the type of each feature
(endogenous genes, mitochondrial genes, spike-in transcripts and repeats).
Spike-ins and repeats are stored as separate entries in the \code{\link[SingleCellExperiment]{altExps}}.

Column metadata is provided in the same form as supplied in \url{http://linnarssonlab.org/cortex/}.
This contains information such as the cell diameter and the published cell type annotations.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

Spike-in metadata is added using \code{\link{ERCCSpikeInConcentrations}},
with molecule counts computed using a volume of 9 nL per cell at a dilution of 1:20000.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/zeisel-brain}.
}
\examples{
sce <- ZeiselBrainData()

}
\references{
Zeisel A et al. (2015). 
Brain structure. Cell types in the mouse cortex and hippocampus revealed by single-cell RNA-seq. 
\emph{Science} 347(6226), 1138-42.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listPaths.R
\name{listPaths}
\alias{listPaths}
\title{List all paths for a dataset}
\usage{
listPaths(
  name,
  version,
  package = "scRNAseq",
  cache = cacheDirectory(),
  overwrite = FALSE,
  include.metadata = FALSE
)
}
\arguments{
\item{name}{String containing the name of the dataset.}

\item{version}{String containing the version of the dataset.}

\item{package}{String containing the name of the package.}

\item{cache, overwrite}{Arguments to pass to \code{\link[gypsum]{saveVersion}} or \code{\link[gypsum]{saveFile}}.}

\item{include.metadata}{Logical scalar indicating whether to report the metadata for each subdataset.}
}
\value{
If \code{include.metadata=FALSE}, a character vector containing the paths to subdatasets within the specified version of the dataset.
If \code{name} does not contain any subdatasets, \code{NA} is returned.

Otherwise, a \link[S4Vectors]{DFrame} is returned containing the metadata for each subdataset, e.g., the title and description.
More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}.
}
\description{
List the available paths to subdatasets within a version of a given dataset.
}
\examples{
listPaths("he-organs-2020", "2023-12-21")
listPaths("he-organs-2020", "2023-12-21", include.metadata=TRUE)
listPaths("zeisel-brain-2015", "2023-12-14") # no subdatasets
listPaths("zeisel-brain-2015", "2023-12-14", include.metadata=TRUE) 

}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GrunHSCData.R
\name{GrunHSCData}
\alias{GrunHSCData}
\title{Obtain the Grun HSC data}
\usage{
GrunHSCData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse haematopoietic stem cell single-cell RNA-seq data from Grun et al. (2016).
}
\details{
Row metadata contains the symbol and chromosomal location for each gene.
Column metadata contains the extraction protocol used for each sample, as described in GSE76983.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/grun-hsc}.
}
\examples{
sce <- GrunHSCData()

}
\references{
Grun D et al. (2016). 
De novo prediction of stem cell identity using single-cell transcriptome data. 
\emph{Cell Stem Cell} 19(2), 266-277.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listVersions.R
\name{listVersions}
\alias{listVersions}
\alias{fetchLatestVersion}
\title{List available versions}
\usage{
listVersions(name)

fetchLatestVersion(name)
}
\arguments{
\item{name}{String containing the name of the dataset.}
}
\value{
For \code{listVersions}, a character vector containing the names of the available versions of the dataset.

For \code{fetchLatestVersion}, a string containing the name of the latest version.
}
\description{
List the available and latest versions for a dataset.
}
\examples{
listVersions("zeisel-brain-2015")
fetchLatestVersion("zeisel-brain-2015")

}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GiladiHSCData.R
\name{GiladiHSCData}
\alias{GiladiHSCData}
\title{Obtain the Giladi HSC data}
\usage{
GiladiHSCData(
  mode = c("rna", "crispr"),
  filtered = TRUE,
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{mode}{Character vector specifying which modalities should be returned.}

\item{filtered}{Logical scalar indicating whether to filter out cells that were not used by the authors.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers, when \code{mode} contains \code{"rna"}.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned, when \code{mode} contains \code{"rna"}.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a matrix of UMI counts for the scRNA-seq or CRISPR-seq data.
Alternatively, an object with both count matrices, where the second modality is stored as an alternative Experiment.
}
\description{
Obtain the mouse haematopoietic stem cell single-cell RNA-seq and CRISPR-seq dataset from Giladi et al. (2018).
}
\details{
Column metadata is scraped from GEO using the author-supplied TSV of per-cell annotations. 
This contains information such as the batch of origin for each cell plus an array of FACS measurements per cell.

If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
Otherwise, an additional \code{filtered} field will be present in the \code{\link[SummarizedExperiment]{colData}}, indicating whether the cell was retained by the authors. 

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
For row names with multiple semi-colon-delimited symbols, the last symbol is used for matching against the Ensembl annotation.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
This is only relevant when \code{ensembl=TRUE}.

If \code{mode} contains multiple modalities, the intersection of cells that are present in both modalities is returned.
This is because not all cells have data across both modalities.
If \code{mode} contains only one modality, all cells for that modality are returned.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/giladi-hsc}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- GiladiHSCData()
}
}
\references{
Giladi A et al. (2018).
Single-cell characterization of haematopoietic progenitors and their trajectories in homeostasis and perturbed haematopoiesis. 
\emph{Nat Cell Biol} 20, 836-846
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HuCortexData.R
\name{HuCortexData}
\alias{HuCortexData}
\title{Obtain the Hu cortex data}
\usage{
HuCortexData(
  mode = c("ctx", "3T3"),
  samples = NULL,
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{mode}{Character vector indicating whether to return data for the 3T3 cells or the mouse cortex.}

\item{samples}{Character vector indicating whether to return data for specific samples, see Details.
If specified, this overrides \code{mode}.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse cortex single-nuclei RNA-seq data from Hu et al. (2017).
}
\details{
Column metadata includes the mode and sample corresponding to each cell/nuclei.
Available samples are:
\itemize{
\item \code{"cell-3T3"} and \code{"nuclei-3T3"}, generated from the 3T3 cell line.
\item \code{"nuclei-ctx-X"}, nuclei generated from the cortex of animal number X (from 1 to 13).
\item \code{"nuclei-ctx-salineX"} or \code{"nuclei-ctx-PTZX"}, nuclei generated from the cortex of saline- or PTZ-treated mice.
X represents the replicate number and can be 1 or 2.
}

If multiple modes are requested, counts are only reported for the intersection of genes across all modes.
This is because the gene annotation in the original count matrices differs across modes.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/wu-kidney}.
}
\examples{
sce <- HuCortexData("3T3")

}
\references{
Hu P et al. (2017). 
Dissecting cell-type composition and activity-dependent transcriptional state in mammalian brains by massively parallel single-nucleus RNA-seq.
\emph{Mol. cell} 68, 1006-1015.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/SegerstolpePancreasData.R
\name{SegerstolpePancreasData}
\alias{SegerstolpePancreasData}
\title{Obtain the Segerstolpe pancreas data}
\usage{
SegerstolpePancreasData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Download the human pancreas single-cell RNA-seq (scRNA-seq) dataset from Segerstolpe et al. (2016)
}
\details{
Row data contains fields for the gene symbol and RefSeq transcript IDs corresponding to each gene.
The rows of the output object are named with the symbol, but note that these are not unique.

Column metadata were extracted from the \code{Characteristics} fields of the SDRF file for ArrayExpress E-MTAB-5061.
This contains information such as the cell type labels and patient status.

Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
Estimated numbers of spike-in molecules are provided in the \code{\link[SummarizedExperiment]{rowData}} of this entry. 
Note that these concentrations are incorrect for donor H1, as 100 uL of spike-in mixture were added for this donor, rather than 25 uL for all others.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/segerstolpe-pancreas}.
}
\examples{
sce <- SegerstolpePancreasData()

}
\references{
Segerstolpe A et al. (2016). 
Single-cell transcriptome profiling of human pancreatic islets in health and type 2 diabetes. 
\emph{Cell Metab.} 24(4), 593-607.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/TasicBrainData.R
\name{TasicBrainData}
\alias{TasicBrainData}
\title{Obtain the Tasic brain data}
\usage{
TasicBrainData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq data from Tasic et al. (2015).
}
\details{
Column metadata is provided in the same form as supplied in GSE71585.
This contains information such as the reporter gene expressed in each cell, the mouse line, dissection type and so on.

Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
Note that some of the spike-in rows have \code{NA} observations for some (but not all) cells.

The last 9 columns (containing \code{_CTX_} in their names) correspond to no-cell control libraries.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/tasic-brain}.
}
\examples{
sce <- TasicBrainData()

}
\references{
Tasic A et al. (2016). 
Adult mouse cortical cell taxonomy revealed by single cell transcriptomics.
\emph{Nat. Neurosci.} 19(2), 335-46.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LawlorPancreasData.R
\name{LawlorPancreasData}
\alias{LawlorPancreasData}
\title{Obtain the Lawlor pancreas data}
\usage{
LawlorPancreasData(legacy = FALSE)
}
\arguments{
\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Provides the human pancreas single-cell RNA-seq data from Lawlor et al. (2017).
}
\details{
Column metadata is provided in the same form as supplied in GSE86469. 
This contains information such as the cell type labels and patient status.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/lawlor-pancreas}.
}
\examples{
sce <- LawlorPancreasData()

}
\references{
Lawlor N et al. (2017). 
Single-cell transcriptomes identify human islet cell signatures and reveal cell-type-specific expression changes in type 2 diabetes.
\emph{Genome Res.} 27(2), 208-222.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/surveyDatasets.R
\name{surveyDatasets}
\alias{surveyDatasets}
\title{Survey of dataset metadata}
\usage{
surveyDatasets(cache = cacheDirectory(), overwrite = FALSE, latest = TRUE)
}
\arguments{
\item{cache, overwrite}{Arguments to pass to \code{\link[gypsum]{fetchMetadataDatabase}}.}

\item{latest}{Whether to only consider the latest version of each dataset.}
}
\value{
A \link[S4Vectors]{DataFrame} where each row corresponds to a (sub)dataset, containing various columns of metadata.
Some columns may be lists to capture 1:many mappings.
}
\description{
Metadata survey for all available datasets in the \pkg{scRNAseq} package.
}
\details{
The returned DataFrame contains the usual suspects like the title and description for each dataset,
the number of rows and columns, the organisms and genome builds involved,
whether the dataset has any pre-computed reduced dimensions, and so on.
More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 

If a dataset contains multiple subdatasets, each subdataset is reported as a separate row in the DataFrame.
The location of subdataset is provided in the \code{path} column.
If a dataset does not contain any subdatasets, the \code{path} entry will be set to \code{NA}.
}
\examples{
surveyDatasets()

}
\seealso{
\code{\link{searchDatasets}}, to search on the metadata for specific datasets.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/GrunPancreasData.R
\name{GrunPancreasData}
\alias{GrunPancreasData}
\title{Obtain the Grun pancreas data}
\usage{
GrunPancreasData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human pancreas single-cell RNA-seq data from Grun et al. (2016).
}
\details{
Row metadata contains fields for the symbol and chromosomal location of each gene,
as derived from the row names.

Column metadata is derived from the column names of the count matrix with the sample annotations in GSE81076.
This includes the donor identity for each cell and the type of sample.

The \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}} contains count data for the ERCC spike-in transcripts.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/grun-pancreas}.
}
\examples{
sce <- GrunPancreasData()

}
\references{
Grun D et al. (2016). 
De novo prediction of stem cell identity using single-cell transcriptome data. 
\emph{Cell Stem Cell} 19(2), 266-277.
}
\author{
Aaron Lun,
using additional metadata obtained by Vladimir Kiselev.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BuettnerESCData.R
\name{BuettnerESCData}
\alias{BuettnerESCData}
\title{Obtain the Buettner ESC data}
\usage{
BuettnerESCData(remove.htseq = TRUE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{remove.htseq}{Logical scalar indicating whether HT-seq alignment statistics should be removed.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse embryonic stem cell single-cell RNA-seq data from Buettner et al. (2015).
}
\details{
Rows corresponding to HT-seq's alignment statistics are removed by default.
These can be retained by setting \code{remove.htseq=FALSE}.

Column metadata contains the experimentally determined cell cycle phase for each cell.

Counts for ERCC spike-ins are stored in the \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}}.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/buettner-esc}.
}
\examples{
sce <- BuettnerESCData()

}
\references{
Buettner F et al. (2015).
Computational analysis of cell-to-cell heterogeneity in single-cell RNA-sequencing data reveals hidden subpopulations of cells.
\emph{Nat. Biotechnol.} 33(2), 155-160.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HermannSpermatogenesisData.R
\name{HermannSpermatogenesisData}
\alias{HermannSpermatogenesisData}
\title{Obtain the Hermann spermatogenesis data}
\usage{
HermannSpermatogenesisData(strip = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{strip}{Logical scalar indicating whether to strip the \code{.X} notation from the row names.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.
Only used if \code{strip=TRUE}.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with two matrices, containing spliced and unspliced counts, respectively.
}
\description{
Obtain the mouse spermatogenesis single-cell RNA-seq data from Hermann et al. (2018).
}
\details{
Column metadata contains cell types provided by the data generators
at \url{https://data.mendeley.com/datasets/kxd5f8vpt4/1#file-fe79c10b-c42e-472e-9c7e-9a9873d9b3d8}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/hermann-spermatogenesis}.
}
\examples{
sce <- HermannSpermatogenesisData()

}
\references{
Hermann B.P. et al. (2018).
The Mammalian Spermatogenesis Single-Cell Transcriptome, from Spermatogonial Stem Cells to Spermatids. 
\emph{Cell Rep.} 25(6), 1650-1667.e8.
}
\author{
Charlotte Soneson
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/NowakowskiCortexData.R
\name{NowakowskiCortexData}
\alias{NowakowskiCortexData}
\title{Obtain the Nowakowski cortex data}
\usage{
NowakowskiCortexData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of TPMs.
The \code{\link[SingleCellExperiment]{reducedDims}} contains an assortment of dimensionality reduction results.
}
\description{
Obtain the human cortex single-cell RNA-seq dataset from Nowakowski et al. (2017).
}
\details{
Column metadata includes the presumed cell type (\code{WGCNAcluster}), patient and tissue region of origin. 
A variety of dimensionality reduction results are also provided.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
This is only performed when \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/nowakowski-cortex}.
}
\examples{
sce <- NowakowskiCortexData()

}
\references{
Nowakowski S et al. (2017). 
Spatiotemporal gene expression trajectories reveal developmental hierarchies of the human cortex. 
\emph{Science} 358, 1318-1323.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BhaduriOrganoidData.R
\name{BhaduriOrganoidData}
\alias{BhaduriOrganoidData}
\title{Obtain the Bhaduri cortical organoid data}
\usage{
BhaduriOrganoidData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of normalized expression values.
}
\description{
Obtain the human cortical organoid single-cell RNA-seq dataset from Bhaduri et al. (2020).
}
\details{
Column data contains sample-level information.
In theory, there is also cell-level metadata for this dataset but it could not be unambiguously mapped to the column names.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/bhaduri-organoid}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- BhaduriOrganoidData()
}
}
\references{
Bhaduri A et al. (2020). 
Cell stress in cortical organoids impairs molecular subtype specification.
\emph{Nature} 578(7793), 142-148.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZhongPrefrontalData.R
\name{ZhongPrefrontalData}
\alias{ZhongPrefrontalData}
\title{Obtain the Zhong prefrontal cortex data}
\usage{
ZhongPrefrontalData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human prefrontal cortex single-cell RNA-seq dataset from Zhong et al. (2018).
}
\details{
Column metadata is scraped from GEO and includes week of gestation, gender and likely cell type.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
This is only performed when \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/zhong-prefrontal}.
}
\examples{
sce <- ZhongPrefrontalData()

}
\references{
Zhong S et al. (2018). 
A single-cell RNA-seq survey of the developmental landscape of the human prefrontal cortex. 
\emph{Nature} 555, 524-528.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/XinPancreasData.R
\name{XinPancreasData}
\alias{XinPancreasData}
\title{Obtain the Xin pancreas data}
\usage{
XinPancreasData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of RPKMs.
}
\description{
Obtain the human pancreas single-cell RNA-seq dataset from Xin et al. (2016).
}
\details{
Row data contains fields for the Entrez ID and symbol for each gene.
Column metadata was obtained from the authors (indirectly, via the Hemberg group) 
and contains information such as the cell type labels and donor status.

If \code{ensembl=TRUE}, the Entrez IDs are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/xin-pancreas}.
}
\examples{
sce <- XinPancreasData()

}
\references{
Xin A et al. (2016).
RNA sequencing of single human islet cells reveals type 2 diabetes genes.
\emph{Cell Metab.} 24(4), 608-615.
}
\author{
Aaron Lun,
using additional metadata obtained by Vladimir Kiselev.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZilionisLungData.R
\name{ZilionisLungData}
\alias{ZilionisLungData}
\title{Obtain the Zilionis lung cancer data}
\usage{
ZilionisLungData(
  which = c("human", "mouse"),
  ensembl = FALSE,
  location = TRUE,
  filter = FALSE,
  legacy = FALSE
)
}
\arguments{
\item{which}{String specifying the species to get data for.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{filter}{Logical scalar indicating if the filtered subset should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the human/mouse lung cancer single-cell RNA-seq data from Zilionis et al. (2019).
}
\details{
Column metadata is provided and contains information on the library, donor ID/animal ID, replicate and tissue.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

If \code{filter=TRUE}, only cells that have been used in the original analysis are returned.
The cells used are specified in the \code{Used} column of the \code{\link[SummarizedExperiment]{colData}}.

The \code{\link[SingleCellExperiment]{reducedDim}} contains coordinates of SPRING representations.
This may be filled with \code{NA}s for SPRING coordinates computed on a subset of cells (specified in \code{\link[SummarizedExperiment]{colData}}).

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/zilionis-lung}.
}
\examples{
sce.human <- ZilionisLungData()

sce.mouse <- ZilionisLungData("mouse")

}
\references{
Zilionis R et al. (2019). 
Single-cell transcriptomics of human and mouse lung cancers reveals 
conserved myeloid populations across individuals and species. 
\emph{Immunity} 50(5), 1317-1334.
}
\author{
Jens Preussner
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RichardTCellData.R
\name{RichardTCellData}
\alias{RichardTCellData}
\title{Obtain the Richard T cell data}
\usage{
RichardTCellData(location = TRUE, legacy = FALSE)
}
\arguments{
\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse CD8+ T cell single-cell RNA-seq data from Richard et al. (2018).
}
\details{
Column metadata is provided in the same form as supplied in E-MTAB-6051.
This contains information such as the stimulus, time after stimulation, age of the mice and sequencing batch.

Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/richard-tcell}.
}
\examples{
sce <- RichardTCellData()

}
\references{
Richard AC et al. (2018). 
T cell cytolytic capacity is independent of initial stimulation strength. 
\emph{Nat. Immunol.} 19(8), 849-858.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/FletcherOlfactoryData.R
\name{FletcherOlfactoryData}
\alias{FletcherOlfactoryData}
\title{Obtain the Fletcher Olfactory data}
\usage{
FletcherOlfactoryData(
  filtered = TRUE,
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{filtered}{Logical scalar indicating whether to filter out cells that were not used by the authors.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse olfactory epithelial HBC stem cell differentiation dataset from Fletcher et al. (2017).
}
\details{
Column metadata is scraped from GEO, using both the author-supplied \dQuote{phenoData} per-cell annotations and the author-supplied \dQuote{protocolData} per-cell annotations. 
The former includes information about the animals and the instruments, while the latter contains QC statistics.

We also included the clustering results from the authors' analysis.

If \code{filtered=TRUE}, only the cells used by the authors in their cluster analysis are returned. 
Otherwise, the cells not used by the authors will have \code{NA} in the clustering columns of the \code{\link[SummarizedExperiment]{colData}}.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object. 
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/fletcher-olfactory}.
}
\examples{
sce <- FletcherOlfactoryData()

}
\references{
Fletcher R et al. (2017). 
Deconstructing olfactory stem cell trajectories at single-cell resolution. 
\emph{Cell Stem Cell} 20, 817-30.
}
\author{
Davide Risso
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/StoeckiusHashingData.R
\name{StoeckiusHashingData}
\alias{StoeckiusHashingData}
\title{Obtain the Stoeckius cell hashing data}
\usage{
StoeckiusHashingData(
  type = c("pbmc", "mixed"),
  mode = NULL,
  ensembl = FALSE,
  location = TRUE,
  strip.metrics = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{type}{String specifying the dataset to obtain.}

\item{mode}{String specifying the data modalities to obtain, see Details.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{strip.metrics}{Logical scalar indicating whether quality control metrics should be removed from the HTO/ADT counts.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a matrix of UMI counts corresponding to the first \code{mode},
plus any number of alternative Experiments containing the remaining \code{mode}s.
If multiple \code{mode}s are specified, the output object only contains the intersection of their column names.
}
\description{
Obtain the (mostly human) cell hashing single-cell RNA-seq data from Stoeckius et al. (2018).
}
\details{
When \code{type="pbmc"}, the \code{mode} can be one or more of:
\itemize{
\item \code{"human"}, the RNA counts for human genes.
\item \code{"mouse"}, the RNA counts for mouse genes.
Present as the PBMC dataset is actually a mixture of human PBMCs and unlabelled mouse cells.
\item \code{"hto"}, the HTO counts.
\item \code{"adt1"}, counts for the first set of ADTs (immunoglobulin controls).
\item \code{"adt2"}, counts for the second set of ADTs (cell type-specific markers).
}
If \code{mode=NULL}, the default is to use \code{"human"}, \code{"mouse"} and \code{"hto"}.

When \code{type="mixed"}, the \code{mode} can be one or more of:
\itemize{
\item \code{"rna"}, the RNA counts for the genes;
\item \code{"hto"}, the HTO counts.
}
If \code{mode=NULL}, the default is to use \code{"rna"} and \code{"hto"}.

If \code{ensembl=TRUE}, gene symbols for the RNA counts are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE} and only for the RNA counts.

For the HTO and ADT matrices, some rows correspond to quality control metrics.
If \code{strip.metrics=TRUE}, these rows are removed so that only data for actual HTOs or ADTs are present.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/nestorowa-hsc}.
}
\examples{
sce.pbmc <- StoeckiusHashingData()
sce.pbmc

sce.mixed <- StoeckiusHashingData(type="mixed")
sce.mixed

}
\references{
Stoeckius et al. (2018). 
Cell Hashing with barcoded antibodies enables multiplexing and doublet detection for single cell genomics.
\emph{Genome Biol.} 19, 224.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/saveDataset.R
\name{saveDataset}
\alias{saveDataset}
\title{Save a dataset to disk}
\usage{
saveDataset(x, path, metadata)
}
\arguments{
\item{x}{A \link[SummarizedExperiment]{SummarizedExperiment} or one of its subclasses.}

\item{path}{String containing the path to a new directory in which to save \code{x}.
Any existing directory is removed before saving \code{x}.}

\item{metadata}{Named list containing metadata for this dataset,
see the schema returned by \code{\link[gypsum]{fetchMetadataSchema}()}.
Note that the \code{applications.takane} property will be automatically added by this function and does not have to be supplied.}
}
\value{
\code{x} and its metadata are saved into \code{path}, and \code{NULL} is invisibly returned.
}
\description{
Save a single-cell dataset to disk, usually in preparation for upload.
}
\examples{
library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=matrix(rpois(1000, lambda=1), 100, 10)))
rownames(sce) <- sprintf("GENE_\%i", seq_len(nrow(sce)))
colnames(sce) <- head(LETTERS, 10)

meta <- list(
    title="My dataset",
    description="This is my dataset",
    taxonomy_id="10090",
    genome="GRCh38",
    sources=list(list(provider="GEO", id="GSE12345")),
    maintainer_name="Shizuka Mogami",
    maintainer_email="mogami.shizuka@765pro.com"
)

tmp <- tempfile()
saveDataset(sce, tmp, meta)
list.files(tmp, recursive=TRUE)
alabaster.base::readObject(tmp)

}
\seealso{
\url{https://github.com/ArtifactDB/bioconductor-metadata-index}, on the expected schema for the metadata.

\code{\link{polishDataset}}, to polish \code{x} before saving it.

\code{\link[gypsum]{uploadDirectory}}, to upload the saved contents.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/RomanovBrainData.R
\name{RomanovBrainData}
\alias{RomanovBrainData}
\title{Obtain the Romanov brain data}
\usage{
RomanovBrainData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq dataset from Romanov et al. (2017).
}
\details{
Column metadata is provided in the same form as supplied in GSE74672.
This contains information such as the reporter gene expressed in each cell, the mouse line, dissection type and so on.

Counts for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.
Note that some of the spike-in rows have \code{NA} observations for some (but not all) cells.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/romanov-brain}.
}
\examples{
sce <- RomanovBrainData()

}
\references{
Romanov RA et al. (2017).
Molecular interrogation of hypothalamic organization reveals distinct dopamine neuronal subtypes. 
\emph{Nat. Neurosci.} 20, 176-188.
}
\author{
Aaron Lun,
based on code by Vladimir Kiselev and Tallulah Andrews.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/PollenGliaData.R
\name{PollenGliaData}
\alias{PollenGliaData}
\title{Obtain the Pollen radial glia data}
\usage{
PollenGliaData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the human radial glia single-cell RNA-seq dataset from Pollen et al. (2017).
}
\details{
Column metadata includes the anatomical source, sample of origin, presumed cell type and assorted alignment statistics.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
This is only performed when \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/pollen-glia}.
}
\examples{
sce <- PollenGliaData()

}
\references{
Pollen A et al. (2017). 
Molecular identity of human outer radial glia during cortical development.
\emph{Cell} 163, 55-67.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MessmerESCData.R
\name{MessmerESCData}
\alias{MessmerESCData}
\title{Obtain the Messmer ESC data}
\usage{
MessmerESCData(location = TRUE, legacy = FALSE)
}
\arguments{
\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the human embryonic stem cell single-cell RNA-seq data from Messmer et al. (2019).
}
\details{
Row data contains a single \code{"Length"} field describing the total exonic length of each feature.

Column metadata is provided in the same form as supplied in E-MTAB-6819.
This contains information such as the cell phenotype (naive or primed) and the batch of origin.
Note that counts for technical replicates have already been summed together.

Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/messmer-esc}.
}
\examples{
sce <- MessmerESCData()

}
\references{
Messmer T et al. (2019). 
Transcriptional heterogeneity in naive and primed human pluripotent stem cells at single-cell resolution. 
\emph{Cell Rep} 26(4), 815-824.e4
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/polishDataset.R
\name{polishDataset}
\alias{polishDataset}
\title{Polish dataset for saving}
\usage{
polishDataset(
  x,
  strip.inner.names = TRUE,
  reformat.assay.by.density = 0.3,
  attempt.integer.conversion = TRUE,
  remove.altexp.coldata = TRUE,
  forbid.nested.altexp = TRUE
)
}
\arguments{
\item{x}{A \link[SummarizedExperiment]{SummarizedExperiment} or one of its subclasses.}

\item{strip.inner.names}{Logical scalar indicating whether to strip redundant names from internal objects, 
e.g., dimnames of the assays, row names of reduced dimensions, column names of alternative experiments.
This saves some space in the on-disk representation.}

\item{reformat.assay.by.density}{Numeric scalar indicating whether to optimize assay formats based on the density of non-zero values.
Assays with densities above this number are converted to ordinary dense arrays (if they are not already), while those with lower densities are converted to sparse matrices.
This can be disabled by setting it to \code{NULL}.}

\item{attempt.integer.conversion}{Logical scalar indicating whether to convert double-precision assays containing integer values to actually have the integer type.
This can improve efficiency of downstream applications by avoiding the need to operate in double precision.}

\item{remove.altexp.coldata}{Logical scalar indicating whether column data for alternative experiments should be removed.
This defaults to \code{TRUE} as the alternative experiment column data is usually redundant with that of the main experiment.}

\item{forbid.nested.altexp}{Logical scalar indicating whether nested alternative experiments (i.e., alternative experiments of alternative experiments) should be forbidden.
This defaults to \code{TRUE} as nested alternative experiments are usually the result of some mistake in \code{\link[SingleCellExperiment]{altExp}} preparation.}
}
\value{
A modified copy of \code{x}.
}
\description{
Prepare a \link[SummarizedExperiment]{SummarizedExperiment} or \link[SingleCellExperiment]{SingleCellExperiment} to be saved with \code{\link{saveDataset}}.
This performs some minor changes to improve storage efficiency.
}
\examples{
mat <- matrix(rpois(1000, lambda=0.2), 100, 10) * 1.0
rownames(mat) <- sprintf("GENE_\%i", seq_len(nrow(mat)))
colnames(mat) <- head(LETTERS, 10)

library(SingleCellExperiment)
sce <- SingleCellExperiment(list(counts=mat))
str(assay(sce, withDimnames=FALSE))

polished <- polishDataset(sce)
str(assay(polished, withDimnames=FALSE))

}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/HeOrganAtlasData.R
\name{HeOrganAtlasData}
\alias{HeOrganAtlasData}
\title{Obtain the He organ atlas data}
\usage{
HeOrganAtlasData(
  tissue = c("Bladder", "Blood", "Common.bile.duct", "Esophagus", "Heart", "Liver",
    "Lymph.node", "Marrow", "Muscle", "Rectum", "Skin", "Small.intestine", "Spleen",
    "Stomach", "Trachea"),
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{tissue}{Character vector specifying the tissues to return.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human cortex single-nuclei RNA-seq data from Hu et al. (2017).
}
\details{
Column data contains the tissue of origin, a variety of per-cell QC metrics well as some cell type annotations.
The reclustered annotations required some assembly:
\itemize{
\item \code{reclustered.broad} was generated based on whether the barcode was present in each \code{*_meta.data.txt} file at \url{https://github.com/bei-lab/scRNA-AHCA}.
\item For each barcode that was present in one of those files, \code{reclustered.fine} was generated based on the label in the \code{annotation} field inside that file.
}

If multiple tissues are requested, counts are only reported for the intersection of genes across all tissues.
This is because the gene annotation in the original count matrices differs across tissues.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/he-organ-atlas}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- HeOrganAtlasData()
}

}
\references{
He S et al. (2020).
Single-cell transcriptome profiling of an adult human cell atlas of 15 major organs. 
\emph{Genome Biol} 21, 1:294.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LaMannoBrainData.R
\name{LaMannoBrainData}
\alias{LaMannoBrainData}
\title{Obtain the La Manno brain data}
\usage{
LaMannoBrainData(
  which = c("human-es", "human-embryo", "human-ips", "mouse-adult", "mouse-embryo"),
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{which}{A string specifying which dataset should be obtained.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse/human brain scRNA-seq data from La Manno et al. (2016).
}
\details{
Column metadata is provided in the same form as supplied in the supplementary tables in GSE71585.
This contains information such as the time point and cell type.

The various settings of \code{which} will obtain different data sets.
\itemize{
\item \code{"human-es"}, human embryonic stem cells.
\item \code{"human-embryo"}, human embryo midbrain.
\item \code{"human-ips"}, human induced pluripotent stem cells.
\item \code{"mouse-adult"}, mouse adult dopaminergic neurons.
\item \code{"mouse-embryo"}, mouse embryo midbrain.
}
Unfortunately, each of these datasets uses a different set of features.
If multiple datasets are to be used simultaneously, users will have to decide how to merge them,
e.g., by taking the intersection of common features across all datasets.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/lamanno-brain}.
}
\examples{
sce.h.es <- LaMannoBrainData()

sce.h.em <- LaMannoBrainData("human-embryo")

sce.h.ip <- LaMannoBrainData("human-ips")

sce.m.ad <- LaMannoBrainData("mouse-adult")

sce.m.em <- LaMannoBrainData("mouse-embryo")

}
\references{
La Manno A et al. (2016).
Molecular diversity of midbrain development in mouse, human, and stem cells. 
\emph{Cell} 167(2), 566-580.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LedergorMyelomaData.R
\name{LedergorMyelomaData}
\alias{LedergorMyelomaData}
\title{Obtain the Ledergor Myeloma data}
\usage{
LedergorMyelomaData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the human multiple myeloma single-cell RNA-seq data from Ledergor et al. (2018).
}
\details{
Column metadata was created from the sample metadata file in GSE117156. It contains an 'Experiment_ID' column,
from which the tissue and subject of origin were extracted, as well as the condition and treatment status of the subject.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/ledergor-myeloma}.
}
\examples{
sce <- LedergorMyelomaData()

}
\references{
Ledergor G et al. (2018)
Single cell dissection of plasma cell heterogeneity in symptomatic and
asymptomatic myeloma. \emph{Nat Med.} 24(12), 1867–1876.
}
\author{
Milan Malfait
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ChenBrainData.R
\name{ChenBrainData}
\alias{ChenBrainData}
\title{Obtain the Chen brain data}
\usage{
ChenBrainData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq data from Chen et al. (2017).
}
\details{
Column metadata is provided in the same form as supplied in GSE87544.
This contains the putative cell type assigned by the original authors.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/chen-brain}.
}
\examples{
sce <- ChenBrainData()

}
\references{
Chen R et al. (2017).
Single-Cell RNA-Seq reveals hypothalamic cell diversity. 
\emph{Cell Rep.} 18, 3227-3241.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/searchDatasets.R
\name{searchDatasets}
\alias{searchDatasets}
\title{Search dataset metadata}
\usage{
searchDatasets(
  query,
  cache = cacheDirectory(),
  overwrite = FALSE,
  latest = TRUE
)
}
\arguments{
\item{query}{String containing a query in a human-readable syntax or a \link[gypsum]{gypsum.search.clause}, see Examples.}

\item{cache, overwrite}{Arguments to pass to \code{\link[gypsum]{fetchMetadataDatabase}}.}

\item{latest}{Whether to only consider the latest version of each dataset.}
}
\value{
A \link[S4Vectors]{DataFrame} where each row corresponds to a (sub)dataset, containing various columns of metadata.
Some columns may be lists to capture 1:many mappings.
}
\description{
Search for datasets of interest based on matching text in the associated metadata.
}
\details{
The returned DataFrame contains the usual suspects like the title and description for each dataset,
the number of rows and columns, the organisms and genome builds involved,
whether the dataset has any pre-computed reduced dimensions, and so on.
More details can be found in the Bioconductor metadata schema at \url{https://github.com/ArtifactDB/bioconductor-metadata-index}. 

If a dataset contains multiple subdatasets, each subdataset is reported as a separate row in the DataFrame.
The location of subdataset is provided in the \code{path} column.
If a dataset does not contain any subdatasets, the \code{path} entry will be set to \code{NA}.
}
\examples{
searchDatasets("brain")[,c("name", "title")]
searchDatasets("Neuro\%")[,c("name", "title")]
searchDatasets("taxonomy_id:10090")[,c("name", "title")]
searchDatasets("(genome: GRCm38 AND neuro\%) OR pancrea\%")[,c("name", "title")]

# We can also use gypsum search clauses via the gsc() function.
# Here, 'asset' is analogous to the 'name' of the dataset.
searchDatasets(gsc(asset="he-organs-2020"))[,c("path")]
searchDatasets(
    (gsc(asset="\%brain\%", partial=TRUE) |
    gsc(asset="\%neur\%", partial=TRUE) |
    gsc(text="\%neur\%", partial=TRUE) |
    gsc(text="\%brain\%", partial=TRUE)) &
    gsc(field="taxonomy_id", text="10090")
)[,c("name", "title")]

}
\seealso{
\code{\link{surveyDatasets}}, to easily obtain a listing of all available datasets.

\code{\link[gypsum]{translateTextQuery}}, for details on the human-readable query syntax.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MuraroPancreasData.R
\name{MuraroPancreasData}
\alias{MuraroPancreasData}
\title{Obtain the Muraro pancreas data}
\usage{
MuraroPancreasData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human pancreas single-cell RNA-seq data from Muraro et al. (2016).
}
\details{
Row data contains fields for the symbol and chromosomal location of each gene.

Column metadata is derived from the columns of the count matrix provided in GSE85241,
with additional cell type labels obtained from the authors (indirectly, via the Hemberg group).
Some cells have \code{NA} labels and were presumably removed prior to downstream analyses.

Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/muraro-pancreas}.
}
\examples{
sce <- MuraroPancreasData()

}
\references{
Muraro MJ et al. (2016). 
A single-cell transcriptome atlas of the human pancreas.
\emph{Cell Syst.} 3(4), 385-394.
}
\author{
Aaron Lun,
using additional metadata obtained by Vladimir Kiselev.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BacherTCellData.R
\name{BacherTCellData}
\alias{BacherTCellData}
\title{Obtain the Bacher T cell data}
\usage{
BacherTCellData(filtered = TRUE, ensembl = FALSE, location = TRUE)
}
\arguments{
\item{filtered}{Logical scalar indicating whether to filter out cells that were not used by the authors.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}
}
\value{
A \linkS4class{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human COVID T cell single-cell RNA-seq dataset from Bacher et al. (2020).
}
\details{
Column metadata is scraped from GEO, using both the author-supplied TSV of per-cell annotations and the sample-level metadata.
This contains information such as the diagnosis, severity, WHO class, clustering and clonotype.

If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
Otherwise, an additional \code{filtered} field will be present in the \code{\link{colData}}, indicating whether the cell was retained by the authors. 

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/bacher-tcell}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- BacherTCellData()
}

}
\references{
Bacher P et al. (2020). 
Low avidity T cell responses to SARS-CoV-2 in unexposed individuals and severe COVID-19
\emph{Immunity} 53, 1258-1271
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ERCCSpikeInConcentrations.R
\name{ERCCSpikeInConcentrations}
\alias{ERCCSpikeInConcentrations}
\title{Obtain ERCC concentrations}
\usage{
ERCCSpikeInConcentrations(
  volume = NULL,
  dilution = NULL,
  mix = c("1", "2"),
  legacy = FALSE
)
}
\arguments{
\item{volume}{Numeric scalar specifying the added volume (in nanoliters) of ERCC spike-in mixture.
Only used if \code{dilution} is specified.}

\item{dilution}{Numeric scalar specifying the dilution factor used for the added volume of the spike-in mixture.
Only used if \code{volume} is specified.}

\item{mix}{String specifying whether to compute the number of molecules for mix 1 or 2.
Only used if both \code{dilution} and \code{volume} are specified.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[S4Vectors]{DataFrame} object with one row per ERCC spike-in transcript.
This contains information such as the spike-in concentration in each mix.
}
\description{
Obtain ERCC spike-in concentrations from the Thermo Fisher Scientific website.
}
\details{
If \code{volume} and \code{dilution} are specified,
an additional column is added to the output specifying the number of molecules of spike-in transcipt for the specified mix.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/ercc-concentrations}.
}
\examples{
df <- ERCCSpikeInConcentrations()

}
\author{
Alan O'Callaghan
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fetchDataset.R
\name{fetchDataset}
\alias{fetchDataset}
\alias{fetchMetadata}
\title{Fetch a dataset from the gypsum backend}
\usage{
fetchDataset(
  name,
  version,
  path = NA,
  package = "scRNAseq",
  cache = cacheDirectory(),
  overwrite = FALSE,
  realize.assays = FALSE,
  realize.reduced.dims = TRUE,
  ...
)

fetchMetadata(
  name,
  version,
  path = NA,
  package = "scRNAseq",
  cache = cacheDirectory(),
  overwrite = FALSE
)
}
\arguments{
\item{name}{String containing the name of the dataset.}

\item{version}{String containing the version of the dataset.}

\item{path}{String containing the path to a subdataset, if \code{name} consists of multiple subdatasets.
Defaults to \code{NA} if no subdatasets are present.}

\item{package}{String containing the name of the package.}

\item{cache, overwrite}{Arguments to pass to \code{\link[gypsum]{saveVersion}} or \code{\link[gypsum]{saveFile}}.}

\item{realize.assays, realize.reduced.dims}{Logical scalars indicating whether to realize assays and reduced dimensions into memory.
Dense and sparse \link[alabaster.matrix]{ReloadedArray} objects are converted into ordinary arrays and dgCMatrix objects, respectively.}

\item{...}{Further arguments to pass to \code{\link[alabaster.base]{readObject}}.}
}
\value{
\code{fetchDataset} returns the dataset as a \link[SummarizedExperiment]{SummarizedExperiment} or one of its subclasses.

\code{fetchMetadata} returns a named list of metadata for the specified dataset.
}
\description{
Fetch a dataset (or its metadata) from the gypsum backend.
}
\examples{
fetchDataset("zeisel-brain-2015", "2023-12-14")
fetchMetadata("zeisel-brain-2015", "2023-12-14")

}
\seealso{
\url{https://github.com/ArtifactDB/bioconductor-metadata-index}, on the expected schema for the metadata.

\code{\link{saveDataset}} and \code{\link[gypsum]{uploadDirectory}}, to save and upload a dataset.

\code{\link{surveyDatasets}} and \code{\link{listVersions}}, to get possible values for \code{name} and \code{version}.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BachMammaryData.R
\name{BachMammaryData}
\alias{BachMammaryData}
\title{Obtain the Bach mammary data}
\usage{
BachMammaryData(
  samples = c("NP_1", "NP_2", "G_1", "G_2", "L_1", "L_2", "PI_1", "PI_2"),
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{samples}{A character vector with at least one element, specifying which samples(s) to retrieve.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse mammary gland single-cell RNA-seq data from Bach et al. (2017).
}
\details{
Column metadata is extracted from the sample annotation in GSE106273,
and refers to the developmental stage of the mammary gland.

If multiple samples are specified in \code{samples}, the count matrices will be \code{cbind}ed together.
Cells originating from different samples are identifiable by the \code{"Sample"} field in the column metadata.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/bach-mammary}.
}
\examples{
sce <- BachMammaryData(samples="NP_1")

}
\references{
Bach K et al. (2017).
Differentiation dynamics of mammary epithelial cells revealed by single-cell RNA sequencing. 
\emph{Nat Commun.} 8(1), 2128
}
\author{
Aaron Lun
}

\name{scRNAseq-package}
\alias{scRNAseq-package}
\alias{scRNAseq}
\alias{fluidigm}
\alias{th2}
\alias{allen}

\docType{package}
\title{
\packageTitle{scRNAseq}
}
\description{
\packageDescription{scRNAseq}
}
\details{

This package contains a collection of three publicly available single-cell RNA-seq datasets.

The dataset \code{fluidigm} contains 65 cells from Pollen et al. (2014), each sequenced at high and low coverage.

The dataset \code{th2} contains 96 T helper cells from Mahata et al. (2014).

The dataset \code{allen} contains 379 cells from the mouse visual cortex. This is a subset of the data published in Tasic et al. (2016).

See the package vignette for details on the pre-processing of the data.

}
\author{
\packageAuthor{scRNAseq}

Maintainer: \packageMaintainer{scRNAseq}
}

\references{

Pollen, Nowakowski, Shuga, Wang, Leyrat, Lui, Li, Szpankowski, Fowler, Chen, Ramalingam, Sun, Thu, Norris, Lebofsky, Toppani, Kemp II, Wong, Clerkson, Jones, Wu, Knutsson, Alvarado, Wang, Weaver, May, Jones, Unger, Kriegstein, West. Low-coverage single-cell mRNA sequencing reveals cellular heterogeneity and activated signaling pathways in developing cerebral cortex. Nature Biotechnology, 32, 1053-1058 (2014).

Mahata, Zhang, Kolodziejczyk, Proserpio, Haim-Vilmovsky, Taylor, Hebenstreit, Dingler, Moignard, Gottgens, Arlt, McKenzie, Teichmann. Single-Cell RNA Sequencing Reveals T Helper Cells Synthesizing Steroids De Novo to Contribute to Immune Homeostasis. Cell Reports, 7(4): 1130–1142 (2014).

Tasic, Menon, Nguyen, Kim, Jarsky, Yao, Levi, Gray, Sorensen, Dolbeare, Bertagnolli, Goldy, Shapovalova, Parry, Lee, Smith, Bernard, Madisen, Sunkin, Hawrylycz, Koch, Zeng. Adult mouse cortical cell taxonomy revealed by single cell transcriptomics. Nature Neuroscience, 19, 335–346 (2016).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MacoskoRetinaData.R
\name{MacoskoRetinaData}
\alias{MacoskoRetinaData}
\title{Obtain the Macosko retina data}
\usage{
MacoskoRetinaData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse retina single-cell RNA-seq data from Macosko et al. (2016).
}
\details{
Column metadata contains the cluster identity as reported in the paper.
Note that some cells will have \code{NA} identities as they are present in the count matrix but not in the metadata file.
These are presumably low-quality cells that were discarded prior to clustering.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/macosko-retina}.
}
\examples{
sce <- MacoskoRetinaData()

}
\references{
Macosko E et al. (2016). 
Highly parallel genome-wide expression profiling of individual cells using nanoliter droplets. 
\emph{Cell} 161(5), 1202-1214.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZeiselNervousData.R
\name{ZeiselNervousData}
\alias{ZeiselNervousData}
\title{Obtain the Zeisel nervous system data}
\usage{
ZeiselNervousData(location = TRUE, legacy = FALSE)
}
\arguments{
\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse nervous system single-cell RNA-seq dataset from Zeisel et al. (2018).
}
\details{
Row data contains the gene symbol as well as some relevant per-gene statistics,
e.g., the squared coefficient of variance, mean, and whether it was selected for downstream analyses.

Column data contains a wide variety of fields including patient-level information, sample-level sequencing statistics and many flavors of cell type classification.
Note that many numeric columns may have \code{NA} values if they could not be successfully parsed form the source file.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/zeisel-nervous}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- ZeiselNervousData()
}
}
\references{
Zeisel A et al. (2018). 
Molecular architecture of the mouse nervous system.
\emph{Cell} 174(4), 999-1014.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/MarquesBrainData.R
\name{MarquesBrainData}
\alias{MarquesBrainData}
\title{Obtain the Marques brain data}
\usage{
MarquesBrainData(ensembl = FALSE, location = TRUE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}
}
\value{
A \linkS4class{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq data from Marques et al. (2016).
}
\details{
Column metadata is provided in the same form as supplied in GSE75330. 
This contains information such as the cell type and age/sex of the mouse of origin for each cell.

Note that some genes may be present in multiple rows corresponding to different genomic locations.
These additional rows are identified by a \code{_loc[2-9]} suffix in their row names.
Users may wish to consider either removing them or merging them, e.g., with \code{scater::sumCountsAcrossFeatures}.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.
All searching is performed after removing the \code{_loc[2-9]} suffix. 

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/marques-brain}.
}
\examples{
sce <- MarquesBrainData()

}
\references{
Marques A et al. (2016).
Oligodendrocyte heterogeneity in the mouse juvenile and adult central nervous system. 
\emph{Science} 352(6291), 1326-1329.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/listDatasets.R
\name{listDatasets}
\alias{listDatasets}
\title{List all available datasets}
\usage{
listDatasets()
}
\value{
A \link[S4Vectors]{DataFrame} where each row corresponds to a dataset, containing the fields:
\itemize{
\item \code{Reference}, a Markdown-formatted citation to \code{scripts/ref.bib} in the \pkg{scRNAseq} installation directory.
\item \code{Taxonomy}, an identifier for the organism.
\item \code{Part}, the part of the organism being studied.
\item \code{Number}, the total number of cells in the dataset.
\item \code{Call}, the relevant R call required to construct the dataset.
}
}
\description{
Summary information for all available datasets in the \pkg{scRNAseq} package.
}
\details{
A study may contribute multiple datasets if they cannot be reasonably combined (e.g., different species).
The reported number of cells refers only to the dataset as it is stored in \pkg{scRNAseq};
this may be different to the number of cells used by the authors in their analysis, e.g., due to filtering.
}
\examples{
listDatasets()

}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/UsoskinBrainData.R
\name{UsoskinBrainData}
\alias{UsoskinBrainData}
\title{Obtain the Usoskin brain data}
\usage{
UsoskinBrainData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of RPMs.
}
\description{
Obtain the mouse brain single-cell RNA-seq dataset from Usoskin et al. (2015).
}
\details{
Column metadata is provided in the same form as supplied in External Table 2 of \url{http://linnarssonlab.org/drg/}.
This contains information such as the library of origin and the cell type.

The count matrix contains information for repeats, marked with \code{r_} prefixes in the row names;
as well as mitochondrial transcripts, marked with \code{mt-} prefixes.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/usoskin-brain}.
}
\examples{
sce <- UsoskinBrainData()

}
\references{
Usoskin A et al. (2015).
Unbiased classification of sensory neuron types by large-scale single-cell RNA sequencing. 
\emph{Nat. Neurosci.} 18(1), 145-53.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/AztekinTailData.R
\name{AztekinTailData}
\alias{AztekinTailData}
\title{Obtain the Aztekin tail data}
\usage{
AztekinTailData(legacy = FALSE)
}
\arguments{
\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the Xenopus tail single-cell RNA-seq data from Aztekin et al. (2019).
}
\details{
Column metadata is provided in the same form as supplied in E-MTAB-7761.
This contains information such as the treatment condition, batch, putative cell type, putative cell cycle phase.

The UMAP results are available as the \code{"UMAP"} entry in the \code{\link[SingleCellExperiment]{reducedDims}}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/aztekin-tail}.
}
\examples{
sce <- AztekinTailData()

}
\references{
Aztekin C et al. (2019).
Identification of a regeneration-organizing cell in the Xenopus tail.
\emph{Science} 364(6441), 653-658
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/CampbellBrainData.R
\name{CampbellBrainData}
\alias{CampbellBrainData}
\title{Obtain the Campbell brain data}
\usage{
CampbellBrainData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the row names of the returned object should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq data from Campbell et al. (2017).
}
\details{
Column metadata is provided in the same form as supplied in GSE93374.
This contains information such as the diet of the mice, sex and proposed cell type for each cell.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/campbell-brain}.
}
\examples{
sce <- CampbellBrainData()

}
\references{
Campbell R et al. (2017).
A molecular census of arcuate hypothalamus and median eminence cell types. 
\emph{Nat. Neurosci.} 20, 484-496.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BaronPancreasData.R
\name{BaronPancreasData}
\alias{BaronPancreasData}
\title{Obtain the Baron pancreas data}
\usage{
BaronPancreasData(
  which = c("human", "mouse"),
  ensembl = FALSE,
  location = TRUE
)
}
\arguments{
\item{which}{String specifying the species to get data for.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}
}
\value{
A \linkS4class{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the human/mouse pancreas single-cell RNA-seq data from Baron et al. (2017).
}
\details{
Column metadata is provided in the same form as supplied in GSE84133.
This contains information such as the cell type labels and donor ID (for humans) or strain (for mouse).

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/baron-pancreas}.
}
\examples{
sce.human <- BaronPancreasData()

sce.mouse <- BaronPancreasData("mouse")

}
\references{
Baron M et al. (2017). 
Single-cell transcriptomic map of the human and mouse pancreas reveals inter- and intra-cell population structure. 
\emph{Cell Syst.} 3(4), 346-360.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ShekharRetinaData.R
\name{ShekharRetinaData}
\alias{ShekharRetinaData}
\title{Obtain the Shekhar retina data}
\usage{
ShekharRetinaData(ensembl = FALSE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse retina single-cell RNA-seq dataset from Shekhar et al. (2016).
}
\details{
Column metadata contains the cluster identities as reported in the paper.
Note that some cells will have \code{NA} identities as they are present in the count matrix but not in the metadata file.
These are presumably low-quality cells that were discarded prior to clustering.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/shekhar-retina}.
}
\examples{
sce <- ShekharRetinaData()

}
\references{
Shekhar K et al. (2016). 
Comprehensive classification of retinal bipolar neurons by single-cell transcriptomics.
\emph{Cell} 166(5), 1308-1323.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ErnstSpermatogenesisData.R
\name{ErnstSpermatogenesisData}
\alias{ErnstSpermatogenesisData}
\title{Obtain the Ernst spermatogenesis data}
\usage{
ErnstSpermatogenesisData(
  method = c("emptyDrops", "Cellranger"),
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{method}{String indicating which cell caller to obtain results for.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse spermatogenesis single-cell RNA-seq dataset from Ernst et al. (2019).
}
\details{
This study contains two analyses done with datasets from different cell calling algorithms.
One uses Cellranger version 2 while the other uses \code{emptyDrops} from \pkg{DropletUtils}.

Column metadata includes sample information, per-cell QC metrics and cell type labels.
In particular, the sample label specifies the developmental stage of the mouse.

Note that \code{method="Cellranger"} contains additional data for Tc1 mice.
These mice have an additional human chromosome 21 inserted alongside the usual mouse chromosomes.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/ernst-spermatogenesis}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- ErnstSpermatogenesisData()
}

}
\references{
Ernst C et al. (2019). 
Staged developmental mapping and X chromosome transcriptional dynamics during mouse spermatogenesis.
\emph{Nat Commun} 10, 1251
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/countErccMolecules.R
\name{countErccMolecules}
\alias{countErccMolecules}
\title{Obtain ERCC molecule counts from concentrations}
\usage{
countErccMolecules(volume, dilution, mix = c("1", "2"), ...)
}
\arguments{
\item{volume}{Numeric scalar specifying the added volume (in microliters) of ERCC spike-in mixture.}

\item{dilution}{Numeric scalar specifying the dilution factor used for the added volume of the spike-in mixture.}

\item{mix}{String specifying whether to compute the number of molecules for mix 1 or 2.}

\item{...}{Further arguments to pass to \code{\link{fetchDataset}}.}
}
\value{
A \link[S4Vectors]{DataFrame} object with one row per ERCC spike-in transcript.
This contains the estimated concentration and molecule count for each transcript.
}
\description{
Compute the number of molecules for each transcript in the ERCC spike-in mixture,
based on their published concentration as well as the volume of the diluted mixture added to each cell.
}
\examples{
countErccMolecules(volume = 9, dilution = 300000)

}
\author{
Aaron Lun, based on code from Alan O'Callaghan
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/KolodziejczykESCData.R
\name{KolodziejczykESCData}
\alias{KolodziejczykESCData}
\title{Obtain the Kolodziejcyzk ESC data}
\usage{
KolodziejczykESCData(remove.htseq = TRUE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{remove.htseq}{Logical scalar indicating whether HT-seq alignment statistics should be removed.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse embryonic stem cell single-cell RNA-seq data from Kolodziejczyk et al. (2015).
}
\details{
Column metadata is generated from the column names,
and contains the culture conditions and the plate of origin for each cell.

Count data for ERCC spike-ins are stored in the \code{"ERCC"} entry in the \code{\link[SingleCellExperiment]{altExps}}.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/kolodziejczyk-esc}.
}
\examples{
sce <- KolodziejczykESCData()

}
\references{
Messmer T et al. (2019). 
Transcriptional heterogeneity in naive and primed human pluripotent stem cells at single-cell resolution. 
\emph{Cell Rep} 26(4), 815-824.e4
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ReprocessedData.R
\name{ReprocessedAllenData}
\alias{ReprocessedAllenData}
\alias{ReprocessedTh2Data}
\alias{ReprocessedFluidigmData}
\title{Reprocessed single-cell data sets}
\usage{
ReprocessedAllenData(
  assays = NULL,
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)

ReprocessedTh2Data(
  assays = NULL,
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)

ReprocessedFluidigmData(
  assays = NULL,
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{assays}{Character vector specifying one or more assays to return.
Choices are \code{"tophat_counts"}, \code{"cufflinks_fpkm"}, \code{"rsem_counts"} and \code{"rsem_tpm"}.
If \code{NULL}, all assays are returned.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object containing one or more expression matrices of counts and/or TPMs,
depending on \code{assays}.
}
\description{
Obtain the legacy count matrices for three publicly available single-cell RNA-seq datasets.
Raw sequencing data were downloaded from NCBI's SRA or from EBI's ArrayExpress,
aligned to the relevant genome build and used to quantify gene expression.
}
\details{
\code{ReprocessedFluidigmData} returns a dataset of 65 human neural cells from Pollen et al. (2014), 
each sequenced at high and low coverage (SRA accession SRP041736).

\code{ReprocessedTh2Data} returns a dataset of 96 mouse T helper cells from Mahata et al. (2014),
obtained from ArrayExpress accession E-MTAB-2512.
Spike-in counts are stored in the \code{"ERCC"} entry of the \code{\link[SingleCellExperiment]{altExps}}.

\code{ReprocessedAllenData} return a dataset of 379 mouse brain cells from Tasic et al. (2016).
This is a re-processed subset of the data from \code{\link{TasicBrainData}},
and contains spike-in information stored as in the \code{\link[SingleCellExperiment]{altExps}}.

In each dataset, the first columns of the \code{colData} are sample quality metrics from FastQC and Picard.
The remaining fields were obtained from the original study in their GEO/SRA submission
and/or as Supplementary files in the associated publication.
These two categories of \code{colData} are distinguished by a \code{which_qc} element in the \code{\link[S4Vectors]{metadata}},
which contains the names of the quality-related columns in each object.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/legacy-allen},
\code{scRNAseq/legacy-fluidigm} or \code{scRNAseq/legacy-th2}.
}
\section{Pre-processing details}{

FASTQ files were either obtained directly from ArrayExpress, 
or converted from SRA files (downloaded from the Sequence Read Archive) using the SRA Toolkit.

Reads were aligned with TopHat (v. 2.0.11) to the appropriate reference genome (GRCh38 for human samples, GRCm38 for mouse). 
RefSeq mouse gene annotation (GCF_000001635.23_GRCm38.p3) was downloaded from NCBI on Dec. 28, 2014. 
RefSeq human gene annotation (GCF_000001405.28) was downloaded from NCBI on Jun. 22, 2015.

featureCounts (v. 1.4.6-p3) was used to compute gene-level read counts.
Cufflinks (v. 2.2.0) was used to compute gene-leve FPKMs.
Reads were also mapped to the transcriptome using RSEM (v. 1.2.19) to compute read counts and TPM's.

FastQC (v. 0.10.1) and Picard (v. 1.128) were used to compute sample quality control (QC) metrics. 
However, no filtering on the QC metrics has been performed for any dataset.
}

\examples{
sce <- ReprocessedAllenData()

}
\references{
Pollen AA et al. (2014). 
Low-coverage single-cell mRNA sequencing reveals cellular heterogeneity and activated signaling pathways in developing cerebral cortex. 
\emph{Nat. Biotechnol.} 32(10), 1053-8.

Mahata B et al. (2014).
Single-cell RNA sequencing reveals T helper cells synthesizing steroids de novo to contribute to immune homeostasis. 
\emph{Cell Rep}, 7(4), 1130-42.

Tasic A et al. (2016). 
Adult mouse cortical cell taxonomy revealed by single cell transcriptomics.
\emph{Nat. Neurosci.} 19(2), 335-46.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/JessaBrainData.R
\name{JessaBrainData}
\alias{JessaBrainData}
\title{Obtain the Jessa brain data}
\usage{
JessaBrainData(filtered = TRUE, location = TRUE, legacy = FALSE)
}
\arguments{
\item{filtered}{Logical scalar indicating whether to filter out cells that were not used by the authors.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the mouse brain single-cell RNA-seq dataset from Jessa et al. (2019).
}
\details{
If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
Otherwise, an additional \code{filtered} field will be present in the \code{\link[SummarizedExperiment]{colData}}, indicating whether the cell was retained by the authors. 

The column data contains sample of origin, some QC metrics and various cluster assignments for each cell.
Cluster assignments starting with \code{Sample_*} are derived from per-sample analyses and cannot be compared sensibly across samples.
Other clusterings (\code{Forebrain_*} and \code{Pons_*}) are derived from joint analyses across all samples involving the named tissue.

The \code{reducedDims} of the output contains various dimensionality reduction results.
Coordinates for entries prefixed with \code{Sample_*} were generated from per-sample analyses and cannot be compared across samples.
Coordinates for entries prefixed with \code{Forebrain_*} and \code{Pons_*} were generated from joint analyses from the corresponding tissue. 

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/jessa-brain}.
}
\examples{
if (.Machine$sizeof.pointer > 4) { # too large for 32-bit machines!
    sce <- JessaBrainData()
} 
}
\references{
Jessa S et al. (2019).
Stalled developmental programs at the root of pediatric brain tumors
\emph{Nat Genet} 51, 1702-1713
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/WuKidneyData.R
\name{WuKidneyData}
\alias{WuKidneyData}
\title{Obtain the Wu kidney data}
\usage{
WuKidneyData(
  mode = c("healthy", "disease"),
  ensembl = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{mode}{String indicating whether to return data for healthy and/or diseased donors.}

\item{ensembl}{Logical scalar indicating whether the output row names should contain Ensembl identifiers.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the mouse kidney single-nuclei RNA-seq data from Wu et al. (2019).
}
\details{
Column metadata includes the single-cell technology and whether they came from a diseased or healthy individual.

If \code{mode} specifies both healthy and disease donors,
counts are only reported for the intersection of genes that are present for both donors.
This is because the original count matrices had differences in their annotation.

If \code{ensembl=TRUE}, the gene symbols are converted to Ensembl IDs in the row names of the output object.
Rows with missing Ensembl IDs are discarded, and only the first occurrence of duplicated IDs is retained.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/wu-kidney}.
}
\examples{
sce <- WuKidneyData("disease")

}
\references{
Wu H et al. (2019). 
Advantages of single-nucleus over single-cell RNA sequencing of adult kidney: rare cell types and novel cell states revealed in fibrosis.
\emph{J. Am. Soc. Nephrol.} 30, 23-32.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BunisHSPCData.R
\name{BunisHSPCData}
\alias{BunisHSPCData}
\title{Obtain the Bunis haematopoietic stem and progenitor cell data}
\usage{
BunisHSPCData(filtered = TRUE, legacy = FALSE)
}
\arguments{
\item{filtered}{Logical scalar or "cells" indicating whether to:
\itemize{
\item \code{TRUE}: filter out cells that were not used by the authors.
\item \code{"cells"}: filter out empty droplets as filtered out by cell ranger.
\item \code{FALSE}: no filtering
}}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \linkS4class{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human fetal, newborn, and adult haematopoietic stem and progenitor cell single-cell RNA-seq dataset from Bunis et al. (2021).
}
\details{
Column metadata is recreated from GEO using the author-supplied TSV of per-cell annotations, or retrieved from a processed version of the data shared by authors via figshare.
This contains information such as the tissue & sample of origin, age group, likely cell type, and Developmental Stage Scoring.
Within DevStageScoring element of the column metadata are the applied results ('<cell_type>_scores') of random forest regression trained on the fetal (score = 0) and adult (score = 1) cells of individual cell types indicated by ('<cell_type>_inTraining').

If \code{filtered=TRUE}, only the cells used by the authors in their final analysis are returned.
Otherwise, an additional \code{retained} field will be present in the \code{\link{colData}}, indicating whether the cell was retained by the authors.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/bunis-hspc}.
}
\examples{
sce <- BunisHSPCData()

}
\references{
Bunis DG et al. (2021). 
Single-Cell Mapping of Progressive Fetal-to-Adult Transition in Human Naive T Cells
\emph{Cell Rep.} 34(1): 108573
}
\author{
Daniel Bunis
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/LunSpikeInData.R
\name{LunSpikeInData}
\alias{LunSpikeInData}
\title{Obtain the Lun spike-in data}
\usage{
LunSpikeInData(
  which = c("416b", "tropho"),
  split.oncogene = FALSE,
  location = TRUE,
  legacy = FALSE
)
}
\arguments{
\item{which}{String specifying whether the 416B or trophoblast data should be obtained.}

\item{split.oncogene}{Logical scalar indicating whether the oncogene should be split to a separate \code{\link[SingleCellExperiment]{altExp}}.}

\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of read counts.
}
\description{
Obtain the spike-in single-cell RNA-seq data from Lun et al. (2017).
}
\details{
Row data contains a single \code{"Length"} field describing the total exonic length of each feature.

Column metadata is provided in the same form as supplied in E-MTAB-5522.
This contains information such as the cell type, plate of origin, spike-in addition order and oncogene induction. 

Two sets of spike-ins were added to each cell in each dataset.
These are available as the \code{"SIRV"} and \code{"ERCC"} entries in the \code{\link[SingleCellExperiment]{altExps}}.

If \code{split.oncogene=TRUE} and \code{which="416b"},
the CBFB-MYH11-mcherry oncogene is moved to extra \code{"oncogene"} entry in the \code{\link[SingleCellExperiment]{altExps}}.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/lun-spikein}.
}
\examples{
sce <- LunSpikeInData()

sce <- LunSpikeInData("tropho")

}
\references{
Lun ATL et al. (2017). 
Assessing the reliability of spike-in normalization for analyses of single-cell RNA sequencing data. 
\emph{Genome Res.} 27(11), 1795-1806.
}
\author{
Aaron Lun
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/ZhaoImmuneLiverData.R
\name{ZhaoImmuneLiverData}
\alias{ZhaoImmuneLiverData}
\title{Obtain the Zhao immune liver data}
\usage{
ZhaoImmuneLiverData(location = TRUE, filter = FALSE, legacy = FALSE)
}
\arguments{
\item{location}{Logical scalar indicating whether genomic coordinates should be returned.}

\item{filter}{Logical scalar indicating if the filtered subset should be returned.}

\item{legacy}{Logical scalar indicating whether to pull data from ExperimentHub.
By default, we use data from the gypsum backend.}
}
\value{
A \link[SingleCellExperiment]{SingleCellExperiment} object with a single matrix of UMI counts.
}
\description{
Obtain the human liver immune single-cell RNA-seq data from Zhao et al. (2020).
}
\details{
Column metadata contains various cell labels as provided by the authors.
Some of these labels required assembly on our part:
\itemize{
\item The \code{broad} label was assigned to each barcode based on whether that barcode was present in each \code{*_identities.tsv.gz} in GSE125188's supplementary files.
\item For each cell barcode that was present in one of these files, the \code{fine} label was generated from the \code{Group} annotations inside that file.
}
We guessed the \code{sample} for each cell by assuming that the GEM group numbers match the order of samples in GSE125188.
We also assumed that \dQuote{donor 4} is a typo, given that the paper only mentions 3 donors.

If \code{location=TRUE}, the coordinates of the Ensembl gene models are stored in the \code{\link[SummarizedExperiment]{rowRanges}} of the output.
Note that this is only performed if \code{ensembl=TRUE}.

If \code{filter=TRUE}, only cells that have been used in the original analysis are returned.
Otherwise, the cells used are specified in the \code{retained} column of the \code{\link[SummarizedExperiment]{colData}}.

All data are downloaded from ExperimentHub and cached for local re-use.
Specific resources can be retrieved by searching for \code{scRNAseq/zhao-immune-liver}.
}
\examples{
sce.zhao <- ZhaoImmuneLiverData()

}
\references{
Zhao J et al. (2020). 
Single-cell RNA sequencing reveals the heterogeneity of liver-resident immune cells in human.
\emph{Cell Discov} 6, 22.
}
\author{
Aaron Lun
}
